@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/arc.c,v 2.67 1993/04/28 05:06:11 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/arc.c,v 2.79 1994/05/02 00:53:02 william Exp $";
 #endif
 
 #include <math.h>
@@ -33,6 +34,11 @@ static char RCSid[] =
 #include "const.h"
 #include "types.h"
 
+#ifndef _NO_EXTERN
+#include "arc.e"
+#endif
+#include "attr.e"
+#include "choice.e"
 #include "cmd.e"
 #include "color.e"
 #include "cursor.e"
@@ -41,6 +47,8 @@ static char RCSid[] =
 #include "file.e"
 #include "grid.e"
 #include "mainloop.e"
+#include "mark.e"
+#include "msg.e"
 #include "obj.e"
 #include "pattern.e"
 #include "poly.e"
@@ -51,6 +59,8 @@ static char RCSid[] =
 #include "special.e"
 #include "xpixmap.e"
 
+extern int	atoi ARGS_DECL((char *));
+
 #define EXPAND_BBOX(bbox,x,y) \
    if ((x)<(bbox)->ltx) (bbox)->ltx=(x); if ((y)<(bbox)->lty) (bbox)->lty=(y); \
    if ((x)>(bbox)->rbx) (bbox)->rbx=(x); if ((y)>(bbox)->rby) (bbox)->rby=(y)
@@ -424,7 +434,7 @@ void DumpArcArrow (FP,Angle,Dir,X,Y,XRad,YRad,AW,AH,Pen,ColorIndex)
          case SOLIDPAT: fprintf (FP, "   closepath fill\n"); break;
          case BACKPAT: break;
          default:
-            fprintf (FP, "   closepath eoclip\n");
+            fprintf (FP, "   closepath eoclip newpath\n");
             DumpPatFill (FP, Pen, 8, bbox, "   ");
             break;
       }
@@ -478,7 +488,7 @@ void DumpArcPath (FP,ObjPtr,xc,yc,xr,yr,dir,angle1,angle2,width,pen,dash)
       default:
          if (colorDump)
          {
-            fprintf (FP, "   flattenpath strokepath clip\n");
+            fprintf (FP, "   flattenpath strokepath clip newpath\n");
             DumpPatFill (FP, pen, 8, ObjPtr->bbox, "   ");
          }
          else
@@ -562,7 +572,7 @@ void DumpArcObj (FP, ObjPtr)
          DumpArcPSPath (FP,xc,yc,xr,yr,dir,angle1,angle2,TRUE,"   ","      ");
          if (colorDump)
          {
-            fprintf (FP, "   closepath clip\n");
+            fprintf (FP, "   closepath clip newpath\n");
             DumpPatFill (FP, fill, 8, ObjPtr->bbox, "   ");
          }
          else
@@ -822,10 +832,10 @@ void ContinueArc (OrigX, OrigY)
    int	OrigX, OrigY;
 {
    int 			grid_x, grid_y, first_x = 0, first_y = 0;
-   int 			end_x, end_y, saved_x, saved_y;
+   int 			end_x, end_y, saved_x, saved_y, dx, dy, radius;
    int 			done = FALSE, drawing_arc = FALSE;
    int			dir = INVALID, ltx, lty, w, h, angle1, angle2=0;
-   char			msg[80];
+   char			msg[80], buf[80];
    struct ObjRec	* obj_ptr;
    XGCValues		values;
    XEvent		input, ev;
@@ -848,11 +858,15 @@ void ContinueArc (OrigX, OrigY)
    grid_y = saved_y = OrigY; 
    XDrawLine (mainDisplay, drawWindow, drawGC, OrigX, OrigY, saved_x, saved_y);
 
+   SaveStatusStrings ();
+   sprintf (buf, "r=0,[%+1d,%+1d]", ABS_X(grid_x), ABS_Y(grid_y));
+   StartShowMeasureCursor (grid_x, grid_y, buf, TRUE);
    XGrabPointer (mainDisplay, drawWindow, FALSE,
          PointerMotionMask | ButtonPressMask | ButtonReleaseMask,
          GrabModeAsync, GrabModeAsync, None, handCursor, CurrentTime);
    
    Msg ("Please specify the start of an arc.");
+   SetMouseStatus ("Set start of arc", "Cancel", "Cancel");
    while (!done)
    {
       XNextEvent (mainDisplay, &input);
@@ -861,33 +875,92 @@ void ContinueArc (OrigX, OrigY)
          ExposeEventHandler (&input, TRUE);
       else if (input.type == ButtonPress)
       {
-         if (drawing_arc)
+         if (input.xbutton.button != Button1)
          {
             XUngrabPointer (mainDisplay, CurrentTime);
-            XDrawArc (mainDisplay, drawWindow, drawGC, ltx, lty, w, h,
-                  angle1, angle2);
+            if (drawing_arc)
+            {
+               XDrawArc (mainDisplay, drawWindow, drawGC, ltx, lty, w, h,
+                     angle1, angle2);
+               sprintf (buf, "r=%1d,degree=%1d", w>>1, abs(angle2>>6));
+            }
+            else
+            {
+               XDrawLine (mainDisplay, drawWindow, drawGC, OrigX, OrigY,
+                     saved_x, saved_y);
+               dx = saved_x - OrigX;
+               dy = saved_y - OrigY;
+               radius = (int)(sqrt((double)(dx*dx+dy*dy)));
+               w = h = (radius<<1);
+               sprintf (buf, "r=%1d,[%+1d,%+1d]", radius,
+                     ABS_X(saved_x), ABS_Y(saved_y));
+            }
+            EndShowMeasureCursor (saved_x, saved_y, buf, TRUE);
             done = TRUE;
+            angle2 = 0;
             Msg ("");
          }
          else
          {
-            XDrawLine (mainDisplay, drawWindow, drawGC, OrigX, OrigY,
-                  saved_x, saved_y);
-            first_x = saved_x;
-            first_y = saved_y;
-            drawing_arc = TRUE;
-            if (OrigX == grid_x && OrigY == grid_y)
-            {  /* fake it as if the 1st point is ok but the 2nd point is bad */
+            if (drawing_arc)
+            {
                XUngrabPointer (mainDisplay, CurrentTime);
-               grid_x = first_x;
-               grid_y = first_y;
+               XDrawArc (mainDisplay, drawWindow, drawGC, ltx, lty, w, h,
+                     angle1, angle2);
+               sprintf (buf, "r=%1d,degree=%1d", w>>1, abs(angle2>>6));
+               EndShowMeasureCursor (saved_x, saved_y, buf, TRUE);
                done = TRUE;
+               Msg ("");
+            }
+            else
+            {
+               dx = saved_x - OrigX;
+               dy = saved_y - OrigY;
+               radius = (int)(sqrt((double)(dx*dx+dy*dy)));
+               w = h = (radius<<1);
+               sprintf (buf, "r=%1d,[%+1d,%+1d]", radius,
+                     ABS_X(saved_x), ABS_Y(saved_y));
+               ShowMeasureCursor (saved_x, saved_y, buf, TRUE);
+               XDrawLine (mainDisplay, drawWindow, drawGC, OrigX, OrigY,
+                     saved_x, saved_y);
+               first_x = saved_x;
+               first_y = saved_y;
+               drawing_arc = TRUE;
+               if (OrigX == grid_x && OrigY == grid_y)
+               {  /* fake it as if the 1st point is ok but not the 2nd point */
+                  XUngrabPointer (mainDisplay, CurrentTime);
+                  grid_x = first_x;
+                  grid_y = first_y;
+                  done = TRUE;
+               }
+               SetMouseStatus ("Set end of arc", "Cancel", "Cancel");
+               Msg ("Please specify the end of the arc.");
+               sprintf (buf, "r=%1d,degree=0", radius);
+               ShowMeasureCursor (saved_x, saved_y, buf, TRUE);
+               if (done)
+               {
+                  sprintf (buf, "r=%1d,degree=0", radius);
+                  EndShowMeasureCursor (saved_x, saved_y, buf, TRUE);
+               }
             }
-            Msg ("Please specify the end of the arc.");
          }
       }
       else if (input.type == MotionNotify)
       {
+         if (drawing_arc)
+         {
+            sprintf (buf, "r=%1d,degree=%1d", w>>1, abs(angle2>>6));
+            ShowMeasureCursor (saved_x, saved_y, buf, TRUE);
+         }
+         else
+         {
+            dx = saved_x - OrigX;
+            dy = saved_y - OrigY;
+            sprintf (buf, "r=%1d,[%+1d,%+1d]",
+                  (int)(sqrt((double)(dx*dx+dy*dy))),
+                  ABS_X(saved_x), ABS_Y(saved_y));
+            ShowMeasureCursor (saved_x, saved_y, buf, TRUE);
+         }
          motion_ev = &(input.xmotion);
          end_x = motion_ev->x;
          end_y = motion_ev->y;
@@ -928,10 +1001,26 @@ void ContinueArc (OrigX, OrigY)
                      saved_x, saved_y);
             }
          }
+         if (drawing_arc)
+         {
+            sprintf (buf, "r=%1d,degree=%1d", w>>1, abs(angle2>>6));
+            ShowMeasureCursor (saved_x, saved_y, buf, TRUE);
+         }
+         else
+         {
+            dx = saved_x - OrigX;
+            dy = saved_y - OrigY;
+            sprintf (buf, "r=%1d,[%+1d,%+1d]",
+                  (int)(sqrt((double)(dx*dx+dy*dy))),
+                  ABS_X(saved_x), ABS_Y(saved_y));
+            ShowMeasureCursor (saved_x, saved_y, buf, TRUE);
+         }
          MarkRulers (grid_x, grid_y);
          while (XCheckMaskEvent (mainDisplay, PointerMotionMask, &ev)) ;
       }
    }
+   RestoreStatusStrings ();
+   SetMouseStatus (NULL, NULL, NULL);
    if (angle2 == 0)
       Msg ("No arc created.");
    else
@@ -970,20 +1059,22 @@ void SaveArcObj (FP, ObjPtr)
 {
    register struct ArcRec	* arc_ptr = ObjPtr->detail.a;
 
-   fprintf (FP, "arc('%s',", colorMenuItems[ObjPtr->color]);
-   fprintf (FP, "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,\
+   if (fprintf (FP, "arc('%s',", colorMenuItems[ObjPtr->color]) == EOF)
+      writeFileFailed = TRUE;
+   if (fprintf (FP, "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,\
 %1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,",
          arc_ptr->fill, arc_ptr->width, arc_ptr->pen, arc_ptr->dash,
          arc_ptr->ltx, arc_ptr->lty, arc_ptr->xc, arc_ptr->yc,
          arc_ptr->x1, arc_ptr->y1, arc_ptr->x2, arc_ptr->y2,
          arc_ptr->dir, arc_ptr->w, arc_ptr->h, arc_ptr->angle1, arc_ptr->angle2,
          ObjPtr->id, ObjPtr->rotation, arc_ptr->style, arc_ptr->aw,
-         arc_ptr->ah, ObjPtr->locked);
+         arc_ptr->ah, ObjPtr->locked) == EOF)
+      writeFileFailed = TRUE;
    SaveAttrs (FP, ObjPtr->lattr);
-   fprintf (FP, ")");
+   if (fprintf (FP, ")") == EOF) writeFileFailed = TRUE;
 }
 
-#define GETVALUE(val,name) ScanValue("%d", (char *) &(val), name, "arc")
+#define GETVALUE(val,name) ScanValue("%d", &(val), name, "arc")
 
 void ReadArcObj (Inbuf, ObjPtr)
    char			* Inbuf;
