@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/attr.c,v 2.64 1993/05/27 04:52:07 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/attr.c,v 2.77 1994/04/04 00:49:06 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -35,7 +36,11 @@ static char RCSid[] =
 #include "const.h"
 #include "types.h"
 
+#ifndef _NO_EXTERN
+#include "attr.e"
+#endif
 #include "auxtext.e"
+#include "button.e"
 #include "choice.e"
 #include "cmd.e"
 #include "color.e"
@@ -45,14 +50,18 @@ static char RCSid[] =
 #include "dup.e"
 #include "file.e"
 #include "font.e"
+#include "grid.e"
 #include "mainloop.e"
 #include "mainmenu.e"
 #include "mark.e"
 #include "menu.e"
+#include "move.e"
 #include "msg.e"
 #include "names.e"
 #include "obj.e"
 #include "raster.e"
+#include "rect.e"
+#include "ruler.e"
 #include "select.e"
 #include "setup.e"
 #include "stk.e"
@@ -180,18 +189,13 @@ void UpdateAttr (TextPtr, AttrPtr)
       strcpy (AttrPtr->s, s);
    }
 
-   if (!(AttrPtr->shown))
-      TextPtr->first->s[0] = '\0';
-   else
+   if (AttrPtr->nameshown)
    {
-      if (AttrPtr->nameshown)
-      {
-         strcat (name, s);
-         strcpy (TextPtr->first->s, name);
-      }
-      else
-          strcpy (TextPtr->first->s, s);
+      strcat (name, s);
+      strcpy (TextPtr->first->s, name);
    }
+   else
+       strcpy (TextPtr->first->s, s);
    UpdTextBBox (AttrPtr->obj);
 }
 
@@ -417,15 +421,16 @@ void SaveAttr (FP, AttrPtr)
    FILE			* FP;
    struct AttrRec	* AttrPtr;
 {
-   fprintf (FP, "attr(\"");
+   if (fprintf (FP, "attr(\"") == EOF) writeFileFailed = TRUE;
    SaveString (FP, AttrPtr->name);
-   fprintf (FP, "\", \"");
+   if (fprintf (FP, "\", \"") == EOF) writeFileFailed = TRUE;
    SaveString (FP, AttrPtr->s);
-   fprintf (FP, "\", %1d, %1d, %1d,\n",
-         AttrPtr->shown, AttrPtr->nameshown, AttrPtr->inherited);
+   if (fprintf (FP, "\", %1d, %1d, %1d,\n",
+         AttrPtr->shown, AttrPtr->nameshown, AttrPtr->inherited) == EOF)
+      writeFileFailed = TRUE;
   
    SaveTextObj (FP, AttrPtr->obj);
-   fprintf (FP, ")");         
+   if (fprintf (FP, ")") == EOF) writeFileFailed = TRUE;
 }
 
 void SaveAttrs (FP, BotAttrPtr)
@@ -435,19 +440,24 @@ void SaveAttrs (FP, BotAttrPtr)
  
    struct AttrRec       * ptr;
 
-   fprintf (FP, "[\n");
+   if (fprintf (FP, "[\n") == EOF) writeFileFailed = TRUE;
 
    for (ptr = BotAttrPtr; ptr != NULL; ptr = ptr->prev)
-     {
-       SaveAttr (FP, ptr);
-       if (ptr->prev != NULL)
-         fprintf (FP, ",\n");
-     }
+   {
+      SaveAttr (FP, ptr);
+      if (ptr->prev != NULL)
+         if (fprintf (FP, ",\n") == EOF)
+            writeFileFailed = TRUE;
+   }
 
    if (BotAttrPtr == NULL)
-      fprintf (FP, "]");
+   {
+      if (fprintf (FP, "]") == EOF) writeFileFailed = TRUE;
+   }
    else
-      fprintf (FP, "\n]");
+   {
+      if (fprintf (FP, "\n]") == EOF) writeFileFailed = TRUE;
+   }
 }
 
 static
@@ -471,7 +481,7 @@ char * ReadAttrString (Str)
    return (s);
 }
 
-#define GETVALUE(val,name) ScanValue("%d", (char *) &(val), name, "attribute")
+#define GETVALUE(val,name) ScanValue("%d", &(val), name, "attribute")
 
 int ReadAttr (FP, AttrPtr)
    FILE			* FP;
@@ -1091,10 +1101,13 @@ int MoveOneAttr (ObjPtr, AttrPtr)
    int          	x, y, grid_x, grid_y, dx, dy, placing = TRUE;
    int          	ltx, lty, rbx, rby;
    int			orig_x, orig_y, grid_orig_x, grid_orig_y;
+   char			buf[80];
    XEvent		input, ev;
 
    text_obj_ptr = AttrPtr->obj;
    Msg ("LEFT--show and move.  MIDDLE--toggle name shown.  RIGHT--hide attr.");
+   SetMouseStatus ("Show/move an attribute", "Toggle name shown",
+         "Hide an attribute");
 
    orig_x = OFFSET_X(text_obj_ptr->x);
    orig_y = OFFSET_Y(text_obj_ptr->y);
@@ -1113,15 +1126,21 @@ int MoveOneAttr (ObjPtr, AttrPtr)
    grid_x = grid_orig_x; grid_y = grid_orig_y;
 
    SelBox (drawWindow, revDefaultGC, ltx+dx, lty+dy, rbx+dx, rby+dy);
+   sprintf (buf, "dx=+0,dy=+0");
+   StartShowMeasureCursor (ltx, lty-defaultFontHeight, buf, TRUE);
    while (placing)
    {
       XNextEvent (mainDisplay, &input);
 
       if (input.type == Expose || input.type == VisibilityNotify)
       {
+         sprintf (buf, "dx=%+1d,dy=%+1d", ABS_SIZE(dx), ABS_SIZE(dy));
+         ShowMeasureCursor (ltx+dx, lty-defaultFontHeight+dy, buf, TRUE);
          SelBox (drawWindow, revDefaultGC, ltx+dx, lty+dy, rbx+dx, rby+dy);
          ExposeEventHandler (&input, TRUE);
          SelBox (drawWindow, revDefaultGC, ltx+dx, lty+dy, rbx+dx, rby+dy);
+         sprintf (buf, "dx=%+1d,dy=%+1d", ABS_SIZE(dx), ABS_SIZE(dy));
+         ShowMeasureCursor (ltx+dx, lty-defaultFontHeight+dy, buf, TRUE);
       }
       else if (input.type == MotionNotify)
       {
@@ -1131,10 +1150,14 @@ int MoveOneAttr (ObjPtr, AttrPtr)
 
          if (grid_x != grid_orig_x+dx || grid_y != grid_orig_y+dy)
          {
+            sprintf (buf, "dx=%+1d,dy=%+1d", ABS_SIZE(dx), ABS_SIZE(dy));
+            ShowMeasureCursor (ltx+dx, lty-defaultFontHeight+dy, buf, TRUE);
             SelBox (drawWindow, revDefaultGC, ltx+dx, lty+dy, rbx+dx, rby+dy);
             dx = grid_x - grid_orig_x;
             dy = grid_y - grid_orig_y;
             SelBox (drawWindow, revDefaultGC, ltx+dx, lty+dy, rbx+dx, rby+dy);
+            sprintf (buf, "dx=%+1d,dy=%+1d", ABS_SIZE(dx), ABS_SIZE(dy));
+            ShowMeasureCursor (ltx+dx, lty-defaultFontHeight+dy, buf, TRUE);
             MarkRulers (grid_x, grid_y);
          }
          while (XCheckMaskEvent (mainDisplay, PointerMotionMask, &ev)) ;
@@ -1144,7 +1167,10 @@ int MoveOneAttr (ObjPtr, AttrPtr)
          XUngrabPointer (mainDisplay, CurrentTime);
          placing = FALSE;
          SelBox (drawWindow, revDefaultGC, ltx+dx, lty+dy, rbx+dx, rby+dy);
+         sprintf (buf, "dx=%+1d,dy=%+1d", ABS_SIZE(dx), ABS_SIZE(dy));
+         EndShowMeasureCursor (ltx+dx, lty-defaultFontHeight+dy, buf, TRUE);
          Msg ("");
+         SetMouseStatus ("", "", "");
          switch (input.xbutton.button)
          {
             case Button1:
@@ -1218,7 +1244,8 @@ int ChangeAttrJust (ObjPtr, AttrPtr)
    XEvent		input, ev;
 
    text_obj_ptr = AttrPtr->obj;
-   Msg ("LEFT--left, MIDDLE--center, RIGHT--right justified.");
+   SetMouseStatus ("Left justify an attribute", "Center justify an attribute",
+         "Right justify an attribute");
 
    orig_x = OFFSET_X(text_obj_ptr->x);
    orig_y = OFFSET_Y(text_obj_ptr->y);
@@ -1237,6 +1264,7 @@ int ChangeAttrJust (ObjPtr, AttrPtr)
    grid_x = grid_orig_x; grid_y = grid_orig_y;
 
    SelBox (drawWindow, revDefaultGC, ltx+dx, lty+dy, rbx+dx, rby+dy);
+   StartShowMeasureCursor (ltx, lty, NULL, TRUE);
    while (placing)
    {
       XNextEvent (mainDisplay, &input);
@@ -1268,7 +1296,9 @@ int ChangeAttrJust (ObjPtr, AttrPtr)
          XUngrabPointer (mainDisplay, CurrentTime);
          placing = FALSE;
          SelBox (drawWindow, revDefaultGC, ltx+dx, lty+dy, rbx+dx, rby+dy);
+         EndShowMeasureCursor (ltx+dx, lty+dy, NULL, TRUE);
          Msg ("");
+         SetMouseStatus ("", "", "");
          old_just = text_obj_ptr->detail.t->just;
          switch (input.xbutton.button)
          {
@@ -1296,7 +1326,7 @@ void MoveAttr ()
 {
    struct ObjRec	* obj_ptr;
    struct AttrRec	* attr_ptr, * attr_ptr1;
-   int			num_attrs = 0, i, index, x, y;
+   int			num_attrs = 0, i, index=INVALID, x, y;
    int			sel_ltx, sel_lty, sel_rbx, sel_rby;
    int			* fore_colors, * pixel_ptr, * valid, * flag_ptr;
    int			len1, len2, * init_rv;
@@ -1342,13 +1372,16 @@ void MoveAttr ()
    }
 
    Msg ("Hold down left button to see attributes.");
-   strcpy (buf, "Left button move/see attributes.  ");
-   strcat (buf, "Middle button change attribute justifications.");
+   strcpy (buf, "Left button show/move an attribute.  ");
+   strcat (buf, "Middle button change an attribute justification.");
    Msg (buf);
+   SetMouseStatus ("Show/move an attribute",
+         "Change an attribute justification", "(none)");
    button = CornerLoop (&x, &y);
    activeMenu = INVALID;
-   index = TextMenuLoop (x, y, attrStrs, num_attrs, fore_colors, valid, init_rv,
-         MULTICOLOR);
+   if (button == Button1 || button == Button2)
+      index = TextMenuLoop (x, y, attrStrs, num_attrs, fore_colors, valid,
+            init_rv, NULL, MULTICOLOR, FALSE);
    if (index != INVALID)
    {
       attr_ptr1 = attr_ptr;
@@ -1604,29 +1637,45 @@ static XComposeStatus	c_stat;
 static DspList		* fileAttrNameDspPtr=NULL;
 
 static
-DspList * FileAttrNameListing ()
+DspList * FileAttrNameListing (ObjPtr)
+   struct ObjRec	* ObjPtr;
 {
    register int		i;
    DspList		* dsp_ptr;
    struct AttrRec	* attr_ptr;
 
-   for (nameEntries=0, attr_ptr=tgifObj->fattr; attr_ptr!=NULL;
+   for (nameEntries=0, attr_ptr=ObjPtr->fattr; attr_ptr!=NULL;
          attr_ptr=attr_ptr->next)
       nameEntries++;
 
    fileAttrNameDspPtr = (DspList *) calloc (nameEntries, sizeof(DspList));
-   for (i=1, dsp_ptr=fileAttrNameDspPtr, attr_ptr=tgifObj->fattr;
+   for (i=1, dsp_ptr=fileAttrNameDspPtr, attr_ptr=ObjPtr->fattr;
          i<=nameEntries; i++, dsp_ptr++, attr_ptr=attr_ptr->next)
    {
       strcpy (dsp_ptr->itemstr, attr_ptr->obj->detail.t->first->s);
+      /* use the directory field for inherited */
+      dsp_ptr->directory = attr_ptr->inherited;
       dsp_ptr->next = ((i == nameEntries) ? NULL : &dsp_ptr[1]);
    }
    return (fileAttrNameDspPtr);
 }
 
 static
-int EditFileAttrNames (TopStr)
-   char	* TopStr;
+int IsFirstEqChar (s, index)
+   char	* s;
+   int	index;
+{
+   s = &s[index];
+   if (*s != '=') return (FALSE);
+   for (--index, --s; index >= 0; index--, s--)
+      if (*s == '=') return (FALSE);
+   return (TRUE);
+}
+
+static
+int EditAttrNames (TopStr, ObjAttrDspList)
+   char		* TopStr;
+   DspList	* ObjAttrDspList;
 {
    int			button_widths, str_width, graph_width;
    int			str_start, button_start, graph_start;
@@ -1834,7 +1883,10 @@ int EditFileAttrNames (TopStr)
                (buf[0] == '\177' && (key_sym & 0x7f)=='\177') ||
                key_sym==XK_Left)
          {
-            if (nameMarked != INVALID && name_index != 0)
+            if (nameMarked != INVALID && name_index != 0 &&
+                  (ObjAttrDspList == NULL ||
+                  !ObjAttrDspList[nameMarked].directory ||
+                  !IsFirstEqChar (name, name_index-1)))
             {
                name[--name_index] = '\0';
                if (nameMarked != INVALID)
@@ -1967,11 +2019,13 @@ void EditFileAttrs ()
             "( <CR> or <ESC> to continue )", attrDummyStr);
       return;
    }
-   dsp_ptr = FileAttrNameListing ();
+   dsp_ptr = FileAttrNameListing (tgifObj);
+   ignoreDirectoryFlagInMakeNameDspItemArray = TRUE;
    nameDspPtr = MakeNameDspItemArray (nameEntries, dsp_ptr);
+   ignoreDirectoryFlagInMakeNameDspItemArray = FALSE;
    nameFirst = 0;
    nameMarked = 0;
-   if (EditFileAttrNames ("Edit File Attributes..."))
+   if (EditAttrNames ("Edit File Attributes...", NULL))
    {
       int		modified=FALSE;
       struct AttrRec	* attr_ptr, * next_attr;
@@ -2023,3 +2077,86 @@ void EditFileAttrs ()
    nameDspPtr = NULL;
    Msg ("");
 }
+
+void EditAttrs ()
+{
+   register int	i;
+   DspList	* dsp_ptr;
+
+   if (topSel == NULL || topSel != botSel)
+   {
+      Dialog ("Must select one object for EditAttrs().",
+            "( <CR> or <ESC> to continue )", attrDummyStr);
+      return;
+   }
+   if (topSel->obj->fattr == NULL)
+   {
+      Dialog ("Selected object contains no attributes.",
+            "( <CR> or <ESC> to continue )", attrDummyStr);
+      return;
+   }
+   dsp_ptr = FileAttrNameListing (topSel->obj);
+   ignoreDirectoryFlagInMakeNameDspItemArray = TRUE;
+   nameDspPtr = MakeNameDspItemArray (nameEntries, dsp_ptr);
+   ignoreDirectoryFlagInMakeNameDspItemArray = FALSE;
+   nameFirst = 0;
+   nameMarked = 0;
+   if (EditAttrNames ("Edit Object Attributes...", dsp_ptr))
+   {
+      int		modified=FALSE;
+      int		sel_ltx, sel_lty, sel_rbx, sel_rby;
+      struct AttrRec	* attr_ptr, * next_attr;
+
+      sel_ltx = selLtX; sel_lty = selLtY; sel_rbx = selRbX; sel_rby = selRbY;
+      HighLightReverse ();
+      PrepareToReplaceAnObj (topSel->obj);
+
+      for (attr_ptr=topSel->obj->fattr, i=0; attr_ptr!=NULL;
+            attr_ptr=next_attr, i++)
+      {
+         int	blank_str=BlankStr(nameDspPtr[i]);
+
+         next_attr = attr_ptr->next;
+         if (blank_str &&
+               attr_ptr->obj->detail.t->first==attr_ptr->obj->detail.t->last ||
+               strcmp (attr_ptr->obj->detail.t->first->s, nameDspPtr[i]) != 0)
+         {
+            modified = TRUE;
+            if (blank_str &&
+                  attr_ptr->obj->detail.t->first==attr_ptr->obj->detail.t->last)
+            {
+               UnlinkAttr (attr_ptr);
+               FreeTextObj (attr_ptr->obj);
+               FreeAttr (attr_ptr);
+            }
+            else
+            {
+               strcpy (attr_ptr->obj->detail.t->first->s, nameDspPtr[i]);
+               UpdateAttr (attr_ptr->obj->detail.t, attr_ptr);
+            }
+         }
+      }
+      if (modified)
+      {
+         AdjObjBBox (topSel->obj);
+         RecordReplaceAnObj (topSel->obj);
+         UpdSelBBox ();
+         RedrawAreas (botObj, sel_ltx-GRID_ABS_SIZE(1),
+                  sel_lty-GRID_ABS_SIZE(1),
+                  sel_rbx+GRID_ABS_SIZE(1), sel_rby+GRID_ABS_SIZE(1),
+                  selLtX-GRID_ABS_SIZE(1), selLtY-GRID_ABS_SIZE(1),
+                  selRbX+GRID_ABS_SIZE(1), selRbY+GRID_ABS_SIZE(1));
+         SetFileModified (TRUE);
+         justDupped = FALSE;
+         Msg ("Object attribute updated.");
+      }
+      else
+         AbortPrepareCmd (CMD_REPLACE);
+      HighLightForward ();
+   }
+   cfree (dsp_ptr);
+   cfree (*nameDspPtr);
+   cfree (nameDspPtr);
+   nameDspPtr = NULL;
+   Msg ("");
+}
