@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/auxtext.c,v 2.35 1993/04/28 06:25:34 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/auxtext.c,v 2.50 1994/03/21 04:53:29 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -34,6 +35,9 @@ static char RCSid[] =
 #include "const.h"
 #include "types.h"
 
+#ifndef _NO_EXTERN
+#include "auxtext.e"
+#endif
 #include "color.e"
 #include "file.e"
 #include "font.e"
@@ -44,6 +48,7 @@ static char RCSid[] =
 #include "raster.e"
 #include "setup.e"
 #include "text.e"
+#include "xbitmap.e"
 #include "xpixmap.e"
 
 GC	rotateGC = NULL;
@@ -55,6 +60,20 @@ int	groupedTextEditable = FALSE;
 
 struct SelRec	* outerSel=NULL, * innerSel=NULL;
 
+int TextWidth (xfs, s, len, lextra, rextra)
+   XFontStruct	* xfs;
+   char		* s;
+   int		len, * lextra, * rextra;
+{
+   int		dir, asc, des;
+   XCharStruct	xcs;
+
+   XTextExtents (canvasFontPtr, s, len, &dir, &asc, &des, &xcs);
+   if (lextra != NULL) *lextra = (-xcs.lbearing);
+   if (rextra != NULL) *rextra = (xcs.rbearing-xcs.width);
+   return (xcs.width);
+}
+
 void TranslateKeys (s, key_sym)
    char		* s;
    KeySym	* key_sym;
@@ -178,17 +197,30 @@ int UpdTextBBox (ObjPtr)
 {
    register int		num_lines;
    struct StrRec	* s_ptr = ObjPtr->detail.t->first;
-   int			max_len = 0, w;
+   int			max_len=0, w;
+
+   if (ObjPtr->detail.t->read_only)
+   {
+      SetTextBBox (ObjPtr, ObjPtr->detail.t->just, ObjPtr->detail.t->orig_w,
+            ObjPtr->detail.t->orig_h, ObjPtr->detail.t->rotate);
+      return (TRUE);
+   }
 
    SaveCurFont ();
    curFont = ObjPtr->detail.t->font;
-   curFontDPI = ObjPtr->detail.t->dpi;
    curStyle = ObjPtr->detail.t->style;
    curSize = ObjPtr->detail.t->size;
    textJust = ObjPtr->detail.t->just;
    textVSpace = ObjPtr->detail.t->v_space;
+   attemptingToSetFontProperty = TRUE;
    SetCanvasFont ();
+   attemptingToSetFontProperty = FALSE;
 
+   if (canvasFontSize == INVALID || curSize != canvasFontSize)
+   {
+      RestoreCurFont ();
+      return (INVALID);
+   }
    if (textCursorH+textVSpace <= 0)
    {
       RestoreCurFont ();
@@ -250,17 +282,19 @@ int PixelOnOff (image, col_start, row_start, scale)
 void MakeCachedTextBitmap (ObjPtr)
    struct ObjRec	* ObjPtr;
 {
-   register int		m, n, r, c;
-   int			w=0, h=0, num_cols=0, num_rows=0, scale;
-   int			yinc, y, rotate;
-   int			flat=FALSE, skinny=FALSE;
+   register int		r, c;
+   int			w=0, h=0, num_cols=0, num_rows=0;
+   int			yinc, y, rotate, max_str_w=0, str_w;
+   int			flat=FALSE, skinny=FALSE, start_row, start_col;
    char			* s, msg[MAXSTRING];
+   struct MtrxRec	mtrx;
    Pixmap		dest_bitmap;
    XImage		* src_image, * dest_image;
    struct StrRec	* s_ptr;
    struct TextRec	* text_ptr = ObjPtr->detail.t;
    XGCValues		values;
 
+
    if (text_ptr->cached_bitmap!=None && text_ptr->cached_zoomed==zoomedIn &&
          text_ptr->cached_zoom==zoomScale && (text_ptr->rotate==ROTATE0 ||
          text_ptr->cached_rotate==text_ptr->rotate))
@@ -278,17 +312,43 @@ void MakeCachedTextBitmap (ObjPtr)
 
    rotate = text_ptr->rotate;
 
+   if (text_ptr->read_only)
+   {
+      max_str_w = 0;
+      for (s_ptr = text_ptr->first; s_ptr != NULL; s_ptr = s_ptr->next)
+      {
+         str_w = XTextWidth (canvasFontPtr, s_ptr->s, strlen (s_ptr->s));
+         if (str_w > max_str_w) max_str_w = str_w;
+      }
+   }
+
    switch (rotate)
    {
       case ROTATE0:
       case ROTATE180:
-         w = ObjPtr->obbox.rbx - ObjPtr->obbox.ltx;
-         h = ObjPtr->obbox.rby - ObjPtr->obbox.lty;
+         if (text_ptr->read_only)
+         {
+            w = max_str_w;
+            h = text_ptr->lines*textCursorH+(text_ptr->lines-1)*textVSpace;
+         }
+         else
+         {
+            w = ObjPtr->obbox.rbx - ObjPtr->obbox.ltx;
+            h = ObjPtr->obbox.rby - ObjPtr->obbox.lty;
+         }
          break;
       case ROTATE90:
       case ROTATE270:
-         w = ObjPtr->obbox.rby - ObjPtr->obbox.lty;
-         h = ObjPtr->obbox.rbx - ObjPtr->obbox.ltx;
+         if (text_ptr->read_only)
+         {
+            w = max_str_w;
+            h = text_ptr->lines*textCursorH+(text_ptr->lines-1)*textVSpace;
+         }
+         else
+         {
+            w = ObjPtr->obbox.rby - ObjPtr->obbox.lty;
+            h = ObjPtr->obbox.rbx - ObjPtr->obbox.ltx;
+         }
          break;
    }
 
@@ -342,49 +402,38 @@ void MakeCachedTextBitmap (ObjPtr)
    switch (rotate)
    {
       case ROTATE0:
-      case ROTATE180: num_cols=w; num_rows=h; break;
+      case ROTATE180:
+         if (text_ptr->read_only)
+         {
+            num_cols=ObjPtr->obbox.rbx - ObjPtr->obbox.ltx;
+            num_rows=ObjPtr->obbox.rby - ObjPtr->obbox.lty;
+         }
+         else
+         {
+            num_cols=w; num_rows=h;
+         }
+         break;
 
       case ROTATE90:
-      case ROTATE270: num_cols=h; num_rows=w; break;
+      case ROTATE270:
+         if (text_ptr->read_only)
+         {
+            num_cols=ObjPtr->obbox.rbx - ObjPtr->obbox.ltx;
+            num_rows=ObjPtr->obbox.rby - ObjPtr->obbox.lty;
+         }
+         else
+         {
+            num_cols=h; num_rows=w;
+         }
+         break;
    }
    if (zoomedIn || zoomScale == 0)
    {
-      scale = 1<<zoomScale;
       num_cols <<= zoomScale;
       num_rows <<= zoomScale;
-      dest_bitmap = XCreatePixmap (mainDisplay,mainWindow,num_cols,num_rows,1);
-      XSetForeground (mainDisplay, rotateGC, 0);
-      XFillRectangle (mainDisplay,dest_bitmap,rotateGC,0,0,num_cols,num_rows);
-      dest_image = XGetImage (mainDisplay, dest_bitmap, 0, 0, num_cols,
-            num_rows, 1, ZPixmap);
-
-      for (r = 0; r < h; r++)
-         for (c = 0; c < w; c++)
-            if (XGetPixel (src_image,c,r) == 1)
-            {
-               for (m = 0; m < scale; m++)
-                  for (n = 0; n < scale; n++)
-                     switch (rotate)
-                     {
-                        case ROTATE0:
-                           XPutPixel (dest_image,c*scale+m,r*scale+n,1);
-                           break;
-                        case ROTATE90:
-                           XPutPixel (dest_image,(h-r-1)*scale+m,c*scale+n,1);
-                           break;
-                        case ROTATE180:
-                           XPutPixel (dest_image,
-                                 (w-c-1)*scale+m,(h-r-1)*scale+n,1);
-                           break;
-                        case ROTATE270:
-                           XPutPixel (dest_image,r*scale+m,(w-c-1)*scale+n,1);
-                           break;
-                     }
-            }
    }
    else
    {
-      scale = 1<<zoomScale;
       num_cols >>= zoomScale;
       num_rows >>= zoomScale;
       if (num_cols == 0)
@@ -397,52 +446,41 @@ void MakeCachedTextBitmap (ObjPtr)
          flat = TRUE;
          num_rows = 1;
       }
-      dest_bitmap = XCreatePixmap (mainDisplay,mainWindow,num_cols,num_rows,1);
-      XSetForeground (mainDisplay, rotateGC, 0);
-      XFillRectangle (mainDisplay,dest_bitmap,rotateGC,0,0,num_cols,num_rows);
-      dest_image = XGetImage (mainDisplay, dest_bitmap, 0, 0, num_cols,
-            num_rows, 1, ZPixmap);
+   }
+   dest_bitmap = XCreatePixmap (mainDisplay,mainWindow,num_cols,num_rows,1);
+   XSetForeground (mainDisplay, rotateGC, 0);
+   XFillRectangle (mainDisplay,dest_bitmap,rotateGC,0,0,num_cols,num_rows);
+   dest_image = XGetImage (mainDisplay, dest_bitmap, 0, 0, num_cols,
+         num_rows, 1, ZPixmap);
 
-      if (!flat && !skinny)
+   if (!flat && !skinny)
+   {
+      mtrx.image_w = (float)w;
+      mtrx.image_h = (float)h;
+      mtrx.w = (float)num_cols;
+      mtrx.h = (float)num_rows;
+      mtrx.rotate = rotate;
+      mtrx.flip = NO_FLIP;
+
+      CalcTransform (&mtrx);
+
+      start_col = (mtrx.transformed_w >= 0.0) ? 0 : (-num_cols)+1;
+      start_row = (mtrx.transformed_h >= 0.0) ? 0 : (-num_rows)+1;
+
+      for (r = 0; r < num_rows; r++)
       {
-         int	zoomed_text_w=0, zoomed_text_h=0;
+         float	part_x, part_y;
 
-         switch (rotate)
+         part_x = (r+start_row)*mtrx.rev_m[1][0];
+         part_y = (r+start_row)*mtrx.rev_m[1][1];
+         for (c = 0; c < num_cols; c++)
          {
-            case ROTATE0:
-            case ROTATE180:
-               zoomed_text_w = num_cols; zoomed_text_h = num_rows;
-               break;
+            int	x, y;
 
-            case ROTATE90:
-            case ROTATE270:
-               zoomed_text_w = num_rows; zoomed_text_h = num_cols;
-               break;
-         }
-
-         for (r = 0; r < zoomed_text_h; r++)
-         {
-            for (c = 0; c < zoomed_text_w; c++)
-            {
-               if (PixelOnOff (src_image,scale*c,scale*r,scale) == 1)
-               {
-                  switch (rotate)
-                  {
-                     case ROTATE0:
-                        XPutPixel (dest_image,c,r,1);
-                        break;
-                     case ROTATE90:
-                        XPutPixel (dest_image,num_cols-r-1,c,1);
-                        break;
-                     case ROTATE180:
-                        XPutPixel (dest_image,num_cols-c-1,num_rows-r-1,1);
-                        break;
-                     case ROTATE270:
-                        XPutPixel (dest_image,r,num_rows-c-1,1);
-                        break;
-                  }
-               }
-            }
+            x = (int)((c+start_col)*mtrx.rev_m[0][0]+part_x);
+            y = (int)((c+start_col)*mtrx.rev_m[0][1]+part_y);
+            if (x>=0 && x<w && y>=0 && y<h && XGetPixel(src_image,x,y)==1)
+               XPutPixel (dest_image, c, r, 1);
          }
       }
    }
@@ -700,7 +738,7 @@ void DumpTextPath (FP, ObjPtr, TextPtr)
             case JUST_C: fprintf(FP,") tgifcentertext "); break;
             case JUST_R: fprintf(FP,") tgifrighttext "); break;
          }
-         fprintf(FP,"true charpath clip\n");
+         fprintf(FP,"true charpath clip newpath\n");
       }
       else
       {
@@ -736,7 +774,7 @@ void DumpTextObj (FP, ObjPtr)
    FILE				* FP;
    register struct ObjRec	* ObjPtr;
 {
-   int			font_size, color_index, fill;
+   int			color_index, fill;
    int			ltx, lty, rbx, rby;
    char			font_str[MAXSTRING], msg[MAXSTRING];
    struct TextRec	* text_ptr = ObjPtr->detail.t;
@@ -747,7 +785,6 @@ void DumpTextObj (FP, ObjPtr)
    if (!PRTGIF) SaveCurFont ();
 
    curFont = text_ptr->font;
-   curFontDPI = text_ptr->dpi;
    curStyle = text_ptr->style;
    curSize = text_ptr->size;
    textJust = text_ptr->just;
@@ -757,12 +794,11 @@ void DumpTextObj (FP, ObjPtr)
 
    fill = text_ptr->fill;
 
-   if (PRTGIF)
+   if (PRTGIF || text_ptr->read_only)
    {
       canvasFontAsc = text_ptr->asc;
       canvasFontDes = text_ptr->des;
       textCursorH = canvasFontAsc + canvasFontDes;
-      pointSize = (curFontDPI == FONT_DPI_75) ? pointSize75 : pointSize100;
    }
    else
       SetCanvasFont ();
@@ -838,7 +874,7 @@ void DumpTextObj (FP, ObjPtr)
          fprintf (FP, "%1d %1d lineto\n", ltx, rby);
          if (colorDump)
          {
-            fprintf (FP, "   closepath eoclip\n");
+            fprintf (FP, "   closepath eoclip newpath\n");
             DumpPatFill (FP, fill, 8, ObjPtr->bbox, "   ");
          }
          else
@@ -868,7 +904,10 @@ void DumpTextObj (FP, ObjPtr)
       }
    }
 
-   GetPSFontStr (curFont, curStyle, font_str);
+   if (PRTGIF && text_ptr->font_name != NULL)
+      sprintf (font_str, "/%s", text_ptr->font_name);
+   else
+      GetPSFontStr (curFont, curStyle, font_str);
    fprintf (FP, "%s", font_str); 
 
    if (NeedEncode (curFont, curStyle))
@@ -876,9 +915,10 @@ void DumpTextObj (FP, ObjPtr)
    else
       fprintf (FP, " "); 
 
-   font_size = pointSize[curSize];
+   UpdateDocumentFonts (&font_str[1]);
+
    fprintf (FP, "findfont [%1d 0 0 -%1d 0 0] makefont setfont\n",
-         font_size, font_size);
+         curSize, curSize);
 
    if (colorDump && penPat > BACKPAT)
    {
@@ -906,18 +946,26 @@ void SaveString (FP, S)
    for ( ; *S != '\0'; S++)
    {
       if (*S == '\\')
-         fprintf (FP, "%s", "\\\\");
+      {
+         if (fprintf (FP, "%s", "\\\\") == EOF) writeFileFailed = TRUE;
+      }
       else if (*S == '"')
       {
          if (doubleQuoteDoubleQuote)
-            fprintf (FP, "%s", "\"\"");
+         {
+            if (fprintf (FP, "%s", "\"\"") == EOF) writeFileFailed = TRUE;
+         }
          else
-            fprintf (FP, "%s", "\\\"");
+         {
+            if (fprintf (FP, "%s", "\\\"") == EOF) writeFileFailed = TRUE;
+         }
       }
       else if ((*S) & 0x80)
-         fprintf (FP, "\\%o", (*S)&0xff);
+      {
+         if (fprintf (FP, "\\%o", (*S)&0xff) == EOF) writeFileFailed = TRUE;
+      }
       else
-         putc (*S, FP);
+         if (putc (*S, FP) == EOF) writeFileFailed = TRUE;
    }
 }
 
@@ -927,27 +975,32 @@ void SaveTextObj (FP, ObjPtr)
 {
    register struct TextRec	* text_ptr = ObjPtr->detail.t;
    register struct StrRec	* s_ptr;
+   char				font_str[MAXSTRING];
 
-   fprintf (FP, "text('%s',", colorMenuItems[ObjPtr->color]);
-   fprintf (FP,
-         "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,[\n",
-         ObjPtr->x, ObjPtr->y, text_ptr->font, text_ptr->style, text_ptr->size,
+   GetPSFontStr (text_ptr->font, text_ptr->style, font_str);
+         /* font_str starts with the '/' character */
+   if (fprintf (FP, "text('%s',", colorMenuItems[ObjPtr->color]) == EOF)
+      writeFileFailed = TRUE;
+   if (fprintf (FP,
+         "%1d,%1d,'%s',%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,[\n",
+         ObjPtr->x, ObjPtr->y, &font_str[1], text_ptr->style, text_ptr->size,
          text_ptr->lines, text_ptr->just, text_ptr->rotate, text_ptr->pen,
          ObjPtr->obbox.rbx-ObjPtr->obbox.ltx,
-         ObjPtr->obbox.rby-ObjPtr->obbox.lty, ObjPtr->id, text_ptr->dpi,
+         ObjPtr->obbox.rby-ObjPtr->obbox.lty, ObjPtr->id, 0, /* dummy dpi */
          text_ptr->asc, text_ptr->des, text_ptr->fill, text_ptr->v_space,
-         ObjPtr->rotation, ObjPtr->locked);
+         ObjPtr->rotation, ObjPtr->locked) == EOF)
+      writeFileFailed = TRUE;
 
    for (s_ptr = text_ptr->first; s_ptr->next != NULL; s_ptr = s_ptr->next)
    {
-      fprintf (FP, "\t\"");
+      if (fprintf (FP, "\t\"") == EOF) writeFileFailed = TRUE;
       SaveString (FP, s_ptr->s);
-      fprintf (FP, "\",\n");
+      if (fprintf (FP, "\",\n") == EOF) writeFileFailed = TRUE;
    }
 
-   fprintf (FP, "\t\"");
+   if (fprintf (FP, "\t\"") == EOF) writeFileFailed = TRUE;
    SaveString (FP, s_ptr->s);
-   fprintf (FP, "\"])");
+   if (fprintf (FP, "\"])") == EOF) writeFileFailed = TRUE;
 }
 
 char * ReadString (Str)
@@ -1008,7 +1061,7 @@ int PaintLeftText (Str, Just, Rotate, LtX, LtY)
       LtX = ZOOMED_SIZE(LtX);
       LtY = ZOOMED_SIZE(LtY);
       amount = XTextWidth (canvasFontPtr, Str, strlen (Str));
-      BlurText (drawWindow, drawGC, Just, Rotate, LtX, LtY,
+      BlurText (drawWindow, drawGC, LtX, LtY,
             (zoomedIn ? ((amount<<zoomScale)+1) : (amount>>zoomScale)+1),
             (zoomedIn ? (textCursorH<<zoomScale)+1 :
                         (textCursorH>>zoomScale)+1));
@@ -1045,7 +1098,6 @@ void RepaintFirstStr (ObjPtr, Str)
 
    SaveCurFont ();
    curFont = ObjPtr->detail.t->font;
-   curFontDPI = ObjPtr->detail.t->dpi;
    curStyle = ObjPtr->detail.t->style;
    curSize = ObjPtr->detail.t->size;
    textJust = ObjPtr->detail.t->just;
