@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,9 +25,10 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/choice.c,v 2.58.2.6 1993/03/17 06:33:29 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/choice.c,v 2.70 1994/04/23 05:01:36 william Exp $";
 #endif
 
+#include <stdio.h>
 #include <X11/Xlib.h>
 #include "const.h"
 #include "types.h"
@@ -34,10 +36,16 @@ static char RCSid[] =
 #include "align.e"
 #include "arc.e"
 #include "box.e"
+#ifndef _NO_EXTERN
+#include "choice.e"
+#endif
 #include "color.e"
 #include "cursor.e"
+#include "edit.e"
+#include "file.e"
 #include "font.e"
 #include "grid.e"
+#include "mainloop.e"
 #include "mainmenu.e"
 #include "mark.e"
 #include "msg.e"
@@ -50,6 +58,8 @@ static char RCSid[] =
 #include "rcbox.e"
 #include "select.e"
 #include "setup.e"
+#include "special.e"
+#include "stretch.e"
 #include "text.e"
 
 #define MODE_ROW 0
@@ -136,8 +146,6 @@ int	curChoice = NOTHING;
 static GC	choiceGC;
 static Pixmap	choiceBackingPixmap;
 
-extern void	ShowWhereToPrint ();
-
 void InitChoice ()
 {
    XGCValues	values;
@@ -336,11 +344,7 @@ void ShowTextSize ()
 
    XClearArea (mainDisplay, choiceWindow, TEXT_SIZE_COL*choiceImageW,
          TEXT_SIZE_ROW*choiceImageH, choiceImageW, choiceImageH, FALSE);
-   switch (curFontDPI)
-   {
-      case FONT_DPI_75: sprintf (s, "%1d", pointSize75[curSize]); break;
-      case FONT_DPI_100: sprintf (s, "%1d", pointSize100[curSize]); break;
-   }
+   sprintf (s, "%1d", curSize);
    len = strlen (s);
    w = XTextWidth (defaultFontPtr, s, len);
    x = TEXT_SIZE_COL*choiceImageW + ((choiceImageW-w)>>1);
@@ -550,6 +554,19 @@ void ShowPageLayout ()
          choiceImageW, choiceImageH);
 }
 
+static struct MouseStatusStrRec choiceMouseStatus[] = {
+   { "select/move/resize objects", "Main Menu", "Mode Menu" },
+   { "enter text", "Main Menu", "Mode Menu" },
+   { "draw rectangles", "Main Menu", "Mode Menu" },
+   { "draw ovals", "Main Menu", "Mode Menu" },
+   { "draw poly/open splines", "Main Menu", "Mode Menu" },
+   { "draw polygon/closed splines", "Main Menu", "Mode Menu" },
+   { "draw arcs", "Main Menu", "Mode Menu" },
+   { "draw rcboxes", "Main Menu", "Mode Menu" },
+   { "freehand poly/open splines", "Main Menu", "Mode Menu" },
+   { "select/move vertices", "Main Menu", "Mode Menu" }
+};
+
 void SetCurChoice (NewChoice)
    int	NewChoice;
 {
@@ -631,6 +648,9 @@ void SetCurChoice (NewChoice)
 
    ShowMode ();
    UpdateSubMenu (MENU_MODE);
+
+   SetMouseStatus (choiceMouseStatus[curChoice].l,
+         choiceMouseStatus[curChoice].m, choiceMouseStatus[curChoice].r);
 }
 
 static int	prevChoice = NOTHING;
@@ -853,6 +873,110 @@ void ZoomLoop (button_ev)
    PreciseZoom (new_zoomed_in, new_zoom_scale, FALSE);
 }
 
+static struct MouseStatusStrRec modeMouseStatus[] = {
+   { "text mode", "Mode Menu", "vertex mode" },
+   { "rectangle mode", "Mode Menu", "select mode" },
+   { "oval mode", "Mode Menu", "text mode" },
+   { "poly mode", "Mode Menu", "rectangle mode" },
+   { "polygon mode", "Mode Menu", "oval mode" },
+   { "arc mode", "Mode Menu", "poly mode" },
+   { "rcbox mode", "Mode Menu", "polygon mode" },
+   { "freehand mode", "Mode Menu", "arc mode" },
+   { "vertex mode", "Mode Menu", "rcbox mode" },
+   { "select mode", "Mode Menu", "freehand mode" }
+};
+static struct MouseStatusStrRec colorMouseStatus[] = {
+   { "next color", "Color Menu", "prev color" }
+};
+static struct MouseStatusStrRec hAlignMouseStatus[] = {
+   { "next horizontal align", "HoriAlign Menu", "prev horizontal align" }
+};
+static struct MouseStatusStrRec vAlignMouseStatus[] = {
+   { "next vertical align", "VertAlign Menu", "prev vertical align" }
+};
+static struct MouseStatusStrRec pageMouseStatus[] = {
+   { "next page", "Page Menu", "prev page" },
+   { "drawing size", "Page Menu", "drawing size" }
+};
+static struct MouseStatusStrRec pageLayoutMouseStatus[] = {
+   { "tiled page mode", "PageLayout Menu", "tiled page mode" },
+   { "stacked page mode", "PageLayout Menu", "stacked page mode" }
+};
+static struct MouseStatusStrRec justMouseStatus[] = {
+   { "center justified", "TextStyle Menu", "right justified" },
+   { "right justified", "TextStyle Menu", "left justified" },
+   { "left justified", "TextStyle Menu", "center justified" }
+};
+static struct MouseStatusStrRec fontMouseStatus[] = {
+   { "next font", "Font Menu", "prev font" }
+};
+static struct MouseStatusStrRec vspaceMouseStatus[] = {
+   { "inc text vspace", "modify text vspace", "dec text vspace" }
+};
+static struct MouseStatusStrRec textSizeMouseStatus[] = {
+   { "next text size", "TextSize Menu", "prev text size" }
+};
+static struct MouseStatusStrRec rotateMouseStatus[] = {
+   { "text rotate = 90", "Arrange Menu", "text rotate = 270" },
+   { "text rotate = 180", "Arrange Menu", "no text rotate" },
+   { "text rotate = 270", "Arrange Menu", "text rotate = 90" },
+   { "no text rotate", "Arrange Menu", "text rotate = 180" }
+};
+static struct MouseStatusStrRec editMouseStatus[] = {
+   { "(none)", "Edit Menu", "(none)" }
+};
+static struct MouseStatusStrRec specialMouseStatus[] = {
+   { "(none)", "Special Menu", "(none)" }
+};
+static struct MouseStatusStrRec lineWidthMouseStatus[] = {
+   { "next line width", "LineWidth Menu", "prev line width" }
+};
+static struct MouseStatusStrRec lineStyleMouseStatus[] = {
+   { "next line style", "LineStyle Menu", "prev line style" }
+};
+static struct MouseStatusStrRec lineTypeMouseStatus[] = {
+   { "next line type", "LineType Menu", "prev line type" }
+};
+static struct MouseStatusStrRec lineDashMouseStatus[] = {
+   { "next line dash", "LineDash Menu", "prev line dash" }
+};
+static struct MouseStatusStrRec rcbRadiusMouseStatus[] = {
+   { "inc rcbox radius", "modify rcbox radius", "dec rcbox radius" }
+};
+static struct MouseStatusStrRec zoomMouseStatus[] = {
+   { "zoom in", "modify zoom", "zoom out" }
+};
+static struct MouseStatusStrRec moveModeMouseStatus[] = {
+   { "unconstrained move mode", "MoveMode Menu", "unconstrained move mode" },
+   { "constrained move mode", "MoveMode Menu", "constrained move mode" }
+};
+static struct MouseStatusStrRec bwPrintMouseStatus[] = {
+   { "EPS print mode", "Layout Menu", "EPSI print mode" },     /* printer */
+   { "PS print mode", "Layout Menu", "printer print mode" },   /* EPS */
+   { "XBM print mode", "Layout Menu", "EPS print mode" },      /* PS */
+   { "ASCII print mode", "Layout Menu", "PS print mode" },     /* XBM */
+   { "EPSI print mode", "Layout Menu", "XBM print mode" },     /* ASCII */
+   { "printer print mode", "Layout Menu", "ASCII print mode" } /* EPSI */
+};
+static struct MouseStatusStrRec colorPrintMouseStatus[] = {
+   { "EPS print mode", "Layout Menu", "EPSI print mode" },     /* printer */
+   { "PS print mode", "Layout Menu", "printer print mode" },   /* EPS */
+   { "XPM print mode", "Layout Menu", "EPS print mode" },      /* PS */
+   { "ASCII print mode", "Layout Menu", "PS print mode" },     /* XPM */
+   { "EPSI print mode", "Layout Menu", "XPM print mode" },     /* ASCII */
+   { "printer print mode", "Layout Menu", "ASCII print mode" } /* EPSI */
+};
+static struct MouseStatusStrRec fileMouseStatus[] = {
+   { "landscape", "File Menu", "landscape" },
+   { "portrait", "File Menu", "portrait" }
+};
+static struct MouseStatusStrRec fillMouseStatus[] = {
+   { "next fill pattern", "Fill Menu", "prev fill pattern" }
+};
+static struct MouseStatusStrRec penMouseStatus[] = {
+   { "next pen pattern", "Pen Menu", "prev pen pattern" }
+};
+
 int ChoiceEventHandler (input)
    XEvent	* input;
 {
@@ -869,6 +993,125 @@ int ChoiceEventHandler (input)
       while (XCheckWindowEvent (mainDisplay, choiceWindow, ExposureMask, &ev)) ;
       RedrawChoiceWindow ();
    }
+   else if (input->type == EnterNotify)
+   {
+      SetMouseStatus ("", "", "");
+   }
+   else if (input->type == MotionNotify)
+   {
+      xindex = (int)(input->xmotion.x / choiceImageW);
+      yindex = (int)(input->xmotion.y / choiceImageH);
+      choice = (xindex<<CHOICE_SHIFT)|yindex;
+      switch (choice)
+      {
+         case CHOICE_MODE:
+            SetMouseStatus (modeMouseStatus[curChoice].l,
+                  modeMouseStatus[curChoice].m, modeMouseStatus[curChoice].r);
+            break;
+         case CHOICE_COLOR:
+            SetMouseStatus (colorMouseStatus[0].l,
+                  colorMouseStatus[0].m, colorMouseStatus[0].r);
+            break;
+         case CHOICE_HORI_ALIGN:
+            SetMouseStatus (hAlignMouseStatus[0].l,
+                  hAlignMouseStatus[0].m, hAlignMouseStatus[0].r);
+            break;
+         case CHOICE_VERT_ALIGN:
+            SetMouseStatus (vAlignMouseStatus[0].l,
+                  vAlignMouseStatus[0].m, vAlignMouseStatus[0].r);
+            break;
+         case CHOICE_PAGE:
+            SetMouseStatus (pageMouseStatus[pageLayoutMode].l,
+                  pageMouseStatus[pageLayoutMode].m,
+                  pageMouseStatus[pageLayoutMode].r);
+            break;
+         case CHOICE_PAGELAYOUT:
+            SetMouseStatus (pageLayoutMouseStatus[pageLayoutMode].l,
+                  pageLayoutMouseStatus[pageLayoutMode].m,
+                  pageLayoutMouseStatus[pageLayoutMode].r);
+            break;
+         case CHOICE_JUST:
+            SetMouseStatus (justMouseStatus[textJust].l,
+                  justMouseStatus[textJust].m, justMouseStatus[textJust].r);
+            break;
+         case CHOICE_FONT:
+            SetMouseStatus (fontMouseStatus[0].l,
+                  fontMouseStatus[0].m, fontMouseStatus[0].r);
+            break;
+         case CHOICE_VSPACE:
+            SetMouseStatus (vspaceMouseStatus[0].l,
+                  vspaceMouseStatus[0].m, vspaceMouseStatus[0].r);
+            break;
+         case CHOICE_TEXT_SIZE:
+            SetMouseStatus (textSizeMouseStatus[0].l,
+                  textSizeMouseStatus[0].m, textSizeMouseStatus[0].r);
+            break;
+         case CHOICE_ROTATE:
+            SetMouseStatus (rotateMouseStatus[curRotate].l,
+                  rotateMouseStatus[curRotate].m,
+                  rotateMouseStatus[curRotate].r);
+            break;
+         case CHOICE_EDIT:
+            SetMouseStatus (editMouseStatus[0].l,
+                  editMouseStatus[0].m, editMouseStatus[0].r);
+            break;
+         case CHOICE_SPECIAL:
+            SetMouseStatus (specialMouseStatus[0].l,
+                  specialMouseStatus[0].m, specialMouseStatus[0].r);
+            break;
+         case CHOICE_LINE_WIDTH:
+            SetMouseStatus (lineWidthMouseStatus[0].l,
+                  lineWidthMouseStatus[0].m, lineWidthMouseStatus[0].r);
+            break;
+         case CHOICE_LINE_STYLE:
+            SetMouseStatus (lineStyleMouseStatus[0].l,
+                  lineStyleMouseStatus[0].m, lineStyleMouseStatus[0].r);
+            break;
+         case CHOICE_LINE_TYPE:
+            SetMouseStatus (lineTypeMouseStatus[0].l,
+                  lineTypeMouseStatus[0].m, lineTypeMouseStatus[0].r);
+            break;
+         case CHOICE_DASH:
+            SetMouseStatus (lineDashMouseStatus[0].l,
+                  lineDashMouseStatus[0].m, lineDashMouseStatus[0].r);
+            break;
+         case CHOICE_RADIUS:
+            SetMouseStatus (rcbRadiusMouseStatus[0].l,
+                  rcbRadiusMouseStatus[0].m, rcbRadiusMouseStatus[0].r);
+            break;
+         case CHOICE_ZOOM:
+            SetMouseStatus (zoomMouseStatus[0].l,
+                  zoomMouseStatus[0].m, zoomMouseStatus[0].r);
+            break;
+         case CHOICE_MOVE_MODE:
+            SetMouseStatus (moveModeMouseStatus[moveMode].l,
+                  moveModeMouseStatus[moveMode].m,
+                  moveModeMouseStatus[moveMode].r);
+            break;
+         case CHOICE_PRINT:
+            if (colorDump)
+               SetMouseStatus (colorPrintMouseStatus[whereToPrint].l,
+                     colorPrintMouseStatus[whereToPrint].m,
+                     colorPrintMouseStatus[whereToPrint].r);
+            else
+               SetMouseStatus (bwPrintMouseStatus[whereToPrint].l,
+                     bwPrintMouseStatus[whereToPrint].m,
+                     bwPrintMouseStatus[whereToPrint].r);
+            break;
+         case CHOICE_FILE:
+            SetMouseStatus (fileMouseStatus[pageStyle].l,
+                  fileMouseStatus[pageStyle].m, fileMouseStatus[pageStyle].r);
+            break;
+         case CHOICE_FILL:
+            SetMouseStatus (fillMouseStatus[0].l,
+                  fillMouseStatus[0].m, fillMouseStatus[0].r);
+            break;
+         case CHOICE_PEN:
+            SetMouseStatus (penMouseStatus[0].l,
+                  penMouseStatus[0].m, penMouseStatus[0].r);
+            break;
+      }
+   }
    else if (input->type == ButtonPress)
    {
       Msg ("");
@@ -890,15 +1133,17 @@ int ChoiceEventHandler (input)
       {
          case CHOICE_MODE:
             if (delta == 0)
-               ModeMenu (x, y);
+               ModeMenu (x, y, FALSE);
             else
                SetCurChoice ((curChoice+MAXCHOICES+delta) % MAXCHOICES);
+            SetMouseStatus (modeMouseStatus[curChoice].l,
+                  modeMouseStatus[curChoice].m, modeMouseStatus[curChoice].r);
             break;
          case CHOICE_COLOR:
             if (colorDisplay)
             {
                if (delta == 0)
-                  ColorMenu (x, y);
+                  ColorMenu (x, y, FALSE);
                else
                {
                   if (curChoice != NOTHING) SetCurChoice (NOTHING);
@@ -907,22 +1152,28 @@ int ChoiceEventHandler (input)
                   if (topSel != NULL) ChangeAllSelColor (colorIndex);
                }
             }
+            SetMouseStatus (colorMouseStatus[0].l,
+                  colorMouseStatus[0].m, colorMouseStatus[0].r);
             break;
          case CHOICE_HORI_ALIGN:
             if (delta == 0)
-               HoriAlignMenu (x, y);
+               HoriAlignMenu (x, y, FALSE);
             else
                HoriAlignSubMenu ((horiAlign+MAXALIGNS+delta) % MAXALIGNS);
+            SetMouseStatus (hAlignMouseStatus[0].l,
+                  hAlignMouseStatus[0].m, hAlignMouseStatus[0].r);
             break;
          case CHOICE_VERT_ALIGN:
             if (delta == 0)
-               VertAlignMenu (x, y);
+               VertAlignMenu (x, y, FALSE);
             else
                VertAlignSubMenu ((vertAlign+MAXALIGNS+delta) % MAXALIGNS);
+            SetMouseStatus (vAlignMouseStatus[0].l,
+                  vAlignMouseStatus[0].m, vAlignMouseStatus[0].r);
             break;
          case CHOICE_PAGE:
             if (delta == 0)
-               PageMenu (x, y);
+               PageMenu (x, y, FALSE);
             else if (pageLayoutMode == PAGE_STACK)
             {
                if (delta == 1)
@@ -932,16 +1183,22 @@ int ChoiceEventHandler (input)
             }
             else
                SpecifyDrawingSize ();
+            SetMouseStatus (pageMouseStatus[pageLayoutMode].l,
+                  pageMouseStatus[pageLayoutMode].m,
+                  pageMouseStatus[pageLayoutMode].r);
             break;
          case CHOICE_PAGELAYOUT:
             if (delta == 0)
-               PageLayoutMenu (x, y);
+               PageLayoutMenu (x, y, FALSE);
             else
                PageLayoutSubMenu (!pageLayoutMode);
+            SetMouseStatus (pageLayoutMouseStatus[pageLayoutMode].l,
+                  pageLayoutMouseStatus[pageLayoutMode].m,
+                  pageLayoutMouseStatus[pageLayoutMode].r);
             break;
          case CHOICE_JUST:
             if (delta == 0)
-               StyleMenu (x, y);
+               StyleMenu (x, y, FALSE);
             else
             {
                if (curChoice == DRAWTEXT)
@@ -960,12 +1217,16 @@ int ChoiceEventHandler (input)
                UpdateSubMenu (MENU_STYLE);
                if (topSel != NULL) ChangeFontJust (textJust);
             }
+            SetMouseStatus (justMouseStatus[textJust].l,
+                  justMouseStatus[textJust].m, justMouseStatus[textJust].r);
             break;
          case CHOICE_FONT:
             if (delta == 0)
-               FontMenu (x, y);
+               FontMenu (x, y, FALSE);
             else
             {
+               int	saved_font=curFont;
+
                if (curChoice == DRAWTEXT)
                {
                   if (TieLooseEnds ())
@@ -977,13 +1238,27 @@ int ChoiceEventHandler (input)
                      SetCurChoice (NOTHING);
                   RemoveAllSel ();
                }
-               curFont = (curFont+MAXFONTS+delta) % MAXFONTS;
+               curFont = (curFont+numFonts+delta) % numFonts;
+               attemptingToSetFontProperty = TRUE;
                SetCanvasFont ();
+               attemptingToSetFontProperty = FALSE;
+               if (canvasFontSize == INVALID)
+               {
+                  char  msg[80];
+
+                  sprintf (msg, "%s-%1d not available.", fontMenuStr[curFont],
+                        curSize);
+                  Dialog (msg, NULL, NULL);
+                  curFont = saved_font;
+                  SetCanvasFont ();
+               }
                ShowCurFont ();
                ShowTextSize ();
                UpdateSubMenu (MENU_FONT);
                if (topSel != NULL) ChangeFont (curFont);
             }
+            SetMouseStatus (fontMouseStatus[0].l,
+                  fontMouseStatus[0].m, fontMouseStatus[0].r);
             break;
          case CHOICE_VSPACE:
             if (delta == 0)
@@ -1011,13 +1286,15 @@ int ChoiceEventHandler (input)
                   if (topSel != NULL) ChangeVSpace (textVSpace);
                }
             }
+            SetMouseStatus (vspaceMouseStatus[0].l,
+                  vspaceMouseStatus[0].m, vspaceMouseStatus[0].r);
             break;
          case CHOICE_TEXT_SIZE:
             if (delta == 0)
-               SizeMenu (x, y);
+               SizeMenu (x, y, FALSE);
             else
             {
-               int	cur_index;
+               int	cur_index, saved_index;
 
                if (curChoice == DRAWTEXT)
                {
@@ -1030,20 +1307,37 @@ int ChoiceEventHandler (input)
                      SetCurChoice (NOTHING);
                   RemoveAllSel ();
                }
-               cur_index = GetSizeMenuIndex ();
-               cur_index = (cur_index+2*MAXFONTSIZES-1+delta) %
-                     (2*MAXFONTSIZES-1);
-               curSize = ParseSizeMenuIndex (cur_index, &curFontDPI);
+               cur_index = saved_index = GetSizeMenuIndex ();
+               if (cur_index == INVALID)
+                  cur_index = 0;
+               else
+                  cur_index = (cur_index+numFontSizes+delta) % numFontSizes;
+               curSize = fontSizes[cur_index];
+               attemptingToSetFontProperty = TRUE;
                SetCanvasFont ();
+               attemptingToSetFontProperty = FALSE;
+               if (canvasFontSize == INVALID)
+               {
+                  char	msg[80];
+
+                  sprintf (msg, "%s-%1d not available.", fontMenuStr[curFont],
+                        curSize);
+                  Dialog (msg, NULL, NULL);
+                  cur_index = saved_index;
+                  curSize = fontSizes[cur_index];
+                  SetCanvasFont ();
+               }
                ShowCurFont ();
                ShowTextSize ();
                UpdateSubMenu (MENU_SIZE);
-               if (topSel != NULL) ChangeFontSize (curSize);
+               if (topSel != NULL) ChangeFontSize (cur_index);
             }
+            SetMouseStatus (textSizeMouseStatus[0].l,
+                  textSizeMouseStatus[0].m, textSizeMouseStatus[0].r);
             break;
          case CHOICE_ROTATE:
             if (delta == 0)
-               ArrangeMenu (x, y);
+               ArrangeMenu (x, y, FALSE);
             else
             {
                if (curChoice == DRAWTEXT)
@@ -1067,12 +1361,23 @@ int ChoiceEventHandler (input)
                      RotateCounter ();
                }
             }
+            SetMouseStatus (rotateMouseStatus[curRotate].l,
+                  rotateMouseStatus[curRotate].m,
+                  rotateMouseStatus[curRotate].r);
+            break;
+         case CHOICE_EDIT:
+            if (delta == 0) EditMenu (x, y, FALSE);
+            SetMouseStatus (editMouseStatus[0].l,
+                  editMouseStatus[0].m, editMouseStatus[0].r);
+            break;
+         case CHOICE_SPECIAL:
+            if (delta == 0) SpecialMenu (x, y, FALSE);
+            SetMouseStatus (specialMouseStatus[0].l,
+                  specialMouseStatus[0].m, specialMouseStatus[0].r);
             break;
-         case CHOICE_EDIT: if (delta == 0) EditMenu (x, y); break;
-         case CHOICE_SPECIAL: if (delta == 0) SpecialMenu (x, y); break;
          case CHOICE_LINE_WIDTH:
             if (delta == 0)
-               LineWidthMenu (x, y);
+               LineWidthMenu (x, y, FALSE);
             else
             {
                lineWidth = (lineWidth+maxLineWidths+delta) % maxLineWidths;
@@ -1080,10 +1385,12 @@ int ChoiceEventHandler (input)
                UpdateSubMenu (MENU_LINEWIDTH);
                if (topSel != NULL) ChangeAllSelLineWidth (lineWidth);
             }
+            SetMouseStatus (lineWidthMouseStatus[0].l,
+                  lineWidthMouseStatus[0].m, lineWidthMouseStatus[0].r);
             break;
          case CHOICE_LINE_STYLE:
             if (delta == 0)
-               LineStyleMenu (x, y);
+               LineStyleMenu (x, y, FALSE);
             else
             {
                lineStyle = (lineStyle+MAXLINESTYLES+delta) % MAXLINESTYLES;
@@ -1091,10 +1398,12 @@ int ChoiceEventHandler (input)
                UpdateSubMenu (MENU_LINESTYLE);
                if (topSel != NULL) ChangeAllSelLineStyle (lineStyle);
             }
+            SetMouseStatus (lineStyleMouseStatus[0].l,
+                  lineStyleMouseStatus[0].m, lineStyleMouseStatus[0].r);
             break;
          case CHOICE_LINE_TYPE:
             if (delta == 0)
-               LineTypeMenu (x, y);
+               LineTypeMenu (x, y, FALSE);
             else
             {
                curSpline = (curSpline+MAXLINETYPES+delta) % MAXLINETYPES;
@@ -1102,10 +1411,12 @@ int ChoiceEventHandler (input)
                UpdateSubMenu (MENU_LINETYPE);
                if (topSel != NULL) ChangeAllSelLineType (curSpline);
             }
+            SetMouseStatus (lineTypeMouseStatus[0].l,
+                  lineTypeMouseStatus[0].m, lineTypeMouseStatus[0].r);
             break;
          case CHOICE_DASH:
             if (delta == 0)
-               LineDashMenu (x, y);
+               LineDashMenu (x, y, FALSE);
             else
             {
                curDash = (curDash+MAXDASHES+delta) % MAXDASHES;
@@ -1113,6 +1424,8 @@ int ChoiceEventHandler (input)
                UpdateSubMenu (MENU_LINEDASH);
                if (topSel != NULL) ChangeAllSelDashes (curDash);
             }
+            SetMouseStatus (lineDashMouseStatus[0].l,
+                  lineDashMouseStatus[0].m, lineDashMouseStatus[0].r);
             break;
          case CHOICE_RADIUS:
             if (delta == 0)
@@ -1126,6 +1439,8 @@ int ChoiceEventHandler (input)
                   if (topSel != NULL) ChangeAllSelRCBRadius (rcbRadius);
                }
             }
+            SetMouseStatus (rcbRadiusMouseStatus[0].l,
+                  rcbRadiusMouseStatus[0].m, rcbRadiusMouseStatus[0].r);
             break;
          case CHOICE_ZOOM:
             switch (delta)
@@ -1134,26 +1449,39 @@ int ChoiceEventHandler (input)
                case 0: ZoomLoop (button_ev); break;
                case 1: ZoomOut (); break;
             }
+            SetMouseStatus (zoomMouseStatus[0].l,
+                  zoomMouseStatus[0].m, zoomMouseStatus[0].r);
             break;
          case CHOICE_MOVE_MODE:
             if (delta == 0)
-               MoveModeMenu (x, y);
+               MoveModeMenu (x, y, FALSE);
             else
                ToggleMoveMode ();
+            SetMouseStatus (moveModeMouseStatus[moveMode].l,
+                  moveModeMouseStatus[moveMode].m,
+                  moveModeMouseStatus[moveMode].r);
             break;
          case CHOICE_PRINT:
             if (delta == 0)
-               LayoutMenu (x, y);
+               LayoutMenu (x, y, FALSE);
             else
             {
                whereToPrint = (whereToPrint+MAXWHERETOPRINT+delta) %
                      MAXWHERETOPRINT-1;
                ToggleWhereToPrint ();
             }
+            if (colorDump)
+               SetMouseStatus (colorPrintMouseStatus[whereToPrint].l,
+                     colorPrintMouseStatus[whereToPrint].m,
+                     colorPrintMouseStatus[whereToPrint].r);
+            else
+               SetMouseStatus (bwPrintMouseStatus[whereToPrint].l,
+                     bwPrintMouseStatus[whereToPrint].m,
+                     bwPrintMouseStatus[whereToPrint].r);
             break;
          case CHOICE_FILE:
             if (delta == 0)
-               return (FileMenu (x, y));
+               return (FileMenu (x, y, FALSE));
             else
             {
                switch (pageStyle)
@@ -1164,10 +1492,12 @@ int ChoiceEventHandler (input)
                      ChangePageStyle (PORTRAIT, "Portrait"); break;
                }
             }
+            SetMouseStatus (fileMouseStatus[pageStyle].l,
+                  fileMouseStatus[pageStyle].m, fileMouseStatus[pageStyle].r);
             break;
          case CHOICE_FILL:
             if (delta == 0)
-               FillMenu (x, y);
+               FillMenu (x, y, FALSE);
             else
             {
                if (curChoice == DRAWTEXT)
@@ -1185,10 +1515,12 @@ int ChoiceEventHandler (input)
                ShowFill ();
                if (topSel != NULL) ChangeAllSelFill (objFill);
             }
+            SetMouseStatus (fillMouseStatus[0].l,
+                  fillMouseStatus[0].m, fillMouseStatus[0].r);
             break;
          case CHOICE_PEN:
             if (delta == 0)
-               PenMenu (x, y);
+               PenMenu (x, y, FALSE);
             else
             {
                if (curChoice == DRAWTEXT)
@@ -1206,6 +1538,8 @@ int ChoiceEventHandler (input)
                ShowPen ();
                if (topSel != NULL) ChangeAllSelPen (penPat);
             }
+            SetMouseStatus (penMouseStatus[0].l,
+                  penMouseStatus[0].m, penMouseStatus[0].r);
             break;
       }
    }
