@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,26 +25,35 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/color.c,v 2.56 1993/02/24 07:18:16 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/color.c,v 2.68 1994/05/01 23:22:57 william Exp $";
 #endif
 
 #include <stdio.h>
 #include <X11/Xlib.h>
+#include <X11/Xutil.h>
 #include "const.h"
 #include "types.h"
 
 #include "choice.e"
 #include "cmd.e"
+#ifndef _NO_EXTERN
+#include "color.e"
+#endif
+#include "drawing.e"
 #include "mainloop.e"
 #include "mark.e"
 #include "menu.e"
+#include "msg.e"
 #include "obj.e"
+#include "pattern.e"
 #include "raster.e"
 #include "select.e"
 #include "setup.e"
 #include "text.e"
 #include "xpixmap.e"
 
+extern int	atoi ARGS_DECL((char *));
+
 #define COLORSTRLEN 80
 
 #define WHITE_COLOR_INDEX 8
@@ -57,8 +67,9 @@ int	* xorColorPixels = NULL;
 char	myFgColorStr[COLORSTRLEN];
 char	myBgColorStr[COLORSTRLEN];
 char	* * colorMenuItems = NULL;
-XColor	* tgifColors;
-XColor	* tgifRequestedColors;
+XColor	* tgifColors=NULL;
+XColor	* tgifRequestedColors=NULL;
+XColor	myBgColor;
 int	maxRGB = 0;
 int	colorDump = FALSE;
 
@@ -72,8 +83,9 @@ static char	* defaultColorMenuItems[MAXCOLORS] =
    "white", "black", "DarkSlateGray"
 };
 
-void DefaultColorArrays (Entries, ForePixels, Valid, InitRV)
+void DefaultColorArrays (Entries, ForePixels, Valid, InitRV, StatusStr)
    int	Entries, * * ForePixels, * * Valid, * * InitRV;
+   char	* * * StatusStr;
 {
    register int	i, * fore_pixels, pixel, * valid, * init_rv;
 
@@ -87,6 +99,15 @@ void DefaultColorArrays (Entries, ForePixels, Valid, InitRV)
       *valid++ = TRUE;
       *init_rv++ = FALSE;
    }
+   if (StatusStr != NULL)
+   {
+      *StatusStr = (char **) calloc (Entries, sizeof(char *));
+      for (i=0; i < Entries; i++)
+      {
+         (*StatusStr)[i] = (char *) calloc (MAXSTRING+1, sizeof(char));
+         *(*StatusStr)[i] = '\0';
+      }
+   }
 }
 
 static
@@ -161,7 +182,17 @@ void InitColor ()
    if (cmdLineRV != INVALID)
       reverseVideo = cmdLineRV;
    else if ((c_ptr = XGetDefault(mainDisplay,TOOL_NAME,"ReverseVideo")) != NULL)
-      reverseVideo = (strcmp (c_ptr, "on") == 0);
+   {
+      reverseVideo = FALSE;
+      if (strcmp (c_ptr, "on") == 0 || strcmp (c_ptr, "On") == 0 ||
+            strcmp (c_ptr, "True") == 0 || strcmp (c_ptr, "true") == 0)
+         reverseVideo = TRUE;
+   }
+
+   colorDump = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"InitialPrintInColor")) != NULL)
+      if ((strcmp (c_ptr, "True") == 0) || (strcmp (c_ptr, "true") == 0))
+         colorDump = TRUE;
 
    if ((c_ptr = XGetDefault (mainDisplay, TOOL_NAME, "DefaultColorIndex"))
          != NULL)
@@ -309,23 +340,27 @@ void InitColor ()
          tgifRequestedColors[index].green = green_req;
          tgifRequestedColors[index].blue = blue_req;
 
-         if (strcmp (colorMenuItems[i], fg_color) == 0)
+         if (MyStrICmp (colorMenuItems[i], fg_color) == 0)
          {
             myFgPixel = color.pixel;
             strcpy (myFgColorStr, fg_color);
 
             fg_allocated = TRUE;
          }
-         if (strcmp (colorMenuItems[i], bg_color) == 0)
+         if (MyStrICmp (colorMenuItems[i], bg_color) == 0)
          {
             myBgPixel = color.pixel;
+            myBgColor.pixel = color.pixel;
+            myBgColor.red = color.red;
+            myBgColor.green = color.green;
+            myBgColor.blue = color.blue;
             strcpy (myBgColorStr, bg_color);
 
             bg_allocated = TRUE;
             bg_gray = (((long)tgifColors[i].red)<<2) +
                   (((long)tgifColors[i].green)<<3) + ((long)tgifColors[i].blue);
          }
-         if (strcmp (colorMenuItems[i], brdr_color) == 0)
+         if (MyStrICmp (colorMenuItems[i], brdr_color) == 0)
          {
             myBorderPixel = color.pixel;
             brdr_allocated = TRUE;
@@ -381,6 +416,10 @@ void InitColor ()
             exit (-1);
          }
          myBgPixel = color.pixel;
+         myBgColor.pixel = color.pixel;
+         myBgColor.red = color.red;
+         myBgColor.green = color.green;
+         myBgColor.blue = color.blue;
          strcpy (myBgColorStr, bg_color);
          if (looking_for_default_color)
             bg_gray = (((long)color.red)<<2) + (((long)color.green)<<3) +
@@ -720,7 +759,7 @@ void ChangeAllSelColor (ColorIndex)
    int				changed=FALSE;
    XGCValues			values;
 
-   if (topSel == NULL)
+   if (topSel == NULL || stickyMenuSelection)
    {
       text_cursor_shown = textCursorShown;
       text_obj_created = TieLooseEnds ();
@@ -733,7 +772,7 @@ void ChangeAllSelColor (ColorIndex)
       }
       else
          textCursorShown = FALSE;
-      return;
+      if (topSel == NULL) return;
    }
 
    values.foreground = colorPixels[ColorIndex];
@@ -786,22 +825,40 @@ void SetUpColorMenuPixmap (fore_colors, init_rv, pixmap, rows, cols)
    *rows = (maxColors <= 10) ? maxColors : 10;
 }
 
-void ColorMenu (X, Y)
-   int	X, Y;
+int ColorMenu (X, Y, TrackMenubar)
+   int	X, Y, TrackMenubar;
 {
-   register int	index;
+   register int	i;
    Pixmap	* pixmap;
-   int		* fore_colors, * init_rv, rows, cols;
+   int		index, * fore_colors, * init_rv, rows, cols;
+   char		* * desc;
 
-   if (!colorDisplay) return;
+   if (!colorDisplay) return (INVALID);
 
+   desc = (char **) calloc (maxColors+1, sizeof(char*));
+   if (desc == NULL) fprintf (stderr, "Can not caloc().\n");
+   for (i=0; i < maxColors; i++)
+   {
+      desc[i] = (char*) calloc (80, sizeof(char));
+      if (desc[i] == NULL) fprintf (stderr, "Can not caloc().\n");
+      sprintf (desc[i], "Set color to '%s'", colorMenuItems[i]);
+   }
+   desc[i] = NULL;
    SetUpColorMenuPixmap (&fore_colors, &init_rv, &pixmap, &rows, &cols);
    activeMenu = MENU_COLOR;
    index = PxMpMenuLoop (X, Y, choiceImageW, choiceImageH, rows, cols,
-         maxColors, fore_colors, pixmap, init_rv, MULTICOLOR);
+         maxColors, fore_colors, pixmap, init_rv, desc, MULTICOLOR,
+         TrackMenubar);
    cfree (pixmap);
 
-   if (index != INVALID) ChangeAllSelColor (index);
+   if (index >= 0) ChangeAllSelColor (index);
+   if (desc != NULL)
+   {
+      for (i=0; i < maxColors; i++)
+         if (desc[i] != NULL) cfree (desc[i]);
+      cfree (desc);
+   }
+   return (index);
 }
 
 void CleanUpColors ()
@@ -899,7 +956,8 @@ int UpdatePixel (ObjPtr, OldColorStr)
             XPutImage (mainDisplay,pixmap,xpmGC,image,0,0,0,0,image_w,image_h);
             picture_changed = TRUE;
          }
-         cfree (from_pixels, to_pixels);
+         cfree (from_pixels);
+         cfree (to_pixels);
          break;
       default:
          index = QuickFindColorIndex (OldColorStr[ObjPtr->color], &new_alloc);
