@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/drawing.c,v 2.115.1.17 1993/06/07 02:57:45 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/drawing.c,v 2.138 1994/04/09 05:08:42 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -44,6 +45,9 @@ static char RCSid[] =
 #include "copypaste.e"
 #include "cursor.e"
 #include "dialog.e"
+#ifndef _NO_EXTERN
+#include "drawing.e"
+#endif
 #include "dup.e"
 #include "edit.e"
 #include "eps.e"
@@ -82,6 +86,8 @@ static char RCSid[] =
 
 int		intrCheckInterval=10;
 int		pasteInDrawTextMode=FALSE;
+int		pasteFromFileInDrawTextMode=FALSE;
+char		pasteFromFileName[MAXPATHLENGTH+1];
 int		copyInDrawTextMode=FALSE;
 int		numRedrawBBox=INVALID;
 int		numClipRecs=0;
@@ -230,6 +236,25 @@ void RedrawDummyWindow1 ()
       HideInterrupt ();
 }
 
+void DummiesEventHandler (input)
+   XEvent	* input;
+{
+   if (input->xany.window == dummyWindow1)
+   {
+      if (input->type == Expose)
+         RedrawDummyWindow1 ();
+      else if (input->type == EnterNotify)
+      {
+         if (intrShown)
+            SetMouseStatus ("interrupt", "interrupt", "interrupt");
+         else
+            SetMouseStatus ("(none)", "(none)", "(none)");
+      }
+   }
+   else if (input->xany.window == dummyWindow2)
+      SetMouseStatus ("(none)", "(none)", "(none)");
+}
+
 int CheckInterrupt ()
 {
    if (intrCheckInterval <= 0) return (FALSE);
@@ -1056,6 +1081,30 @@ int ShortHand (input)
             DetachFileAttrs ();
          else if (strcmp (name, "EditFileAttrs()") == 0)
             EditFileAttrs ();
+         else if (strcmp (name, "PrintSelectedObjs()") == 0)
+            PrintSelectedObjs ();
+         else if (strcmp (name, "InputPolyPts()") == 0)
+            InputPolyPts ();
+         else if (strcmp (name, "InputPolygonPts()") == 0)
+            InputPolygonPts ();
+         else if (strcmp (name, "EditAttrs()") == 0)
+            EditAttrs ();
+         else if (strcmp (name, "ConvertIntSpline()") == 0)
+            ConvertIntSpline ();
+         else if (strcmp (name, "PasteFromFile()") == 0)
+         {
+            if (!PasteFromFile ()) return (INVALID);
+         }
+         else if (strcmp (name, "ToggleShowMeasurement()") == 0)
+            ToggleShowMeasurement ();
+         else if (strcmp (name, "Cut()") == 0)
+            CutToCutBuffer ();
+         else if (strcmp (name, "ToggleSmoothHinge()") == 0)
+            ToggleSmoothHinge ();
+         else if (strcmp (name, "ToggleShowMenubar()") == 0)
+            ToggleShowMenubar ();
+         else if (strcmp (name, "ToggleShowStatus()") == 0)
+            ToggleShowStatus ();
       }
       return (BAD);
    }
@@ -1093,6 +1142,45 @@ int DrawingEventHandler (input)
       }
       return (INVALID);
    }
+   else if (input->type == EnterNotify)
+   {
+      switch (curChoice)
+      {
+         case NOTHING:
+            SetMouseStatus ("select/move/resize objects", "Main Menu",
+                  "Mode Menu");
+            break;
+         case DRAWTEXT:
+            SetMouseStatus ("enter text", "Main Menu", "Mode Menu");
+            break;
+         case DRAWBOX:
+            SetMouseStatus ("draw rectangles", "Main Menu", "Mode Menu");
+            break;
+         case DRAWCIRCLE:
+            SetMouseStatus ("draw ovals", "Main Menu", "Mode Menu");
+            break;
+         case DRAWPOLY:
+            SetMouseStatus ("draw poly/open splines", "Main Menu", "Mode Menu");
+            break;
+         case DRAWPOLYGON:
+            SetMouseStatus ("draw polygon/closed splines", "Main Menu",
+                  "Mode Menu");
+            break;
+         case DRAWARC:
+            SetMouseStatus ("draw arcs", "Main Menu", "Mode Menu");
+            break;
+         case DRAWRCBOX:
+            SetMouseStatus ("draw rcboxes", "Main Menu", "Mode Menu");
+            break;
+         case FREEHAND:
+            SetMouseStatus ("freehand poly/open splines", "Main Menu",
+                 "Mode Menu");
+            break;
+         case VERTEXMODE:
+            SetMouseStatus ("select/move vertices", "Main Menu", "Mode Menu");
+            break;
+      }
+   }
    else if (input->type == MotionNotify)
    {
       while (XCheckWindowEvent (mainDisplay,drawWindow,PointerMotionMask,&ev)) ;
@@ -1123,7 +1211,7 @@ int DrawingEventHandler (input)
          return (MainMenu ());
       else if (button_ev->button == Button3)
       {
-         ModeMenu (button_ev->x_root, button_ev->y_root);
+         ModeMenu (button_ev->x_root, button_ev->y_root, FALSE);
          return (INVALID);
       }
       Msg ("");
