@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,9 +25,10 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/dup.c,v 2.48.1.2 1993/03/08 02:16:44 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/dup.c,v 2.61 1994/03/21 04:54:08 william Exp $";
 #endif
 
+#include <stdio.h>
 #include <X11/Xlib.h>
 #include "const.h"
 #include "types.h"
@@ -35,7 +37,13 @@ static char RCSid[] =
 #include "choice.e"
 #include "cmd.e"
 #include "drawing.e"
+#ifndef _NO_EXTERN
+#include "dup.e"
+#endif
 #include "grid.e"
+#include "mark.e"
+#include "move.e"
+#include "msg.e"
 #include "obj.e"
 #include "page.e"
 #include "raster.e"
@@ -44,8 +52,6 @@ static char RCSid[] =
 #include "xbitmap.e"
 #include "xpixmap.e"
 
-extern struct ObjRec	* DupObj ();
-
 int	justDupped = FALSE;
 int	useRecentDupDistance = TRUE;
 int	dupDx = INVALID, dupDy = INVALID;
@@ -153,10 +159,16 @@ void DupPolyObj (PolyPtr, ObjPtr)
    poly_ptr = (struct PolyRec *) calloc (1, sizeof(struct PolyRec));
    num_pts = poly_ptr->n = PolyPtr->n;
    v = (XPoint *) calloc (num_pts+1, sizeof(XPoint));
+   if (PolyPtr->smooth != NULL)
+      poly_ptr->smooth = (char *) calloc (num_pts+1, sizeof(char));
+   else
+      poly_ptr->smooth = NULL;
    for (i = 0; i < num_pts; i++)
    {
       v[i].x = PolyPtr->vlist[i].x;
       v[i].y = PolyPtr->vlist[i].y;
+      if (PolyPtr->smooth != NULL && poly_ptr->smooth != NULL)
+         poly_ptr->smooth[i] = PolyPtr->smooth[i];
    }
    poly_ptr->vlist = v;
    poly_ptr->style = PolyPtr->style;
@@ -165,7 +177,8 @@ void DupPolyObj (PolyPtr, ObjPtr)
    poly_ptr->ah = PolyPtr->ah;
    poly_ptr->pen = PolyPtr->pen;
    poly_ptr->fill = PolyPtr->fill;
-   if ((poly_ptr->curved = PolyPtr->curved) == LT_SPLINE)
+   poly_ptr->curved = PolyPtr->curved;
+   if (PolyPtr->sn != 0 && PolyPtr->svlist != NULL)
    {
       poly_ptr->sn = num_pts = PolyPtr->sn;
       v = (XPoint *) calloc (num_pts+1, sizeof(XPoint));
@@ -176,6 +189,28 @@ void DupPolyObj (PolyPtr, ObjPtr)
       }
       poly_ptr->svlist = v;
    }
+   else
+   {
+      poly_ptr->sn = 0;
+      poly_ptr->svlist = NULL;
+   }
+   if (PolyPtr->curved == LT_INTSPLINE && PolyPtr->intn != 0 &&
+         PolyPtr->intvlist != NULL)
+   {
+      poly_ptr->intn = num_pts = PolyPtr->intn;
+      v = (XPoint *) calloc (num_pts+1, sizeof(XPoint));
+      for (i = 0; i < num_pts; i++)
+      {
+         v[i].x = PolyPtr->intvlist[i].x;
+         v[i].y = PolyPtr->intvlist[i].y;
+      }
+      poly_ptr->intvlist = v;
+   }
+   else
+   {
+      poly_ptr->intn = 0;
+      poly_ptr->intvlist = NULL;
+   }
    poly_ptr->dash = PolyPtr->dash;
 
    ObjPtr->detail.p = poly_ptr;
@@ -193,16 +228,23 @@ void DupPolygonObj (PolygonPtr, ObjPtr)
    polygon_ptr = (struct PolygonRec *) calloc (1, sizeof(struct PolygonRec));
    num_pts = polygon_ptr->n = PolygonPtr->n;
    v = (XPoint *) calloc (num_pts+1, sizeof(XPoint));
+   if (PolygonPtr->smooth != NULL)
+      polygon_ptr->smooth = (char *) calloc (num_pts+1, sizeof(char));
+   else
+      polygon_ptr->smooth = NULL;
    for (i = 0; i < num_pts; i++)
    {
       v[i].x = PolygonPtr->vlist[i].x;
       v[i].y = PolygonPtr->vlist[i].y;
+      if (PolygonPtr->smooth != NULL && polygon_ptr->smooth != NULL)
+         polygon_ptr->smooth[i] = PolygonPtr->smooth[i];
    }
    polygon_ptr->vlist = v;
    polygon_ptr->fill = PolygonPtr->fill;
    polygon_ptr->width = PolygonPtr->width;
    polygon_ptr->pen = PolygonPtr->pen;
-   if ((polygon_ptr->curved = PolygonPtr->curved) == LT_SPLINE)
+   polygon_ptr->curved = PolygonPtr->curved;
+   if (PolygonPtr->sn != 0 && PolygonPtr->svlist != NULL)
    {
       polygon_ptr->sn = num_pts = PolygonPtr->sn;
       v = (XPoint *) calloc (num_pts+1, sizeof(XPoint));
@@ -213,6 +255,28 @@ void DupPolygonObj (PolygonPtr, ObjPtr)
       }
       polygon_ptr->svlist = v;
    }
+   else
+   {
+      polygon_ptr->sn = 0;
+      polygon_ptr->svlist = NULL;
+   }
+   if (PolygonPtr->curved == LT_INTSPLINE && PolygonPtr->intn != 0 &&
+         PolygonPtr->intvlist != NULL)
+   {
+      polygon_ptr->intn = num_pts = PolygonPtr->intn;
+      v = (XPoint *) calloc (num_pts+1, sizeof(XPoint));
+      for (i = 0; i < num_pts; i++)
+      {
+         v[i].x = PolygonPtr->intvlist[i].x;
+         v[i].y = PolygonPtr->intvlist[i].y;
+      }
+      polygon_ptr->intvlist = v;
+   }
+   else
+   {
+      polygon_ptr->intn = 0;
+      polygon_ptr->intvlist = NULL;
+   }
    polygon_ptr->dash = PolygonPtr->dash;
 
    ObjPtr->detail.g = polygon_ptr;
@@ -327,6 +391,8 @@ void DupXBmObj (XBmPtr, ObjPtr)
    xbm_ptr->bitmap = None;
    xbm_ptr->cached_bitmap = None;
 
+   *xbm_ptr->write_date = '\0';
+   xbm_ptr->save_epsf = XBmPtr->save_epsf;
    xbm_ptr->real_type = XBmPtr->real_type;
 
    if (XBmPtr->real_type==XBM_XBM || XBmPtr->real_type==XBM_EPS &&
@@ -368,6 +434,7 @@ void DupXBmObj (XBmPtr, ObjPtr)
       xbm_ptr->urx = XBmPtr->urx; xbm_ptr->ury = XBmPtr->ury;
       xbm_ptr->filename = (char *) calloc (len+1, sizeof(char));
       strcpy (xbm_ptr->filename, XBmPtr->filename);
+      strcpy (xbm_ptr->write_date, XBmPtr->write_date);
       xbm_ptr->num_epsf_lines = num_epsf_lines;
       xbm_ptr->epsflines = (char * *) calloc (num_epsf_lines, sizeof(char *));
       for (i=0; i < num_epsf_lines; i++)
@@ -490,7 +557,6 @@ void DupTextObj (TextPtr, ObjPtr)
    text_ptr->font = TextPtr->font;
    text_ptr->style = TextPtr->style;
    text_ptr->size = TextPtr->size;
-   text_ptr->dpi = TextPtr->dpi;
    text_ptr->rotate = TextPtr->rotate;
    text_ptr->pen = TextPtr->pen;
    text_ptr->fill = TextPtr->fill;
@@ -499,6 +565,11 @@ void DupTextObj (TextPtr, ObjPtr)
    text_ptr->cached_zoom = 0;
    text_ptr->cached_bitmap = None;
 
+   text_ptr->read_only = TextPtr->read_only;
+   text_ptr->orig_w = TextPtr->orig_w;
+   text_ptr->orig_h = TextPtr->orig_h;
+   text_ptr->font_name = NULL;
+
    ObjPtr->detail.t = text_ptr;
 }
 
