@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,13 +25,14 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/eps.c,v 2.8.1.11 1993/05/28 07:43:04 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/eps.c,v 2.16 1994/03/21 04:54:18 william Exp $";
 #endif
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <time.h>
 #include <stdio.h>
+#include <string.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include "const.h"
@@ -40,7 +42,11 @@ static char RCSid[] =
 #include "color.e"
 #include "cursor.e"
 #include "dialog.e"
+#include "drawing.e"
 #include "dup.e"
+#ifndef _NO_EXTERN
+#include "eps.e"
+#endif
 #include "file.e"
 #include "grid.e"
 #include "mark.e"
@@ -51,7 +57,11 @@ static char RCSid[] =
 #include "setup.e"
 #include "xbitmap.e"
 
-extern double	atof();
+#ifdef __hpux
+extern double	atof ARGS_DECL((const char *));
+#else
+extern double	atof ARGS_DECL((char *));
+#endif
 
 typedef struct LineRec {
    char			* s;
@@ -683,6 +693,8 @@ void DumpEPSObj (FP, ObjPtr)
    fprintf (FP, "/tgifopcount%1d count 1 sub def\n", xbm_ptr->epsf_level);
    fprintf (FP, "userdict begin\n");
    fprintf (FP, "/showpage {} def\n");
+   fprintf (FP, "/letter {} def\n");
+   fprintf (FP, "/legal {} def\n");
    fprintf (FP, "1 %1d %1d div %1d mul 100 div div dup neg scale\n",
          psDotsPerInch, PIX_PER_INCH, printMag);
    fprintf (FP, "%1d %s mul neg %1d %s mul neg translate\n", psDotsPerInch,
