@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,32 +25,58 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/exec.c,v 2.0 1993/06/07 03:07:09 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/exec.c,v 2.18 1994/03/21 04:54:22 william Exp $";
 #endif
 
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/time.h>
+
+#ifdef _AIX
+#ifndef _IBMESA
+#include <sys/select.h>
+#endif
+#else
+#ifdef _BSD
+#include <sys/select.h>
+#endif
+#endif
+
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include "const.h"
 #include "types.h"
 
 #include "attr.e"
+#include "cmd.e"
+#include "cursor.e"
 #include "dialog.e"
 #include "drawing.e"
 #include "dup.e"
 #include "eps.e"
+#ifndef _NO_EXTERN
+#include "exec.e"
+#endif
 #include "file.e"
+#include "mainloop.e"
 #include "menu.e"
+#include "move.e"
+#include "msg.e"
 #include "obj.e"
 #include "select.e"
 #include "setup.e"
 #include "stretch.e"
 #include "xbitmap.e"
+#include "xpixmap.e"
 
-extern char *mktemp();
+extern char	* mktemp ARGS_DECL((char *Template));
+#ifndef _NO_EXTERN
+extern int	fork ARGS_DECL((void));
+extern int	system ARGS_DECL((char *));
+extern int	unlink ARGS_DECL((char *));
+#endif
+extern int	atoi ARGS_DECL((char *));
 
 #define TOK_INVALID	(INVALID)
 #define TOK_STR		0
@@ -64,7 +91,20 @@ extern char *mktemp();
 
 static char execDummyStr[MAXSTRING+1];
 
-struct ObjRec	* FindObjWithName (BotObj, ObjName)
+#ifdef NO_STRSTR
+char * strstr (s1, s2)
+   char	* s1, * s2;
+{
+   int	len=strlen(s2);
+
+   for ( ; *s1 != '\0'; s1++)
+      if (strncmp (s1, s2, len) == 0)
+         return (s1);
+   return (NULL);
+}
+#endif /* NO_STRSTR */
+
+struct ObjRec * FindObjWithName (BotObj, ObjName)
    struct ObjRec	* BotObj;
    char			* ObjName;
 {
@@ -579,6 +619,7 @@ int ExecUseTemplate (buf, obj_ptr)
          char		tmp_buf[MAXSTRING+1];
          struct StrRec	* str_ptr;
 
+         writeFileFailed = FALSE;
          while (fgets (tmp_buf, MAXSTRING, in_fp) != NULL)
          {
             len = strlen (tmp_buf);
@@ -588,32 +629,43 @@ int ExecUseTemplate (buf, obj_ptr)
                if (len==len_to_match && strcmp(tmp_buf,real_match_str)==0)
                {
                   if (*attr_ptr->s != '\0')
-                     fprintf (out_fp, "%s\n", attr_ptr->s);
+                     if (fprintf (out_fp, "%s\n", attr_ptr->s) == EOF)
+                        writeFileFailed = TRUE;
                   if (attr_ptr->obj->detail.t->first != NULL)
                      for (str_ptr = attr_ptr->obj->detail.t->first->next;
                            str_ptr != NULL; str_ptr = str_ptr->next)
-                        fprintf (out_fp, "%s\n", str_ptr->s);
+                        if (fprintf (out_fp, "%s\n", str_ptr->s) == EOF)
+                           writeFileFailed = TRUE;
                }
-               else
-                  fprintf (out_fp, "%s\n", tmp_buf);
+               else if (fprintf (out_fp, "%s\n", tmp_buf) == EOF)
+                  writeFileFailed = TRUE;
             }
             else if (len==len_to_match && strcmp(tmp_buf,real_match_str)==0)
             {
                if (*attr_ptr->s != '\0')
-                  fprintf (out_fp, "%s\n", attr_ptr->s);
+                  if (fprintf (out_fp, "%s\n", attr_ptr->s) == EOF)
+                     writeFileFailed = TRUE;
                if (attr_ptr->obj->detail.t->first != NULL)
                   for (str_ptr = attr_ptr->obj->detail.t->first->next;
                         str_ptr != NULL; str_ptr = str_ptr->next)
-                     fprintf (out_fp, "%s\n", str_ptr->s);
+                     if (fprintf (out_fp, "%s\n", str_ptr->s) == EOF)
+                        writeFileFailed = TRUE;
             }
-            else
-               fprintf (out_fp, "%s\n", tmp_buf);
+            else if (fprintf (out_fp, "%s\n", tmp_buf) == EOF)
+               writeFileFailed = TRUE;
          }
          fclose (in_fp);
          fclose (out_fp);
-         cfree (match_str);
+         cfree (real_match_str);
          cfree (real_template_name);
          cfree (real_file_name);
+         if (writeFileFailed)
+         {
+            sprintf (msg, "Fail to write to '%s'.  File system may be full.",
+                  real_file_name);
+            Dialog (msg, NULL, NULL);
+            return (FALSE);
+         }
          return (TRUE);
       }
    }
@@ -630,7 +682,7 @@ int ExecUseTemplate (buf, obj_ptr)
       sprintf (msg, "Can not open '%s' for read.", real_template_name );
       Dialog (msg, "( <CR> or <ESC> to abort execution )", execDummyStr );
    }
-   if (real_match_str) cfree (match_str);
+   if (real_match_str) cfree (real_match_str);
    if (real_template_name) cfree (real_template_name);
    if (real_file_name) cfree (real_file_name);
    if (out_fp != NULL) fclose (out_fp);
@@ -736,7 +788,7 @@ int ExecUpdEpsChild (buf, obj_ptr)
 
    AdjObjBBox (obj_ptr);
 
-   RecordNewObjCmd ();
+   RecordReplaceAnObj (obj_ptr);
    RedrawAreas (botObj, ltx-GRID_ABS_SIZE(1), lty-GRID_ABS_SIZE(1),
       rbx+GRID_ABS_SIZE(1), rby+GRID_ABS_SIZE(1),
       obj_ptr->bbox.ltx-GRID_ABS_SIZE(1), obj_ptr->bbox.lty-GRID_ABS_SIZE(1),
@@ -965,14 +1017,74 @@ int ExecFlipDeck (buf, obj_ptr)
             {
                XEvent	ev;
 
-               if (XCheckMaskEvent (mainDisplay,
-                     ExposureMask | VisibilityChangeMask, &ev))
+               while (XCheckMaskEvent (mainDisplay, StructureNotifyMask, &ev))
+               {
+                  if (iconWindowShown)
+                  {
+                     if (ev.xany.window == iconBaseWindow &&
+                           ev.type == UnmapNotify ||
+                           ev.xany.window == mainWindow &&
+                           ev.type == MapNotify)
+                     {
+                        XPutBackEvent (mainDisplay, &ev);
+                        looping = FALSE;
+                        rc = FALSE;
+                        break;
+                     }
+                  }
+                  else
+                  {
+                     if (ev.xany.window == iconBaseWindow &&
+                           ev.type == MapNotify ||
+                           ev.xany.window == mainWindow &&
+                           ev.type == UnmapNotify)
+                     {
+                        XPutBackEvent (mainDisplay, &ev);
+                        looping = FALSE;
+                        rc = FALSE;
+                        break;
+                     }
+                  }
+               }
+               if (!looping) break;
+               while (XCheckMaskEvent (mainDisplay, VisibilityChangeMask, &ev))
+               {
+                  if (iconWindowShown)
+                  {
+                     if (ev.xany.window == mainWindow &&
+                           ev.type == VisibilityNotify &&
+                           ev.xvisibility.state == VisibilityUnobscured)
+                     {
+                        XPutBackEvent (mainDisplay, &ev);
+                        looping = FALSE;
+                        rc = FALSE;
+                        break;
+                     }
+                     else
+                        ExposeEventHandler (&ev, TRUE);
+                  }
+                  else
+                  {
+                     if (ev.xany.window == iconBaseWindow &&
+                           ev.type == VisibilityNotify &&
+                           ev.xvisibility.state == VisibilityUnobscured)
+                     {
+                        XPutBackEvent (mainDisplay, &ev);
+                        looping = FALSE;
+                        rc = FALSE;
+                        break;
+                     }
+                     else
+                        ExposeEventHandler (&ev, TRUE);
+                  }
+               }
+               if (!looping) break;
+               if (XCheckMaskEvent (mainDisplay, ExposureMask, &ev))
                {
                   ExposeEventHandler (&ev, TRUE);
-                  while (XCheckMaskEvent (mainDisplay,
-                        ExposureMask | VisibilityChangeMask, &ev)) ;
+                  while (XCheckMaskEvent (mainDisplay, ExposureMask, &ev)) ;
                }
-               else if (XCheckMaskEvent (mainDisplay,
+               if (XCheckMaskEvent (mainDisplay,
                      ButtonPressMask | KeyPressMask, &ev))
                {
                   Msg ("flip_deck():  User interrupt.");
@@ -981,7 +1093,7 @@ int ExecFlipDeck (buf, obj_ptr)
                   break;
                }
             }
-         } while (ms_interval > 0);
+         } while (looping && ms_interval > 0);
       }
    }
 
