@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,9 +25,10 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/grid.c,v 2.83 1993/06/16 02:23:43 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/grid.c,v 2.94 1994/05/03 04:49:10 william Exp $";
 #endif
 
+#include <stdio.h>
 #include <X11/Xlib.h>
 #include "const.h"
 #include "types.h"
@@ -37,6 +39,9 @@ static char RCSid[] =
 #include "drawing.e"
 #include "dup.e"
 #include "file.e"
+#ifndef _NO_EXTERN
+#include "grid.e"
+#endif
 #include "mainmenu.e"
 #include "menu.e"
 #include "msg.e"
@@ -66,8 +71,8 @@ int	mapShown = TRUE;
 char * layoutMenuStr[] =
       {  "+Grid        #i",
          "-Grid        #d",
-         "HideGrid",
-         "SnapOff",
+         "HideGrid     #g",
+         "SnapOff     ^#g",
          "ScrollToOrig #^",
          "SaveOrigin     ",
          "ZoomIn       #z",
@@ -77,14 +82,43 @@ char * layoutMenuStr[] =
          "Landscape",
          "Portrait",
          "SetReduction #%",
-         "RawPSFile",
-         "ColorPS",
-         "ConstMove",
-         "MetricGrid",
-         "HideBit/Pixmap",
-         "UseGrayScale"
+         "RawPSFile   ^#x",
+         "ColorPS     ^#k",
+         "ConstMove    #@",
+         "MetricGrid  ^#-",
+         "HideBit/Pixmap ",
+         "UseGrayScale   ",
+         "ShowMeasurement",
+         "ShowMenubar    ",
+         "ShowStatus     "
       };
 #define MAX_LAYOUT_STR 16 /* longest string length in layoutMenuStr[] */
+static char * layoutMenuDescription[] =
+      {  "Increment grid size",
+         "Decrement grid size",
+         "Hide/show grid lines",
+         "Toggle the snapping to grid points effect",
+         "Scroll to the 'origin' of the drawing",
+         "Save current location as the 'origin' of the drawing",
+         "Zoom in",
+         "Zoom out",
+         "No zoom",
+         "Zoom way out to see the whole drawing",
+         "Use landscape page style",
+         "Use portrait page style",
+         "Specify reduction/magnification of the whole drawing",
+         "Set printing mode/format",
+         "Toggle between color and black & white printing modes",
+         "Toggle between constrained and unconstrained move modes",
+         "Toggle between English and Metric grid systems",
+         "Toggle between hiding/showing detailed bitmap/pixmap objects",
+         "Use gray scales for tiling patterns to speed up printing",
+         "Show cursor measurements",
+         "Show menubar window",
+         "Show status window",
+         NULL
+      };
+#define MAX_LAYOUT_DESC 80 /* longest string length in layoutMenuStr[] */
 
 void MyHDotLine (Win, Y, XStart, XEnd)
    Window	Win;
@@ -305,31 +339,72 @@ void DrawPageLines (Win, LtX, LtY, W, H)
    }
 }
 
-static char showGridStr[]         = "ShowGrid     #g";
-static char hideGridStr[]         = "HideGrid     #g";
+static char showGridStr[]        = "ShowGrid     #g";
+static char hideGridStr[]        = "HideGrid     #g";
+static char showGridDesc[]       = "Show grid lines";
+static char hideGridDesc[]       = "Hide grid lines";
 
-static char snapOnStr[]           = "SnapOn      ^#g";
-static char snapOffStr[]          = "SnapOff     ^#g";
+static char snapOnStr[]          = "SnapOn      ^#g";
+static char snapOffStr[]         = "SnapOff     ^#g";
+static char snapOnDesc[]         = "Turn on snapping to grid points";
+static char snapOffDesc[]        = "Turn off snapping to grid points";
 
-static char printerStr[]          = "Printer     ^#x";
-static char latexFigStr[]         = "LaTeXFig    ^#x";
-static char psFileStr[]           = "RawPSFile   ^#x";
-static char xbmStr[]              = "XBitmap     ^#x";
+static char printerStr[]         = "Printer     ^#x";
+static char latexFigStr[]        = "LaTeXFig    ^#x";
+static char psFileStr[]          = "RawPSFile   ^#x";
+static char xbmStr[]             = "XBitmap     ^#x";
+static char xpmStr[]             = "XPixmap     ^#x";
+static char textFileStr[]        = "TextFile    ^#x";
+static char epsiFileStr[]        = "EPSI        ^#x";
+static char printerDesc[]        = "Set printing mode to print to a printer";
+static char latexFigDesc[]       = "Set printing mode to generate EPS files";
+static char psFileDesc[]         = "Set printing mode to generate PS files";
+static char xbmDesc[]            = "Set printing mode to generate bitmap files";
+static char xpmDesc[]            = "Set printing mode to generate pixmap files";
+static char textFileDesc[]       = "Set printing mode to generate ASCII files";
+static char epsiFileDesc[]       = "Set printing mode to generate EPSI files";
 
-static char bwPostScriptStr[]     = "BlkWhtPS    ^#k";
-static char colorPostScriptStr[]  = "ColorPS     ^#k";
+static char bwPostScriptStr[]    = "BlkWhtPS    ^#k";
+static char colorPostScriptStr[] = "ColorPS     ^#k";
+static char bwPostScriptDesc[]   = "Set black & white printing mode";
+static char colorPostScriptDesc[]= "Set color printing mode";
 
-static char constMoveStr[]        = "ConstMove    #@";
-static char unConstMoveStr[]      = "UnConstMove  #@";
+static char constMoveStr[]       = "ConstMove    #@";
+static char unConstMoveStr[]     = "UnConstMove  #@";
+static char constMoveDesc[]      = "Constrained move mode";
+static char unConstMoveDesc[]    = "Unconstrained move mode";
 
-static char englishGridSysStr[]   = "EnglishGrid ^#-";
-static char metricGridSysStr[]    = "MetricGrid  ^#-";
+static char englishGridSysStr[]  = "EnglishGrid ^#-";
+static char metricGridSysStr[]   = "MetricGrid  ^#-";
+static char englishGridSysDesc[] = "Use English grid system";
+static char metricGridSysDesc[]  = "Use Metric grid system";
 
-static char showMapStr[]          = "ShowBit/Pixmap";
-static char hideMapStr[]          = "HideBit/Pixmap";
+static char showMapStr[]         = "ShowBit/Pixmap";
+static char hideMapStr[]         = "HideBit/Pixmap";
+static char showMapDesc[]        = "Show detailed bitmap/pixmap objects";
+static char hideMapDesc[]        = "Show outlines for bitmap/pixmap objects";
 
-static char useGrayStr[]          = "UseGrayScale";
-static char noGrayStr[]           = "NoGrayScale";
+static char useGrayStr[]         = "UseGrayScale";
+static char noGrayStr[]          = "NoGrayScale";
+static char useGrayDesc[]        =
+      "Use gray scales for tiling patterns to speed up printing";
+static char noGrayDesc[]         =
+      "Use tiling patterns as is (may be slow for printing";
+
+static char hideMeasureStr[]     = "HideMeasurement";
+static char showMeasureStr[]     = "ShowMeasurement";
+static char hideMeasureDesc[]    = "Hide cursor measurements";
+static char showMeasureDesc[]    = "Show cursor measurements";
+
+static char hideMenubarStr[]     = "HideMenubar";
+static char showMenubarStr[]     = "ShowMenubar";
+static char hideMenubarDesc[]    = "Hide menubar window";
+static char showMenubarDesc[]    = "Show menubar window";
+
+static char hideStatusStr[]      = "HideStatus";
+static char showStatusStr[]      = "ShowStatus";
+static char hideStatusDesc[]     = "Hide status window";
+static char showStatusDesc[]     = "Show status window";
 
 void ToggleGridSystem ()
 {
@@ -428,6 +503,7 @@ void ToggleColorPostScript ()
    else
       Msg ("Will print in black-and-white (PS and bitmap).");
    ShowWhereToPrint ();
+   SetFileModified (TRUE);
    UpdateSubMenu (MENU_LAYOUT);
 }
 
@@ -465,6 +541,28 @@ void ToggleUseGray ()
    UpdateSubMenu (MENU_LAYOUT);
 }
 
+void ToggleShowMenubar ()
+{
+   noMenubar = !noMenubar;
+   if (noMenubar)
+      XUnmapWindow (mainDisplay, menubarWindow);
+   else
+      XMapWindow (mainDisplay, menubarWindow);
+   Reconfigure (TRUE);
+   UpdateSubMenu (MENU_LAYOUT);
+}
+
+void ToggleShowStatus ()
+{
+   noStatusWindow = !noStatusWindow;
+   if (noStatusWindow)
+      XUnmapWindow (mainDisplay, statusWindow);
+   else
+      XMapWindow (mainDisplay, statusWindow);
+   Reconfigure (TRUE);
+   UpdateSubMenu (MENU_LAYOUT);
+}
+
 void ToggleWhereToPrint ()
 {
    whereToPrint = (whereToPrint+1) % MAXWHERETOPRINT;
@@ -481,6 +579,10 @@ void ToggleWhereToPrint ()
          else
             Msg ("Will print X11 Bitmap file.");
          break;
+      case TEXT_FILE: Msg ("Will print ASCII text file."); break;
+      case EPSI_FILE:
+         Msg ("Will print EPSI file (EPS with preview bitmap).");
+         break;
    }
    ShowWhereToPrint ();
    UpdateSubMenu (MENU_LAYOUT);
@@ -944,86 +1046,179 @@ void LayoutSubMenu (index)
       case LAYOUT_TOGGLE_GRID_SYSTEM: ToggleGridSystem (); break;
       case LAYOUT_TOGGLE_MAP_SHOWN: ToggleMapShown (); break;
       case LAYOUT_TOGGLE_USE_GRAY: ToggleUseGray (); break;
+      case LAYOUT_TOGGLE_SHOW_MEASURE: ToggleShowMeasurement (); break;
+      case LAYOUT_TOGGLE_SHOW_MENUBAR: ToggleShowMenubar (); break;
+      case LAYOUT_TOGGLE_SHOW_STATUS: ToggleShowStatus (); break;
    }
 }
 
-void SetUpLayoutMenuStr (menu_strings)
-   char	* * menu_strings;
+void SetUpLayoutMenuStr (menu_strings, menu_description)
+   char	* * menu_strings, * * menu_description;
 {
    register int	i;
-   char		* str_ptr;
+   char		* str_ptr, * desc_ptr;
 
    str_ptr = (char *) calloc (MAX_LAYOUT_STR*MAXLAYOUTMENUS, sizeof(char));
-   for (i = 0; i < MAXLAYOUTMENUS; i++, str_ptr += MAX_LAYOUT_STR)
+   desc_ptr = (char *) calloc (MAX_LAYOUT_DESC*MAXLAYOUTMENUS, sizeof(char));
+   for (i = 0; i < MAXLAYOUTMENUS; i++, str_ptr += MAX_LAYOUT_STR,
+         desc_ptr += MAX_LAYOUT_DESC)
    {
       menu_strings[i] = str_ptr;
       strcpy (menu_strings[i], layoutMenuStr[i]);
+      menu_description[i] = desc_ptr;
+      strcpy (menu_description[i], layoutMenuDescription[i]);
    }
    if (gridShown)
+   {
       strcpy (menu_strings[LAYOUT_GRID], hideGridStr);
+      strcpy (menu_description[LAYOUT_GRID], hideGridDesc);
+   }
    else
+   {
       strcpy (menu_strings[LAYOUT_GRID], showGridStr);
+      strcpy (menu_description[LAYOUT_GRID], showGridDesc);
+   }
    if (gridOn)
+   {
       strcpy (menu_strings[LAYOUT_SNAP], snapOffStr);
+      strcpy (menu_description[LAYOUT_SNAP], snapOffDesc);
+   }
    else
+   {
       strcpy (menu_strings[LAYOUT_SNAP], snapOnStr);
+      strcpy (menu_description[LAYOUT_SNAP], snapOnDesc);
+   }
    switch (whereToPrint)
    {
       case PRINTER:
          strcpy (menu_strings[LAYOUT_TOGGLE_WHERE_TO_PRINT], latexFigStr);
+         strcpy (menu_description[LAYOUT_TOGGLE_WHERE_TO_PRINT], latexFigDesc);
          break;
       case LATEX_FIG:
          strcpy (menu_strings[LAYOUT_TOGGLE_WHERE_TO_PRINT], psFileStr);
+         strcpy (menu_description[LAYOUT_TOGGLE_WHERE_TO_PRINT], psFileDesc);
          break;
       case PS_FILE:
-         strcpy (menu_strings[LAYOUT_TOGGLE_WHERE_TO_PRINT], xbmStr);
+         if (colorDump)
+         {
+            strcpy (menu_strings[LAYOUT_TOGGLE_WHERE_TO_PRINT], xpmStr);
+            strcpy (menu_description[LAYOUT_TOGGLE_WHERE_TO_PRINT], xpmDesc);
+         }
+         else
+         {
+            strcpy (menu_strings[LAYOUT_TOGGLE_WHERE_TO_PRINT], xbmStr);
+            strcpy (menu_description[LAYOUT_TOGGLE_WHERE_TO_PRINT], xbmDesc);
+         }
          break;
       case XBM_FILE:
+         strcpy (menu_strings[LAYOUT_TOGGLE_WHERE_TO_PRINT], textFileStr);
+         strcpy (menu_description[LAYOUT_TOGGLE_WHERE_TO_PRINT], textFileDesc);
+         break;
+      case TEXT_FILE:
+         strcpy (menu_strings[LAYOUT_TOGGLE_WHERE_TO_PRINT], epsiFileStr);
+         strcpy (menu_description[LAYOUT_TOGGLE_WHERE_TO_PRINT], epsiFileDesc);
+         break;
+      case EPSI_FILE:
          strcpy (menu_strings[LAYOUT_TOGGLE_WHERE_TO_PRINT], printerStr);
+         strcpy (menu_description[LAYOUT_TOGGLE_WHERE_TO_PRINT], printerDesc);
          break;
    }
    if (colorDump)
+   {
       strcpy (menu_strings[LAYOUT_TOGGLE_COLOR_PS], bwPostScriptStr);
+      strcpy (menu_description[LAYOUT_TOGGLE_COLOR_PS], bwPostScriptDesc);
+   }
    else
+   {
       strcpy (menu_strings[LAYOUT_TOGGLE_COLOR_PS], colorPostScriptStr);
+      strcpy (menu_description[LAYOUT_TOGGLE_COLOR_PS], colorPostScriptDesc);
+   }
    switch (moveMode)
    {
       case CONST_MOVE:
          strcpy (menu_strings[LAYOUT_TOGGLE_MOVE_MODE], unConstMoveStr);
+         strcpy (menu_description[LAYOUT_TOGGLE_MOVE_MODE], unConstMoveDesc);
          break;
       case UNCONST_MOVE:
          strcpy (menu_strings[LAYOUT_TOGGLE_MOVE_MODE], constMoveStr);
+         strcpy (menu_description[LAYOUT_TOGGLE_MOVE_MODE], constMoveDesc);
          break;
    }
    switch (gridSystem)
    {
       case ENGLISH_GRID:
          strcpy (menu_strings[LAYOUT_TOGGLE_GRID_SYSTEM], metricGridSysStr);
+         strcpy (menu_description[LAYOUT_TOGGLE_GRID_SYSTEM],
+               metricGridSysDesc);
          break;
       case METRIC_GRID:
          strcpy (menu_strings[LAYOUT_TOGGLE_GRID_SYSTEM], englishGridSysStr);
+         strcpy (menu_description[LAYOUT_TOGGLE_GRID_SYSTEM],
+               englishGridSysDesc);
          break;
    }
    if (mapShown)
+   {
       strcpy (menu_strings[LAYOUT_TOGGLE_MAP_SHOWN], hideMapStr);
+      strcpy (menu_description[LAYOUT_TOGGLE_MAP_SHOWN], hideMapDesc);
+   }
    else
+   {
       strcpy (menu_strings[LAYOUT_TOGGLE_MAP_SHOWN], showMapStr);
-
+      strcpy (menu_description[LAYOUT_TOGGLE_MAP_SHOWN], showMapDesc);
+   }
    if (useGray)
+   {
       strcpy (menu_strings[LAYOUT_TOGGLE_USE_GRAY], noGrayStr);
+      strcpy (menu_description[LAYOUT_TOGGLE_USE_GRAY], noGrayDesc);
+   }
    else
+   {
       strcpy (menu_strings[LAYOUT_TOGGLE_USE_GRAY], useGrayStr);
+      strcpy (menu_description[LAYOUT_TOGGLE_USE_GRAY], useGrayDesc);
+   }
+   if (showMeasurement)
+   {
+      strcpy (menu_strings[LAYOUT_TOGGLE_SHOW_MEASURE], hideMeasureStr);
+      strcpy (menu_description[LAYOUT_TOGGLE_SHOW_MEASURE], hideMeasureDesc);
+   }
+   else
+   {
+      strcpy (menu_strings[LAYOUT_TOGGLE_SHOW_MEASURE], showMeasureStr);
+      strcpy (menu_description[LAYOUT_TOGGLE_SHOW_MEASURE], showMeasureDesc);
+   }
+   if (noMenubar)
+   {
+      strcpy (menu_strings[LAYOUT_TOGGLE_SHOW_MENUBAR], showMenubarStr);
+      strcpy (menu_description[LAYOUT_TOGGLE_SHOW_MENUBAR], showMenubarDesc);
+   }
+   else
+   {
+      strcpy (menu_strings[LAYOUT_TOGGLE_SHOW_MENUBAR], hideMenubarStr);
+      strcpy (menu_description[LAYOUT_TOGGLE_SHOW_MENUBAR], hideMenubarDesc);
+   }
+   if (noStatusWindow)
+   {
+      strcpy (menu_strings[LAYOUT_TOGGLE_SHOW_STATUS], showStatusStr);
+      strcpy (menu_description[LAYOUT_TOGGLE_SHOW_STATUS], showStatusDesc);
+   }
+   else
+   {
+      strcpy (menu_strings[LAYOUT_TOGGLE_SHOW_STATUS], hideStatusStr);
+      strcpy (menu_description[LAYOUT_TOGGLE_SHOW_STATUS], hideStatusDesc);
+   }
 }
 
-void LayoutMenu (X, Y)
-   int	X, Y;
+int LayoutMenu (X, Y, TrackMenubar)
+   int	X, Y, TrackMenubar;
 {
    int		index, * fore_colors, * valid, * init_rv;
    char		* menu_strings[MAXLAYOUTMENUS];
+   char		* menu_description[MAXLAYOUTMENUS];
 
-   SetUpLayoutMenuStr (menu_strings);
+   SetUpLayoutMenuStr (menu_strings, menu_description);
 
-   DefaultColorArrays (MAXLAYOUTMENUS, &fore_colors, &valid, &init_rv);
+   DefaultColorArrays (MAXLAYOUTMENUS, &fore_colors, &valid, &init_rv, NULL);
 
    switch (pageStyle)
    {
@@ -1032,11 +1227,14 @@ void LayoutMenu (X, Y)
    }
    activeMenu = MENU_LAYOUT;
    index = TextMenuLoop (X, Y, menu_strings, MAXLAYOUTMENUS,
-         fore_colors, valid, init_rv, SINGLECOLOR);
+         fore_colors, valid, init_rv, menu_description, SINGLECOLOR,
+         TrackMenubar);
 
    cfree (menu_strings[0]);
+   cfree (menu_description[0]);
 
-   if (index != INVALID) LayoutSubMenu (index);
+   if (index >= 0) LayoutSubMenu (index);
+   return (index);
 }
 
 void GridXY (X, Y, GridX, GridY)
@@ -1101,17 +1299,26 @@ void MoveModeSubMenu (index)
    UpdateSubMenu (MENU_MOVEMODE);
 }
 
-void MoveModeMenu (X, Y)
-   int	X, Y;
+static char * moveModeMenuDescription[] =
+{
+   "Constrained move mode",
+   "Unconstrained move mode",
+   NULL
+};
+
+int MoveModeMenu (X, Y, TrackMenubar)
+   int	X, Y, TrackMenubar;
 {
    int	index, * fore_colors, * valid, * init_rv;
 
-   DefaultColorArrays (MAXMOVEMODES, &fore_colors, &valid, &init_rv);
+   DefaultColorArrays (MAXMOVEMODES, &fore_colors, &valid, &init_rv, NULL);
    cfree (valid);
    init_rv[moveMode] = TRUE;
    activeMenu = MENU_MOVEMODE;
    index = PxMpMenuLoop (X, Y, choiceImageW, choiceImageH, MAXMOVEMODES, 1,
-         MAXMOVEMODES, fore_colors, moveModePixmap, init_rv, SINGLECOLOR);
+         MAXMOVEMODES, fore_colors, moveModePixmap, init_rv,
+         moveModeMenuDescription, SINGLECOLOR, TrackMenubar);
 
-   if (index != INVALID) MoveModeSubMenu (index);
+   if (index >= 0) MoveModeSubMenu (index);
+   return (index);
 }
