@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,11 +25,12 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/mainloop.c,v 2.79.1.17 1993/06/10 03:10:06 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/mainloop.c,v 2.100 1994/04/04 00:49:06 william Exp $";
 #endif
 
 #include <stdio.h>
 #include <signal.h>
+#include <string.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #ifdef USE_XT_INITIALIZE
@@ -50,6 +52,9 @@ static char RCSid[] =
 #include "file.e"
 #include "font.e"
 #include "grid.e"
+#ifndef _NO_EXTERN
+#include "mainloop.e"
+#endif
 #include "mainmenu.e"
 #include "menu.e"
 #include "msg.e"
@@ -68,9 +73,15 @@ static char RCSid[] =
 #include "xbitmap.e"
 #include "xpixmap.e"
 
+extern int	atoi ARGS_DECL((char *));
+#ifndef _NO_EXTERN
+extern int	fork ARGS_DECL((void));
+extern int	system ARGS_DECL((char *));
+#endif
+
 #ifdef USE_XAPPLRESDIR
-extern char	* getenv ();
-extern int	putenv ();
+extern char	* getenv ARGS_DECL((char *));
+extern int	putenv ARGS_DECL((char *));
 #else
 #ifdef USE_XT_INITIALIZE
 Widget		toplevel;
@@ -230,6 +241,7 @@ void CleanUp ()
    CleanUpDrawingWindow ();
    CleanUpPage ();
    CleanUpStk ();
+   CleanUpStatus ();
    CleanUpChoices ();
 
    CleanUpScrolls ();
@@ -256,6 +268,7 @@ void CleanUp ()
       XDestroyWindow (mainDisplay, iconBaseWindow);
       iconWindowCreated = FALSE;
    }
+   CleanUpResiduals ();
    XDestroyWindow (mainDisplay, mainWindow);
 #ifndef DONT_FREE_COLORMAP
    if (newColormapUsed) XFreeColormap (mainDisplay, mainColormap);
@@ -288,12 +301,12 @@ void ExposeEventHandler (input, recursive)
    else if (input->xany.window == vRuleWindow)
    {
       while (XCheckWindowEvent (mainDisplay, vRuleWindow, ExposureMask, &ev)) ;
-      RedrawVRuler (&input);
+      RedrawVRuler ();
    }
    else if (input->xany.window == hRuleWindow)
    {
       while (XCheckWindowEvent (mainDisplay, hRuleWindow, ExposureMask, &ev)) ;
-      RedrawHRuler (&input);
+      RedrawHRuler ();
    }
    else if (input->xany.window == iconWindow && input->type == Expose)
    {
@@ -305,6 +318,11 @@ void ExposeEventHandler (input, recursive)
       while (XCheckWindowEvent (mainDisplay, titleWindow, ExposureMask, &ev)) ;
       RedrawTitleWindow ();
    }
+   else if (input->xany.window == menubarWindow)
+   {
+      while (XCheckWindowEvent(mainDisplay,menubarWindow,ExposureMask,&ev)) ;
+      RedrawMenubarWindow ();
+   }
    else if (input->xany.window == msgWindow)
    {
       while (XCheckWindowEvent (mainDisplay, msgWindow, ExposureMask, &ev)) ;
@@ -320,6 +338,11 @@ void ExposeEventHandler (input, recursive)
       while (XCheckWindowEvent (mainDisplay, hSBarWindow, ExposureMask, &ev)) ;
       RedrawHScrollWindow ();
    }
+   else if (input->xany.window == statusWindow)
+   {
+      while (XCheckWindowEvent (mainDisplay, statusWindow, ExposureMask, &ev)) ;
+      RedrawStatusWindow ();
+   }
    else if (input->xany.window == dummyWindow1)
    {
       while (XCheckWindowEvent (mainDisplay, dummyWindow1, ExposureMask, &ev)) ;
@@ -331,15 +354,22 @@ void ExposeEventHandler (input, recursive)
       RedrawMainMenuWindow ();
    }
    else if (input->xany.window==mainWindow && input->type==VisibilityNotify &&
-         input->xvisibility.state==VisibilityUnobscured && !iconWindowShown)
+         input->xvisibility.state==VisibilityUnobscured)
    {
+      XEvent	tmp_ev;
+
       while (XCheckWindowEvent (mainDisplay, mainWindow, VisibilityChangeMask,
-            &ev)) ;
-      if (pinnedMainMenu) XMapRaised (mainDisplay, mainMenuWindow);
-      for (i = 0; i < numExtraWins; i++)
-         if (extraWinInfo[i].window != None && extraWinInfo[i].mapped &&
-               extraWinInfo[i].raise)
-            XMapRaised (mainDisplay, extraWinInfo[i].window);
+            &tmp_ev)) ;
+      if (iconWindowShown)
+         UnIconify ();
+      else
+      {
+         if (pinnedMainMenu) XMapRaised (mainDisplay, mainMenuWindow);
+         for (i = 0; i < numExtraWins; i++)
+            if (extraWinInfo[i].window != None && extraWinInfo[i].mapped &&
+                  extraWinInfo[i].raise)
+               XMapRaised (mainDisplay, extraWinInfo[i].window);
+      }
    }
    else
    {
@@ -382,8 +412,9 @@ void MainLoop (Op, FileName, FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3)
          sprintf (initMsg1, "%s Version %s (patchlevel %1d)", TOOL_NAME,
                version_string, TGIF_PATCHLEVEL);
       sprintf (initMsg2, "%s%s",
-            "Copyright (C) 1990, 1991, 1992, William Chia-Wei Cheng ",
+            "Copyright (C) 1990-1994, William Chia-Wei Cheng ",
             "(william@cs.UCLA.edu)");
+      TwoLineMsg (initMsg1, initMsg2);
       fprintf (stderr, "%s\n", initMsg1);
       fprintf (stderr, "%s\n", initMsg2);
       fflush(stderr);
@@ -470,11 +501,12 @@ void MainLoop (Op, FileName, FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3)
 
       Setup ();
       InitExtraWinInfo ();
-      TwoLineMsg (initMsg1, initMsg2);
       quitDraw = FALSE;
 
       if (FileName[0] != '\0')
       {
+         int obj_file=TRUE;
+
          sprintf (obj_ext_str, ".%s", OBJ_FILE_EXT);
          obj_ext_len = strlen (obj_ext_str);
          len = strlen (FileName);
@@ -483,9 +515,21 @@ void MainLoop (Op, FileName, FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3)
                strcmp (&FileName[len-obj_ext_len], obj_ext_str) == 0)
             strcpy (file_name, FileName);
          else
-            sprintf (file_name, "%s.%s", FileName, OBJ_FILE_EXT);
+         {
+            sprintf (obj_ext_str, ".%s", SYM_FILE_EXT);
+            obj_ext_len = strlen (obj_ext_str);
 
-         if (short_name = IsPrefix (bootDir, file_name, &rest)) ++rest;
+            if (len >= obj_ext_len &&
+                  strcmp (&FileName[len-obj_ext_len], obj_ext_str) == 0)
+            {
+               strcpy (file_name, FileName);
+               obj_file = FALSE;
+            }
+            else
+               sprintf (file_name, "%s.%s", FileName, OBJ_FILE_EXT);
+         }
+
+         if ((short_name = IsPrefix (bootDir, file_name, &rest))) ++rest;
          if ((fp = fopen (file_name, "r")) == NULL)
          {
             lastPageNum = 1;
@@ -502,8 +546,13 @@ void MainLoop (Op, FileName, FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3)
             else
                sprintf (full_name, "%s/%s", curDir, file_name);
 
-            SetCurDir (full_name);
-            *curSymDir = '\0';
+            if (obj_file)
+            {
+               SetCurDir (full_name);
+               *curSymDir = '\0';
+            }
+            else
+               SetCurSymDir (full_name);
             curFileDefined = TRUE;
 
             SetFileModified (FALSE);
@@ -515,6 +564,7 @@ void MainLoop (Op, FileName, FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3)
          }
          else
          {
+            SetFileModified (FALSE);
             if (short_name)
                sprintf (s, "Loading '%s' ...", rest);
             else
@@ -530,6 +580,7 @@ void MainLoop (Op, FileName, FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3)
             SetWatchCursor (mainWindow);
 
             readingPageNum = 0;
+            loadedCurPageNum = 0;
             while ((read_status = ReadObj (fp, &obj_ptr)) == TRUE)
                if (obj_ptr != NULL)
                {
@@ -555,17 +606,26 @@ void MainLoop (Op, FileName, FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3)
                else
                   sprintf (full_name, "%s/%s", curDir, file_name);
 
-               SetCurDir (full_name);
-               *curSymDir = '\0';
+               if (obj_file)
+               {
+                  SetCurDir (full_name);
+                  *curSymDir = '\0';
+               }
+               else
+                  SetCurSymDir (full_name);
                curFileDefined = TRUE;
 
-               SetFileModified (FALSE);
                if (short_name)
                   sprintf (s, "Current file is '%s'.", rest);
                else
                   sprintf (s, "Current file is '%s'.", file_name);
                Msg (s);
             }
+            if (loadedCurPageNum <= 0)
+            {
+               loadedCurPageNum = curPageNum = lastPageNum = 1;
+               InitPage ();
+            }
             GotoPageNum (loadedCurPageNum);
             SetDefaultCursor (mainWindow);
             SetDefaultCursor (drawWindow);
@@ -730,22 +790,42 @@ void MainLoop (Op, FileName, FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3)
             return;
          }
       }
-      else if (input.xany.window == vRuleWindow)
-         RedrawVRuler (&input);
-      else if (input.xany.window == hRuleWindow)
-         RedrawHRuler (&input);
+      else if (input.xany.window == vRuleWindow ||
+            input.xany.window == hRuleWindow)
+         RulersEventHandler (&input);
       else if (input.xany.window == iconWindow ||
             input.xany.window == iconBaseWindow)
          IconEventHandler (&input);
       else if (input.xany.window == titleWindow)
          TitleEventHandler (&input);
+      else if (menubarWindow != None && input.xany.window == menubarWindow)
+      {
+         if ((rc = MenubarEventHandler (&input)) != INVALID)
+         {
+            AllocStrings (FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3);
+            strcpy (*FuncStr, fileMenuStr[rc]);
+            strcpy (*Str1, curDomainName);
+            sprintf (*Menu1, "tmpmodel.%s", OBJ_FILE_EXT);
+            strcpy (*Str2, "");
+            for (c_ptr = *FuncStr; *c_ptr != '\0'; c_ptr++)
+               if (*c_ptr == ' ')
+               {
+                  *c_ptr = '\0';
+                  break;
+               }
+            return;
+         }
+      }
       else if (input.xany.window == msgWindow)
          MsgEventHandler (&input);
       else if (input.xany.window == vSBarWindow ||
             input.xany.window == hSBarWindow)
          ScrollEventHandler (&input);
-      else if (input.xany.window == dummyWindow1)
-         RedrawDummyWindow1 ();
+      else if (statusWindow != None && input.xany.window == statusWindow)
+         StatusEventHandler (&input);
+      else if (input.xany.window == dummyWindow1 ||
+            input.xany.window == dummyWindow2)
+         DummiesEventHandler (&input);
       else if (input.xany.type == MappingNotify)
          XRefreshKeyboardMapping (&(input.xmapping));
       else if (pinnedMainMenu && input.xany.window == mainMenuWindow)
