@@ -1,9 +1,9 @@
 /*
- * Author:	Kou1 Ma2da (matsuda@ccs.mt.nec.co.jp)
- * Modified By:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Original draft:	Kou1 Ma2da (matsuda@ccs.mt.nec.co.jp)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -13,7 +13,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -25,7 +26,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/mainmenu.c,v 1.51.1.8 1993/06/07 02:42:38 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/mainmenu.c,v 2.7 1994/04/04 00:49:06 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -43,7 +44,11 @@ static char RCSid[] =
 #include "font.e"
 #include "grid.e"
 #include "mainloop.e"
+#ifndef _NO_EXTERN
+#include "mainmenu.e"
+#endif
 #include "menu.e"
+#include "msg.e"
 #include "page.e"
 #include "pattern.e"
 #include "raster.e"
@@ -51,6 +56,8 @@ static char RCSid[] =
 #include "special.e"
 #include "text.e"
 
+extern int	atoi ARGS_DECL((char *));
+
 #define DEF_MAINMENUPINDIST 80
 
 int	pinnedMainMenu = FALSE;
@@ -122,7 +129,8 @@ void InitMainMenu ()
    XSetTransientForHint (mainDisplay, mainMenuWindow, mainWindow);
 
    XSelectInput (mainDisplay, mainMenuWindow, StructureNotifyMask |
-         ExposureMask | ButtonPressMask | ButtonReleaseMask | KeyPressMask);
+         ExposureMask | ButtonPressMask | ButtonReleaseMask | KeyPressMask |
+         EnterWindowMask);
 
    sizehints.flags = PSize | PMinSize | PMaxSize;
    sizehints.width = sizehints.min_width = sizehints.max_width =
@@ -265,6 +273,11 @@ int MainMenuEventHandler (input)
 
       RedrawMainMenuWindow ();
    }
+   else if (input->type == EnterNotify)
+   {
+      SetMouseStatus ("Move pinned menu", "Execute Menu Command",
+            "Close pinned menu");
+   }
    else if (input->type == ConfigureNotify)
    {
       mainMenuX = input->xconfigure.x;
@@ -284,27 +297,27 @@ int MainMenuEventHandler (input)
             y = button_ev->y_root;
             switch (index)
             {
-               case MENU_MODE: ModeMenu (x, y); break;
-               case MENU_FILE: rc = FileMenu (x, y); break;
-               case MENU_EDIT: EditMenu (x, y); break;
-               case MENU_LAYOUT: LayoutMenu (x, y); break;
-               case MENU_MOVEMODE: MoveModeMenu (x, y); break;
-               case MENU_ARRANGE: ArrangeMenu (x, y); break;
-               case MENU_HORIALIGN: HoriAlignMenu (x, y); break;
-               case MENU_VERTALIGN: VertAlignMenu (x, y); break;
-               case MENU_FONT: FontMenu (x, y); break;
-               case MENU_STYLE: StyleMenu (x, y); break;
-               case MENU_SIZE: SizeMenu (x, y); break;
-               case MENU_LINEDASH: LineDashMenu (x, y); break;
-               case MENU_LINESTYLE: LineStyleMenu (x, y); break;
-               case MENU_LINETYPE: LineTypeMenu (x, y); break;
-               case MENU_LINEWIDTH: LineWidthMenu (x, y); break;
-               case MENU_FILL: FillMenu (x, y); break;
-               case MENU_PEN: PenMenu (x, y); break;
-               case MENU_COLOR: ColorMenu (x, y); break;
-               case MENU_SPECIAL: SpecialMenu (x, y); break;
-               case MENU_PAGE: PageMenu (x, y); break;
-               case MENU_PAGELAYOUT: PageLayoutMenu (x, y); break;
+               case MENU_MODE: ModeMenu (x, y, FALSE); break;
+               case MENU_FILE: rc = FileMenu (x, y, FALSE); break;
+               case MENU_EDIT: EditMenu (x, y, FALSE); break;
+               case MENU_LAYOUT: LayoutMenu (x, y, FALSE); break;
+               case MENU_MOVEMODE: MoveModeMenu (x, y, FALSE); break;
+               case MENU_ARRANGE: ArrangeMenu (x, y, FALSE); break;
+               case MENU_HORIALIGN: HoriAlignMenu (x, y, FALSE); break;
+               case MENU_VERTALIGN: VertAlignMenu (x, y, FALSE); break;
+               case MENU_FONT: FontMenu (x, y, FALSE); break;
+               case MENU_STYLE: StyleMenu (x, y, FALSE); break;
+               case MENU_SIZE: SizeMenu (x, y, FALSE); break;
+               case MENU_LINEDASH: LineDashMenu (x, y, FALSE); break;
+               case MENU_LINESTYLE: LineStyleMenu (x, y, FALSE); break;
+               case MENU_LINETYPE: LineTypeMenu (x, y, FALSE); break;
+               case MENU_LINEWIDTH: LineWidthMenu (x, y, FALSE); break;
+               case MENU_FILL: FillMenu (x, y, FALSE); break;
+               case MENU_PEN: PenMenu (x, y, FALSE); break;
+               case MENU_COLOR: ColorMenu (x, y, FALSE); break;
+               case MENU_SPECIAL: SpecialMenu (x, y, FALSE); break;
+               case MENU_PAGE: PageMenu (x, y, FALSE); break;
+               case MENU_PAGELAYOUT: PageLayoutMenu (x, y, FALSE); break;
             }
             break;
       }
@@ -536,52 +549,58 @@ void RedrawSubMenu (menu_index)
    int	menu_index;
 {
    char		* layout_menu_strings[MAXLAYOUTMENUS];
-   int		* fore_colors, * valid, * init_rv, rows, cols;
+   char		* layout_menu_description[MAXLAYOUTMENUS];
+   int		* fore_colors, * valid, * init_rv, rows, cols, index;
    Pixmap	*pixmap;
    Window	win=subMenuInfo[menu_index].win;
 
    switch (menu_index)
    {
       case MENU_MODE:
-         DefaultColorArrays (MAXCHOICES, &fore_colors, &valid, &init_rv);
+         DefaultColorArrays (MAXCHOICES, &fore_colors, &valid, &init_rv, NULL);
          cfree (valid);
          init_rv[curChoice] = TRUE;
          PaintPxMpSubMenu (win, choiceImageW, choiceImageH, MAXCHOICES, 1,
                MAXCHOICES, fore_colors, choicePixmap, init_rv, SINGLECOLOR);
          break;
       case MENU_FILE:
-         DefaultColorArrays (FILEMENUENTRIES, &fore_colors, &valid, &init_rv);
+         DefaultColorArrays (FILEMENUENTRIES, &fore_colors, &valid, &init_rv,
+               NULL);
          PaintTextSubMenu (win, subMenuInfo[menu_index].w, fileMenuStr,
                FILEMENUENTRIES, fore_colors, valid, init_rv);
          break;
       case MENU_EDIT:
-         DefaultColorArrays (MAXEDITMENUS, &fore_colors, &valid, &init_rv);
+         DefaultColorArrays (MAXEDITMENUS, &fore_colors, &valid, &init_rv,
+               NULL);
          PaintTextSubMenu (win, subMenuInfo[menu_index].w, editMenuStr,
                MAXEDITMENUS, fore_colors, valid, init_rv);
          break;
       case MENU_STYLE:
          DefaultColorArrays (MAXFONTSTYLES+MAXJUSTS+1, &fore_colors, &valid,
-               &init_rv);
+               &init_rv, NULL);
          init_rv[curStyle] = TRUE;
          init_rv[MAXFONTSTYLES+1+textJust] = TRUE;
          PaintTextSubMenu (win, subMenuInfo[menu_index].w, styleMenuStr,
                MAXFONTSTYLES+MAXJUSTS+1, fore_colors, valid, init_rv);
          break;
       case MENU_SIZE:
-         DefaultColorArrays (2*MAXFONTSIZES-1, &fore_colors, &valid, &init_rv);
-         init_rv[GetSizeMenuIndex ()] = TRUE;
+         DefaultColorArrays (numFontSizes, &fore_colors, &valid, &init_rv,
+               NULL);
+         index = GetSizeMenuIndex ();
+         if (index != INVALID) init_rv[index] = TRUE;
          PaintTextSubMenu (win, subMenuInfo[menu_index].w, sizeMenuStr,
-               2*MAXFONTSIZES-1, fore_colors, valid, init_rv);
+               numFontSizes, fore_colors, valid, init_rv);
          break;
       case MENU_FONT:
-         DefaultColorArrays (MAXFONTS, &fore_colors, &valid, &init_rv);
+         DefaultColorArrays (numFonts, &fore_colors, &valid, &init_rv, NULL);
          init_rv[curFont] = TRUE;
          PaintTextSubMenu (win, subMenuInfo[menu_index].w, fontMenuStr,
-               MAXFONTS, fore_colors, valid, init_rv);
+               numFonts, fore_colors, valid, init_rv);
          break;
       case MENU_LAYOUT:
-         SetUpLayoutMenuStr (layout_menu_strings);
-         DefaultColorArrays (MAXLAYOUTMENUS, &fore_colors, &valid, &init_rv);
+         SetUpLayoutMenuStr (layout_menu_strings, layout_menu_description);
+         DefaultColorArrays (MAXLAYOUTMENUS, &fore_colors, &valid, &init_rv,
+               NULL);
          switch (pageStyle)
          {
             case PORTRAIT: init_rv[LAYOUT_PORT] = TRUE; break;
@@ -590,41 +609,45 @@ void RedrawSubMenu (menu_index)
          PaintTextSubMenu (win, subMenuInfo[menu_index].w, layout_menu_strings,
                MAXLAYOUTMENUS, fore_colors, valid, init_rv);
          cfree (layout_menu_strings[0]);
+         cfree (layout_menu_description[0]);
          break;
       case MENU_ARRANGE:
-         DefaultColorArrays (MAXARRANGEMENUS, &fore_colors, &valid, &init_rv);
+         DefaultColorArrays (MAXARRANGEMENUS, &fore_colors, &valid, &init_rv,
+               NULL);
          PaintTextSubMenu (win, subMenuInfo[menu_index].w, arrangeMenuStr,
                MAXARRANGEMENUS, fore_colors, valid, init_rv);
          break;
       case MENU_MOVEMODE:
-         DefaultColorArrays (MAXMOVEMODES, &fore_colors, &valid, &init_rv);
+         DefaultColorArrays (MAXMOVEMODES, &fore_colors, &valid, &init_rv,
+               NULL);
          cfree (valid);
          init_rv[moveMode] = TRUE;
          PaintPxMpSubMenu (win, choiceImageW, choiceImageH, MAXMOVEMODES, 1,
                MAXMOVEMODES, fore_colors, moveModePixmap, init_rv, SINGLECOLOR);
          break;
       case MENU_HORIALIGN:
-         DefaultColorArrays (MAXALIGNS, &fore_colors, &valid, &init_rv);
+         DefaultColorArrays (MAXALIGNS, &fore_colors, &valid, &init_rv, NULL);
          cfree (valid);
          init_rv[horiAlign] = TRUE;
          PaintPxMpSubMenu (win, choiceImageW, choiceImageH, MAXALIGNS, 1,
                MAXALIGNS, fore_colors, alignHoriPixmap, init_rv, SINGLECOLOR);
          break;
       case MENU_VERTALIGN:
-         DefaultColorArrays (MAXALIGNS, &fore_colors, &valid, &init_rv);
+         DefaultColorArrays (MAXALIGNS, &fore_colors, &valid, &init_rv, NULL);
          cfree (valid);
          init_rv[vertAlign] = TRUE;
          PaintPxMpSubMenu (win, choiceImageW, choiceImageH, MAXALIGNS, 1,
                MAXALIGNS, fore_colors, alignVertPixmap, init_rv, SINGLECOLOR);
          break;
       case MENU_FILL:
-         DefaultColorArrays (MAXPATTERNS, &fore_colors, &valid, &init_rv);
+         DefaultColorArrays (MAXPATTERNS, &fore_colors, &valid, &init_rv, NULL);
          cfree (valid);
          PaintPxMpSubMenu (win, choiceImageW, choiceImageH, 8, 4,
                MAXPATTERNS, fore_colors, patPixmap, init_rv, SINGLECOLOR);
          break;
       case MENU_LINEWIDTH:
-         DefaultColorArrays (maxLineWidths, &fore_colors, &valid, &init_rv);
+         DefaultColorArrays (maxLineWidths, &fore_colors, &valid, &init_rv,
+               NULL);
          cfree (valid);
          init_rv[lineWidth] = TRUE;
          PaintPxMpSubMenu (win, menuImageW, menuImageH, maxLineWidths, 1,
@@ -632,7 +655,8 @@ void RedrawSubMenu (menu_index)
                SINGLECOLOR);
          break;
       case MENU_LINESTYLE:
-         DefaultColorArrays (MAXLINESTYLES, &fore_colors, &valid, &init_rv);
+         DefaultColorArrays (MAXLINESTYLES, &fore_colors, &valid, &init_rv,
+               NULL);
          cfree (valid);
          init_rv[lineStyle] = TRUE;
          PaintPxMpSubMenu (win, menuImageW, menuImageH, MAXLINESTYLES, 1,
@@ -640,7 +664,8 @@ void RedrawSubMenu (menu_index)
                SINGLECOLOR);
          break;
       case MENU_LINETYPE:
-         DefaultColorArrays (MAXLINETYPES, &fore_colors, &valid, &init_rv);
+         DefaultColorArrays (MAXLINETYPES, &fore_colors, &valid, &init_rv,
+               NULL);
          cfree (valid);
          init_rv[curSpline] = TRUE;
          PaintPxMpSubMenu (win, menuImageW, menuImageH, MAXLINETYPES, 1,
@@ -648,14 +673,14 @@ void RedrawSubMenu (menu_index)
                SINGLECOLOR);
          break;
       case MENU_LINEDASH:
-         DefaultColorArrays (MAXDASHES, &fore_colors, &valid, &init_rv);
+         DefaultColorArrays (MAXDASHES, &fore_colors, &valid, &init_rv, NULL);
          cfree (valid);
          init_rv[curDash] = TRUE;
          PaintPxMpSubMenu (win, menuImageW, menuImageH, MAXDASHES, 1,
                MAXDASHES, fore_colors, dashPixmap, init_rv, SINGLECOLOR);
          break;
       case MENU_PEN:
-         DefaultColorArrays (MAXPATTERNS, &fore_colors, &valid, &init_rv);
+         DefaultColorArrays (MAXPATTERNS, &fore_colors, &valid, &init_rv, NULL);
          cfree (valid);
          PaintPxMpSubMenu (win, choiceImageW, choiceImageH, 8, 4,
                MAXPATTERNS, fore_colors, patPixmap, init_rv, SINGLECOLOR);
@@ -665,14 +690,14 @@ void RedrawSubMenu (menu_index)
          {
             case PAGE_STACK:
                DefaultColorArrays (MAXPAGESTACKMENUS, &fore_colors, &valid,
-                     &init_rv);
+                     &init_rv, NULL);
                PaintTextSubMenu (win, subMenuInfo[menu_index].w,
                      pageStackMenuStr, MAXPAGESTACKMENUS, fore_colors, valid,
                      init_rv);
                break;
             case PAGE_TILE:
                DefaultColorArrays (MAXPAGETILEMENUS, &fore_colors, &valid,
-                     &init_rv);
+                     &init_rv, NULL);
                PaintTextSubMenu (win, subMenuInfo[menu_index].w,
                      pageTileMenuStr, MAXPAGETILEMENUS, fore_colors, valid,
                      init_rv);
@@ -680,7 +705,8 @@ void RedrawSubMenu (menu_index)
          }
          break;
       case MENU_PAGELAYOUT:
-         DefaultColorArrays (MAXPAGELAYOUTMODES,&fore_colors,&valid,&init_rv);
+         DefaultColorArrays (MAXPAGELAYOUTMODES, &fore_colors, &valid, &init_rv,
+               NULL);
          cfree (valid);
          init_rv[pageLayoutMode] = TRUE;
          PaintPxMpSubMenu (win, choiceImageW, choiceImageH, MAXPAGELAYOUTMODES,
@@ -694,7 +720,8 @@ void RedrawSubMenu (menu_index)
          cfree (pixmap);
          break;
       case MENU_SPECIAL:
-         DefaultColorArrays (MAXSPECIALMENUS, &fore_colors, &valid, &init_rv);
+         DefaultColorArrays (MAXSPECIALMENUS, &fore_colors, &valid, &init_rv,
+               NULL);
          PaintTextSubMenu (win, subMenuInfo[menu_index].w, specialMenuStr,
                MAXSPECIALMENUS, fore_colors, valid, init_rv);
          break;
@@ -817,6 +844,11 @@ int SubMenuEventHandler (input)
       ComputeSubMenuWinXY (win, &(subMenuInfo[menu_index].x),
             &(subMenuInfo[menu_index].y));
    }
+   else if (input->type == EnterNotify)
+   {
+      SetMouseStatus ("Move pinned menu", "Execute Menu Command",
+            "Close pinned menu");
+   }
    else if (input->type == ButtonPress)
    {
       button_ev = &(input->xbutton);
@@ -858,7 +890,7 @@ int SubMenuEventHandler (input)
                         case MENU_FILE: return (FileSubMenu (up_row));
                         case MENU_EDIT: EditSubMenu (up_row); break;
                         case MENU_STYLE: StyleSubMenu (up_row); break;
-                        case MENU_SIZE: SizeSubMenu (up_row); break;
+                        case MENU_SIZE: ChangeFontSize (up_row); break;
                         case MENU_FONT: ChangeFont (up_row); break;
                         case MENU_LAYOUT: LayoutSubMenu (up_row); break;
                         case MENU_ARRANGE: ArrangeSubMenu (up_row); break;
@@ -949,10 +981,12 @@ void RealizeSubMenuWindow (win_x, win_y, win_w, win_h)
 #ifdef MAPBEFORESELECT
    XMapWindow (mainDisplay, win);
    XSelectInput (mainDisplay, win, StructureNotifyMask |
-         ExposureMask | ButtonPressMask | ButtonReleaseMask | KeyPressMask);
+         ExposureMask | ButtonPressMask | ButtonReleaseMask | KeyPressMask |
+         EnterWindowMask);
 #else
    XSelectInput (mainDisplay, win, StructureNotifyMask |
-         ExposureMask | ButtonPressMask | ButtonReleaseMask | KeyPressMask);
+         ExposureMask | ButtonPressMask | ButtonReleaseMask | KeyPressMask |
+         EnterWindowMask);
    XMapWindow (mainDisplay, win);
 #endif
 
