@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/names.c,v 2.93.1.5 1993/04/23 02:11:46 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/names.c,v 2.107 1994/03/21 04:55:29 william Exp $";
 #endif
 
 #include <sys/types.h>
@@ -32,7 +33,7 @@ static char RCSid[] =
 #include "vms_compat.h"
 #define DIR_ENTRY struct dirent
 #else
-#ifdef ibm-rt
+#ifdef ibm
 #include <sys/dir.h>
 #define DIR_ENTRY struct direct
 #else
@@ -48,6 +49,10 @@ static char RCSid[] =
 #include <sys/dir.h>
 #define DIR_ENTRY struct direct
 #else
+#ifdef sequent
+#include <sys/dir.h>
+#define DIR_ENTRY struct direct
+#else
 #include <dirent.h>
 #define DIR_ENTRY struct dirent
 #endif
@@ -55,7 +60,10 @@ static char RCSid[] =
 #endif
 #endif
 #endif
+#endif
 #include <sys/stat.h>
+#include <stdio.h>
+#include <string.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
@@ -70,12 +78,17 @@ static char RCSid[] =
 #include "font.e"
 #include "mainloop.e"
 #include "mainmenu.e"
+#include "menu.e"
 #include "msg.e"
+#ifndef _NO_EXTERN
 #include "names.e"
+#endif
 #include "raster.e"
+#include "rect.e"
 #include "setup.e"
 
-extern char	* getenv ();
+extern char	* getenv ARGS_DECL((char *));
+extern int	atoi ARGS_DECL((char *));
 
 char	curDomainName[MAXPATHLENGTH];
 char	curDomainPath[MAXPATHLENGTH];
@@ -103,6 +116,9 @@ int	nameDspWinH;		/* height of display window */
 int	nameDspWinW;		/* width of display window */
 
 int	justClicked=FALSE;
+int	domainInResource=TRUE;
+
+int	ignoreDirectoryFlagInMakeNameDspItemArray=FALSE;
 
 static int	nameScrollAreaH;	/* heighr of the grey scroll area */
 
@@ -126,6 +142,7 @@ char * ReadPath (path_str, dir_str)
    register char        * s1, * s2;
 
    s1 = path_str;
+   while (*s1==' ' || *s1=='\t' || *s1=='\n') s1++;
    if (*s1 == '~')
    {
       strcpy (dir_str, homeDir);
@@ -151,6 +168,7 @@ void ParseSymPath (path_str)
 {
    register char        * s;
    register int         i;
+   int			path_len=0;
    char                 dir_str[MAXPATHLENGTH];
 
    for (i = 0, s = path_str; *s != '\0'; )
@@ -160,6 +178,7 @@ void ParseSymPath (path_str)
    }
    symPath = (char * *) calloc (i, sizeof (char *));
    symPathNumEntries = i;
+   *curDomainPath = '\0';
    for (i = 0, s = path_str; *s != '\0'; )
    {
       s = ReadPath (s, dir_str);
@@ -167,10 +186,19 @@ void ParseSymPath (path_str)
       {
          symPath[i] = (char *) calloc (MAXPATHLENGTH+1, sizeof (char));
          strcpy (symPath[i], dir_str);
+         if (path_len == 0)
+         {
+            sprintf (&curDomainPath[path_len], "%s", dir_str);
+            path_len += strlen (dir_str);
+         }
+         else
+         {
+            sprintf (&curDomainPath[path_len], ":%s", dir_str);
+            path_len += strlen (dir_str)+1;
+         }
          i++;
       }
    }
-   strcpy (curDomainPath, path_str);
 }
 
 static
@@ -311,8 +339,16 @@ DspList * DirListing (Path, ExtStr, OtherExtStr)
       strcat (path, "/");
    }
 
-   sprintf (ext_str, ".%s", ExtStr);
-   ext_len = strlen (ext_str);
+   if (ExtStr != NULL)
+   {
+      sprintf (ext_str, ".%s", ExtStr);
+      ext_len = strlen (ext_str);
+   }
+   else
+   {
+      *ext_str = '\0';
+      ext_len = 0;
+   }
    if (*OtherExtStr == '\0')
    {
       *other_ext_str = '\0';
@@ -339,7 +375,38 @@ DspList * DirListing (Path, ExtStr, OtherExtStr)
    {
       len = strlen (d->d_name);
       if (*OtherExtStr == '\0') other_ext_len = len;
-      if (len > ext_len && strcmp (ext_str, &d->d_name[len-ext_len]) == 0 ||
+      if (ext_len == 0)
+      {
+#ifdef VMS
+         if (len > 4 && (strcmp (".dir", &d->d_name[len-4]) == 0))
+         {
+            d->d_name[len-4] = '/';
+            d->d_name[len-4+1] = '\0';
+            dsp_ptr = (DspList *) calloc (1, sizeof(DspList));
+            dsp_ptr->directory = TRUE;
+            strcpy (dsp_ptr->itemstr, d->d_name);
+         }
+         else if (strcmp (d->d_name, ".") == 0)
+#else
+         if (strcmp (d->d_name, ".") == 0)
+#endif /* VMS */
+            continue;
+         else
+         {
+            sprintf (s, "%s%s", path, d->d_name);
+            stat (s, &stat_buf);
+            dsp_ptr = (DspList *) calloc (1, sizeof(DspList));
+            if (stat_buf.st_mode & S_IFDIR)
+            {
+               dsp_ptr->directory = TRUE;
+               strcat (d->d_name, "/");
+            }
+            else
+               dsp_ptr->directory = FALSE;
+            strcpy (dsp_ptr->itemstr, d->d_name);
+         }
+      }
+      else if (len > ext_len && strcmp (ext_str,&d->d_name[len-ext_len]) == 0 ||
             len > other_ext_len &&
             strcmp (other_ext_str,&d->d_name[len-other_ext_len]) == 0)
       {
@@ -462,6 +529,8 @@ void InitNames ()
    nameDspWinH = nameDspH + 2 * brdrW;
    nameScrollAreaH = nameDspH;
 
+   ignoreDirectoryFlagInMakeNameDspItemArray = FALSE;
+
    *curDomainName = '\0';
    *curDomainPath = '\0';
    *curSymDir = '\0';
@@ -484,28 +553,63 @@ void InitNames ()
       if ((strcmp (c_ptr, "true") == 0) || (strcmp (c_ptr, "True") == 0))
          importFromLibrary = TRUE;
 
+   domainInResource = TRUE;
    if ((c_ptr = XGetDefault (mainDisplay, TOOL_NAME, "DefaultDomain")) != NULL)
    {
-      sprintf (domain_str, "Domain%s", c_ptr);
+      int	default_domain=atoi (c_ptr);
+
+      sprintf (domain_str, "DomainPath%1d", default_domain);
       if ((c_ptr = XGetDefault (mainDisplay, TOOL_NAME, domain_str)) != NULL)
       {
-         if (*c_ptr != '\0')
+         char	* c_ptr1;
+
+         while (*c_ptr==' ' || *c_ptr=='\t' || *c_ptr=='\n') c_ptr++;
+         if (*c_ptr != '\0' && (c_ptr1=strchr (c_ptr, ':')) != NULL)
          {
-            ConvertToUpperCase (TOOL_NAME, cap_tool_name);
-            strcpy (curDomainName, c_ptr);
-            sprintf (sym_path, "%s_%s", cap_tool_name, c_ptr);
+            int	len=c_ptr1-c_ptr;
+
+            strncpy (curDomainName, c_ptr, len);
+            curDomainName[len] = '\0';
+            c_ptr = (++c_ptr1);
             default_found = TRUE;
          }
       }
+      else
+      {
+         sprintf (domain_str, "Domain%1d", default_domain);
+         if ((c_ptr = XGetDefault (mainDisplay, TOOL_NAME, domain_str)) != NULL)
+         {
+            if (*c_ptr != '\0')
+            {
+               ConvertToUpperCase (TOOL_NAME, cap_tool_name);
+               strcpy (curDomainName, c_ptr);
+               sprintf (sym_path, "%s_%s", cap_tool_name, c_ptr);
+               default_found = TRUE;
+               domainInResource = FALSE;
+            }
+         }
+      }
    }
+   if (default_found)
+   {
+      if (domainInResource)
+         ParseSymPath (c_ptr);
+      else if ((c_ptr=getenv (sym_path)) == NULL ||
+            strlen (c_ptr) >= MAXPATHLENGTH-1)
+         ParseSymPath (".");
+      else
+         ParseSymPath (c_ptr);
+   }
+   else
+      ParseSymPath (".");
 
-   if (!default_found || (c_ptr = getenv (sym_path)) == NULL)
+/* if (!default_found || (c_ptr = getenv (sym_path)) == NULL)
       ParseSymPath (".");
    else
       if (strlen (c_ptr) >= MAXPATHLENGTH-1)
          ParseSymPath (".");
       else 
-         ParseSymPath (c_ptr);
+         ParseSymPath (c_ptr); */
    if ((topOfSymLinkList = SymbolListing ()) != NULL) BuildSymbolList ();
 }
 
@@ -536,6 +640,7 @@ void CleanUpNames ()
    if (symbolList != NULL) cfree (symbolList);
    if (dirList != NULL) cfree (dirList);
    symbolList = dirList = NULL;
+   ignoreDirectoryFlagInMakeNameDspItemArray = FALSE;
 }
 
 void RedrawNameScrollWin ()
@@ -679,7 +784,7 @@ char * * MakeNameDspItemArray (Entries, DLPtr)
    {
       dsp_ptr[i] = c_ptr;
       len = strlen (DLPtr->itemstr);
-      if (!DLPtr->directory)
+      if (!ignoreDirectoryFlagInMakeNameDspItemArray && !DLPtr->directory)
       {
          for (j = len; j >= 0 && DLPtr->itemstr[j] != '/'; j--) ;
          if (j >= 0)
@@ -1114,8 +1219,10 @@ int DirNames (TopStr, ExtStr, SelStr, JustSetDir)
    SetWatchCursor (mainWindow);
    SetWatchCursor (nameBaseWin);
    Msg ("Generating list of file names, please wait ...");
-   if (strcmp (ExtStr, EPSF_FILE_EXT) == 0)
+   if (ExtStr != NULL && strcmp (ExtStr, EPSF_FILE_EXT) == 0)
       strcpy (other_ext_str, PS_FILE_EXT);
+   else if (ExtStr != NULL && strcmp (ExtStr, OBJ_FILE_EXT) == 0)
+      strcpy (other_ext_str, SYM_FILE_EXT);
    else
       *other_ext_str = '\0';
    if ((topOfDirLinkList = DirListing (dir_name,ExtStr,other_ext_str)) == NULL)
@@ -2152,6 +2259,44 @@ int SelectFileName (MsgStr, SelStr)
    return (index);
 }
 
+int SelectFileNameToPaste (MsgStr, SelStr)
+   char	* MsgStr, * SelStr;
+{
+   int		index, saved_num_dir_entries, just_set_dir;
+   DspList	* dsp_ptr, * next_dsp;
+   char		saved_cur_dir[MAXPATHLENGTH];
+
+   strcpy (saved_cur_dir, curDir);
+   saved_num_dir_entries = numDirEntries;
+   if ((index = DirNames (MsgStr,NULL,SelStr,&just_set_dir)) == INVALID)
+   {
+      if (just_set_dir)
+      {
+         strcpy (curDir, SelStr);
+         BuildDirList ();
+         if (strcmp (saved_cur_dir, curDir) != 0 && DirInSymPath ("."))
+            UpdateSymInfo ();
+         RedrawTitleWindow ();
+         Msg ("");
+      }
+      else
+      {
+         numDirEntries = saved_num_dir_entries;
+         for (dsp_ptr = topOfDirLinkList; dsp_ptr != NULL; dsp_ptr = next_dsp)
+         {
+            next_dsp = dsp_ptr->next;
+            cfree (dsp_ptr);
+         }
+         topOfDirLinkList = NULL;
+      }
+      *SelStr = '\0';
+      return (INVALID);
+   }
+   BuildDirList ();
+   Msg ("");
+   return (index);
+}
+
 int SelectFileNameToImport (MsgStr, ExtStr, SelStr)
    char	* MsgStr, * ExtStr, * SelStr;
 {
@@ -2227,6 +2372,8 @@ int DirInSymPath (DirName)
    return (FALSE);
 }
 
+static char	* * tmpDomainName=NULL;
+
 static
 DspList * DomainListing (Entries)
    int	* Entries;
@@ -2242,13 +2389,28 @@ DspList * DomainListing (Entries)
    else
       *Entries = atoi (c_ptr)+1;
 
+   tmpDomainName = (char **) calloc (*Entries, sizeof(char *));
+   for (i = 0; i < *Entries; i++) tmpDomainName[i] = NULL;
+
    head_ptr = tail_ptr = NULL;
    for (i = 0; i < *Entries; i++)
    {
       if (i != (*Entries)-1)
       {
-         sprintf (s, "Domain%1d", i);
-         if ((c_ptr = XGetDefault (mainDisplay, TOOL_NAME, s)) == NULL)
+         char	* c_ptr1;
+
+         if (domainInResource)
+         {
+            sprintf (s, "DomainPath%1d", i);
+            if ((c_ptr=XGetDefault (mainDisplay, TOOL_NAME, s)) != NULL)
+               while (*c_ptr==' ' || *c_ptr=='\t' || *c_ptr=='\n') c_ptr++;
+         }
+         else
+         {
+            sprintf (s, "Domain%1d", i);
+            c_ptr = XGetDefault (mainDisplay, TOOL_NAME, s);
+         }
+         if (c_ptr == NULL || *c_ptr == '\0')
          {
             for ( ; head_ptr != NULL; head_ptr = next_dsp)
             {
@@ -2257,11 +2419,24 @@ DspList * DomainListing (Entries)
             }
             return (NULL);
          }
-         else if (strcmp (c_ptr, "Examples") == 0)
-            seen_examples = TRUE;
-
          dsp_ptr = (DspList *) calloc (1, sizeof(DspList));
-         strcpy (dsp_ptr->itemstr, c_ptr);
+         tmpDomainName[i] = (char *) calloc (strlen(c_ptr)+1, sizeof(char));
+         if ((c_ptr1=strchr (c_ptr, ':')) != NULL)
+         {
+            int	len=c_ptr1-c_ptr;
+
+            strncpy (dsp_ptr->itemstr, c_ptr, len);
+            strncpy (tmpDomainName[i], c_ptr, len);
+            dsp_ptr->itemstr[len] = '\0';
+            tmpDomainName[i][len] = '\0';
+         }
+         else
+         {
+            strcpy (dsp_ptr->itemstr, c_ptr);
+            strcpy (tmpDomainName[i], c_ptr);
+         }
+         if (strcmp (dsp_ptr->itemstr, "Examples") == 0)
+            seen_examples = TRUE;
       }
       else if (seen_examples)
       {
@@ -2272,6 +2447,8 @@ DspList * DomainListing (Entries)
       {
          dsp_ptr = (DspList *) calloc (1, sizeof(DspList));
          strcpy (dsp_ptr->itemstr, "Examples");
+         tmpDomainName[i] = (char *) calloc (strlen("Examples")+1,sizeof(char));
+         strcpy (tmpDomainName[i], "Examples");
       }
 
       if (head_ptr == NULL)
@@ -2303,6 +2480,7 @@ DspList * DomainListing (Entries)
 int SelectDomain (SelStr)
    char	* SelStr;
 {
+   register int	i;
    int		index;
    DspList	* dsp_ptr, * next_dsp;
 
@@ -2310,6 +2488,14 @@ int SelectDomain (SelStr)
 
    if ((dsp_ptr = DomainListing (&nameEntries)) == NULL)
    {
+      if (tmpDomainName != NULL)
+      {
+         for (i=0; i < nameEntries; i++)
+            if (tmpDomainName[i] != NULL)
+               cfree (tmpDomainName[i]);
+         cfree (tmpDomainName);
+      }
+      tmpDomainName = NULL;
       Msg ("No domain names found.");
       *SelStr = '\0';
       return (INVALID);
@@ -2339,6 +2525,24 @@ int SelectDomain (SelStr)
 
    Msg ("");
 
+   index = INVALID;
+   if (*SelStr != '\0' && nameEntries > 0 && tmpDomainName != NULL)
+   {
+      for (i=0; i<nameEntries; i++)
+         if (tmpDomainName[i] != NULL && strcmp (tmpDomainName[i], SelStr) == 0)
+         {
+            index = i;
+            break;
+         }
+   }
+   if (tmpDomainName != NULL)
+   {
+      for (i=0; i < nameEntries; i++)
+         if (tmpDomainName[i] != NULL)
+            cfree (tmpDomainName[i]);
+      cfree (tmpDomainName);
+      tmpDomainName = NULL;
+   }
    return (index);
 }
 
@@ -2655,7 +2859,7 @@ void SetCurDir (FileName)
    char	* FileName;
 {
    register int	i;
-   char		file_name[MAXPATHLENGTH];
+   char		file_name[MAXPATHLENGTH+1];
 
    strcpy (file_name, FileName);
    ShrinkName (file_name);
@@ -2677,11 +2881,37 @@ void SetCurDir (FileName)
    }
 }
 
+void SetCurSymDir (FileName)
+   char	* FileName;
+{
+   register int	i;
+   char		file_name[MAXPATHLENGTH+1];
+
+   strcpy (file_name, FileName);
+   ShrinkName (file_name);
+
+   for (i = strlen(file_name)-1; i>=0 && file_name[i]!='/'; i--) ;
+
+   if (i < 0)
+   {
+      TwoLineMsg ("Error:  No '/' found in SetCurSymDir ().",
+                  "        curSymDir set to '.'.");
+      strcpy (curSymDir, ".");
+      strcpy (curFileName, FileName);
+   }
+   else
+   {
+      strcpy (curFileName, &file_name[i+1]);
+      file_name[i] = '\0';
+      strcpy (curSymDir, file_name);
+   }
+}
+
 void SetCurImportDir (FileName)
    char	* FileName;
 {
    register int	i;
-   char		file_name[MAXPATHLENGTH];
+   char		file_name[MAXPATHLENGTH+1];
 
    strcpy (file_name, FileName);
    ShrinkName (file_name);
