@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,9 +25,10 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/obj.c,v 2.35.2.4 1993/05/23 22:56:10 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/obj.c,v 2.44 1994/03/21 04:55:35 william Exp $";
 #endif
 
+#include <stdio.h>
 #include <math.h>
 #include <X11/Xlib.h>
 #include "const.h"
@@ -38,6 +40,9 @@ static char RCSid[] =
 #include "box.e"
 #include "cursor.e"
 #include "group.e"
+#ifndef _NO_EXTERN
+#include "obj.e"
+#endif
 #include "oval.e"
 #include "page.e"
 #include "poly.e"
@@ -149,10 +154,16 @@ void AdjObjBBox (ObjPtr)
    {
       case OBJ_POLY:
          poly_ptr = ObjPtr->detail.p;
-         CalcPolyBBox (ObjPtr->obbox, poly_ptr->n, poly_ptr->vlist,
-               poly_ptr->style, poly_ptr->width, poly_ptr->aw, poly_ptr->ah,
-               &(ObjPtr->bbox.ltx), &(ObjPtr->bbox.lty), &(ObjPtr->bbox.rbx),
-               &(ObjPtr->bbox.rby));
+         if (poly_ptr->curved != LT_INTSPLINE)
+            CalcPolyBBox (ObjPtr->obbox, poly_ptr->n, poly_ptr->vlist,
+                  poly_ptr->style, poly_ptr->width, poly_ptr->aw, poly_ptr->ah,
+                  &(ObjPtr->bbox.ltx), &(ObjPtr->bbox.lty), &(ObjPtr->bbox.rbx),
+                  &(ObjPtr->bbox.rby));
+         else
+            CalcPolyBBox (ObjPtr->obbox, poly_ptr->intn, poly_ptr->intvlist,
+                  poly_ptr->style, poly_ptr->width, poly_ptr->aw, poly_ptr->ah,
+                  &(ObjPtr->bbox.ltx), &(ObjPtr->bbox.lty), &(ObjPtr->bbox.rbx),
+                  &(ObjPtr->bbox.rby));
          break;
       case OBJ_ARC:
          CalcArcBBox (ObjPtr->detail.a, ObjPtr->obbox, &(ObjPtr->bbox));
@@ -258,22 +269,34 @@ void AdjObjSplineVs (ObjPtr)
    switch (ObjPtr->type)
    {
       case OBJ_POLY:
-         if (ObjPtr->detail.p->curved)
+         switch (ObjPtr->detail.p->curved)
          {
-            if (ObjPtr->detail.p->svlist != NULL)
-               cfree (ObjPtr->detail.p->svlist);
-            ObjPtr->detail.p->svlist = MakeSplinePolyVertex (
-                  &(ObjPtr->detail.p->sn), drawOrigX, drawOrigY,
-                  ObjPtr->detail.p->n, ObjPtr->detail.p->vlist);
-         }
-         else
-         {
-            if (ObjPtr->detail.p->svlist != NULL)
-               cfree (ObjPtr->detail.p->svlist);
-            ObjPtr->detail.p->svlist = MakePolyVertex (
-                  drawOrigX, drawOrigY, ObjPtr->detail.p->n,
-                  ObjPtr->detail.p->vlist);
-            ObjPtr->detail.p->sn = ObjPtr->detail.p->n;
+            case LT_STRAIGHT:
+               if (ObjPtr->detail.p->svlist != NULL)
+                  cfree (ObjPtr->detail.p->svlist);
+               ObjPtr->detail.p->svlist = MakeMultiSplinePolyVertex (
+                     &(ObjPtr->detail.p->sn), ObjPtr->detail.p->smooth,
+                     drawOrigX, drawOrigY,
+                     ObjPtr->detail.p->n, ObjPtr->detail.p->vlist);
+               break;
+            case LT_SPLINE:
+               if (ObjPtr->detail.p->svlist != NULL)
+                  cfree (ObjPtr->detail.p->svlist);
+               ObjPtr->detail.p->svlist = MakeMultiSplinePolyVertex (
+                     &(ObjPtr->detail.p->sn), ObjPtr->detail.p->smooth,
+                     drawOrigX, drawOrigY,
+                     ObjPtr->detail.p->n, ObjPtr->detail.p->vlist);
+               break;
+            case LT_INTSPLINE:
+               if (ObjPtr->detail.p->svlist != NULL)
+                  cfree (ObjPtr->detail.p->svlist);
+               if (ObjPtr->detail.p->intvlist != NULL)
+                  cfree (ObjPtr->detail.p->intvlist);
+               ObjPtr->detail.p->svlist = MakeIntSplinePolyVertex (
+                     &(ObjPtr->detail.p->sn), &(ObjPtr->detail.p->intn),
+                     &(ObjPtr->detail.p->intvlist), drawOrigX, drawOrigY,
+                     ObjPtr->detail.p->n, ObjPtr->detail.p->vlist);
+               break;
          }
 
          if (ObjPtr->detail.p->style != LS_PLAIN)
@@ -287,6 +310,16 @@ void AdjObjSplineVs (ObjPtr)
             if (ObjPtr->detail.p->asvlist != NULL)
                cfree (ObjPtr->detail.p->asvlist);
 
+            if (poly_ptr->curved != LT_INTSPLINE)
+            {
+               v = poly_ptr->vlist;
+               n = poly_ptr->n;
+            }
+            else
+            {
+               v = poly_ptr->intvlist;
+               n = poly_ptr->intn;
+            }
             v_copy = (XPoint *) calloc (n+1, sizeof(XPoint));
             for (i = 0; i < n; i++)
             {
@@ -319,35 +352,52 @@ void AdjObjSplineVs (ObjPtr)
                v_copy[n-1].x = round(v_copy[n-1].x-RETREAT*aw*cos);
                v_copy[n-1].y = round(v_copy[n-1].y-RETREAT*aw*sin);
             }
-            if (ObjPtr->detail.p->curved)
-               poly_ptr->asvlist = MakeSplinePolyVertex (&(poly_ptr->asn),
-                     drawOrigX, drawOrigY, n, v_copy);
-            else
+            switch (ObjPtr->detail.p->curved)
             {
-               poly_ptr->asvlist = MakePolyVertex (drawOrigX, drawOrigY,
-                     n, v_copy);
-               poly_ptr->asn = poly_ptr->n;
+               case LT_STRAIGHT:
+               case LT_SPLINE:
+                  poly_ptr->asvlist = MakeMultiSplinePolyVertex (
+                        &(poly_ptr->asn), poly_ptr->smooth,
+                        drawOrigX, drawOrigY,
+                        n, v_copy);
+                  break;
+               case LT_INTSPLINE:
+                  poly_ptr->asvlist = MakeSplinePolyVertex (&(poly_ptr->asn),
+                        drawOrigX, drawOrigY, n, v_copy);
+                  break;
             }
             cfree (v_copy);
          }
          break;
       case OBJ_POLYGON:
-         if (ObjPtr->detail.g->curved)
+         switch (ObjPtr->detail.g->curved)
          {
-            if (ObjPtr->detail.g->svlist != NULL)
-               cfree (ObjPtr->detail.g->svlist);
-            ObjPtr->detail.g->svlist = MakeSplinePolygonVertex (
-                  &(ObjPtr->detail.g->sn), drawOrigX, drawOrigY,
-                  ObjPtr->detail.g->n, ObjPtr->detail.g->vlist);
-         }
-         else
-         {
-            if (ObjPtr->detail.g->svlist != NULL)
-               cfree (ObjPtr->detail.g->svlist);
-            ObjPtr->detail.g->svlist = MakePolygonVertex (
-                  drawOrigX, drawOrigY, ObjPtr->detail.g->n,
-                  ObjPtr->detail.g->vlist);
-            ObjPtr->detail.g->sn = ObjPtr->detail.g->n;
+            case LT_STRAIGHT:
+               if (ObjPtr->detail.g->svlist != NULL)
+                  cfree (ObjPtr->detail.g->svlist);
+               ObjPtr->detail.g->svlist = MakeMultiSplinePolygonVertex (
+                     &(ObjPtr->detail.g->sn), ObjPtr->detail.g->smooth,
+                     drawOrigX, drawOrigY,
+                     ObjPtr->detail.g->n, ObjPtr->detail.g->vlist);
+               break;
+            case LT_SPLINE:
+               if (ObjPtr->detail.g->svlist != NULL)
+                  cfree (ObjPtr->detail.g->svlist);
+               ObjPtr->detail.g->svlist = MakeMultiSplinePolygonVertex (
+                     &(ObjPtr->detail.g->sn), ObjPtr->detail.g->smooth,
+                     drawOrigX, drawOrigY,
+                     ObjPtr->detail.g->n, ObjPtr->detail.g->vlist);
+               break;
+            case LT_INTSPLINE:
+               if (ObjPtr->detail.g->svlist != NULL)
+                  cfree (ObjPtr->detail.g->svlist);
+               if (ObjPtr->detail.g->intvlist != NULL)
+                  cfree (ObjPtr->detail.g->intvlist);
+               ObjPtr->detail.g->svlist = MakeIntSplinePolygonVertex (
+                     &(ObjPtr->detail.g->sn), &(ObjPtr->detail.g->intn),
+                     &(ObjPtr->detail.g->intvlist), drawOrigX, drawOrigY,
+                     ObjPtr->detail.g->n, ObjPtr->detail.g->vlist);
+               break;
          }
          break;
       case OBJ_SYM:
@@ -397,7 +447,8 @@ void AdjObjCache (ObjPtr)
    switch (ObjPtr->type)
    {
       case OBJ_TEXT:
-         if (zoomScale!=0 || ObjPtr->detail.t->rotate!=ROTATE0)
+         if (zoomScale!=0 || ObjPtr->detail.t->rotate!=ROTATE0 ||
+               ObjPtr->detail.t->read_only)
          {
             text_ptr = ObjPtr->detail.t;
 
