@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,22 +25,27 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/oval.c,v 2.38 1993/04/28 06:25:25 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/oval.c,v 2.49 1994/03/21 04:55:42 william Exp $";
 #endif
 
-#include <math.h>
 #include <stdio.h>
+#include <math.h>
 #include <X11/Xlib.h>
 #include "const.h"
 #include "types.h"
 
+#include "attr.e"
 #include "cmd.e"
 #include "color.e"
 #include "cursor.e"
 #include "file.e"
 #include "grid.e"
 #include "mainloop.e"
+#include "msg.e"
 #include "obj.e"
+#ifndef _NO_EXTERN
+#include "oval.e"
+#endif
 #include "pattern.e"
 #include "poly.e"
 #include "raster.e"
@@ -129,7 +135,7 @@ void DumpOvalPath (FP, ObjPtr, Xc, Yc, A, B, Width, Pen, Dash)
       default:
          if (colorDump)
          {
-            fprintf (FP, "flattenpath strokepath clip\n");
+            fprintf (FP, "flattenpath strokepath clip newpath\n");
             DumpPatFill (FP, Pen, 8, ObjPtr->bbox, "   ");
          }
          else
@@ -200,7 +206,7 @@ void DumpOvalObj (FP, ObjPtr)
                   ((float)tgifColors[color_index].green/maxRGB),
                   ((float)tgifColors[color_index].blue/maxRGB));
             fprintf (FP, "   %s\n", s);
-            fprintf (FP, "   closepath eoclip\n");
+            fprintf (FP, "   closepath eoclip newpath\n");
             DumpPatFill (FP, fill, 8, ObjPtr->bbox, "   ");
          }
          else
@@ -342,6 +348,7 @@ void ContinueOval (OrigX, OrigY)
    int 			end_x, end_y, grid_x, grid_y;
    int 			done = FALSE;
    int			xor_pixel;
+   char			buf[80];
    struct BBRec		bbox;
    XGCValues		values;
    XEvent		input, ev;
@@ -367,6 +374,8 @@ void ContinueOval (OrigX, OrigY)
 
    grid_x = end_x = OrigX;
    grid_y = end_y = OrigY; 
+   sprintf (buf, "0x0%+1d%+1d", ABS_X(grid_x), ABS_Y(grid_y));
+   StartShowMeasureCursor (grid_x, grid_y, buf, TRUE);
    BeginIntervalRulers (grid_x, grid_y, grid_x, grid_y);
    XGrabPointer (mainDisplay, drawWindow, FALSE,
          PointerMotionMask | ButtonReleaseMask,
@@ -383,6 +392,9 @@ void ContinueOval (OrigX, OrigY)
          XUngrabPointer (mainDisplay, CurrentTime);
          MyOval (drawWindow, drawGC, bbox);
          EndIntervalRulers (grid_x, grid_y);
+         sprintf (buf, "%1dx%1d%+1d%+1d", ABS_SIZE(abs(bbox.rbx-OrigX)),
+               ABS_SIZE(abs(bbox.rby-OrigY)), ABS_X(bbox.rbx), ABS_Y(bbox.rby));
+         EndShowMeasureCursor (bbox.rbx, bbox.rby, buf, TRUE);
          done = TRUE;
       }
       else if (input.type == MotionNotify)
@@ -391,12 +403,41 @@ void ContinueOval (OrigX, OrigY)
          end_x = motion_ev->x;
          end_y = motion_ev->y;
          GridXY (end_x, end_y, &grid_x, &grid_y);
+         if (motion_ev->state & (ShiftMask | ControlMask))
+         {
+            int	w, h, pos_w=TRUE, pos_h=TRUE;
+
+            w = grid_x - bbox.ltx;
+            h = grid_y - bbox.lty;
+            if (w < 0)
+            {
+               w = (-w);
+               pos_w = FALSE;
+            }
+            if (h < 0)
+            {
+               h = (-h);
+               pos_h = FALSE;
+            }
+            if (w > h)
+               grid_x = (pos_w ? (bbox.ltx+h) : (bbox.ltx-h));
+            else
+               grid_y = (pos_h ? (bbox.lty+w) : (bbox.lty-w));
+         }
          if (grid_x != bbox.rbx || grid_y != bbox.rby)
          {
+            sprintf (buf, "%1dx%1d%+1d%+1d", ABS_SIZE(abs(bbox.rbx-OrigX)),
+                  ABS_SIZE(abs(bbox.rby-OrigY)), ABS_X(bbox.rbx),
+                  ABS_Y(bbox.rby));
+            ShowMeasureCursor (bbox.rbx, bbox.rby, buf, TRUE);
             MyOval (drawWindow, drawGC, bbox);
             bbox.rbx = grid_x;
             bbox.rby = grid_y;
             MyOval (drawWindow, drawGC, bbox);
+            sprintf (buf, "%1dx%1d%+1d%+1d", ABS_SIZE(abs(bbox.rbx-OrigX)),
+                  ABS_SIZE(abs(bbox.rby-OrigY)), ABS_X(bbox.rbx),
+                  ABS_Y(bbox.rby));
+            ShowMeasureCursor (bbox.rbx, bbox.rby, buf, TRUE);
          }
          DrawIntervalRulers (OrigX, OrigY, grid_x, grid_y);
          while (XCheckMaskEvent (mainDisplay, PointerMotionMask, &ev)) ;
@@ -442,17 +483,19 @@ void SaveOvalObj (FP, ObjPtr)
    FILE			* FP;
    struct ObjRec	* ObjPtr;
 {
-   fprintf (FP, "oval('%s',", colorMenuItems[ObjPtr->color]);
-   fprintf (FP, "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,",
+   if (fprintf (FP, "oval('%s',", colorMenuItems[ObjPtr->color]) == EOF)
+      writeFileFailed = TRUE;
+   if (fprintf (FP, "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,",
          ObjPtr->obbox.ltx, ObjPtr->obbox.lty, ObjPtr->obbox.rbx,
          ObjPtr->obbox.rby, ObjPtr->detail.o->fill, ObjPtr->detail.o->width,
          ObjPtr->detail.o->pen, ObjPtr->id, ObjPtr->detail.o->dash,
-         ObjPtr->rotation, ObjPtr->locked);
+         ObjPtr->rotation, ObjPtr->locked) == EOF)
+      writeFileFailed = TRUE;
    SaveAttrs (FP, ObjPtr->lattr);
-   fprintf (FP, ")");
+   if (fprintf (FP, ")") == EOF) writeFileFailed = TRUE;
 }
 
-#define GETVALUE(val,name) ScanValue("%d", (char *) &(val), name, "oval")
+#define GETVALUE(val,name) ScanValue("%d", &(val), name, "oval")
 
 void ReadOvalObj (Inbuf, ObjPtr)
    char			* Inbuf;
