@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/pattern.c,v 2.35 1992/03/31 07:55:03 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/pattern.c,v 2.46 1994/04/04 00:49:06 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -43,7 +44,11 @@ static char RCSid[] =
 #include "mainmenu.e"
 #include "mark.e"
 #include "menu.e"
+#include "msg.e"
 #include "obj.e"
+#ifndef _NO_EXTERN
+#include "pattern.e"
+#endif
 #include "poly.e"
 #include "raster.e"
 #include "select.e"
@@ -61,6 +66,8 @@ int     rcbRadius = DEF_RCB_RADIUS;
 int     useGray = FALSE;
 char    patternStr[] = "8 1 0 72 300 32 div div tgifsetpattern";
 
+int	stickyMenuSelection=FALSE;
+
 static int	tileAsGrayDetected=FALSE;
 static int	canFakeGray=FALSE;
 static char	*grayStr[9] =
@@ -176,19 +183,36 @@ void EndGrayDetection ()
    }
 }
 
-void ModeMenu (X, Y)
-   int	X, Y;
+static char * modeMenuDescription[] =
+{
+   "Enter select/move/resize object mode",
+   "Enter draw text mode",
+   "Enter draw rectangle mode",
+   "Enter draw oval mode",
+   "Enter draw poly/open spline mode",
+   "Enter draw polygon/close spline mode",
+   "Enter draw arc mode",
+   "Enter draw rcbox mode",
+   "Enter draw freehand poly/open spline mode",
+   "Enter select/move vertices mode",
+   NULL
+};
+
+int ModeMenu (X, Y, TrackMenubar)
+   int	X, Y, TrackMenubar;
 {
    int		index, * fore_colors, * valid, * init_rv;
 
-   DefaultColorArrays (MAXCHOICES, &fore_colors, &valid, &init_rv);
+   DefaultColorArrays (MAXCHOICES, &fore_colors, &valid, &init_rv, NULL);
    cfree (valid);
    init_rv[curChoice] = TRUE;
    activeMenu = MENU_MODE;
    index = PxMpMenuLoop (X, Y, choiceImageW, choiceImageH, MAXCHOICES, 1,
-         MAXCHOICES, fore_colors, choicePixmap, init_rv, SINGLECOLOR);
+         MAXCHOICES, fore_colors, choicePixmap, init_rv, modeMenuDescription,
+         SINGLECOLOR, TrackMenubar);
 
-   if (index != INVALID) SetCurChoice (index);
+   if (index >= 0) SetCurChoice (index);
+   return (index);
 }
 
 static
@@ -288,11 +312,12 @@ void ChangeAllSelFill (FillIndex)
    register struct SelRec	* sel_ptr;
    int				changed=FALSE, ltx, lty, rbx, rby;
 
-   if (topSel == NULL)
+   if (topSel == NULL || stickyMenuSelection)
    {
       objFill = FillIndex;
       ShowFill ();
-      return;
+      UpdateSubMenu (MENU_FILL);
+      if (topSel == NULL) return;
    }
 
    HighLightReverse ();
@@ -323,18 +348,22 @@ void ChangeAllSelFill (FillIndex)
    HighLightForward ();
 }
 
-void FillMenu (X, Y)
-   int	X, Y;
+static char * fillMenuDescription[] = { "\nSet fill pattern\n", NULL };
+
+int FillMenu (X, Y, TrackMenubar)
+   int	X, Y, TrackMenubar;
 {
    int		index, * fore_colors, * valid, * init_rv;
 
-   DefaultColorArrays (MAXPATTERNS, &fore_colors, &valid, &init_rv);
+   DefaultColorArrays (MAXPATTERNS, &fore_colors, &valid, &init_rv, NULL);
    cfree (valid);
    activeMenu = MENU_FILL;
    index = PxMpMenuLoop (X, Y, choiceImageW, choiceImageH, 8, 4, MAXPATTERNS,
-         fore_colors, patPixmap, init_rv, SINGLECOLOR);
+         fore_colors, patPixmap, init_rv, fillMenuDescription, SINGLECOLOR,
+         TrackMenubar);
 
-   if (index != INVALID) ChangeAllSelFill (index);
+   if (index >= 0) ChangeAllSelFill (index);
+   return (index);
 }
 
 static
@@ -353,6 +382,12 @@ int ChangeObjLineStyle (ObjPtr, StyleIndex)
             ObjPtr->detail.p->style = StyleIndex;
             changed = TRUE;
             AdjObjSplineVs (ObjPtr);
+            if (ObjPtr->detail.p->curved != LT_INTSPLINE)
+               UpdPolyBBox (ObjPtr, ObjPtr->detail.p->n,
+                     ObjPtr->detail.p->vlist);
+            else
+               UpdPolyBBox (ObjPtr, ObjPtr->detail.p->intn,
+                     ObjPtr->detail.p->intvlist);
          }
          break;
       case OBJ_ARC:
@@ -380,12 +415,12 @@ void ChangeAllSelLineStyle (StyleIndex)
    register struct SelRec	* sel_ptr;
    int				ltx, lty, rbx, rby, changed=FALSE;
 
-   if (topSel == NULL)
+   if (topSel == NULL || stickyMenuSelection)
    {
       lineStyle = StyleIndex;
       ShowLineStyle ();
       UpdateSubMenu (MENU_LINESTYLE);
-      return;
+      if (topSel == NULL) return;
    }
 
    HighLightReverse ();
@@ -416,6 +451,153 @@ void ChangeAllSelLineStyle (StyleIndex)
    HighLightForward ();
 }
 
+static
+int VerifyType(ObjPtr, TypeIndex)
+   struct ObjRec	* ObjPtr;
+   int			TypeIndex;
+   /* returns TRUE if ObjPtr has the same line type as TypeIndex */
+{
+   register int		i;
+   struct PolyRec	* poly_ptr;
+   struct PolygonRec	* polygon_ptr;
+   int			curved, n;
+   char			* smooth=NULL;
+
+   switch (ObjPtr->type)
+   {
+      case OBJ_POLY:
+         poly_ptr = ObjPtr->detail.p;
+         smooth = poly_ptr->smooth;
+         curved = poly_ptr->curved;
+         n = poly_ptr->n;
+         break;
+      case OBJ_POLYGON:
+         polygon_ptr = ObjPtr->detail.g;
+         smooth = polygon_ptr->smooth;
+         curved = polygon_ptr->curved;
+         n = polygon_ptr->n;
+         break;
+      default: return (FALSE);
+   }
+   switch (TypeIndex)
+   {
+      case LT_STRAIGHT:
+         if (curved==LT_INTSPLINE) return (FALSE);
+         if (curved==LT_STRAIGHT && smooth==NULL) return (TRUE);
+         if (curved==LT_SPLINE && smooth==NULL) return (FALSE);
+         switch (ObjPtr->type)
+         {
+            case OBJ_POLY:
+               for (i=1; i < n-1; i++) if (smooth[i]) return (FALSE);
+               break;
+            case OBJ_POLYGON:
+               for (i=0; i < n; i++) if (smooth[i]) return (FALSE);
+               break;
+         }
+         return TRUE;
+      case LT_SPLINE:
+         if (curved==LT_INTSPLINE) return (FALSE);
+         if (curved==LT_STRAIGHT && smooth==NULL) return (FALSE);
+         if (curved==LT_SPLINE && smooth==NULL) return (TRUE);
+         switch (ObjPtr->type)
+         {
+            case OBJ_POLY:
+               for (i=1; i < n-1; i++) if (!smooth[i]) return (FALSE);
+               break;
+            case OBJ_POLYGON:
+               for (i=0; i < n; i++) if (!smooth[i]) return (FALSE);
+               break;
+         }
+         return TRUE;
+      case LT_INTSPLINE:
+         return (curved == LT_INTSPLINE);
+   }
+   return (FALSE);
+}
+
+static
+void SetObjSmooth(ObjPtr, TypeIndex)
+   struct ObjRec	* ObjPtr;
+   int			TypeIndex;
+{
+   register int		i;
+   int			n, curved;
+   char			* smooth=NULL;
+   struct PolyRec	* poly_ptr=NULL;
+   struct PolygonRec	* polygon_ptr=NULL;
+
+   switch (ObjPtr->type)
+   {
+      case OBJ_POLY:
+         poly_ptr = ObjPtr->detail.p;
+         smooth = poly_ptr->smooth;
+         curved = poly_ptr->curved;
+         n = poly_ptr->n;
+         switch (TypeIndex)
+         {
+            case LT_STRAIGHT:
+               if (smooth == NULL) smooth = (char *) calloc (n+1, sizeof(char));
+               if (smooth == NULL)
+                  fprintf (stderr, "Can not calloc().\n");
+               else
+                  for (i=0; i < n; i++) smooth[i] = FALSE;
+               poly_ptr->smooth = smooth;
+               return;
+            case LT_SPLINE:
+               if (smooth == NULL) smooth = (char *) calloc (n+1, sizeof(char));
+               if (smooth == NULL)
+                  fprintf (stderr, "Can not calloc().\n");
+               else
+               {
+                  smooth[0] = smooth[n-1] = FALSE;
+                  for (i=1; i < n-1; i++) smooth[i] = TRUE;
+               }
+               poly_ptr->smooth = smooth;
+               return;
+            case LT_INTSPLINE:
+               if (smooth != NULL)
+               {
+                  cfree (smooth);
+                  poly_ptr->smooth = NULL;
+               }
+               return;
+         }
+         break;
+      case OBJ_POLYGON:
+         polygon_ptr = ObjPtr->detail.g;
+         smooth = polygon_ptr->smooth;
+         curved = polygon_ptr->curved;
+         n = polygon_ptr->n;
+         switch (TypeIndex)
+         {
+            case LT_STRAIGHT:
+               if (smooth == NULL) smooth = (char *) calloc (n+1, sizeof(char));
+               if (smooth == NULL)
+                  fprintf (stderr, "Can not calloc().\n");
+               else
+                  for (i=0; i < n; i++) smooth[i] = FALSE;
+               polygon_ptr->smooth = smooth;
+               return;
+            case LT_SPLINE:
+               if (smooth == NULL) smooth = (char *) calloc (n+1, sizeof(char));
+               if (smooth == NULL)
+                  fprintf (stderr, "Can not calloc().\n");
+               else
+                  for (i=0; i < n; i++) smooth[i] = TRUE;
+               polygon_ptr->smooth = smooth;
+               return;
+            case LT_INTSPLINE:
+               if (smooth != NULL)
+               {
+                  cfree (smooth);
+                  polygon_ptr->smooth = NULL;
+               }
+               return;
+         }
+         break;
+   }
+}
+
 static
 int ChangeObjLineType (ObjPtr, TypeIndex)
    struct ObjRec	* ObjPtr;
@@ -427,19 +609,33 @@ int ChangeObjLineType (ObjPtr, TypeIndex)
    switch (ObjPtr->type)
    {
       case OBJ_POLY:
-         if (ObjPtr->detail.p->curved != TypeIndex)
+         if (!VerifyType(ObjPtr, TypeIndex))
          {
             ObjPtr->detail.p->curved = TypeIndex;
+            SetObjSmooth(ObjPtr, TypeIndex);
             changed = TRUE;
             AdjObjSplineVs (ObjPtr);
+            if (ObjPtr->detail.p->curved != LT_INTSPLINE)
+               UpdPolyBBox (ObjPtr, ObjPtr->detail.p->n,
+                     ObjPtr->detail.p->vlist);
+            else
+               UpdPolyBBox (ObjPtr, ObjPtr->detail.p->intn,
+                     ObjPtr->detail.p->intvlist);
          }
          break;
       case OBJ_POLYGON:
-         if (ObjPtr->detail.g->curved != TypeIndex)
+         if (!VerifyType(ObjPtr, TypeIndex))
          {
             ObjPtr->detail.g->curved = TypeIndex;
+            SetObjSmooth(ObjPtr, TypeIndex);
             changed = TRUE;
             AdjObjSplineVs (ObjPtr);
+            if (ObjPtr->detail.g->curved != LT_INTSPLINE)
+               UpdPolyBBox (ObjPtr, ObjPtr->detail.g->n,
+                     ObjPtr->detail.g->vlist);
+            else
+               UpdPolyBBox (ObjPtr, ObjPtr->detail.g->intn,
+                     ObjPtr->detail.g->intvlist);
          }
          break;
 
@@ -461,12 +657,12 @@ void ChangeAllSelLineType (TypeIndex)
    register struct SelRec	* sel_ptr;
    int				ltx, lty, rbx, rby, changed=FALSE;
 
-   if (topSel == NULL)
+   if (topSel == NULL || stickyMenuSelection)
    {
       curSpline = TypeIndex;
       ShowLineType ();
       UpdateSubMenu (MENU_LINETYPE);
-      return;
+      if (topSel == NULL) return;
    }
 
    HighLightReverse ();
@@ -516,6 +712,12 @@ int ChangeObjLineWidth (ObjPtr, W, AW, AH)
             ObjPtr->detail.p->ah = AH;
             changed = TRUE;
             AdjObjSplineVs (ObjPtr);
+            if (ObjPtr->detail.p->curved != LT_INTSPLINE)
+               UpdPolyBBox (ObjPtr, ObjPtr->detail.p->n,
+                     ObjPtr->detail.p->vlist);
+            else
+               UpdPolyBBox (ObjPtr, ObjPtr->detail.p->intn,
+                     ObjPtr->detail.p->intvlist);
          }
          break;
       case OBJ_BOX:
@@ -576,12 +778,12 @@ void ChangeAllSelLineWidth (WidthIndex)
    int				ltx, lty, rbx, rby, changed=FALSE;
    int				w, aw, ah;
 
-   if (topSel == NULL)
+   if (topSel == NULL || stickyMenuSelection)
    {
       lineWidth = WidthIndex;
       ShowLineWidth ();
       UpdateSubMenu (MENU_LINEWIDTH);
-      return;
+      if (topSel == NULL) return;
    }
 
    w = curWidthOfLine[WidthIndex];
@@ -686,12 +888,12 @@ void ChangeAllSelDashes (DashIndex)
    register struct SelRec	* sel_ptr;
    int				changed=FALSE;
 
-   if (topSel == NULL)
+   if (topSel == NULL || stickyMenuSelection)
    {
       curDash = DashIndex;
       ShowDash ();
       UpdateSubMenu (MENU_LINEDASH);
-      return;
+      if (topSel == NULL) return;
    }
 
    HighLightReverse ();
@@ -718,64 +920,110 @@ void ChangeAllSelDashes (DashIndex)
    HighLightForward ();
 }
 
-void LineWidthMenu (X, Y)
-   int	X, Y;
+int LineWidthMenu (X, Y, TrackMenubar)
+   int	X, Y, TrackMenubar;
 {
+   register int	i;
    int		index, * fore_colors, * valid, * init_rv;
+   char		* * desc=(char **) calloc (maxLineWidths+1, sizeof(char*));
 
-   DefaultColorArrays (maxLineWidths, &fore_colors, &valid, &init_rv);
+   if (desc == NULL) fprintf (stderr, "Can not calloc().\n");
+   for (i=0; i < maxLineWidths; i++)
+   {
+      desc[i] = (char*) calloc (80, sizeof(char));
+      if (desc[i] == NULL) fprintf (stderr, "Can not caloc().\n");
+      sprintf (desc[i], "Set line width to %1d", curWidthOfLine[i]);
+   }
+   desc[i] = NULL;
+   DefaultColorArrays (maxLineWidths, &fore_colors, &valid, &init_rv, NULL);
    cfree (valid);
    init_rv[lineWidth] = TRUE;
    activeMenu = MENU_LINEWIDTH;
    index = PxMpMenuLoop (X, Y, menuImageW, menuImageH, maxLineWidths, 1,
-         maxLineWidths, fore_colors, lineWidthPixmap, init_rv, SINGLECOLOR);
+         maxLineWidths, fore_colors, lineWidthPixmap, init_rv, desc,
+         SINGLECOLOR, TrackMenubar);
 
-   if (index != INVALID) ChangeAllSelLineWidth (index);
+   if (index >= 0) ChangeAllSelLineWidth (index);
+   if (desc != NULL)
+   {
+      for (i=0; i < maxLineWidths; i++)
+         if (desc[i] != NULL) cfree (desc[i]);
+      cfree (desc);
+   }
+   return (index);
 }
 
-void LineStyleMenu (X, Y)
-   int	X, Y;
+static char * lineStyleMenuDescription[] =
+{
+   "No arrows for a poly/open spline",
+   "An arrow at the end of a poly/open spline",
+   "An arrow at the beginning of a poly/open spline",
+   "Arrows at both ends of a poly/open spline",
+   NULL
+};
+
+int LineStyleMenu (X, Y, TrackMenubar)
+   int	X, Y, TrackMenubar;
 {
    int		index, * fore_colors, * valid, * init_rv;
 
-   DefaultColorArrays (MAXLINESTYLES, &fore_colors, &valid, &init_rv);
+   DefaultColorArrays (MAXLINESTYLES, &fore_colors, &valid, &init_rv, NULL);
    cfree (valid);
    init_rv[lineStyle] = TRUE;
    activeMenu = MENU_LINESTYLE;
    index = PxMpMenuLoop (X, Y, menuImageW, menuImageH, MAXLINESTYLES, 1,
-         MAXLINESTYLES, fore_colors, lineStylePixmap, init_rv, SINGLECOLOR);
+         MAXLINESTYLES, fore_colors, lineStylePixmap, init_rv,
+         lineStyleMenuDescription, SINGLECOLOR, TrackMenubar);
 
-   if (index != INVALID) ChangeAllSelLineStyle (index);
+   if (index >= 0) ChangeAllSelLineStyle (index);
+   return (index);
 }
 
-void LineTypeMenu (X, Y)
-   int	X, Y;
+static char * lineTypeMenuDescription[] =
+{
+   "Straight line segments everywhere",
+   "Open/closed splines",
+   "Interpolated splines",
+   NULL
+};
+
+int LineTypeMenu (X, Y, TrackMenubar)
+   int	X, Y, TrackMenubar;
 {
    int		index, * fore_colors, * valid, * init_rv;
 
-   DefaultColorArrays (MAXLINETYPES, &fore_colors, &valid, &init_rv);
+   DefaultColorArrays (MAXLINETYPES, &fore_colors, &valid, &init_rv, NULL);
    cfree (valid);
    init_rv[curSpline] = TRUE;
    activeMenu = MENU_LINETYPE;
    index = PxMpMenuLoop (X, Y, menuImageW, menuImageH, MAXLINETYPES, 1,
-         MAXLINETYPES, fore_colors, lineTypePixmap, init_rv, SINGLECOLOR);
+         MAXLINETYPES, fore_colors, lineTypePixmap, init_rv,
+         lineTypeMenuDescription, SINGLECOLOR, TrackMenubar);
 
-   if (index != INVALID) ChangeAllSelLineType (index);
+   if (index >= 0) ChangeAllSelLineType (index);
+   return (index);
 }
 
-void LineDashMenu (X, Y)
-   int	X, Y;
+static char * lineDashMenuDescription[] =
+{
+   "\nSet dash pattern for the pen\n", NULL
+};
+
+int LineDashMenu (X, Y, TrackMenubar)
+   int	X, Y, TrackMenubar;
 {
    int		index, * fore_colors, * valid, * init_rv;
 
-   DefaultColorArrays (MAXDASHES, &fore_colors, &valid, &init_rv);
+   DefaultColorArrays (MAXDASHES, &fore_colors, &valid, &init_rv, NULL);
    cfree (valid);
    init_rv[curDash] = TRUE;
    activeMenu = MENU_LINEDASH;
    index = PxMpMenuLoop (X, Y, menuImageW, menuImageH, MAXDASHES, 1,
-         MAXDASHES, fore_colors, dashPixmap, init_rv, SINGLECOLOR);
+         MAXDASHES, fore_colors, dashPixmap, init_rv, lineDashMenuDescription,
+         SINGLECOLOR, TrackMenubar);
 
-   if (index != INVALID) ChangeAllSelDashes (index);
+   if (index >= 0) ChangeAllSelDashes (index);
+   return (index);
 }
 
 static
@@ -855,11 +1103,12 @@ void ChangeAllSelPen (PenIndex)
    register struct SelRec	* sel_ptr;
    int				changed=FALSE;
 
-   if (topSel == NULL)
+   if (topSel == NULL || stickyMenuSelection)
    {
       penPat = PenIndex;
       ShowPen ();
-      return;
+      UpdateSubMenu (MENU_PEN);
+      if (topSel == NULL) return;
    }
 
    HighLightReverse ();
@@ -886,18 +1135,22 @@ void ChangeAllSelPen (PenIndex)
    HighLightForward ();
 }
 
-void PenMenu (X, Y)
-   int	X, Y;
+static char * penMenuDescription[] = { "\nSet pen pattern\n", NULL };
+
+int PenMenu (X, Y, TrackMenubar)
+   int	X, Y, TrackMenubar;
 {
    int		index, * fore_colors, * valid, * init_rv;
 
-   DefaultColorArrays (MAXPATTERNS, &fore_colors, &valid, &init_rv);
+   DefaultColorArrays (MAXPATTERNS, &fore_colors, &valid, &init_rv, NULL);
    cfree (valid);
    activeMenu = MENU_PEN;
    index = PxMpMenuLoop (X, Y, choiceImageW, choiceImageH, 8, 4, MAXPATTERNS,
-         fore_colors, patPixmap, init_rv, SINGLECOLOR);
+         fore_colors, patPixmap, init_rv, penMenuDescription, SINGLECOLOR,
+         TrackMenubar);
 
-   if (index != INVALID) ChangeAllSelPen (index);
+   if (index >= 0) ChangeAllSelPen (index);
+   return (index);
 }
 
 static
@@ -910,14 +1163,26 @@ int ToggleObjLineType (ObjPtr)
    switch (ObjPtr->type)
    {
       case OBJ_POLY:
-         ObjPtr->detail.p->curved = !ObjPtr->detail.p->curved;
+         ObjPtr->detail.p->curved = (ObjPtr->detail.p->curved+1) % MAXLINETYPES;
          changed = TRUE;
          AdjObjSplineVs (ObjPtr);
+         if (ObjPtr->detail.p->curved != LT_INTSPLINE)
+            UpdPolyBBox (ObjPtr, ObjPtr->detail.p->n,
+                  ObjPtr->detail.p->vlist);
+         else
+            UpdPolyBBox (ObjPtr, ObjPtr->detail.p->intn,
+                  ObjPtr->detail.p->intvlist);
          break;
       case OBJ_POLYGON:
-         ObjPtr->detail.g->curved = !ObjPtr->detail.g->curved;
+         ObjPtr->detail.g->curved = (ObjPtr->detail.g->curved+1) % MAXLINETYPES;
          changed = TRUE;
          AdjObjSplineVs (ObjPtr);
+         if (ObjPtr->detail.g->curved != LT_INTSPLINE)
+            UpdPolyBBox (ObjPtr, ObjPtr->detail.g->n,
+                  ObjPtr->detail.g->vlist);
+         else
+            UpdPolyBBox (ObjPtr, ObjPtr->detail.g->intn,
+                  ObjPtr->detail.g->intvlist);
          break;
 
       case OBJ_GROUP:
@@ -939,7 +1204,7 @@ void ToggleAllSelLineType ()
 
    if (topSel == NULL)
    {
-      curSpline = !curSpline;
+      curSpline = (curSpline+1) % MAXLINETYPES;
       ShowLineType ();
       UpdateSubMenu (MENU_LINETYPE);
       return;
@@ -1057,7 +1322,7 @@ int UpdateAnObj (ObjPtr)
 
    if (ChangeObjTextStyle (ObjPtr, curStyle)) changed = TRUE;
    if (ChangeObjTextJust (ObjPtr, textJust)) changed = TRUE;
-   if (ChangeObjTextSize (ObjPtr, curSize)) changed = TRUE;
+   if (ChangeObjTextSize (ObjPtr, GetSizeMenuIndex ())) changed = TRUE;
    if (ChangeObjTextFont (ObjPtr, curFont)) changed = TRUE;
    if (ChangeObjVSpace (ObjPtr, textVSpace)) changed = TRUE;
 
@@ -1077,9 +1342,6 @@ void UpdateSelObjs ()
    for (sel_ptr = botSel; sel_ptr != NULL; sel_ptr = sel_ptr->prev)
    {
       obj_ptr = sel_ptr->obj;
-      if (obj_ptr->type==OBJ_ICON || obj_ptr->type==OBJ_SYM ||
-            obj_ptr->type==OBJ_GROUP)
-         continue;
 
       PrepareToReplaceAnObj (obj_ptr);
       if (UpdateAnObj (obj_ptr))
