@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,21 +25,25 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/polygon.c,v 2.58.1.4 1993/01/03 23:34:24 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/polygon.c,v 2.82 1994/04/18 04:34:13 william Exp $";
 #endif
 
 #include <math.h>
 #include <stdio.h>
+#include <string.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 #include "const.h"
 #include "types.h"
 
+#include "attr.e"
 #include "cmd.e"
 #include "color.e"
 #include "cursor.e"
 #include "dialog.e"
+#include "drawing.e"
+#include "dup.e"
 #include "grid.e"
 #include "file.e"
 #include "mainloop.e"
@@ -46,6 +51,9 @@ static char RCSid[] =
 #include "obj.e"
 #include "pattern.e"
 #include "poly.e"
+#ifndef _NO_EXTERN
+#include "polygon.e"
+#endif
 #include "raster.e"
 #include "ruler.e"
 #include "select.e"
@@ -94,79 +102,168 @@ void DumpPoints (FP, NumPts, V, Indent)
 }
 
 static
-void DumpPolygonPath (FP, ObjPtr, Vs, NumPts, Width, Pen, Curved, Dash)
+void DumpPolygonPath (FP, ObjPtr, Vs, NumPts, Smooth, Width, Pen, Fill,
+      Curved, Dash, Indent)
    FILE			* FP;
    struct ObjRec	* ObjPtr;
    XPoint		* Vs;
-   int			NumPts;
-   int			Width, Pen, Curved, Dash;
+   int			NumPts, Width, Pen, Fill, Curved, Dash, Indent;
+   char			* Smooth;
 {
    register int	i;
+   int		color_index=ObjPtr->color;
 
-   if (Width != 1) fprintf (FP, "   %1d setlinewidth\n", Width);
-   if (Dash != 0)
-   {
-      fprintf (FP, "   [");
-      for (i = 0; i < dashListLength[Dash]-1; i++)
-         fprintf (FP, "%1d ", (int)(dashList[Dash][i]));
-      fprintf (FP, "%1d] 0 setdash\n",
-            (int)(dashList[Dash][dashListLength[Dash]-1]));
+   if (Fill == (-1) && Pen != (-1))
+   {  /* dumping the pen */
+      if (Width != 1)
+      {
+         for (i=0; i < Indent+3; i++) fprintf (FP, " ");
+         fprintf (FP, "%1d setlinewidth\n", Width);
+      }
+      if (Dash != 0)
+      {
+         for (i=0; i < Indent+3; i++) fprintf (FP, " ");
+         fprintf (FP, "[");
+         for (i = 0; i < dashListLength[Dash]-1; i++)
+            fprintf (FP, "%1d ", (int)(dashList[Dash][i]));
+         fprintf (FP, "%1d] 0 setdash\n",
+               (int)(dashList[Dash][dashListLength[Dash]-1]));
+      }
    }
-
-   if (!colorDump && Pen > BACKPAT)
-   {
-      GrayCheck (Pen);
-      if (useGray)
-         fprintf (FP, "   %s setgray\n", GrayStr(Pen));
-      else
-         fprintf (FP, "   pat%1d %s\n", Pen, patternStr);
+   else if (Fill != (-1) && Pen == (-1))
+   {  /* dumping the fill */
+      if (Fill > BACKPAT)
+      {
+         for (i=0; i < Indent; i++) fprintf (FP, " ");
+         fprintf (FP, "gsave\n");
+         if (!colorDump)
+         {
+            GrayCheck (Fill);
+            for (i=0; i < Indent+3; i++) fprintf (FP, " ");
+            if (useGray)
+               fprintf(FP, "%s setgray\n", GrayStr(Fill));
+            else
+               fprintf(FP, "pat%1d %s\n", Fill, patternStr);
+         }
+         else
+         {
+            DumpPolygonPath (FP, ObjPtr, Vs, NumPts, Smooth, Width, (-1),
+                  BACKPAT, Curved, Dash, Indent);
+         }
+      }
    }
-
+   for (i=0; i < Indent+3; i++) fprintf (FP, " ");
+   fprintf (FP, "newpath\n");
+   if (Curved == LT_INTSPLINE)
+      DumpCurvedPolygonPoints (FP, NumPts, Vs, Indent+6);
+   else
+      DumpMultiCurvedPolygonPoints (FP, Smooth, Curved, NumPts, Vs, Indent+6);
+   for (i=0; i < Indent+3; i++) fprintf (FP, " ");
+   fprintf (FP, "closepath ");
+/*
    switch (Curved)
    {
       case LT_STRAIGHT:
+      case LT_SPLINE:
          fprintf (FP, "   newpath\n   %1d %1d moveto\n", Vs[0].x, Vs[0].y);
          DumpPoints (FP, NumPts-1, Vs, 6);
          fprintf (FP, "   closepath ");
          break;
-      case LT_SPLINE:
+      case LT_INTSPLINE:
          fprintf (FP, "   newpath\n");
          DumpCurvedPolygonPoints (FP, NumPts, Vs, 6);
          fprintf (FP, "   closepath ");
          break;
    }
+ */
 
-   switch (Pen)
-   {
-      case SOLIDPAT: fprintf (FP, "stroke\n"); break;
-      case BACKPAT: fprintf (FP, "1 setgray stroke 0 setgray\n"); break;
-      default:
-         if (colorDump)
-         {
-            fprintf (FP, "flattenpath strokepath clip\n");
-            DumpPatFill (FP, Pen, 8, ObjPtr->bbox, "   ");
-         }
-         else
-            fprintf (FP, "stroke\n");
-         break;
+   if (Fill == (-1) && Pen != (-1))
+   {  /* dumping the pen */
+      switch (Pen)
+      {
+         case SOLIDPAT: fprintf (FP, "stroke\n"); break;
+         case BACKPAT: fprintf (FP, "1 setgray stroke 0 setgray\n"); break;
+         default:
+            if (colorDump)
+            {
+               fprintf (FP, "flattenpath strokepath clip newpath\n");
+               for (i=0; i < Indent+3; i++) fprintf (FP, " ");
+               DumpPatFill (FP, Pen, 8, ObjPtr->bbox, "");
+            }
+            else
+            {
+               GrayCheck (Pen);
+               if (useGray)
+                  fprintf (FP, "%s setgray\n", GrayStr(Pen));
+               else
+                  fprintf (FP, "pat%1d %s\n", Pen, patternStr);
+               for (i=0; i < Indent+3; i++) fprintf (FP, " ");
+               fprintf (FP, "stroke\n");
+            }
+            break;
+      }
+      if (Dash != 0)
+      {
+         for (i=0; i < Indent+3; i++) fprintf (FP, " ");
+         fprintf (FP, "[] 0 setdash\n");
+      }
+      if (Width != 1)
+      {
+         for (i=0; i < Indent+3; i++) fprintf (FP, " ");
+         fprintf (FP, "1 setlinewidth\n");
+      }
+   }
+   else if (Fill != (-1) && Pen == (-1))
+   {  /* dumping the fill */
+      switch (Fill)
+      {
+         case SOLIDPAT:
+            fprintf (FP, "eofill\n");
+            break;
+         case BACKPAT:
+            fprintf (FP, "1 setgray eofill\n");
+            for (i=0; i < Indent+3; i++) fprintf (FP, " ");
+            if (colorDump)
+               fprintf (FP, "%.3f %.3f %.3f setrgbcolor\n",
+                     ((float)tgifColors[color_index].red/maxRGB),
+                     ((float)tgifColors[color_index].green/maxRGB),
+                     ((float)tgifColors[color_index].blue/maxRGB));
+            else
+               fprintf (FP, "0 setgray\n");
+            break;
+         default:
+            if (colorDump)
+            {
+               fprintf (FP, "eoclip newpath\n");
+               for (i=0; i < Indent+3; i++) fprintf (FP, " ");
+               DumpPatFill (FP, Fill, 8, ObjPtr->bbox, "");
+            }
+            else
+               fprintf (FP, "eofill\n");
+            for (i=0; i < Indent; i++) fprintf (FP, " ");
+            fprintf (FP, "grestore\n");
+            break;
+      }
    }
-   if (Dash != 0) fprintf (FP, "   [] 0 setdash\n");
-   if (Width != 1) fprintf (FP, "   1 setlinewidth\n");
 }
 
 void DumpPolygonObj (FP, ObjPtr)
    FILE			* FP;
    struct ObjRec	* ObjPtr;
 {
-   XPoint	* v = ObjPtr->detail.g->vlist;
+   XPoint	* v = ObjPtr->detail.g->vlist, * intv;
    int		num_pts = ObjPtr->detail.g->n;
-   int		fill, width, pen, curved, dash, color_index;
+   int		fill, width, pen, curved, dash, color_index, intn;
+   char		* smooth;
 
    fill = ObjPtr->detail.g->fill;
    width = ObjPtr->detail.g->width;
    pen = ObjPtr->detail.g->pen;
    curved = ObjPtr->detail.g->curved;
    dash = ObjPtr->detail.g->dash;
+   intv = ObjPtr->detail.g->intvlist;
+   intn = ObjPtr->detail.g->intn;
+   smooth = ObjPtr->detail.g->smooth;
 
    if (fill == NONEPAT && pen == NONEPAT) return;
 
@@ -180,6 +277,13 @@ void DumpPolygonObj (FP, ObjPtr)
 
    if (fill != NONEPAT)
    {
+      if (curved != LT_INTSPLINE)
+         DumpPolygonPath (FP, ObjPtr, v, num_pts, smooth, width, (-1), fill,
+               curved, dash, (fill > BACKPAT ? 0 : (-3)));
+      else
+         DumpPolygonPath (FP, ObjPtr, intv, intn, smooth, width, (-1), fill,
+               curved, dash, (fill > BACKPAT ? 0 : (-3)));
+#ifdef NOT_DEFINED
       switch (curved)
       {
          case LT_STRAIGHT:
@@ -228,7 +332,7 @@ void DumpPolygonObj (FP, ObjPtr)
                   DumpPoints (FP, num_pts-1, v, 6);
                   if (colorDump)
                   {
-                     fprintf (FP, "   closepath eoclip\n");
+                     fprintf (FP, "   closepath eoclip newpath\n");
                      DumpPatFill (FP, fill, 8, ObjPtr->bbox, "   ");
                   }
                   else
@@ -238,16 +342,23 @@ void DumpPolygonObj (FP, ObjPtr)
             }
             break;
          case LT_SPLINE:
+         case LT_INTSPLINE:
             switch (fill)
             {
                case SOLIDPAT: /* solid black polygon */
                   fprintf (FP, "newpath\n");
-                  DumpCurvedPolygonPoints (FP, num_pts, v, 3);
+                  if (curved == LT_SPLINE)
+                     DumpCurvedPolygonPoints (FP, num_pts, v, 3);
+                  else
+                     DumpCurvedPolygonPoints (FP, intn, intv, 3);
                   fprintf (FP, "closepath eofill\n");
                   break;
                case BACKPAT: /* solid white polygon */
                   fprintf (FP, "newpath\n");
-                  DumpCurvedPolygonPoints (FP, num_pts, v, 3);
+                  if (curved == LT_SPLINE)
+                     DumpCurvedPolygonPoints (FP, num_pts, v, 3);
+                  else
+                     DumpCurvedPolygonPoints (FP, intn, intv, 3);
                   fprintf (FP, "closepath 1 setgray eofill\n");
                   if (colorDump)
                      fprintf (FP, "   %.3f %.3f %.3f setrgbcolor\n",
@@ -270,7 +381,10 @@ void DumpPolygonObj (FP, ObjPtr)
                   else
                   {
                      fprintf (FP, "   newpath\n");
-                     DumpCurvedPolygonPoints (FP, num_pts, v, 6);
+                     if (curved == LT_SPLINE)
+                        DumpCurvedPolygonPoints (FP, num_pts, v, 6);
+                     else
+                        DumpCurvedPolygonPoints (FP, intn, intv, 6);
                      fprintf (FP, "   closepath 1 setgray eofill\n");
                      fprintf (FP, "   %.3f %.3f %.3f setrgbcolor\n",
                            ((float)tgifColors[color_index].red/maxRGB),
@@ -278,10 +392,13 @@ void DumpPolygonObj (FP, ObjPtr)
                            ((float)tgifColors[color_index].blue/maxRGB));
                   }
                   fprintf (FP, "   newpath\n");
-                  DumpCurvedPolygonPoints (FP, num_pts, v, 6);
+                  if (curved == LT_SPLINE)
+                     DumpCurvedPolygonPoints (FP, num_pts, v, 6);
+                  else
+                     DumpCurvedPolygonPoints (FP, intn, intv, 6);
                   if (colorDump)
                   {
-                     fprintf (FP, "   closepath eoclip\n");
+                     fprintf (FP, "   closepath eoclip newpath\n");
                      DumpPatFill (FP, fill, 8, ObjPtr->bbox, "   ");
                   }
                   else
@@ -291,6 +408,7 @@ void DumpPolygonObj (FP, ObjPtr)
             }
             break;
       }
+#endif /* NOT_DEFINED */
    }
 
    if (pen == NONEPAT) { fprintf (FP, "\n"); return; }
@@ -299,14 +417,24 @@ void DumpPolygonObj (FP, ObjPtr)
 
    if (colorDump && pen > BACKPAT)
    {
-      DumpPolygonPath (FP, ObjPtr, v, num_pts, width, BACKPAT, curved, 0);
+      if (curved != LT_INTSPLINE)
+         DumpPolygonPath (FP, ObjPtr, v, num_pts, smooth, width, BACKPAT, (-1),
+               curved, 0, 0);
+      else
+         DumpPolygonPath (FP, ObjPtr, intv, intn, smooth, width, BACKPAT, (-1),
+               curved, 0, 0);
       if (colorDump)
          fprintf (FP, "   %.3f %.3f %.3f setrgbcolor\n",
                ((float)tgifColors[color_index].red/maxRGB),
                ((float)tgifColors[color_index].green/maxRGB),
                ((float)tgifColors[color_index].blue/maxRGB));
    }
-   DumpPolygonPath (FP, ObjPtr, v, num_pts, width, pen, curved, dash);
+   if (curved != LT_INTSPLINE)
+      DumpPolygonPath (FP, ObjPtr, v, num_pts, smooth, width, pen, (-1),
+            curved, dash, 0);
+   else
+      DumpPolygonPath (FP, ObjPtr, intv, intn, smooth, width, pen, (-1),
+            curved, dash, 0);
 
    fprintf (FP, "grestore\n");
 
@@ -373,9 +501,13 @@ void DrawPolygonObj (Win, XOff, YOff, ObjPtr)
    XDrawLines (mainDisplay, Win, drawGC, v, num_pts, CoordModeOrigin);
 }
 
+#define CREATE_RELATIVE (FALSE)
+#define CREATE_ABSOLUTE (TRUE)
+
 static
-void CreatePolygonObj (NumPts)
+void CreatePolygonObj (NumPts, CreateAbsolute)
    int	NumPts;
+   int	CreateAbsolute;
 {
    register int		i;
    struct PtRec		* pt_ptr, * next_pt;
@@ -383,28 +515,40 @@ void CreatePolygonObj (NumPts)
    struct ObjRec	* obj_ptr;
    XPoint		* v;
    int			width, w, ltx, lty, rbx, rby;
+   char			* smooth=NULL;
 
    polygon_ptr = (struct PolygonRec *) calloc (1, sizeof(struct PolygonRec));
    polygon_ptr->n = NumPts;
    v = (XPoint *) calloc (NumPts+1, sizeof(XPoint));
    if (v == NULL) fprintf (stderr, "Can not calloc().\n");
+   if (curSpline != LT_INTSPLINE)
+   {
+      smooth = (char *) calloc (NumPts+1, sizeof(char));
+      if (smooth == NULL) fprintf(stderr, "Can not calloc().\n");
+   }
    pt_ptr = lastPtPtr;
    ltx = rbx = pt_ptr->x;
    lty = rby = pt_ptr->y;
    for (i = NumPts-1; i >= 0; i--, lastPtPtr = next_pt)
    {
       next_pt = lastPtPtr->next;
-      v[i].x = ABS_X(lastPtPtr->x);
-      v[i].y = ABS_Y(lastPtPtr->y);
-      if (lastPtPtr->x < ltx) ltx = lastPtPtr->x;
-      if (lastPtPtr->y < lty) lty = lastPtPtr->y;
-      if (lastPtPtr->x > rbx) rbx = lastPtPtr->x;
-      if (lastPtPtr->y > rby) rby = lastPtPtr->y;
+      v[i].x = CreateAbsolute ? lastPtPtr->x : ABS_X(lastPtPtr->x);
+      v[i].y = CreateAbsolute ? lastPtPtr->y : ABS_Y(lastPtPtr->y);
+      if (curSpline != LT_INTSPLINE)
+      {
+         if (lastPtPtr->x < ltx) ltx = lastPtPtr->x;
+         if (lastPtPtr->y < lty) lty = lastPtPtr->y;
+         if (lastPtPtr->x > rbx) rbx = lastPtPtr->x;
+         if (lastPtPtr->y > rby) rby = lastPtPtr->y;
+         smooth[i] = (curSpline != LT_STRAIGHT);
+      }
       cfree (lastPtPtr);
    }
 
    polygon_ptr->vlist = v;
+   polygon_ptr->smooth = smooth;
    polygon_ptr->svlist = NULL;
+   polygon_ptr->intvlist = NULL;
    polygon_ptr->fill = objFill;
    polygon_ptr->width = width = curWidthOfLine[lineWidth];
    polygon_ptr->pen = penPat;
@@ -413,10 +557,20 @@ void CreatePolygonObj (NumPts)
    obj_ptr = (struct ObjRec *) calloc (1, sizeof(struct ObjRec));
    obj_ptr->color = colorIndex;
    obj_ptr->type = OBJ_POLYGON;
-   obj_ptr->bbox.ltx = obj_ptr->obbox.ltx = obj_ptr->x = ABS_X(ltx);
-   obj_ptr->bbox.lty = obj_ptr->obbox.lty = obj_ptr->y = ABS_Y(lty);
-   obj_ptr->bbox.rbx = obj_ptr->obbox.rbx = ABS_X(rbx);
-   obj_ptr->bbox.rby = obj_ptr->obbox.rby = ABS_Y(rby);
+   if (CreateAbsolute)
+   {
+      obj_ptr->bbox.ltx = obj_ptr->obbox.ltx = obj_ptr->x = ltx;
+      obj_ptr->bbox.lty = obj_ptr->obbox.lty = obj_ptr->y = lty;
+      obj_ptr->bbox.rbx = obj_ptr->obbox.rbx = rbx;
+      obj_ptr->bbox.rby = obj_ptr->obbox.rby = rby;
+   }
+   else
+   {
+      obj_ptr->bbox.ltx = obj_ptr->obbox.ltx = obj_ptr->x = ABS_X(ltx);
+      obj_ptr->bbox.lty = obj_ptr->obbox.lty = obj_ptr->y = ABS_Y(lty);
+      obj_ptr->bbox.rbx = obj_ptr->obbox.rbx = ABS_X(rbx);
+      obj_ptr->bbox.rby = obj_ptr->obbox.rby = ABS_Y(rby);
+   }
    w = HALF_W(width);
    obj_ptr->bbox.ltx -= w;
    obj_ptr->bbox.lty -= w;
@@ -429,6 +583,11 @@ void CreatePolygonObj (NumPts)
    obj_ptr->fattr = obj_ptr->lattr = NULL;
    obj_ptr->detail.g = polygon_ptr;
    AdjObjSplineVs (obj_ptr);
+   if (curSpline != LT_INTSPLINE)
+      UpdPolyBBox (obj_ptr, polygon_ptr->n, polygon_ptr->vlist);
+   else
+      UpdPolyBBox (obj_ptr, polygon_ptr->intn, polygon_ptr->intvlist);
+   AdjObjBBox (obj_ptr);
    AddObj (NULL, topObj, obj_ptr);
 }
 
@@ -444,12 +603,12 @@ void ContinuePolygon (OrigX, OrigY)
    XButtonEvent		* button_ev;
    XMotionEvent		* motion_ev;
    KeySym		key_sym;
-   char			s[80];
+   char			s[80], buf[80];
    int 			end_x, end_y, grid_x, grid_y, done = FALSE;
    int 			saved_x, saved_y, closed=FALSE;
-   int			xor_pixel, num_pts=1, n=2, sn=0, max_n=40;
+   int			xor_pixel, num_pts=1, n=2, sn=0, max_n=40, intn=0;
    struct PtRec		* pt_ptr;
-   XPoint		* v = NULL, * sv = NULL;
+   XPoint		* v = NULL, * sv = NULL, * cntrlv=NULL;
 
    xor_pixel = xorColorPixels[colorIndex];
 
@@ -469,16 +628,29 @@ void ContinuePolygon (OrigX, OrigY)
 
    grid_x = end_x = saved_x = OrigX;
    grid_y = end_y = saved_y = OrigY;
-   if (curSpline == LT_SPLINE && splineRubberband)
+   if (curSpline != LT_STRAIGHT && splineRubberband)
    {
       v = (XPoint *) calloc (max_n+1, sizeof (XPoint));
       if (v == NULL) fprintf (stderr, "Can not calloc().\n");
       v[0].x = v[1].x = v[2].x = ABS_X(OrigX);
       v[0].y = v[1].y = v[2].y = ABS_Y(OrigY);
-      sv = MakeSplinePolygonVertex (&sn, drawOrigX, drawOrigY, n+1, v);
+      switch (curSpline)
+      {
+         case LT_SPLINE:
+            sv = MakeSplinePolygonVertex (&sn, drawOrigX, drawOrigY, n+1, v);
+            break;
+         case LT_INTSPLINE:
+            sv = MakeIntSplinePolygonVertex (&sn, &intn, &cntrlv,
+                  drawOrigX, drawOrigY, n+1, v);
+            break;
+      }
    }
-
+   SaveStatusStrings ();
+   SetMouseStatus ("Add a vertex", "Close the polygon/spline",
+         "Close the polygon/spline");
    Msg ("Click middle or right button to end polygon.");
+   sprintf (buf, "0x0%+1d%+1d", ABS_X(grid_x), ABS_Y(grid_y));
+   StartShowMeasureCursor (grid_x, grid_y, buf, TRUE);
    XGrabPointer (mainDisplay, drawWindow, FALSE,
          PointerMotionMask | ButtonPressMask,
          GrabModeAsync, GrabModeAsync, None, handCursor, CurrentTime);
@@ -490,7 +662,7 @@ void ContinuePolygon (OrigX, OrigY)
       if (input.type == Expose || input.type == VisibilityNotify)
          ExposeEventHandler (&input, TRUE);
       else if ((input.type == KeyPress || input.type == KeyRelease) &&
-            num_pts != 1 && !(curSpline == LT_SPLINE && splineRubberband))
+            num_pts != 1 && !(curSpline != LT_STRAIGHT && splineRubberband))
       {
          XLookupString (&(input.xkey), s, 80-1, &key_sym, &c_stat);
          if (key_sym == XK_Shift_L || key_sym == XK_Shift_R ||
@@ -503,7 +675,10 @@ void ContinuePolygon (OrigX, OrigY)
       }
       else if (input.type == MotionNotify)
       {
-         if (curSpline == LT_SPLINE && splineRubberband)
+         sprintf (buf, "%1dx%1d%+1d%+1d", ABS_SIZE(abs(grid_x-OrigX)),
+               ABS_SIZE(abs(grid_y-OrigY)), ABS_X(grid_x), ABS_Y(grid_y));
+         ShowMeasureCursor (grid_x, grid_y, buf, TRUE);
+         if (curSpline != LT_STRAIGHT && splineRubberband)
             XDrawLines (mainDisplay, drawWindow, drawGC, sv, sn,
                   CoordModeOrigin);
          else
@@ -519,12 +694,22 @@ void ContinuePolygon (OrigX, OrigY)
          end_y = motion_ev->y;
          GridXY (end_x, end_y, &grid_x, &grid_y);
          MarkRulers (grid_x, grid_y);
-         if (curSpline == LT_SPLINE && splineRubberband)
+         if (curSpline != LT_STRAIGHT && splineRubberband)
          {
-            cfree (sv);
+            if (sv != NULL) cfree (sv);
             v[n-1].x = ABS_X(grid_x);
             v[n-1].y = ABS_Y(grid_y);
-            sv = MakeSplinePolygonVertex (&sn, drawOrigX, drawOrigY, n+1, v);
+            switch (curSpline)
+            {
+               case LT_SPLINE:
+                  sv = MakeSplinePolygonVertex (&sn,drawOrigX,drawOrigY,n+1,v);
+                  break;
+               case LT_INTSPLINE:
+                  if (cntrlv != NULL) cfree (cntrlv);
+                  sv = MakeIntSplinePolygonVertex (&sn, &intn, &cntrlv,
+                        drawOrigX, drawOrigY, n+1, v);
+                  break;
+            }
             XDrawLines (mainDisplay, drawWindow, drawGC, sv, sn,
                   CoordModeOrigin);
          }
@@ -544,10 +729,16 @@ void ContinuePolygon (OrigX, OrigY)
             XDrawLine (mainDisplay, drawWindow, drawGC, OrigX, OrigY, grid_x,
                   grid_y);
          }
+         sprintf (buf, "%1dx%1d%+1d%+1d", ABS_SIZE(abs(grid_x-OrigX)),
+               ABS_SIZE(abs(grid_y-OrigY)), ABS_X(grid_x), ABS_Y(grid_y));
+         ShowMeasureCursor (grid_x, grid_y, buf, TRUE);
          while (XCheckMaskEvent (mainDisplay, PointerMotionMask, &ev)) ;
       }
       else if (input.type == ButtonPress)
       {
+         sprintf (buf, "%1dx%1d%+1d%+1d", ABS_SIZE(abs(grid_x-OrigX)),
+               ABS_SIZE(abs(grid_y-OrigY)), ABS_X(grid_x), ABS_Y(grid_y));
+         EndShowMeasureCursor (grid_x, grid_y, buf, TRUE);
          button_ev = &(input.xbutton);
 
          end_x = button_ev->x;
@@ -563,7 +754,7 @@ void ContinuePolygon (OrigX, OrigY)
             lastPtPtr = pt_ptr;
             pt_ptr->x = grid_x;
             pt_ptr->y = grid_y;
-            if (curSpline == LT_SPLINE && splineRubberband)
+            if (curSpline != LT_STRAIGHT && splineRubberband)
             {
                if (n >= max_n-3)
                {
@@ -579,20 +770,31 @@ void ContinuePolygon (OrigX, OrigY)
                n++;
                v[n].x = v[0].x;
                v[n].y = v[0].y;
-               sv = MakeSplinePolygonVertex (&sn, drawOrigX, drawOrigY, n+1, v);
+               switch (curSpline)
+               {
+                  case LT_SPLINE:
+                     sv = MakeSplinePolygonVertex (&sn, drawOrigX, drawOrigY,
+                           n+1, v);
+                     break;
+                  case LT_INTSPLINE:
+                     if (cntrlv != NULL) cfree (cntrlv);
+                     sv = MakeIntSplinePolygonVertex (&sn, &intn, &cntrlv,
+                           drawOrigX, drawOrigY, n+1, v);
+                     break;
+               }
                XDrawLines (mainDisplay, drawWindow, drawGC, sv, sn,
                      CoordModeOrigin);
             }
          }
 
          if (num_pts == 2 && closed && button_ev->button == Button1 &&
-               !(curSpline == LT_SPLINE && splineRubberband))
+               !(curSpline != LT_STRAIGHT && splineRubberband))
             XDrawLine (mainDisplay, drawWindow, drawGC, OrigX, OrigY, grid_x,
                   grid_y);
 
          if (grid_x == startPolygonX && grid_y == startPolygonY)
          {
-            if (curSpline == LT_SPLINE && splineRubberband)
+            if (curSpline != LT_STRAIGHT && splineRubberband)
                XDrawLines (mainDisplay, drawWindow, drawGC, sv, sn,
                      CoordModeOrigin);
             else
@@ -612,10 +814,14 @@ void ContinuePolygon (OrigX, OrigY)
          {
             switch(button_ev->button)
             {
-               case Button1 : OrigX = grid_x; OrigY = grid_y; break;
-               case Button2 :
-               case Button3 :
-                  if (curSpline == LT_SPLINE && splineRubberband)
+               case Button1:
+                  OrigX = grid_x; OrigY = grid_y;
+                  sprintf (buf, "0x0%+1d%+1d", ABS_X(grid_x), ABS_Y(grid_y));
+                  StartShowMeasureCursor (grid_x, grid_y, buf, TRUE);
+                  break;
+               case Button2:
+               case Button3:
+                  if (curSpline != LT_STRAIGHT && splineRubberband)
                      XDrawLines (mainDisplay, drawWindow, drawGC, sv, sn,
                            CoordModeOrigin);
                   else
@@ -647,9 +853,15 @@ void ContinuePolygon (OrigX, OrigY)
       }
    }
    XUngrabPointer (mainDisplay, CurrentTime);
+   RestoreStatusStrings ();
+   SetMouseStatus (NULL, NULL, NULL);
    Msg ("");
 
-   if (curSpline == LT_SPLINE && splineRubberband) { cfree (v); cfree (sv); }
+   if (curSpline != LT_STRAIGHT && splineRubberband)
+   {
+      cfree (v); cfree (sv);
+      if (curSpline == LT_INTSPLINE && cntrlv != NULL) cfree (cntrlv);
+   }
 
    switch (num_pts)
    {
@@ -661,7 +873,7 @@ void ContinuePolygon (OrigX, OrigY)
          cfree (lastPtPtr);
          break;
       default:
-         CreatePolygonObj (num_pts);
+         CreatePolygonObj (num_pts, CREATE_RELATIVE);
          RecordNewObjCmd ();
          RedrawAnArea (botObj, topObj->bbox.ltx-GRID_ABS_SIZE(1),
                topObj->bbox.lty-GRID_ABS_SIZE(1),
@@ -695,6 +907,112 @@ void DrawPolygon (input)
    } 
 }
 
+void InputPolygonPts ()
+{
+   char	inbuf[MAXSTRING+1];
+   int	more_polygon=FALSE;
+
+   MakeQuiescent ();
+
+   do
+   {
+      int		len, ok=TRUE, num_pts=0;
+      int		first_x=0, first_y=0, eof=TRUE;
+      struct PtRec	* pt_ptr;
+
+      more_polygon = FALSE;
+      lastPtPtr = NULL;
+      printf ("Please input pairs of points: ");
+      printf ("(terminate with \"<Cntrl>d<CR>\" or \".<CR>\", ");
+      printf ("continue to next polygon with \";<CR>\")\n");
+      printf ("> ");
+      fflush (stdout);
+      while (ok && fgets (inbuf, MAXSTRING, stdin) != NULL)
+      {
+         if (strcmp (inbuf, ";\n") == 0)
+         {
+            eof = FALSE;
+            more_polygon = TRUE;
+            break;
+         }
+         if (strcmp (inbuf, ".\n") == 0) { eof = FALSE; break; }
+         len = strlen (inbuf);
+         if (len > 0)
+         {
+            char * c_ptr=strtok(inbuf," ,\t\n"), * c_ptr1=NULL;
+   
+            if (c_ptr != NULL)
+               c_ptr1 = strtok(NULL," ,\t\n");
+            if (c_ptr1 != NULL)
+               while (strchr (" ,\t\n", *c_ptr1)) c_ptr1++;
+            while (c_ptr != NULL && c_ptr1 != NULL)
+            {
+               num_pts++;
+               pt_ptr = (struct PtRec *)calloc(1,sizeof(struct PtRec));
+               pt_ptr->next = lastPtPtr;
+               if (sscanf (c_ptr, "%d", &pt_ptr->x) != 1 ||
+                     sscanf (c_ptr1, "%d", &pt_ptr->y) != 1)
+               {
+                  ok = FALSE;
+                  Dialog ("Error reading integer for poly points.",
+                        "( <CR> or <ESC> to continue )", inbuf);
+                  break;
+               }
+               if (lastPtPtr == NULL)
+               {
+                  first_x = pt_ptr->x;
+                  first_y = pt_ptr->y;
+               }
+               lastPtPtr = pt_ptr;
+               c_ptr = strtok(NULL," ,\t\n");
+               if (c_ptr != NULL)
+                  c_ptr1 = strtok(NULL," ,\t\n");
+               if (c_ptr1 != NULL)
+                  while (strchr (" ,\t\n", *c_ptr1)) c_ptr1++;
+            }
+            if (c_ptr != NULL)
+            {
+               ok = FALSE;
+               Dialog ("Error reading integer for poly points.",
+                     "( <CR> or <ESC> to continue )", inbuf);
+            }
+         }
+         printf ("> ");
+         fflush (stdout);
+      }
+      printf ("\n");
+      if (eof) rewind (stdin);
+      if (ok && num_pts > 2)
+      {
+         if (lastPtPtr->x != first_x || lastPtPtr->y != first_y)
+         {
+            num_pts++;
+            pt_ptr = (struct PtRec *)calloc(1,sizeof(struct PtRec));
+            pt_ptr->next = lastPtPtr;
+            pt_ptr->x = first_x;
+            pt_ptr->y = first_y;
+            lastPtPtr = pt_ptr;
+         }
+         CreatePolygonObj (num_pts, CREATE_ABSOLUTE);
+         RecordNewObjCmd ();
+         RedrawAnArea (botObj, topObj->bbox.ltx-GRID_ABS_SIZE(1),
+               topObj->bbox.lty-GRID_ABS_SIZE(1),
+               topObj->bbox.rbx+GRID_ABS_SIZE(1),
+               topObj->bbox.rby+GRID_ABS_SIZE(1));
+         SelectTopObj ();
+         SetFileModified (TRUE);
+         justDupped = FALSE;
+      }
+      if (ok && num_pts <= 2)
+         Dialog ("Too few points.", "( <CR> or <ESC> to continue )", inbuf);
+      for ( ; lastPtPtr != NULL; lastPtPtr=pt_ptr)
+      {
+         pt_ptr = lastPtPtr->next;
+         cfree (pt_ptr);
+      }
+   } while (more_polygon);
+}
+
 void SavePolygonObj (FP, ObjPtr)
    FILE			* FP;
    struct ObjRec	* ObjPtr;
@@ -704,29 +1022,38 @@ void SavePolygonObj (FP, ObjPtr)
    struct PolygonRec	* polygon_ptr = ObjPtr->detail.g;
 
    n = polygon_ptr->n;
-   fprintf (FP, "polygon('%s',%1d,[\n\t",
-         colorMenuItems[ObjPtr->color], polygon_ptr->n);
+   if (fprintf (FP, "polygon('%s',%1d,[\n\t",
+         colorMenuItems[ObjPtr->color], polygon_ptr->n) == EOF)
+      writeFileFailed = TRUE;
    for (i = 0, count = 0; i < n-1; i++)
    {
-      fprintf (FP,"%1d,%1d,",polygon_ptr->vlist[i].x,polygon_ptr->vlist[i].y);
+      if (fprintf (FP, "%1d,%1d,", polygon_ptr->vlist[i].x,
+            polygon_ptr->vlist[i].y) == EOF)
+         writeFileFailed = TRUE;
       if (++count == 8)
       {
          count = 0;
-         fprintf (FP, "\n\t");
+         if (fprintf (FP, "\n\t") == EOF) writeFileFailed = TRUE;
       }
    }
-   fprintf (FP, "%1d,%1d],",
-         polygon_ptr->vlist[n-1].x, polygon_ptr->vlist[n-1].y);
+   if (fprintf (FP, "%1d,%1d],",
+         polygon_ptr->vlist[n-1].x, polygon_ptr->vlist[n-1].y) == EOF)
+      writeFileFailed = TRUE;
 
-   fprintf (FP, "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,",
+   if (fprintf (FP, "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,[",
          polygon_ptr->fill, polygon_ptr->width, polygon_ptr->pen,
          polygon_ptr->curved, ObjPtr->id, polygon_ptr->dash, ObjPtr->rotation,
-         ObjPtr->locked);
+         ObjPtr->locked) == EOF)
+      writeFileFailed = TRUE;
+   if (fprintf (FP, "\n    \"") == EOF) writeFileFailed = TRUE;
+   SaveSmoothHinge (FP, polygon_ptr->curved, polygon_ptr->n,
+         polygon_ptr->smooth);
+   if (fprintf (FP, "\",") == EOF) writeFileFailed = TRUE;
    SaveAttrs (FP, ObjPtr->lattr);
-   fprintf (FP, ")");
+   if (fprintf (FP, ")") == EOF) writeFileFailed = TRUE;
 }
 
-#define GETVALUE(val,name) ScanValue("%d", (char *) &(val), name, "polygon")
+#define GETVALUE(val,name) ScanValue("%d", &(val), name, "polygon")
 
 void ReadPolygonObj (FP, Inbuf, ObjPtr)
    FILE			* FP;
@@ -740,6 +1067,7 @@ void ReadPolygonObj (FP, Inbuf, ObjPtr)
    int			num_pts, ltx=0, lty=0, rbx=0, rby=0, x, y, id=0;
    int			fill, width, pen, w, new_alloc, locked=FALSE;
    int			curved, dash, initialized, rotation, count;
+   char			* smooth=NULL;
 
    *ObjPtr = NULL;
 
@@ -769,11 +1097,15 @@ void ReadPolygonObj (FP, Inbuf, ObjPtr)
    {
       v = (XPoint *) calloc (5, sizeof(XPoint));
       if (v == NULL) fprintf (stderr, "Can not calloc().\n");
+      smooth = (char *) calloc (5, sizeof(char));
+      if (smooth == NULL) fprintf (stderr, "Can not calloc().\n");
    }
    else
    {
       v = (XPoint *) calloc (num_pts+1, sizeof(XPoint));
       if (v == NULL) fprintf (stderr, "Can not calloc().\n");
+      smooth = (char *) calloc (num_pts+1, sizeof(char));
+      if (smooth == NULL) fprintf (stderr, "Can not calloc().\n");
    }
 
    initialized = FALSE;
@@ -850,9 +1182,9 @@ void ReadPolygonObj (FP, Inbuf, ObjPtr)
             fprintf (stderr, "%s\n", msg);
          else
             Msg (msg);
-         v[2].x = v[1].x = v[0].x;
-         v[2].y = v[1].y = v[0].y;
-         num_pts = 3;
+         v[3].x = v[2].x = v[1].x = v[0].x;
+         v[3].y = v[2].y = v[1].y = v[0].y;
+         num_pts = 4;
          break;
       case 2:
          sprintf (msg, "%s [%1d,%1d,%1d,%1d] converted to 3-point polygon.",
@@ -861,9 +1193,20 @@ void ReadPolygonObj (FP, Inbuf, ObjPtr)
             fprintf (stderr, "%s\n", msg);
          else
             Msg (msg);
-         v[2].x = v[0].x;
-         v[2].y = v[0].y;
-         num_pts = 3;
+         v[3].x = v[2].x = v[0].x;
+         v[3].y = v[2].y = v[0].y;
+         num_pts = 4;
+         break;
+      case 3:
+         sprintf (msg, "%s [%1d,%1d,%1d,%1d] converted to 3-point polygon.",
+               "2-point polygon", v[0].x, v[0].y, v[1].x, v[1].y);
+         if (PRTGIF)
+            fprintf (stderr, "%s\n", msg);
+         else
+            Msg (msg);
+         v[3].x = v[2].x = v[0].x;
+         v[3].y = v[2].y = v[0].y;
+         num_pts = 4;
          break;
    }
 
@@ -1004,6 +1347,28 @@ void ReadPolygonObj (FP, Inbuf, ObjPtr)
    }
 
    if (fileVersion <= 16 && width <= 6) width = origWidthOfLine[width];
+
+   if (fileVersion <= 25 && curved > 1) curved = 0;
+   if (curved == LT_INTSPLINE && smooth != NULL)
+   {
+      cfree (smooth);
+      smooth = NULL;
+   }
+   if (fileVersion <= 30)
+   {
+      switch (curved)
+      {
+         case LT_STRAIGHT:
+            for (i=0; i < num_pts; i++) smooth[i] = FALSE;
+            break;
+         case LT_SPLINE:
+            for (i=0; i < num_pts; i++) smooth[i] = TRUE;
+            break;
+      }
+   }
+   else if (!ReadSmoothHinge (FP, curved, num_pts, smooth))
+      return;
+
    fill = UpgradePenFill (fill);
    pen = UpgradePenFill (pen);
 
@@ -1014,7 +1379,9 @@ void ReadPolygonObj (FP, Inbuf, ObjPtr)
    polygon_ptr->dash = dash;
 
    polygon_ptr->vlist = v;
+   polygon_ptr->smooth = smooth;
    polygon_ptr->svlist = NULL;
+   polygon_ptr->intvlist = NULL;
 
    (*ObjPtr)->x = ltx;
    (*ObjPtr)->y = lty;
@@ -1035,13 +1402,19 @@ void ReadPolygonObj (FP, Inbuf, ObjPtr)
    (*ObjPtr)->bbox.rby = rby + w;
    (*ObjPtr)->detail.g = polygon_ptr;
    AdjObjSplineVs (*ObjPtr);
+   if (polygon_ptr->curved != LT_INTSPLINE)
+      UpdPolyBBox (*ObjPtr, polygon_ptr->n, polygon_ptr->vlist);
+   else
+      UpdPolyBBox (*ObjPtr, polygon_ptr->intn, polygon_ptr->intvlist);
 }
 
 void FreePolygonObj (ObjPtr)
    struct ObjRec	* ObjPtr;
 {
+   if (ObjPtr->detail.g->intvlist != NULL) cfree (ObjPtr->detail.g->intvlist);
    if (ObjPtr->detail.g->svlist != NULL) cfree (ObjPtr->detail.g->svlist);
    cfree (ObjPtr->detail.g->vlist);
+   if (ObjPtr->detail.g->smooth != NULL) cfree (ObjPtr->detail.g->smooth);
    cfree (ObjPtr->detail.g);
    cfree (ObjPtr);
 }
