@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/raster.c,v 2.42.1.4 1993/03/16 06:52:49 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/raster.c,v 2.53 1994/04/19 05:04:28 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -38,6 +39,9 @@ static char RCSid[] =
 #include "font.e"
 #include "pattern.e"
 #include "poly.e"
+#ifndef _NO_EXTERN
+#include "raster.e"
+#endif
 #include "setup.e"
 
 #include "xbm/arrow.xbm"
@@ -111,6 +115,7 @@ static char RCSid[] =
 
 #include "xbm/lt0.xbm"
 #include "xbm/lt1.xbm"
+#include "xbm/lt2.xbm"
 
 #include "xbm/ls0.xbm"
 #include "xbm/ls1.xbm"
@@ -137,6 +142,7 @@ static char RCSid[] =
 
 #include "xbm/lt0s.xbm"
 #include "xbm/lt1s.xbm"
+#include "xbm/lt2s.xbm"
 
 #include "xbm/ls0s.xbm"
 #include "xbm/ls1s.xbm"
@@ -157,6 +163,8 @@ static char RCSid[] =
 #include "xbm/latex.xbm"
 #include "xbm/psfile.xbm"
 #include "xbm/xbm.xbm"
+#include "xbm/ascii.xbm"
+#include "xbm/epsi.xbm"
 
 #include "xbm/file.xbm"
 #include "xbm/landscape.xbm"
@@ -176,6 +184,12 @@ static char RCSid[] =
 #include "xbm/rotate_180.xbm"
 #include "xbm/rotate_270.xbm"
 
+#include "xbm/btn1.xbm"
+#include "xbm/btn2.xbm"
+#include "xbm/btn3.xbm"
+
+extern int	atoi ARGS_DECL((char *));
+
 GC	rasterGC;
 
 Pixmap	patPixmap[MAXPATTERNS+1];
@@ -214,6 +228,8 @@ Pixmap	shortLineTypePixmap[MAXLINETYPES];
 Pixmap	shortLineStylePixmap[MAXLINESTYLES];
 Pixmap	shortDashPixmap[MAXDASHES];
 
+Pixmap	statusBtnPixmap[MAX_STATUS_BTNS];
+
 int	maxLineWidths = MAXLINEWIDTHS;
 
 int	pat_w[MAXPATTERNS+1], pat_h[MAXPATTERNS+1];
@@ -401,8 +417,8 @@ void InitPattern ()
    values.background = xorZero;
    values.function = GXxor;
    values.fill_style = FillSolid;
-   revDefaultGC = XCreateGC (mainDisplay, mainWindow,
-         GCForeground | GCBackground | GCFunction | GCFillStyle, &values);
+   revDefaultGC = XCreateGC (mainDisplay, mainWindow, GCForeground |
+         GCBackground | GCFunction | GCFillStyle | GCFont, &values);
 
    pat_w[0] = pat0_width; pat_h[0] = pat0_height; patData[0] = pat0_bits;
    pat_w[1] = pat1_width; pat_h[1] = pat1_height; patData[1] = pat1_bits;
@@ -508,6 +524,8 @@ void InitPattern ()
          lt0_bits, lt0_width, lt0_height);
    lineTypePixmap[1] = XCreateBitmapFromData (mainDisplay, mainWindow,
          lt1_bits, lt1_width, lt1_height);
+   lineTypePixmap[2] = XCreateBitmapFromData (mainDisplay, mainWindow,
+         lt2_bits, lt2_width, lt2_height);
 
    dashPixmap[0] = XCreateBitmapFromData (mainDisplay, mainWindow,
          ld0_bits, ld0_width, ld0_height);
@@ -588,11 +606,17 @@ void InitPattern ()
          mainWindow, psfile_bits, psfile_width, psfile_height);
    whereToPrintPixmap[XBM_FILE] = XCreateBitmapFromData (mainDisplay,
          mainWindow, xbm_bits, xbm_width, xbm_height);
+   whereToPrintPixmap[TEXT_FILE] = XCreateBitmapFromData (mainDisplay,
+         mainWindow, ascii_bits, ascii_width, ascii_height);
+   whereToPrintPixmap[EPSI_FILE] = XCreateBitmapFromData (mainDisplay,
+         mainWindow, epsi_bits, epsi_width, epsi_height);
 
    shortLineTypePixmap[0] = XCreateBitmapFromData (mainDisplay, mainWindow,
          lt0s_bits, lt0s_width, lt0s_height);
    shortLineTypePixmap[1] = XCreateBitmapFromData (mainDisplay, mainWindow,
          lt1s_bits, lt1s_width, lt1s_height);
+   shortLineTypePixmap[2] = XCreateBitmapFromData (mainDisplay, mainWindow,
+         lt2s_bits, lt2s_width, lt2s_height);
 
    shortDashPixmap[0] = XCreateBitmapFromData (mainDisplay, mainWindow,
          ld0s_bits, ld0s_width, ld0s_height);
@@ -649,6 +673,13 @@ void InitPattern ()
    choicePixmap[9] = XCreateBitmapFromData (mainDisplay, mainWindow,
          vmode_bits, vmode_width, vmode_height);
 
+   statusBtnPixmap[0] = XCreateBitmapFromData (mainDisplay, mainWindow,
+         btn1_bits, btn1_width, btn1_height);
+   statusBtnPixmap[1] = XCreateBitmapFromData (mainDisplay, mainWindow,
+         btn2_bits, btn2_width, btn2_height);
+   statusBtnPixmap[2] = XCreateBitmapFromData (mainDisplay, mainWindow,
+         btn3_bits, btn3_width, btn3_height);
+
    InitLines();
 }
 
@@ -724,6 +755,9 @@ void CleanUpRasters ()
    for (i = 0; i < MAXDASHES; i++)
       XFreePixmap (mainDisplay, shortDashPixmap[i]);
 
+   for (i = 0; i < MAX_STATUS_BTNS; i++)
+      XFreePixmap (mainDisplay, statusBtnPixmap[i]);
+
    if (newLineWidthAllocated)
    {
       for (i = 0; i < maxLineWidths; i++)
