@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/rcbox.c,v 2.38 1993/06/03 07:18:58 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/rcbox.c,v 2.48 1994/03/21 04:56:19 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -32,17 +33,24 @@ static char RCSid[] =
 #include "const.h"
 #include "types.h"
 
+#include "attr.e"
+#include "box.e"
 #include "cmd.e"
 #include "color.e"
 #include "cursor.e"
 #include "file.e"
 #include "grid.e"
 #include "mainloop.e"
+#include "msg.e"
 #include "obj.e"
 #include "pattern.e"
 #include "poly.e"
 #include "raster.e"
+#ifndef _NO_EXTERN
+#include "rcbox.e"
+#endif
 #include "ruler.e"
+#include "select.e"
 #include "setup.e"
 #include "xpixmap.e"
 
@@ -213,7 +221,7 @@ void DumpRCBoxPath (FP, ObjPtr, ltx, lty, rbx, rby, r, width, pen, dash)
       default:
          if (colorDump)
          {
-            fprintf (FP, "flattenpath strokepath clip\n");
+            fprintf (FP, "flattenpath strokepath clip newpath\n");
             DumpPatFill (FP, pen, 8, ObjPtr->bbox, "   ");
          }
          else
@@ -292,7 +300,7 @@ void DumpRCBoxObj (FP, ObjPtr)
          DumpRCBoxPSPath (FP, ltx, lty, rbx, rby, r, "   ", "      ");
          if (colorDump)
          {
-            fprintf (FP, "   closepath eoclip\n");
+            fprintf (FP, "   closepath eoclip newpath\n");
             DumpPatFill (FP, fill, 8, ObjPtr->bbox, "   ");
          }
          else
@@ -452,6 +460,7 @@ void ContinueRCBox (OrigX, OrigY)
    int 			end_x, end_y, grid_x, grid_y, saved_x, saved_y;
    int 			done = FALSE;
    int			xor_pixel, radius=ZOOMED_SIZE(rcbRadius);
+   char			buf[80];
    XGCValues		values;
    XEvent		input, ev;
    XMotionEvent		* motion_ev;
@@ -474,6 +483,8 @@ void ContinueRCBox (OrigX, OrigY)
 
    saved_x = grid_x = OrigX;
    saved_y = grid_y = OrigY; 
+   sprintf (buf, "0x0%+1d%+1d", ABS_X(grid_x), ABS_Y(grid_y));
+   StartShowMeasureCursor (grid_x, grid_y, buf, TRUE);
    BeginIntervalRulers (grid_x, grid_y, grid_x, grid_y);
    XGrabPointer (mainDisplay, drawWindow, FALSE,
          PointerMotionMask | ButtonReleaseMask,
@@ -491,6 +502,9 @@ void ContinueRCBox (OrigX, OrigY)
          XUngrabPointer (mainDisplay, CurrentTime);
          MyRCBox (drawWindow,drawGC,OrigX,OrigY,saved_x,saved_y,radius);
          EndIntervalRulers (grid_x, grid_y);
+         sprintf (buf, "%1dx%1d%+1d%+1d", ABS_SIZE(abs(saved_x-OrigX)),
+               ABS_SIZE(abs(saved_y-OrigY)), ABS_X(saved_x), ABS_Y(saved_y));
+         EndShowMeasureCursor (saved_x, saved_y, buf, TRUE);
          done = TRUE;
       }
       else if (input.type == MotionNotify)
@@ -499,13 +513,40 @@ void ContinueRCBox (OrigX, OrigY)
          end_x = motion_ev->x;
          end_y = motion_ev->y;
          GridXY (end_x, end_y, &grid_x, &grid_y);
+         if (motion_ev->state & (ShiftMask | ControlMask))
+         {
+            int w, h, pos_w=TRUE, pos_h=TRUE;
+
+            w = grid_x - OrigX;
+            h = grid_y - OrigY;
+            if (w < 0)
+            {
+               w = (-w);
+               pos_w = FALSE;
+            }
+            if (h < 0)
+            {
+               h = (-h);
+               pos_h = FALSE;
+            }
+            if (w > h)
+               grid_x = (pos_w ? (OrigX+h) : (OrigX-h));
+            else
+               grid_y = (pos_h ? (OrigY+w) : (OrigY-w));
+         }
          if (grid_x != saved_x || grid_y != saved_y)
          {
+            sprintf (buf, "%1dx%1d%+1d%+1d", ABS_SIZE(abs(saved_x-OrigX)),
+                  ABS_SIZE(abs(saved_y-OrigY)), ABS_X(saved_x), ABS_Y(saved_y));
+            ShowMeasureCursor (saved_x, saved_y, buf, TRUE);
             MyRCBox (drawWindow,drawGC,OrigX,OrigY,saved_x,saved_y,radius);
             saved_x = grid_x;
             saved_y = grid_y;
             SetRCBoxVertex (OrigX, OrigY, saved_x, saved_y, radius);
             MyRCBox (drawWindow,drawGC,OrigX,OrigY,saved_x,saved_y,radius);
+            sprintf (buf, "%1dx%1d%+1d%+1d", ABS_SIZE(abs(saved_x-OrigX)),
+                  ABS_SIZE(abs(saved_y-OrigY)), ABS_X(saved_x), ABS_Y(saved_y));
+            ShowMeasureCursor (saved_x, saved_y, buf, TRUE);
          }
          DrawIntervalRulers (OrigX, OrigY, grid_x, grid_y);
          while (XCheckMaskEvent (mainDisplay, PointerMotionMask, &ev)) ;
@@ -543,19 +584,21 @@ void SaveRCBoxObj (FP, ObjPtr)
    FILE			* FP;
    struct ObjRec	* ObjPtr;
 {
-   fprintf (FP, "rcbox('%s',", colorMenuItems[ObjPtr->color]);
-   fprintf (FP, "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,",
+   if (fprintf (FP, "rcbox('%s',", colorMenuItems[ObjPtr->color]) == EOF)
+      writeFileFailed = TRUE;
+   if (fprintf (FP, "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,",
          ObjPtr->obbox.ltx,
          ObjPtr->obbox.lty, ObjPtr->obbox.rbx, ObjPtr->obbox.rby,
          ObjPtr->detail.rcb->fill, ObjPtr->detail.rcb->width,
          ObjPtr->detail.rcb->pen, ObjPtr->detail.rcb->dash,
          ObjPtr->detail.rcb->radius, ObjPtr->id, ObjPtr->rotation,
-         ObjPtr->locked);
+         ObjPtr->locked) == EOF)
+      writeFileFailed = TRUE;
    SaveAttrs (FP, ObjPtr->lattr);
-   fprintf (FP, ")");
+   if (fprintf (FP, ")") == EOF) writeFileFailed = TRUE;
 }
 
-#define GETVALUE(val,name) ScanValue("%d", (char *) &(val), name, "rcbox")
+#define GETVALUE(val,name) ScanValue("%d", &(val), name, "rcbox")
 
 void ReadRCBoxObj (Inbuf, ObjPtr)
    char			* Inbuf;
