@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,9 +25,10 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/scroll.c,v 2.36 1992/10/21 04:45:47 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/scroll.c,v 2.41 1994/04/04 00:49:06 william Exp $";
 #endif
 
+#include <stdio.h>
 #include <X11/Xlib.h>
 #include "const.h"
 #include "types.h"
@@ -36,10 +38,14 @@ static char RCSid[] =
 #include "drawing.e"
 #include "dup.e"
 #include "grid.e"
+#include "mainloop.e"
 #include "msg.e"
 #include "obj.e"
 #include "raster.e"
 #include "ruler.e"
+#ifndef _NO_EXTERN
+#include "scroll.e"
+#endif
 #include "setup.e"
 #include "text.e"
 
@@ -315,7 +321,7 @@ void ScrollUp (button_ev)
       UpdDrawWinBBox ();
       AdjSplineVs ();
       AdjustCurText (0, adjustment);
-      RedrawVRuler ();
+      RedrawRulers ();
       ClearAndRedrawDrawWindow ();
    }
 }
@@ -361,7 +367,7 @@ void ScrollDown (button_ev)
       UpdDrawWinBBox ();
       AdjSplineVs ();
       AdjustCurText (0, -adjustment);
-      RedrawVRuler ();
+      RedrawRulers ();
       ClearAndRedrawDrawWindow ();
    }
 }
@@ -494,7 +500,7 @@ void VSBarHandler (button_ev)
             UpdDrawWinBBox ();
             AdjSplineVs ();
             AdjustCurText (0, ZOOMED_SIZE(adjustment));
-            RedrawVRuler ();
+            RedrawRulers ();
             ClearAndRedrawDrawWindow ();
          }
          else
@@ -526,7 +532,7 @@ void VSBarHandler (button_ev)
             UpdDrawWinBBox ();
             AdjSplineVs ();
             AdjustCurText (0, ZOOMED_SIZE(adjustment));
-            RedrawVRuler ();
+            RedrawRulers ();
             ClearAndRedrawDrawWindow ();
          }
          else
@@ -819,7 +825,11 @@ void ScrollEventHandler (input)
          RedrawVScrollWindow ();
          return;
       }
-
+      else if (input->type == EnterNotify)
+      {
+         SetMouseStatus ("scroll down", "scroll vertically", "scroll up");
+         return;
+      }
       Msg (""); 
       VSBarHandler (&(input->xbutton));
       return;
@@ -832,7 +842,11 @@ void ScrollEventHandler (input)
          RedrawHScrollWindow ();
          return;
       }
-
+      else if (input->type == EnterNotify)
+      {
+         SetMouseStatus ("scroll right", "scroll horizontally", "scroll left");
+         return;
+      }
       Msg (""); 
       HSBarHandler (&(input->xbutton));
       return;
