@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/setup.c,v 2.100.2.21 1993/05/26 04:29:21 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/setup.c,v 2.119 1994/05/01 23:42:23 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -57,6 +58,9 @@ static char RCSid[] =
 #include "ruler.e"
 #include "scroll.e"
 #include "select.e"
+#ifndef _NO_EXTERN
+#include "setup.e"
+#endif
 #include "shortcut.e"
 #include "spline.e"
 #include "stk.e"
@@ -66,8 +70,14 @@ static char RCSid[] =
 
 #include "tgificon.xbm"
 
-extern	char	* getenv ();
-extern	double	atof ();
+
+extern char	* getenv ARGS_DECL((char *));
+#ifdef __hpux
+extern double	atof ARGS_DECL((const char *));
+#else
+extern double	atof ARGS_DECL((char *));
+#endif
+extern int	atoi ARGS_DECL((char *));
 
 #define MENU_IMAGE_W 64
 #define MENU_IMAGE_H 20
@@ -86,11 +96,16 @@ extern	double	atof ();
 #define TITLE_WINDOW_W (HSBAR_W+SCROLLBAR_W+2*BRDR_W)
 #define TITLE_WINDOW_H (MENU_IMAGE_H*2)
 #define TOOL_WINDOW_W (TITLE_WINDOW_W+2*BRDR_W)
-#define TOOL_WINDOW_H (TITLE_WINDOW_H+CHOICE_WINDOW_H+VSBAR_H+SCROLLBAR_W+8*BRDR_W)
+#define TOOL_WINDOW_H \
+      (TITLE_WINDOW_H+CHOICE_WINDOW_H+VSBAR_H+SCROLLBAR_W+8*BRDR_W)
 #define MSG_WINDOW_W (TITLE_WINDOW_W-CHOICE_WINDOW_W-2*BRDR_W)
 #define MSG_WINDOW_H (CHOICE_WINDOW_H)
 #define ICON_WINDOW_W 64
 #define ICON_WINDOW_H 64
+#define MENUBAR_WINDOW_W (TITLE_WINDOW_W)
+#define MENUBAR_WINDOW_H 20
+#define STATUS_WINDOW_W (TITLE_WINDOW_W)
+#define STATUS_WINDOW_H 20
 
 unsigned int	mainWinW = 0;
 unsigned int	mainWinH = 0;
@@ -111,6 +126,17 @@ int	titleWindowW = TITLE_WINDOW_W;
 int	titleWindowH = TITLE_WINDOW_H;
 int	iconWindowW = ICON_WINDOW_W;
 int	iconWindowH = ICON_WINDOW_H;
+int	menubarWindowW = MENUBAR_WINDOW_W;
+int	menubarWindowH = MENUBAR_WINDOW_H;
+int	statusWindowW = STATUS_WINDOW_W;
+int	statusWindowH = STATUS_WINDOW_H;
+int	statusSubWindowW[MAX_STATUS_BTNS];
+int	statusSubWindowH[MAX_STATUS_BTNS];
+
+int	initialMenubarWindowH=MENUBAR_WINDOW_H;
+
+static int	statusSubWindowX[MAX_STATUS_BTNS];
+static int	statusSubWindowY[MAX_STATUS_BTNS];
 
 Display		* mainDisplay=NULL;
 Colormap	mainColormap;
@@ -130,6 +156,8 @@ Window	vRuleWindow=None;
 Window	hRuleWindow=None;
 Window	iconWindow=None;
 Window	iconBaseWindow=None;
+Window	menubarWindow=None;
+Window	statusWindow=None;
 
 #ifndef A4PAPER
 int	onePageWidth = (85*PIX_PER_INCH)/10;
@@ -171,12 +199,18 @@ char	* * symPath;
 int	initDrawWinW, initDrawWinH;
 
 short	handleSize=3;
+int	resizeTextOnStretch=FALSE;
 
 Window	dummyWindow1, dummyWindow2;
 
-static Atom	protocolAtom;
+Window	statusSubWindow[MAX_STATUS_BTNS];
 
-static int	canvasWindowOnly;
+int	noMenubar=FALSE;
+int	noStatusWindow=FALSE;
+
+static Atom	protocolAtom=(Atom)0;
+
+static int	canvasWindowOnly=FALSE;
 
 void UpdDrawWinWH ()
 {
@@ -192,6 +226,35 @@ void UpdDrawWinBBox ()
    drawWinBBox.rby = drawOrigY + drawWinH-1;
 }
 
+#include "xbm/btn1.xbm"
+
+static
+void CalcStatusSubWinGeom ()
+{
+   register int	i;
+   int		left=0, w=(int)(statusWindowW/3), win_y, right=0, win_h;
+
+   if (defaultFontHeight+(brdrW<<1)+2 > btn1_height)
+      win_y = 1;
+   else
+      win_y = (statusWindowH-defaultFontHeight-(brdrW<<1)-2)>>1;
+   win_h = statusWindowH-(win_y<<1)-2;
+   for (i=0; i < MAX_STATUS_BTNS; i++)
+   {
+      int	win_x, win_w;
+
+      right += w;
+      if (right >= statusWindowW) right = statusWindowW-1;
+      win_x = left+(brdrW<<2)+btn1_width+2;
+      win_w = right-left-(brdrW*6)-btn1_width-2;
+      statusSubWindowX[i] = win_x;
+      statusSubWindowY[i] = win_y;
+      statusSubWindowW[i] = win_w;
+      statusSubWindowH[i] = win_h;
+      left += w;
+   }
+}
+
 static
 void InitWinSizes ()
 {
@@ -201,9 +264,15 @@ void InitWinSizes ()
    hSBarW = drawWinW+rulerW+2*brdrW;
    vSBarH = drawWinH+rulerW+2*brdrW;
    titleWindowW = hSBarW+scrollBarW+2*brdrW;
+   menubarWindowW = statusWindowW = titleWindowW;
+   CalcMenubarWindowHeight ();
    msgWindowW = titleWindowW-choiceWindowW-2*brdrW;
    mainWinW = titleWindowW+2*brdrW;
-   mainWinH = titleWindowH+choiceWindowH+vSBarH+scrollBarW+8*brdrW;
+   mainWinH =
+         (noMenubar ? 0 : menubarWindowH+(brdrW<<1)) +
+         (noStatusWindow ? 0 : statusWindowH+(brdrW<<1)) +
+         titleWindowH+choiceWindowH+vSBarH+scrollBarW+8*brdrW;
+   CalcStatusSubWinGeom ();
 }
 
 static
@@ -211,14 +280,19 @@ void InverseInitWinSizes ()
    /* derive other win sizes from mainWinW and mainWinH */
 {
    titleWindowW = mainWinW-2*brdrW;
+   menubarWindowW = statusWindowW = titleWindowW;
+   CalcMenubarWindowHeight ();
    msgWindowW = titleWindowW-choiceWindowW-2*brdrW;
-   vSBarH = mainWinH-titleWindowH-choiceWindowH-scrollBarW-8*brdrW;
+   vSBarH = mainWinH - titleWindowH - choiceWindowH - scrollBarW - 8*brdrW -
+         (noMenubar ? 0 : menubarWindowH+(brdrW<<1)) -
+         (noStatusWindow ? 0 : statusWindowH+(brdrW<<1));
    hSBarW = titleWindowW-scrollBarW-2*brdrW;
    drawWinH = initDrawWinH = vSBarH-rulerW-2*brdrW;
    drawWinW = initDrawWinW = hSBarW-rulerW-2*brdrW;
+   CalcMenubarWindowHeight ();
+   CalcStatusSubWinGeom ();
 }
 
-static
 void ComputeMainWinXY (MainWinX, MainWinY)
    int	* MainWinX, * MainWinY;
 {
@@ -244,31 +318,148 @@ void ComputeMainWinXY (MainWinX, MainWinY)
    }
 }
 
+void Reconfigure (Forced)
+   int	Forced;
+{
+   register int	i;
+   Window	root_win;
+   int		win_x, win_y, main_win_x, main_win_y;
+   unsigned int	win_w, win_h, win_brdr_w, win_d;
+   XEvent	ev;
+
+   ComputeMainWinXY (&main_win_x, &main_win_y);
+   XGetGeometry (mainDisplay, mainWindow, &root_win, &win_x, &win_y, &win_w,
+         &win_h, &win_brdr_w, &win_d);
+   if (!Forced && win_w == mainWinW && win_h == mainWinH)
+   {
+      SaveStackingOrder ();
+      if (pinnedMainMenu)
+         MoveMainMenuWindow (main_win_x, main_win_y);
+      else
+         SaveMainWinPosition (main_win_x, main_win_y);
+      for (i = 0; i < numExtraWins; i++)
+         if (extraWinInfo[i].mapped && extraWinInfo[i].raise &&
+               extraWinInfo[i].window != None)
+            MoveSubMenuWindow (extraWinInfo[i].window);
+      for (i = 0; i < numStacking; i++)
+         XMapRaised (mainDisplay, stackingWins[i]);
+
+      while (XCheckWindowEvent (mainDisplay, mainWindow,
+            VisibilityChangeMask | StructureNotifyMask, &ev)) ;
+
+      XFlush (mainDisplay);
+      return;
+   }
+   if (!Forced && !iconWindowShown)
+   {
+      SaveStackingOrder ();
+      for (i = 0; i < numStacking; i++)
+         XMapRaised (mainDisplay, stackingWins[i]);
+   }
+
+   mainWinW = win_w;
+   mainWinH = win_h;
+
+   menubarWindowW = mainWinW - (brdrW<<1);
+   CalcMenubarWindowHeight ();
+   if (canvasWindowOnly)
+   {
+      initDrawWinW = mainWinW-2*brdrW;
+      initDrawWinH = mainWinH-2*brdrW;
+   }
+   else
+   {
+      initDrawWinW = mainWinW - rulerW - scrollBarW - 6*brdrW;
+      initDrawWinH = mainWinH - titleWindowH -
+            (noMenubar ? 0 : menubarWindowH+(brdrW<<1)) -
+            (noStatusWindow ? 0 : statusWindowH+(brdrW<<1)) -
+            choiceWindowH - rulerW - scrollBarW - 10*brdrW;
+   }
+   drawWinW = initDrawWinW;
+   drawWinH = initDrawWinH;
+   hSBarW = initDrawWinW + rulerW + 2*brdrW;
+   vSBarH = initDrawWinH + rulerW + 2*brdrW;
+   titleWindowW = hSBarW + scrollBarW + 2*brdrW;
+   menubarWindowW = statusWindowW = titleWindowW;
+   CalcMenubarWindowHeight ();
+   msgWindowW = titleWindowW - choiceWindowW - 2*brdrW;
+   CalcStatusSubWinGeom ();
+   if (canvasWindowOnly)
+      XResizeWindow (mainDisplay, drawWindow, drawWinW, drawWinH);
+   else
+   {
+      int	cur_y=0;
+
+      XResizeWindow (mainDisplay, titleWindow, titleWindowW, titleWindowH);
+      cur_y += titleWindowH + (brdrW<<1);
+      if (!noMenubar)
+      {
+         XResizeWindow (mainDisplay, menubarWindow, menubarWindowW,
+               menubarWindowH);
+         cur_y += menubarWindowH + (brdrW<<1);
+      }
+      XMoveResizeWindow (mainDisplay, msgWindow, 0, cur_y,
+            msgWindowW, msgWindowH);
+      XMoveWindow (mainDisplay, choiceWindow, msgWindowW+2*brdrW, cur_y);
+      cur_y += choiceWindowH + (brdrW<<1);
+      XMoveResizeWindow (mainDisplay, dummyWindow1, 0, cur_y,
+            drawWinW, rulerW);
+      XMoveResizeWindow (mainDisplay, hRuleWindow, rulerW+(brdrW<<1), cur_y,
+            drawWinW, rulerW);
+      XMoveResizeWindow (mainDisplay, vSBarWindow, drawWinW+rulerW+4*brdrW,
+            cur_y, scrollBarW, vSBarH);
+      cur_y += rulerW + (brdrW<<1);
+      XMoveResizeWindow (mainDisplay, vRuleWindow, 0, cur_y,
+            rulerW, drawWinH);
+      XMoveResizeWindow (mainDisplay, drawWindow, rulerW+(brdrW<<1), cur_y,
+            drawWinW, drawWinH);
+      cur_y += drawWinH + (brdrW<<1);
+      XMoveResizeWindow (mainDisplay, hSBarWindow, 0, cur_y, hSBarW,
+            scrollBarW);
+      XMoveWindow (mainDisplay, dummyWindow2, drawWinW+rulerW+4*brdrW,
+            cur_y);
+      cur_y += scrollBarW + (brdrW<<1);
+      if (!noStatusWindow)
+      {
+         XMoveResizeWindow (mainDisplay, statusWindow, 0, cur_y,
+               statusWindowW, statusWindowH);
+         for (i=0; i < MAX_STATUS_BTNS; i++)
+            XMoveResizeWindow (mainDisplay, statusSubWindow[i],
+                  statusSubWindowX[i], statusSubWindowY[i],
+                  statusSubWindowW[i], statusSubWindowH[i]);
+      }
+   }
+   UpdDrawWinWH ();
+   UpdDrawWinBBox ();
+   UpdScrollWinWH ();
+   SetDefaultDrawWinClipRecs ();
+   SaveMainWinPosition (main_win_x, main_win_y);
+}
+
 int mainWinEventHandler (input)
    XEvent	* input;
 {
    register int	i;
-   Window	root_win;
-   int		win_x, win_y, main_win_x, main_win_y, configure = FALSE;
-   unsigned int	win_w, win_h, win_brdr_w, win_d;
+   int		configure = FALSE;
    char		* c_ptr, msg[MAXSTRING];
-   XEvent	ev;
 
    if (input->type == UnmapNotify)
       Iconify ();
    else if (input->type == MapNotify)
       UnIconify ();
    else if (input->type == ConfigureNotify)
-   {
-      if (iconWindowCreated && iconWindowShown) UnIconify ();
       configure = TRUE;
-   }
    else if (input->type == VisibilityNotify &&
-         input->xvisibility.state == VisibilityUnobscured && !iconWindowShown)
+         input->xvisibility.state == VisibilityUnobscured)
    {
-      SaveStackingOrder ();
-      for (i = 0; i < numStacking; i++)
-         XMapRaised (mainDisplay, stackingWins[i]);
+      if (iconWindowShown)
+         UnIconify ();
+      else
+      {
+         SaveStackingOrder ();
+         for (i = 0; i < numStacking; i++)
+            XMapRaised (mainDisplay, stackingWins[i]);
+      }
    }
    else if (input->type == ClientMessage)
    {
@@ -286,82 +477,7 @@ int mainWinEventHandler (input)
       XFree ((void *)c_ptr);
    }
 
-   if (configure)
-   {
-      ComputeMainWinXY (&main_win_x, &main_win_y);
-      XGetGeometry (mainDisplay, mainWindow, &root_win, &win_x, &win_y, &win_w,
-            &win_h, &win_brdr_w, &win_d);
-      if (win_w == mainWinW && win_h == mainWinH)
-      {
-         SaveStackingOrder ();
-         if (pinnedMainMenu)
-            MoveMainMenuWindow (main_win_x, main_win_y);
-         else
-            SaveMainWinPosition (main_win_x, main_win_y);
-         for (i = 0; i < numExtraWins; i++)
-            if (extraWinInfo[i].mapped && extraWinInfo[i].raise &&
-                  extraWinInfo[i].window != None)
-               MoveSubMenuWindow (extraWinInfo[i].window);
-         for (i = 0; i < numStacking; i++)
-            XMapRaised (mainDisplay, stackingWins[i]);
-
-         while (XCheckWindowEvent (mainDisplay, mainWindow,
-               VisibilityChangeMask | StructureNotifyMask, &ev)) ;
-
-         XFlush (mainDisplay);
-         return (INVALID);
-      }
-      if (!iconWindowShown)
-      {
-         SaveStackingOrder ();
-         for (i = 0; i < numStacking; i++)
-            XMapRaised (mainDisplay, stackingWins[i]);
-      }
-
-      mainWinW = win_w;
-      mainWinH = win_h;
-      if (canvasWindowOnly)
-      {
-         initDrawWinW = mainWinW-2*brdrW;
-         initDrawWinH = mainWinH-2*brdrW;
-      }
-      else
-      {
-         initDrawWinW = mainWinW - rulerW - scrollBarW - 6*brdrW;
-         initDrawWinH = mainWinH - titleWindowH - choiceWindowH - rulerW -
-               scrollBarW - 10*brdrW;
-      }
-      drawWinW = initDrawWinW;
-      drawWinH = initDrawWinH;
-      hSBarW = initDrawWinW + rulerW + 2*brdrW;
-      vSBarH = initDrawWinH + rulerW + 2*brdrW;
-      titleWindowW = hSBarW + scrollBarW + 2*brdrW;
-      msgWindowW = titleWindowW - choiceWindowW - 2*brdrW;
-      if (canvasWindowOnly)
-         XResizeWindow (mainDisplay, drawWindow, drawWinW, drawWinH);
-      else
-      {
-         XResizeWindow (mainDisplay, titleWindow, titleWindowW, titleWindowH);
-         XResizeWindow (mainDisplay, msgWindow, msgWindowW, msgWindowH);
-         XMoveWindow (mainDisplay, choiceWindow, msgWindowW+2*brdrW,
-               titleWindowH+2*brdrW);
-         XResizeWindow (mainDisplay, hRuleWindow, drawWinW, rulerW);
-         XResizeWindow (mainDisplay, vRuleWindow, rulerW, drawWinH);
-         XResizeWindow (mainDisplay, drawWindow, drawWinW, drawWinH);
-         XMoveResizeWindow (mainDisplay, vSBarWindow, drawWinW+rulerW+4*brdrW,
-               titleWindowH+choiceWindowH+4*brdrW, scrollBarW, vSBarH);
-         XMoveResizeWindow (mainDisplay, hSBarWindow, 0,
-               titleWindowH+choiceWindowH+rulerW+drawWinH+8*brdrW, hSBarW,
-               scrollBarW);
-         XMoveWindow (mainDisplay, dummyWindow2, drawWinW+rulerW+4*brdrW,
-            titleWindowH+choiceWindowH+rulerW+drawWinH+8*brdrW);
-      }
-      UpdDrawWinWH ();
-      UpdDrawWinBBox ();
-      UpdScrollWinWH ();
-      SetDefaultDrawWinClipRecs ();
-      SaveMainWinPosition (main_win_x, main_win_y);
-   }
+   if (configure) Reconfigure (FALSE);
    return (INVALID);
 }
 
@@ -370,12 +486,13 @@ static Window	savedMainWindow=None, savedDrawWindow=None;
 void Setup ()
 {
    int		bitmask = 0, x_neg_in_def = FALSE, y_neg_in_def = FALSE;
-   int		x_in_cmdline = FALSE, y_in_cmdline = FALSE;
+   int		x_in_cmdline=FALSE, y_in_cmdline=FALSE, cur_y=0, i;
    char		* c_ptr;
    int		def_x_neg = 0, def_y_neg = 0;
    XWMHints	wmhints;
    XSizeHints	sizehints;
 
+   *homeDir = *bootDir = '\0';
    if ((c_ptr = XGetDefault (mainDisplay, TOOL_NAME, "Synchronize")) != NULL)
       if ((strcmp (c_ptr, "on") == 0) || (strcmp (c_ptr, "On") == 0))
          XSynchronize (mainDisplay, True);
@@ -416,6 +533,10 @@ void Setup ()
          whereToPrint = PS_FILE;
       else if (strcmp (c_ptr, "Bitmap") == 0)
          whereToPrint = XBM_FILE;
+      else if (strcmp (c_ptr, "Text") == 0)
+         whereToPrint = TEXT_FILE;
+      else if (strcmp (c_ptr, "EPSI") == 0)
+         whereToPrint = EPSI_FILE;
       else
          fprintf (stderr, "Unrecognized WhereToPrint:  %s\n", c_ptr);
    }
@@ -447,7 +568,11 @@ void Setup ()
                TOOL_NAME, c_ptr, "100 is used");
          print_reduction = 100;
       }
-      printMag = print_reduction;
+      if (printMag != print_reduction)
+      {
+         printMag = print_reduction;
+         UpdPageStyle (pageStyle);
+      }
    }
 
    moveMode = UNCONST_MOVE;
@@ -623,6 +748,45 @@ void Setup ()
       }
    }
 
+   stickyMenuSelection = FALSE;
+   if ((c_ptr = XGetDefault (mainDisplay, TOOL_NAME,
+         "StickyMenuSelection")) != NULL)
+   {
+      if (strcmp ("true", c_ptr) == 0 || strcmp ("True", c_ptr) == 0)
+         stickyMenuSelection = TRUE;
+   }
+
+   gridShown = TRUE;
+   if ((c_ptr = XGetDefault (mainDisplay, TOOL_NAME,
+         "InitialShowGrid")) != NULL)
+   {
+      if (strcmp ("false", c_ptr) == 0 || strcmp ("False", c_ptr) == 0)
+         gridShown = FALSE;
+   }
+
+   gridOn = TRUE;
+   if ((c_ptr = XGetDefault (mainDisplay, TOOL_NAME, "InitialSnapOn")) != NULL)
+   {
+      if (strcmp ("false", c_ptr) == 0 || strcmp ("False", c_ptr) == 0)
+         gridOn = FALSE;
+   }
+
+   noMenubar = FALSE;
+   if ((c_ptr = XGetDefault (mainDisplay, TOOL_NAME, "NoMenubar")) != NULL)
+   {
+      if (strcmp ("true", c_ptr) == 0 || strcmp ("True", c_ptr) == 0)
+         noMenubar = TRUE;
+   }
+
+   noStatusWindow = FALSE;
+   if ((c_ptr = XGetDefault (mainDisplay, TOOL_NAME, "NoStatusWindow")) != NULL)
+   {
+      if (strcmp ("true", c_ptr) == 0 || strcmp ("True", c_ptr) == 0)
+         noStatusWindow = TRUE;
+   }
+
+   initialMenubarWindowH = MENUBAR_WINDOW_H;
+
    strcpy (scanFileName, "No File");
    scanLineNum = 0;
 
@@ -638,37 +802,54 @@ void Setup ()
       else
          strcpy (drawPath, TGIF_PATH);
    }
+   else if (strlen (c_ptr) >= 255)
+      strcpy (drawPath, TGIF_PATH);
    else
-      if (strlen (c_ptr) >= 255)
-         strcpy (drawPath, TGIF_PATH);
-      else
-         strcpy (drawPath, c_ptr);
+      strcpy (drawPath, c_ptr);
 
    if ((c_ptr = getenv ("HOME")) == NULL)
       strcpy (homeDir, "/");
+   else if (strlen (c_ptr) >= MAXPATHLENGTH-1)
+      strcpy (homeDir, "/");
    else
-      if (strlen (c_ptr) >= MAXPATHLENGTH-1)
-         strcpy (homeDir, "/");
-      else
-         strcpy (homeDir, c_ptr);
+      strcpy (homeDir, c_ptr);
 
+   if (*bootDir == '\0')
+   {
 #ifdef VMS
-   if (getcwd (bootDir, MAXPATHLENGTH+2, 0) == NULL) strcpy (bootDir, ".");
+      extern char * getcwd ARGS_DECL((char *, int, int));
+
+      if (getcwd (bootDir, MAXPATHLENGTH+2, 0) == NULL) strcpy (bootDir, ".");
 #else
 #ifdef ibm
-   if (getwd (bootDir, MAXPATHLENGTH+2) == NULL) strcpy (bootDir, ".");
+      extern char * getwd ARGS_DECL((char *, int));
+
+      if (getwd (bootDir, MAXPATHLENGTH+2) == NULL) strcpy (bootDir, ".");
 #else
 #ifdef NeXT
-   if (getwd (bootDir, MAXPATHLENGTH+2) == NULL) strcpy (bootDir, ".");
+      extern char * getwd ARGS_DECL((char *, int));
+
+      if (getwd (bootDir, MAXPATHLENGTH+2) == NULL) strcpy (bootDir, ".");
 #else
 #ifdef luna88k
-   if (getwd (bootDir) == NULL) strcpy (bootDir, ".");
+      extern char * getwd ARGS_DECL((char *, int));
+
+      if (getwd (bootDir) == NULL) strcpy (bootDir, ".");
 #else
-   if (getcwd (bootDir, MAXPATHLENGTH+2) == NULL) strcpy (bootDir, ".");
+#ifdef sequent
+      extern char * getwd ARGS_DECL((char *, int));
+
+      if (getwd (bootDir) == NULL) strcpy (bootDir, ".");
+#else
+      extern char * getcwd ARGS_DECL((char *, int));
+
+      if (getcwd (bootDir, MAXPATHLENGTH+2) == NULL) strcpy (bootDir, ".");
 #endif
 #endif
 #endif
 #endif
+#endif
+   }
 
    sizehints.flags = PPosition | PSize | PMinSize;
    sizehints.x = 0;
@@ -716,8 +897,10 @@ void Setup ()
       if (bitmask & YValue) y_in_cmdline = TRUE;
    }
 
-   InitWinSizes ();
    InitFonts ();
+   initialMenubarWindowH = defaultFontHeight;
+   statusWindowH = max(defaultFontHeight+(brdrW<<1)+4, btn1_height+2);
+   InitWinSizes ();
 
    if (canvasWindowOnly)
    {
@@ -752,6 +935,8 @@ void Setup ()
    {
       sizehints.min_width = choiceWindowW+4*brdrW+defaultFontWidth;
       sizehints.min_height =
+            (noMenubar ? 0 : menubarWindowH+(brdrW<<1))+
+            (noStatusWindow ? 0 : statusWindowH+(brdrW<<1))+
             titleWindowH+choiceWindowH+PIX_PER_INCH+rulerW+scrollBarW+10*brdrW;
       if (mainWinW > sizehints.min_width)
          sizehints.width = mainWinW;
@@ -797,62 +982,79 @@ void Setup ()
    InitPage ();
 
    if ((titleWindow = XCreateSimpleWindow (mainDisplay, mainWindow, 0, 0,
-         titleWindowW, titleWindowH, brdrW, myBorderPixel, myBgPixel)) == 0) 
-   { fprintf (stderr, "Could not create menu window!\n"); exit(1); }
+         titleWindowW, titleWindowH, brdrW, myBorderPixel, myBgPixel)) == 0)
+      Error ("setup", "Could not create the title window");
+   cur_y += titleWindowH + (brdrW<<1);
 
-   if ((msgWindow = XCreateSimpleWindow (mainDisplay, mainWindow, 0,
-         titleWindowH+2*brdrW, msgWindowW, msgWindowH, brdrW, myBorderPixel,
+   if ((menubarWindow = XCreateSimpleWindow (mainDisplay, mainWindow, 0,
+         cur_y, menubarWindowW, menubarWindowH, brdrW, myBorderPixel,
          myBgPixel)) == 0)
-   { fprintf (stderr, "Could not create message window!\n"); exit(1); }
+      Error ("setup", "Could not create the menubar window");
+
+   if (!noMenubar)
+      cur_y += menubarWindowH + (brdrW<<1);
+
+   if ((msgWindow = XCreateSimpleWindow (mainDisplay, mainWindow, 0, cur_y,
+         msgWindowW, msgWindowH, brdrW, myBorderPixel, myBgPixel)) == 0)
+      Error ("setup", "Could not create the message window");
 
    if ((choiceWindow = XCreateSimpleWindow (mainDisplay, mainWindow,
-         msgWindowW+2*brdrW, titleWindowH+2*brdrW, choiceWindowW, choiceWindowH,
+         msgWindowW+2*brdrW, cur_y, choiceWindowW, choiceWindowH,
          brdrW, myBorderPixel, myBgPixel)) == 0)
-   { fprintf (stderr, "Could not create choice window!\n"); exit(1); }
+      Error ("setup", "Could not create the choice window");
+   cur_y += msgWindowH + (brdrW<<1);
 
    InitChoice ();
 
+   if ((dummyWindow1 = XCreateSimpleWindow (mainDisplay, mainWindow, 0, cur_y,
+         rulerW, rulerW, brdrW, myBorderPixel, myBgPixel)) == 0) 
+      Error ("setup", "Could not create the left top dummy window");
+
    if ((hRuleWindow = XCreateSimpleWindow (mainDisplay, mainWindow,
-         rulerW+2*brdrW, titleWindowH+choiceWindowH+4*brdrW, drawWinW, rulerW,
+         rulerW+2*brdrW, cur_y, drawWinW, rulerW,
          brdrW, myBorderPixel, myBgPixel)) == 0)
-   { fprintf (stderr, "Could not create horizontal ruler window!\n"); exit(1); }
-
-   if ((vRuleWindow = XCreateSimpleWindow (mainDisplay, mainWindow, 0,
-         titleWindowH+rulerW+choiceWindowH+6*brdrW, rulerW, drawWinH,
-         brdrW, myBorderPixel, myBgPixel)) == 0)
-   { fprintf (stderr, "Could not create vertical ruler window!\n"); exit(1); }
-
-   if ((drawWindow = XCreateSimpleWindow (mainDisplay, mainWindow,
-         rulerW+2*brdrW, titleWindowH+rulerW+choiceWindowH+6*brdrW,
-         drawWinW, drawWinH, brdrW, myBorderPixel, myBgPixel)) == 0) 
-   { fprintf (stderr, "Could not create draw window!\n"); exit(1); }
+      Error ("setup", "Could not create the horizontal ruler window");
 
    if ((vSBarWindow = XCreateSimpleWindow (mainDisplay, mainWindow,
-         4*brdrW+rulerW+drawWinW, titleWindowH+choiceWindowH+4*brdrW,
-         scrollBarW, vSBarH, brdrW, myBorderPixel, myBgPixel)) == 0) 
-   {
-      fprintf (stderr, "Could not create vertical scrollbar window!\n");
-      exit(1);
-   }
+         4*brdrW+rulerW+drawWinW, cur_y, scrollBarW, vSBarH,
+         brdrW, myBorderPixel, myBgPixel)) == 0) 
+      Error ("setup", "Could not create the vertical scrollbar window");
+   cur_y += rulerW + (brdrW<<1);
 
-   if ((hSBarWindow = XCreateSimpleWindow (mainDisplay, mainWindow, 0,
-         titleWindowH+rulerW+drawWinH+choiceWindowH+8*brdrW,
+   if ((vRuleWindow = XCreateSimpleWindow (mainDisplay, mainWindow, 0, cur_y,
+         rulerW, drawWinH, brdrW, myBorderPixel, myBgPixel)) == 0)
+      Error ("setup", "Could not create the vertical ruler window");
+
+   if ((drawWindow = XCreateSimpleWindow (mainDisplay, mainWindow,
+         rulerW+2*brdrW, cur_y, drawWinW, drawWinH,
+         brdrW, myBorderPixel, myBgPixel)) == 0) 
+      Error ("setup", "Could not create the drawing window");
+   cur_y += drawWinH + (brdrW<<1);
+
+   if ((hSBarWindow = XCreateSimpleWindow (mainDisplay, mainWindow, 0, cur_y,
          hSBarW, scrollBarW, brdrW, myBorderPixel, myBgPixel)) == 0) 
-   {
-      fprintf (stderr, "Could not create horizontal scrollbar window!\n");
-      exit(1);
-   }
+      Error ("setup", "Could not create the horizontal scrollbar window");
 
    if ((dummyWindow2 = XCreateSimpleWindow (mainDisplay, mainWindow,
-         rulerW+drawWinW+4*brdrW,
-         titleWindowH+choiceWindowH+rulerW+drawWinH+8*brdrW,
+         rulerW+drawWinW+4*brdrW, cur_y,
          scrollBarW, scrollBarW, brdrW, myBorderPixel, myBgPixel)) == 0) 
-   { fprintf (stderr, "Could not create dummy window!\n"); exit(1); }
+      Error ("setup", "Could not create the right bottom dummy window");
+   cur_y += scrollBarW + (brdrW<<1);
 
-   if ((dummyWindow1 = XCreateSimpleWindow (mainDisplay, mainWindow, 0,
-         titleWindowH+choiceWindowH+4*brdrW, rulerW, rulerW,
-         brdrW, myBorderPixel, myBgPixel)) == 0) 
-   { fprintf (stderr, "Could not create dummy window!\n"); exit(1); }
+   if ((statusWindow = XCreateSimpleWindow (mainDisplay, mainWindow, 0,
+         cur_y, statusWindowW, statusWindowH, brdrW, myBorderPixel,
+         myBgPixel)) == 0)
+      Error ("setup", "Could not create the status window");
+   CalcStatusSubWinGeom ();
+   for (i=0; i < MAX_STATUS_BTNS; i++)
+      if ((statusSubWindow[i] = XCreateSimpleWindow (mainDisplay,
+            statusWindow, statusSubWindowX[i], statusSubWindowY[i],
+            statusSubWindowW[i], statusSubWindowH[i], brdrW,
+            myBorderPixel, myBgPixel)) == 0)
+         Error ("setup", "Could not create the status sub windows");
+
+   if (!noStatusWindow)
+      cur_y += statusWindowH + (brdrW<<1);
 
    if (canvasWindowOnly)
    {
@@ -881,6 +1083,7 @@ void Setup ()
 
    UpdDrawWinBBox ();
    InitTitle ();
+   InitStatus ();
    SetCanvasFont ();
 
 #ifdef MAPBEFORESELECT
@@ -905,83 +1108,124 @@ void Setup ()
       XMapWindow (mainDisplay, drawWindow); 
       XSelectInput (mainDisplay, drawWindow, ButtonReleaseMask |
             ButtonPressMask | PointerMotionMask | KeyPressMask |
-            KeyReleaseMask | ExposureMask);
+            KeyReleaseMask | ExposureMask | EnterWindowMask);
    }
    else
    {
       XMapWindow (mainDisplay, titleWindow);
-      XSelectInput (mainDisplay, titleWindow, ExposureMask);
+      XSelectInput (mainDisplay, titleWindow, ExposureMask | EnterWindowMask);
+
+      if (menubarWindow != None)
+      {
+         XMapWindow (mainDisplay, menubarWindow);
+         XSelectInput (mainDisplay, menubarWindow, ExposureMask |
+               EnterWindowMask | ButtonPressMask | PointerMotionMask);
+      }
 
       XMapWindow (mainDisplay, msgWindow);
       XSelectInput (mainDisplay, msgWindow, ButtonPressMask |
-            ButtonReleaseMask | PointerMotionMask | ExposureMask);
+            ButtonReleaseMask | PointerMotionMask | ExposureMask |
+            EnterWindowMask);
 
       XMapWindow (mainDisplay, choiceWindow);
       XSelectInput (mainDisplay, choiceWindow,
-            ButtonReleaseMask | ButtonPressMask | ExposureMask);
+            ButtonReleaseMask | ButtonPressMask | ExposureMask |
+            PointerMotionMask | EnterWindowMask);
 
       XMapWindow (mainDisplay, hRuleWindow);
-      XSelectInput (mainDisplay, hRuleWindow, ExposureMask);
+      XSelectInput (mainDisplay, hRuleWindow, ExposureMask | EnterWindowMask);
       XMapWindow (mainDisplay, vRuleWindow);
-      XSelectInput (mainDisplay, vRuleWindow, ExposureMask);
+      XSelectInput (mainDisplay, vRuleWindow, ExposureMask | EnterWindowMask);
 
       XMapWindow (mainDisplay, drawWindow); 
       XSelectInput (mainDisplay, drawWindow, ButtonReleaseMask |
             ButtonPressMask | PointerMotionMask | KeyPressMask |
-            KeyReleaseMask | ExposureMask);
+            KeyReleaseMask | ExposureMask | EnterWindowMask);
 
       XMapWindow (mainDisplay, vSBarWindow);
-      XSelectInput (mainDisplay, vSBarWindow, ButtonPressMask | ExposureMask);
+      XSelectInput (mainDisplay, vSBarWindow, ButtonPressMask | ExposureMask |
+            EnterWindowMask);
       XMapWindow (mainDisplay, hSBarWindow);
-      XSelectInput (mainDisplay, hSBarWindow, ButtonPressMask | ExposureMask);
+      XSelectInput (mainDisplay, hSBarWindow, ButtonPressMask | ExposureMask |
+            EnterWindowMask);
       XMapWindow (mainDisplay, dummyWindow1);
-      XSelectInput (mainDisplay, dummyWindow1, ButtonPressMask | ExposureMask);
+      XSelectInput (mainDisplay, dummyWindow1, ButtonPressMask | ExposureMask |
+            EnterWindowMask);
+      XMapWindow (mainDisplay, dummyWindow2);
+      XSelectInput (mainDisplay, dummyWindow2, EnterWindowMask);
+
+      if (statusWindow != None)
+      {
+         XMapWindow (mainDisplay, statusWindow);
+         XSelectInput (mainDisplay, statusWindow, ExposureMask |
+               EnterWindowMask);
+         for (i=0; i < MAX_STATUS_BTNS; i++)
+            XMapWindow (mainDisplay, statusSubWindow[i]);
+      }
    }
 #else
    if (canvasWindowOnly)
    {
       XSelectInput (mainDisplay, drawWindow, ButtonReleaseMask |
             ButtonPressMask | PointerMotionMask | KeyPressMask |
-            KeyReleaseMask | ExposureMask);
+            KeyReleaseMask | ExposureMask | EnterWindowMask);
       XMapWindow (mainDisplay, drawWindow); 
    }
    else
    {
-      XSelectInput (mainDisplay, titleWindow, ExposureMask);
+      XSelectInput (mainDisplay, titleWindow, ExposureMask | EnterWindowMask);
       XMapWindow (mainDisplay, titleWindow);
 
+      if (menubarWindow != None)
+      {
+         XSelectInput (mainDisplay, menubarWindow, ExposureMask |
+               EnterWindowMask | ButtonPressMask | PointerMotionMask);
+         XMapWindow (mainDisplay, menubarWindow);
+      }
+
       XSelectInput (mainDisplay, msgWindow, ButtonPressMask |
-            ButtonReleaseMask | PointerMotionMask | ExposureMask);
+            ButtonReleaseMask | PointerMotionMask | ExposureMask |
+            EnterWindowMask);
       XMapWindow (mainDisplay, msgWindow);
 
       XSelectInput (mainDisplay, choiceWindow,
-            ButtonReleaseMask | ButtonPressMask | ExposureMask);
+            ButtonReleaseMask | ButtonPressMask | ExposureMask |
+            PointerMotionMask | EnterWindowMask);
       XMapWindow (mainDisplay, choiceWindow);
 
-      XSelectInput (mainDisplay, hRuleWindow, ExposureMask);
+      XSelectInput (mainDisplay, hRuleWindow, ExposureMask | EnterWindowMask);
       XMapWindow (mainDisplay, hRuleWindow);
-      XSelectInput (mainDisplay, vRuleWindow, ExposureMask);
+      XSelectInput (mainDisplay, vRuleWindow, ExposureMask | EnterWindowMask);
       XMapWindow (mainDisplay, vRuleWindow);
 
       XSelectInput (mainDisplay, drawWindow, ButtonReleaseMask |
             ButtonPressMask | PointerMotionMask | KeyPressMask |
-            KeyReleaseMask | ExposureMask);
+            KeyReleaseMask | ExposureMask | EnterWindowMask);
       XMapWindow (mainDisplay, drawWindow); 
 
-      XSelectInput (mainDisplay, vSBarWindow, ButtonPressMask | ExposureMask);
+      XSelectInput (mainDisplay, vSBarWindow, ButtonPressMask | ExposureMask |
+            EnterWindowMask);
       XMapWindow (mainDisplay, vSBarWindow);
-      XSelectInput (mainDisplay, hSBarWindow, ButtonPressMask | ExposureMask);
+      XSelectInput (mainDisplay, hSBarWindow, ButtonPressMask | ExposureMask |
+            EnterWindowMask);
       XMapWindow (mainDisplay, hSBarWindow);
-      XSelectInput (mainDisplay, dummyWindow1, ButtonPressMask | ExposureMask);
+      XSelectInput (mainDisplay, dummyWindow1, ButtonPressMask | ExposureMask |
+            EnterWindowMask);
       XMapWindow (mainDisplay, dummyWindow1);
+      XSelectInput (mainDisplay, dummyWindow2, EnterWindowMask);
+      XMapWindow (mainDisplay, dummyWindow2);
+
+      if (statusWindow != None)
+      {
+         XSelectInput (mainDisplay, statusWindow, ExposureMask |
+               EnterWindowMask);
+         XMapWindow (mainDisplay, statusWindow);
+         for (i=0; i < MAX_STATUS_BTNS; i++)
+            XMapWindow (mainDisplay, statusSubWindow[i]);
+      }
    }
 #endif
 
-   if (!canvasWindowOnly)
-   {
-      XMapWindow (mainDisplay, dummyWindow2);
-   }
-
    wmhints.flags = InputHint;
    wmhints.input = True;
 
@@ -1045,6 +1289,11 @@ void Setup ()
    InitText ();
 }
 
+void CleanUpResiduals ()
+{
+   if (savedMainWindow != None) XDestroyWindow (mainDisplay, savedMainWindow);
+}
+
 int TieLooseEnds ()
    /* returns TRUE if something got created */
    /* returns FALSE otherwise */
