@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,10 +25,11 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/special.c,v 2.60.1.3 1993/05/18 04:25:47 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/special.c,v 2.74 1994/04/04 00:49:06 william Exp $";
 #endif
 
 #include <stdio.h>
+#include <string.h>
 #include <X11/Xlib.h>
 #include "const.h"
 #include "types.h"
@@ -37,6 +39,7 @@ static char RCSid[] =
 #include "choice.e"
 #include "cmd.e"
 #include "color.e"
+#include "copypaste.e"
 #include "cursor.e"
 #include "dialog.e"
 #include "drawing.e"
@@ -53,8 +56,12 @@ static char RCSid[] =
 #include "names.e"
 #include "obj.e"
 #include "raster.e"
+#include "ruler.e"
 #include "select.e"
 #include "setup.e"
+#ifndef _NO_EXTERN
+#include "special.e"
+#endif
 #include "stk.e"
 
 char * specialMenuStr[] =
@@ -72,6 +79,7 @@ char * specialMenuStr[] =
       "HideAttr       ^#h",
       "HideAttrName    #j",
       "Move/JustfyAttr #m",
+      "EditAttrs         ",
       "AnimateSend     #e",
       "AnimateFlash    #f",
       "AttachFileAttrs   ",
@@ -79,6 +87,30 @@ char * specialMenuStr[] =
       "EditFileAttrs     ",
       "UpdateSymbols  ^#u"
    };
+static char * specialMenuDescription[] =
+   {  "Turn a selected grouped/icon object into a symbol object",
+      "Turn a selected symbol object into a grouped object",
+      "Instantiate a building-block object from the current domain",
+      "Turn a selected grouped/symbol object into an icon object",
+      "Turn an icon object into a grouped object",
+      "Push into (edit) the symbol file which defines the selected icon object",
+      "Pop back to a high lever (reverse of Push)",
+      "Attach selected text objects as attributes for the non-text object",
+      "Detach all attributes of selected objects",
+      "Make all attributes of selected objects visible",
+      "Make all attribute names of selected objects visible",
+      "Hide all attributes for selected objects",
+      "Hide all attribute names for selected objects",
+      "Move/justify an attribute for a selected object",
+      "Edit attributes of a selected object",
+      "Animate a little token on a selected poly (not very useful)",
+      "Flash a selected poly (not very useful)",
+      "Attach selected text objects as file attributes",
+      "Detach all file attributes",
+      "Edit file attributes",
+      "Refresh selected icon objects from their definition files",
+      NULL
+   };
 
 struct ObjRec * ReadSymbol (FP)
    FILE	* FP;
@@ -92,6 +124,7 @@ struct ObjRec * ReadSymbol (FP)
    SetWatchCursor (mainWindow);
 
    importingFile = TRUE; /* ignore 'state' info but set fileVersion */
+   readingPageNum = 0;
    while ((read_status = ReadObj (FP, &obj_ptr)) == TRUE)
       if (obj_ptr != NULL)
       {
@@ -133,7 +166,7 @@ struct ObjRec * GetObjRepresentation (PathName, SymName)
    char			tmp_filename[MAXPATHLENGTH];
 
    sprintf (file_name, "%s/%s.%s", PathName, SymName, SYM_FILE_EXT);
-   if (short_name = IsPrefix (bootDir, file_name, &rest)) ++rest;
+   if ((short_name = IsPrefix (bootDir, file_name, &rest))) ++rest;
 
    if ((fp = fopen (file_name, "r")) == NULL)
    {
@@ -272,6 +305,60 @@ void PlaceTopObj (ObjPtr)
    XSync (mainDisplay, True);
 }
 
+static
+int UnMakeIconicOnInstantiate (obj_ptr)
+   struct ObjRec	* obj_ptr;
+{
+   register struct AttrRec	* attr_ptr;
+
+   for (attr_ptr=obj_ptr->lattr; attr_ptr!=NULL; attr_ptr=attr_ptr->prev)
+      if (*attr_ptr->name=='\0' &&
+            strcmp (attr_ptr->s, "unmakeiconic_on_instantiate")==0)
+         return (TRUE);
+   return (FALSE);
+}
+
+static
+void DeleteUnMakeIconicOnInstantiateText ()
+{
+   struct SelRec	* sel_ptr;
+   struct ObjRec	* text_obj_ptr=NULL;
+
+   for (sel_ptr=botSel; sel_ptr != NULL; sel_ptr=sel_ptr->prev)
+      if (sel_ptr->obj->type == OBJ_TEXT &&
+            strcmp (sel_ptr->obj->detail.t->first->s,
+            "unmakeiconic_on_instantiate") == 0)
+      {
+         text_obj_ptr = sel_ptr->obj;
+         break;
+      }
+   if (text_obj_ptr == NULL)
+   {
+      char	msg[MAXSTRING+1];
+
+      fprintf (stderr, "Warning:  Can not find the '%s' text object.\n",
+            "unmakeiconic_on_instantiate");
+      fprintf (stderr, "Please send bug report to william@cs.ucla.edu.\n");
+      strcpy (msg,
+            "Possible fatal error.  Please read the terminal error output.");
+      Dialog (msg, NULL, NULL);
+      XFlush (mainDisplay);
+      XSync (mainDisplay, False);
+      return;
+   }
+   MakeQuiescent ();
+   topSel = (struct SelRec *) calloc (1, sizeof(struct SelRec));
+   topSel->next = NULL;
+   topSel->obj = text_obj_ptr;
+   topSel->prev = NULL;
+   botSel = topSel;
+   UpdSelBBox ();
+   HighLightForward ();
+   DelAllSelObj ();
+}
+
+void UnMakeIconic ();
+
 void Instantiate ()
 {
    char			file_name[MAXPATHLENGTH], * rest, msg[MAXSTRING];
@@ -305,7 +392,7 @@ void Instantiate ()
    if (len > ext_len && strcmp (ext_str, &sym_name[len-ext_len]) == 0)
       sym_name[len-ext_len] = '\0';
 
-   if (short_name = IsPrefix (bootDir, file_name, &rest)) ++rest;
+   if ((short_name = IsPrefix (bootDir, file_name, &rest))) ++rest;
 
    if ((fp = fopen (file_name, "r")) == NULL)
    {
@@ -337,11 +424,22 @@ void Instantiate ()
       obj_ptr->detail.r->rotate = ROTATE0;
       obj_ptr->detail.r->flip = NO_FLIP;
       PlaceTopObj (obj_ptr);
+      AssignNewObjIds (obj_ptr);
       AddObj (NULL, topObj, obj_ptr);
       AdjObjBBox (obj_ptr);
 
       SelectTopObj ();
-      RecordNewObjCmd ();
+      if (UnMakeIconicOnInstantiate (obj_ptr))
+      {
+         StartCompositeCmd ();
+         RecordNewObjCmd ();
+         UnMakeIconic ();
+         UngroupSelObj ();
+         DeleteUnMakeIconicOnInstantiateText ();
+         EndCompositeCmd ();
+      }
+      else
+         RecordNewObjCmd ();
       SetFileModified (TRUE);
       justDupped = FALSE;
    }
@@ -363,7 +461,11 @@ void MakeSymbolic ()
       PrepareToRecord (CMD_REPLACE, topSel, botSel, numObjSelected);
 
       if (topSel->obj->type != OBJ_GROUP && topSel->obj->type != OBJ_ICON)
+      {
          GroupSingleObj ();
+         if (topSel->obj->fattr != NULL)
+            Msg ("Object's attributes are promoted to the new symbol object.");
+      }
 
       topSel->obj->type = OBJ_SYM;
       Msg ("Selected object is now SYMBOLIC.");
@@ -482,7 +584,7 @@ void MakeIconic ()
       sprintf (file_name, "%s/%s", curDir, icon_full_name);
 
       if (!OkayToCreateFile (file_name)) return;
-      if (short_name = IsPrefix (bootDir, file_name, &rest)) ++rest;
+      if ((short_name = IsPrefix (bootDir, file_name, &rest))) ++rest;
 
       if ((fp = fopen (file_name, "w")) == NULL)
       {
@@ -509,20 +611,37 @@ void MakeIconic ()
             topSel->obj->type == OBJ_ICON)
          JustMoveSelToTop ();
       else
+      {
          GroupSingleObj ();
+         if (topSel->obj->fattr != NULL)
+            Msg ("Object's attributes are promoted to the new icon object.");
+      }
 
       topSel->obj->type = OBJ_SYM;
       strcpy (topSel->obj->detail.r->s, icon_name);
       topSel->obj->detail.r->rotate = ROTATE0;
       topSel->obj->detail.r->flip = NO_FLIP;
 
+      writeFileFailed = FALSE;
       Save (fp, topSel->obj, 0, 1);
 
-      if (short_name)
-         sprintf (s, "File '%s' created.", rest);
+      if (writeFileFailed)
+      {
+         writeFileFailed = FALSE;
+         sprintf (s, "%s '%s'.  %s.  '%s' deleted.",
+               "Fail to write to", file_name, "File system may be full",
+               file_name);
+         Dialog (s, NULL, NULL);
+         unlink (file_name);
+      }
       else
-         sprintf (s, "File '%s' created.", file_name);
-      Msg (s);
+      {
+         if (short_name)
+            sprintf (s, "File '%s' created.", rest);
+         else
+            sprintf (s, "File '%s' created.", file_name);
+         Msg (s);
+      }
       fclose (fp);
 
       HighLightReverse ();
@@ -600,6 +719,7 @@ void SpecialSubMenu (index)
       case SPECIAL_HIDEATTR: HideAllAttrs (); break;
       case SPECIAL_HIDEATTRNAME: HideAllAttrNames (); break;
       case SPECIAL_MOVEATTR: MoveAttr (); break;
+      case SPECIAL_EDITATTR: EditAttrs (); break;
       case SPECIAL_ANIMATESEND: AnimateSel (); break;
       case SPECIAL_ANIMATEFLASH: FlashSelColor (); break;
       case SPECIAL_ADDFILEATTR: AddFileAttrs (); break;
@@ -609,16 +729,17 @@ void SpecialSubMenu (index)
    }
 }
 
-void SpecialMenu (X, Y)
-   int	X, Y;
+int SpecialMenu (X, Y, TrackMenubar)
+   int	X, Y, TrackMenubar;
 {
    register int	index;
    int		* fore_colors, * valid, * init_rv;
 
-   DefaultColorArrays (MAXSPECIALMENUS, &fore_colors, &valid, &init_rv);
+   DefaultColorArrays (MAXSPECIALMENUS, &fore_colors, &valid, &init_rv, NULL);
    activeMenu = MENU_SPECIAL;
    index = TextMenuLoop (X, Y, specialMenuStr, MAXSPECIALMENUS, fore_colors,
-         valid, init_rv, SINGLECOLOR);
+         valid, init_rv, specialMenuDescription, SINGLECOLOR, TrackMenubar);
 
-   if (index != INVALID) SpecialSubMenu (index);
+   if (index >= 0) SpecialSubMenu (index);
+   return (index);
 }
