@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,11 +25,12 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/stretch.c,v 2.76 1993/03/11 05:56:14 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/stretch.c,v 2.87 1994/03/21 04:57:03 william Exp $";
 #endif
 
-#include <stdio.h>
 #include <math.h>
+#include <stdio.h>
+#include <string.h>
 #include <X11/Xlib.h>
 #include "const.h"
 #include "types.h"
@@ -46,7 +48,9 @@ static char RCSid[] =
 #include "font.e"
 #include "grid.e"
 #include "mainloop.e"
+#include "mark.e"
 #include "move.e"
+#include "msg.e"
 #include "obj.e"
 #include "poly.e"
 #include "raster.e"
@@ -54,6 +58,9 @@ static char RCSid[] =
 #include "select.e"
 #include "setup.e"
 #include "spline.e"
+#ifndef _NO_EXTERN
+#include "stretch.e"
+#endif
 #include "text.e"
 #include "xbitmap.e"
 #include "xpixmap.e"
@@ -158,6 +165,56 @@ int ObjRigid (obj_ptr)
    return (TRUE);
 }
 
+int AutoCenterAttr (obj_ptr)
+   struct ObjRec	* obj_ptr;
+{
+   register struct AttrRec	* attr_ptr;
+
+   for (attr_ptr=obj_ptr->lattr; attr_ptr!=NULL; attr_ptr=attr_ptr->prev)
+      if (*attr_ptr->name=='\0' && strcmp (attr_ptr->s, "auto_center_attr")==0)
+         return (TRUE);
+   return (FALSE);
+}
+
+void CenterObjInOBBox (TextObjPtr, OBBox, BBoxReturn)
+   struct ObjRec	* TextObjPtr;
+   struct BBRec		OBBox, * BBoxReturn;
+{
+   int text_w, text_h, bbox_w, bbox_h, dx, dy;
+
+   if (BBoxReturn != NULL)
+   {
+      BBoxReturn->ltx = TextObjPtr->bbox.ltx;
+      BBoxReturn->lty = TextObjPtr->bbox.lty;
+      BBoxReturn->rbx = TextObjPtr->bbox.rbx;
+      BBoxReturn->rby = TextObjPtr->bbox.rby;
+   }
+   text_w = TextObjPtr->obbox.rbx-TextObjPtr->obbox.ltx;
+   text_h = TextObjPtr->obbox.rby-TextObjPtr->obbox.lty;
+   bbox_w = OBBox.rbx - OBBox.ltx;
+   bbox_h = OBBox.rby - OBBox.lty;
+   if (text_w > bbox_w)
+      dx = OBBox.ltx-((text_w-bbox_w)>>1)-TextObjPtr->obbox.ltx;
+   else
+      dx = OBBox.ltx+((bbox_w-text_w)>>1)-TextObjPtr->obbox.ltx;
+   if (text_h > bbox_h)
+      dy = OBBox.lty-((text_h-bbox_h)>>1)-TextObjPtr->obbox.lty;
+   else
+      dy = OBBox.lty+((bbox_h-text_h)>>1)-TextObjPtr->obbox.lty;
+   MoveObj (TextObjPtr, dx, dy);
+   if (BBoxReturn != NULL)
+   {
+      if (TextObjPtr->bbox.ltx < BBoxReturn->ltx)
+         BBoxReturn->ltx = TextObjPtr->bbox.ltx;
+      if (TextObjPtr->bbox.lty < BBoxReturn->lty)
+         BBoxReturn->lty = TextObjPtr->bbox.lty;
+      if (TextObjPtr->bbox.rbx > BBoxReturn->rbx)
+         BBoxReturn->rbx = TextObjPtr->bbox.rbx;
+      if (TextObjPtr->bbox.rby > BBoxReturn->rby)
+         BBoxReturn->rby = TextObjPtr->bbox.rby;
+   }
+}
+
 struct SelRec * PtInSelMark (XOff, YOff, Corner)
    int	XOff, YOff, * Corner;
    /* XOff and YOff are screen offsets */
@@ -219,9 +276,11 @@ void StretchPoly (XGridOff, YGridOff, ObjPtr, NumPts, V, Index)
    register int	i;
    int		x, y, dx, dy, num = 0, stretching = TRUE;
    int		ltx, lty, rbx, rby, curved=LT_STRAIGHT;
-   int		grid_x = XGridOff, grid_y = YGridOff, sn;
+   int		grid_x = XGridOff, grid_y = YGridOff, sn, intn;
+   int		auto_center_attr = AutoCenterAttr (ObjPtr);
+   char		buf[80], * smooth=NULL;
    XEvent	input, ev;
-   XPoint	* pv = NULL, * sv = NULL;
+   XPoint	* pv=NULL, * sv=NULL, * cntrlv=NULL;
 
    if (ObjPtr->locked)
    {
@@ -233,11 +292,33 @@ void StretchPoly (XGridOff, YGridOff, ObjPtr, NumPts, V, Index)
    {
       case OBJ_POLY:
          curved = ObjPtr->detail.p->curved;
-         if (curved && splineRubberband)
+         if (splineRubberband)
          {
             pv = (XPoint *) calloc (NumPts+1, sizeof (XPoint));
-            for (i = 0; i < NumPts; i++) pv[i] = V[i];
-            sv = MakeSplinePolyVertex (&sn,drawOrigX,drawOrigY,NumPts,pv);
+            if (pv == NULL) fprintf (stderr, "Can not calloc().\n");
+            if (curved != LT_INTSPLINE && ObjPtr->detail.p->smooth != NULL)
+            {
+               smooth = (char *) calloc (NumPts+1, sizeof(char));
+               if (smooth == NULL) fprintf (stderr, "Can not calloc().\n");
+            }
+            for (i = 0; i < NumPts; i++)
+            {
+               pv[i].x = V[i].x;
+               pv[i].y = V[i].y;
+               if (smooth != NULL) smooth[i] = ObjPtr->detail.p->smooth[i];
+            }
+            switch (curved)
+            {
+               case LT_STRAIGHT:
+               case LT_SPLINE:
+                  sv = MakeMultiSplinePolyVertex (&sn, smooth,
+                        drawOrigX, drawOrigY, NumPts, pv);
+                  break;
+               case LT_INTSPLINE:
+                  sv = MakeIntSplinePolyVertex (&sn, &intn, &cntrlv, drawOrigX,
+                        drawOrigY, NumPts, pv);
+                  break;
+            }
          }
          else if (Index == 0 || Index == NumPts-1)
          {
@@ -265,11 +346,33 @@ void StretchPoly (XGridOff, YGridOff, ObjPtr, NumPts, V, Index)
          break;
       case OBJ_POLYGON:
          curved = ObjPtr->detail.g->curved;
-         if (curved && splineRubberband)
+         if (splineRubberband)
          {
             pv = (XPoint *) calloc (NumPts+1, sizeof (XPoint));
-            for (i = 0; i < NumPts; i++) pv[i] = V[i];
-            sv = MakeSplinePolygonVertex (&sn,drawOrigX,drawOrigY,NumPts,pv);
+            if (pv == NULL) fprintf (stderr, "Can not calloc().\n");
+            if (curved != LT_INTSPLINE && ObjPtr->detail.g->smooth != NULL)
+            {
+               smooth = (char *) calloc (NumPts+1, sizeof(char));
+               if (smooth == NULL) fprintf (stderr, "Can not calloc().\n");
+            }
+            for (i = 0; i < NumPts; i++)
+            {
+               pv[i].x = V[i].x;
+               pv[i].y = V[i].y;
+               if (smooth != NULL) smooth[i] = ObjPtr->detail.g->smooth[i];
+            }
+            switch (curved)
+            {
+               case LT_STRAIGHT:
+               case LT_SPLINE:
+                  sv = MakeMultiSplinePolygonVertex (&sn, smooth,
+                        drawOrigX, drawOrigY, NumPts, pv);
+                  break;
+               case LT_INTSPLINE:
+                  sv = MakeIntSplinePolygonVertex (&sn, &intn, &cntrlv,
+                        drawOrigX, drawOrigY, NumPts, pv);
+                  break;
+            }
          }
          else if (Index == 0 || Index == NumPts-1)
          {
@@ -300,6 +403,8 @@ void StretchPoly (XGridOff, YGridOff, ObjPtr, NumPts, V, Index)
          XCheckMaskEvent (mainDisplay, VisibilityChangeMask, &ev))
       ExposeEventHandler (&ev, TRUE);
 
+   sprintf (buf, "dx=+0,dy=+0");
+   StartShowMeasureCursor (grid_x, grid_y, buf, TRUE);
    XGrabPointer (mainDisplay, drawWindow, False,
          PointerMotionMask | ButtonReleaseMask,
          GrabModeAsync, GrabModeAsync, None, handCursor, CurrentTime);
@@ -315,10 +420,13 @@ void StretchPoly (XGridOff, YGridOff, ObjPtr, NumPts, V, Index)
       {
          XUngrabPointer (mainDisplay, CurrentTime);
          stretching = FALSE;
+         sprintf (buf, "dx=%+1d,dy=%+1d", ABS_SIZE(grid_x-XGridOff),
+               ABS_SIZE(grid_y-YGridOff));
+         EndShowMeasureCursor (grid_x, grid_y, buf, TRUE);
 
          if (dx != 0 || dy != 0)
          {
-            if (curved && splineRubberband)
+            if (splineRubberband)
                XDrawLines (mainDisplay, drawWindow, revDefaultGC, sv, sn,
                      CoordModeOrigin);
             else
@@ -335,6 +443,11 @@ void StretchPoly (XGridOff, YGridOff, ObjPtr, NumPts, V, Index)
                case OBJ_POLY:
                   V[Index].x += dx; V[Index].y += dy;
                   AdjObjSplineVs (ObjPtr);
+                  if (ObjPtr->detail.p->curved != LT_INTSPLINE)
+                     UpdPolyBBox (ObjPtr, NumPts, V);
+                  else
+                     UpdPolyBBox (ObjPtr, ObjPtr->detail.p->intn,
+                           ObjPtr->detail.p->intvlist);
                   break;
                case OBJ_POLYGON:
                   V[Index].x += dx; V[Index].y += dy;
@@ -347,9 +460,32 @@ void StretchPoly (XGridOff, YGridOff, ObjPtr, NumPts, V, Index)
                      V[0].x += dx; V[0].y += dy;
                   }
                   AdjObjSplineVs (ObjPtr);
+                  if (ObjPtr->detail.g->curved != LT_INTSPLINE)
+                     UpdPolyBBox (ObjPtr, NumPts, V);
+                  else
+                     UpdPolyBBox (ObjPtr, ObjPtr->detail.g->intn,
+                           ObjPtr->detail.g->intvlist);
                   break;
             }
-            UpdPolyBBox (ObjPtr, NumPts, V);
+            if (auto_center_attr)
+            {
+               struct AttrRec	* attr_ptr=ObjPtr->fattr;
+               int		modified=FALSE;
+
+               for ( ; attr_ptr != NULL; attr_ptr = attr_ptr->next)
+                  if (attr_ptr->shown)
+                  {
+                     struct BBRec	bbox;
+
+                     CenterObjInOBBox (attr_ptr->obj, ObjPtr->obbox, &bbox);
+                     if (bbox.ltx < ltx) ltx = bbox.ltx;
+                     if (bbox.lty < lty) lty = bbox.lty;
+                     if (bbox.rbx > rbx) rbx = bbox.rbx;
+                     if (bbox.rby > rby) rby = bbox.rby;
+                     modified = TRUE;
+                  }
+               if (modified) AdjObjBBox (ObjPtr);
+            }
             RecordReplaceAnObj (ObjPtr);
 
             UpdSelBBox ();
@@ -364,15 +500,24 @@ void StretchPoly (XGridOff, YGridOff, ObjPtr, NumPts, V, Index)
 
             HighLightForward ();
          }
-         if (curved && splineRubberband) { cfree (sv); cfree (pv); }
+         if (splineRubberband)
+         {
+            cfree (sv);
+            cfree (pv);
+            if (smooth != NULL) cfree (smooth);
+            if (curved == LT_INTSPLINE && cntrlv != NULL) cfree (cntrlv);
+         }
       }
       else if (input.type == MotionNotify)
       {
+         sprintf (buf, "dx=%+1d,dy=%+1d", ABS_SIZE(grid_x-XGridOff),
+               ABS_SIZE(grid_y-YGridOff));
+         ShowMeasureCursor (grid_x, grid_y, buf, TRUE);
          x = input.xmotion.x;
          y = input.xmotion.y;
          GridXY (x, y, &grid_x, &grid_y);
 
-         if (curved && splineRubberband)
+         if (splineRubberband)
             XDrawLines (mainDisplay, drawWindow, revDefaultGC, sv, sn,
                   CoordModeOrigin);
          else
@@ -384,7 +529,7 @@ void StretchPoly (XGridOff, YGridOff, ObjPtr, NumPts, V, Index)
          v[1].y = OFFSET_Y(V[Index].y) + dy;
          MarkRulers (v[1].x, v[1].y);
 
-         if (curved && splineRubberband)
+         if (splineRubberband)
          {
             cfree (sv);
             if (ObjPtr->type==OBJ_POLYGON && (Index==0 || Index==NumPts-1))
@@ -400,12 +545,34 @@ void StretchPoly (XGridOff, YGridOff, ObjPtr, NumPts, V, Index)
             switch (ObjPtr->type)
             {
                case OBJ_POLY:
-                  sv = MakeSplinePolyVertex (&sn, drawOrigX, drawOrigY, NumPts,
-                        pv);
+                  switch (curved)
+                  {
+                     case LT_STRAIGHT:
+                     case LT_SPLINE:
+                        sv = MakeMultiSplinePolyVertex (&sn, smooth,
+                              drawOrigX, drawOrigY, NumPts, pv);
+                        break;
+                     case LT_INTSPLINE:
+                        cfree (cntrlv);
+                        sv = MakeIntSplinePolyVertex (&sn, &intn, &cntrlv,
+                              drawOrigX, drawOrigY, NumPts, pv);
+                        break;
+                  }
                   break;
                case OBJ_POLYGON:
-                  sv = MakeSplinePolygonVertex (&sn, drawOrigX, drawOrigY,
-                        NumPts, pv);
+                  switch (curved)
+                  {
+                     case LT_STRAIGHT:
+                     case LT_SPLINE:
+                        sv = MakeMultiSplinePolygonVertex (&sn, smooth,
+                              drawOrigX, drawOrigY, NumPts, pv);
+                        break;
+                     case LT_INTSPLINE:
+                        cfree (cntrlv);
+                        sv = MakeIntSplinePolygonVertex (&sn, &intn, &cntrlv,
+                              drawOrigX, drawOrigY, NumPts, pv);
+                        break;
+                  }
                   break;
             }
             XDrawLines (mainDisplay, drawWindow, revDefaultGC, sv, sn,
@@ -413,6 +580,9 @@ void StretchPoly (XGridOff, YGridOff, ObjPtr, NumPts, V, Index)
          }
          else
             MyDashedLine (drawWindow, revDefaultGC, v, num);
+         sprintf (buf, "dx=%+1d,dy=%+1d", ABS_SIZE(grid_x-XGridOff),
+               ABS_SIZE(grid_y-YGridOff));
+         ShowMeasureCursor (grid_x, grid_y, buf, TRUE);
          while (XCheckMaskEvent (mainDisplay, PointerMotionMask, &ev)) ;
       }
    }
@@ -505,6 +675,28 @@ void SetPivot (Corner, OBBox)
    moveY = OFFSET_Y(moveY);
 }
 
+static void StretchObj ();
+
+static
+void StretchAttr (ObjPtr, Corner, InsideIcon, AutoCenterAttr)
+   struct ObjRec	* ObjPtr;
+   int			Corner, InsideIcon, AutoCenterAttr;
+{
+   struct AttrRec	* attr_ptr=ObjPtr->fattr;
+
+   if (AutoCenterAttr)
+   {
+      for ( ; attr_ptr != NULL; attr_ptr = attr_ptr->next)
+         if (attr_ptr->shown)
+            CenterObjInOBBox (attr_ptr->obj, ObjPtr->obbox, NULL);
+         else
+            StretchObj (attr_ptr->obj, Corner, InsideIcon);
+   }
+   else
+      for ( ; attr_ptr != NULL; attr_ptr = attr_ptr->next)
+         StretchObj (attr_ptr->obj, Corner, InsideIcon);
+}
+
 static
 void StretchObj (ObjPtr, Corner, InsideIcon)
    struct ObjRec	* ObjPtr;
@@ -512,12 +704,12 @@ void StretchObj (ObjPtr, Corner, InsideIcon)
 {
    register int			i;
    register struct ObjRec	* ptr;
-   struct AttrRec		* attr_ptr;
    struct ArcRec		* arc_ptr;
    int				x, y, dx, dy, new_x, new_y, h;
    int				ltx, lty, rbx, rby, xc, yc, icon_is_rigid=TRUE;
    int				real_ltx, real_lty, real_rbx, real_rby;
    int				* flip_ptr=NULL, * rotate_ptr=NULL;
+   int				auto_center_attr = AutoCenterAttr (ObjPtr);
 
    StretchedAbsXY (ObjPtr->obbox.ltx, ObjPtr->obbox.lty, &ltx, &lty);
    StretchedAbsXY (ObjPtr->obbox.rbx, ObjPtr->obbox.rby, &rbx, &rby);
@@ -620,9 +812,10 @@ void StretchObj (ObjPtr, Corner, InsideIcon)
          ObjPtr->obbox.rbx = rbx;
          ObjPtr->obbox.rby = rby;
          AdjObjSplineVs (ObjPtr);
-         attr_ptr = ObjPtr->fattr;
-         for ( ; attr_ptr != NULL; attr_ptr = attr_ptr->next)
-            StretchObj (attr_ptr->obj, Corner, InsideIcon);
+         if (ObjPtr->detail.p->curved == LT_INTSPLINE)
+            UpdPolyBBox (ObjPtr, ObjPtr->detail.p->intn,
+                  ObjPtr->detail.p->intvlist);
+         StretchAttr (ObjPtr, Corner, InsideIcon, auto_center_attr);
          break;
       case OBJ_ARC:
          arc_ptr =  ObjPtr->detail.a;
@@ -658,25 +851,16 @@ void StretchObj (ObjPtr, Corner, InsideIcon)
             arc_ptr->angle2 = -(arc_ptr->angle2);
          }
          UpdArcBBox (ObjPtr);
-
-         attr_ptr = ObjPtr->fattr;
-         for ( ; attr_ptr != NULL; attr_ptr = attr_ptr->next)
-            StretchObj (attr_ptr->obj, Corner, InsideIcon);
+         StretchAttr (ObjPtr, Corner, InsideIcon, auto_center_attr);
          break;
       case OBJ_RCBOX:
-         attr_ptr = ObjPtr->fattr;
-         for ( ; attr_ptr != NULL; attr_ptr = attr_ptr->next)
-            StretchObj (attr_ptr->obj, Corner, InsideIcon);
+         StretchAttr (ObjPtr, Corner, InsideIcon, auto_center_attr);
          break;
       case OBJ_BOX:
-         attr_ptr = ObjPtr->fattr;
-         for ( ; attr_ptr != NULL; attr_ptr = attr_ptr->next)
-            StretchObj (attr_ptr->obj, Corner, InsideIcon);
+         StretchAttr (ObjPtr, Corner, InsideIcon, auto_center_attr);
          break;
       case OBJ_OVAL:
-         attr_ptr = ObjPtr->fattr;
-         for ( ; attr_ptr != NULL; attr_ptr = attr_ptr->next)
-            StretchObj (attr_ptr->obj, Corner, InsideIcon);
+         StretchAttr (ObjPtr, Corner, InsideIcon, auto_center_attr);
          break;
       case OBJ_TEXT: break;
       case OBJ_POLYGON:
@@ -702,26 +886,23 @@ void StretchObj (ObjPtr, Corner, InsideIcon)
          ObjPtr->obbox.rbx = rbx;
          ObjPtr->obbox.rby = rby;
          AdjObjSplineVs (ObjPtr);
-         attr_ptr = ObjPtr->fattr;
-         for ( ; attr_ptr != NULL; attr_ptr = attr_ptr->next)
-            StretchObj (attr_ptr->obj, Corner, InsideIcon);
+         if (ObjPtr->detail.g->curved == LT_INTSPLINE)
+            UpdPolyBBox (ObjPtr, ObjPtr->detail.g->intn,
+                  ObjPtr->detail.g->intvlist);
+         StretchAttr (ObjPtr, Corner, InsideIcon, auto_center_attr);
          break;
       case OBJ_SYM:
       case OBJ_GROUP:
          for (ptr = ObjPtr->detail.r->first; ptr != NULL; ptr = ptr->next)
             StretchObj (ptr, Corner, InsideIcon);
-         attr_ptr = ObjPtr->fattr;
-         for ( ; attr_ptr != NULL; attr_ptr = attr_ptr->next)
-            StretchObj (attr_ptr->obj, Corner, InsideIcon);
+         StretchAttr (ObjPtr, Corner, InsideIcon, auto_center_attr);
          break;
       case OBJ_ICON:
          if (!icon_is_rigid)
          {
             for (ptr = ObjPtr->detail.r->first; ptr != NULL; ptr = ptr->next)
                StretchObj (ptr, Corner, TRUE);
-            attr_ptr = ObjPtr->fattr;
-            for ( ; attr_ptr != NULL; attr_ptr = attr_ptr->next)
-               StretchObj (attr_ptr->obj, Corner, TRUE);
+            StretchAttr (ObjPtr, Corner, InsideIcon, auto_center_attr);
          }
          break;
       case OBJ_XBM:
@@ -733,6 +914,7 @@ void StretchObj (ObjPtr, Corner, InsideIcon)
             ObjPtr->detail.xbm->cached_zoom = 0;
          else
             ObjPtr->detail.xbm->cached_rotate = INVALID;
+         StretchAttr (ObjPtr, Corner, InsideIcon, auto_center_attr);
          break;
       case OBJ_XPM:
          if (ObjPtr->detail.xpm->cached_pixmap != None)
@@ -743,6 +925,7 @@ void StretchObj (ObjPtr, Corner, InsideIcon)
             ObjPtr->detail.xpm->cached_zoom = 0;
          else
             ObjPtr->detail.xpm->cached_rotate = INVALID;
+         StretchAttr (ObjPtr, Corner, InsideIcon, auto_center_attr);
          break;
    }
    AdjObjBBox (ObjPtr);
@@ -868,7 +1051,6 @@ int ConstrainedStretchAllSel (Corner, ltx, lty, rbx, rby)
                      v[index].x += dx;
                }
             }
-
             if (move_first)
             {
                StretchedAbsXY (v[0].x, v[0].y, &x, &y);
@@ -879,11 +1061,44 @@ int ConstrainedStretchAllSel (Corner, ltx, lty, rbx, rby)
                StretchedAbsXY (v[num_pts-1].x, v[num_pts-1].y, &x, &y);
                v[num_pts-1].x = x; v[num_pts-1].y = y;
             }
-
             AdjObjSplineVs (obj_ptr);
-            UpdPolyBBox (obj_ptr, num_pts, v);
+            switch (obj_ptr->type)
+            {
+               case OBJ_POLY:
+                  if (obj_ptr->detail.p->curved != LT_INTSPLINE)
+                     UpdPolyBBox (obj_ptr, num_pts, v);
+                  else
+                     UpdPolyBBox (obj_ptr, obj_ptr->detail.p->intn,
+                           obj_ptr->detail.p->intvlist);
+                  break;
+               case OBJ_POLYGON:
+                  if (obj_ptr->detail.g->curved != LT_INTSPLINE)
+                     UpdPolyBBox (obj_ptr, num_pts, v);
+                  else
+                     UpdPolyBBox (obj_ptr, obj_ptr->detail.g->intn,
+                           obj_ptr->detail.g->intvlist);
+                  break;
+            }
             AdjObjBBox (obj_ptr);
+            if (AutoCenterAttr (obj_ptr))
+            {
+               struct AttrRec	* attr_ptr=obj_ptr->fattr;
+               int		modified=FALSE;
 
+               for ( ; attr_ptr != NULL; attr_ptr = attr_ptr->next)
+                  if (attr_ptr->shown)
+                  {
+                     struct BBRec	bbox;
+
+                     CenterObjInOBBox (attr_ptr->obj, obj_ptr->obbox, &bbox);
+                     if (bbox.ltx < *ltx) *ltx = bbox.ltx;
+                     if (bbox.lty < *lty) *lty = bbox.lty;
+                     if (bbox.rbx > *rbx) *rbx = bbox.rbx;
+                     if (bbox.rby > *rby) *rby = bbox.rby;
+                     modified = TRUE;
+                  }
+               if (modified) AdjObjBBox (obj_ptr);
+            }
             if (obj_ptr->bbox.ltx < *ltx) *ltx = obj_ptr->bbox.ltx;
             if (obj_ptr->bbox.lty < *lty) *lty = obj_ptr->bbox.lty;
             if (obj_ptr->bbox.rbx > *rbx) *rbx = obj_ptr->bbox.rbx;
@@ -950,6 +1165,19 @@ void StretchAllSel (Corner)
    }
 }
 
+static
+void GetMeasurement (ObjPtr, buf)
+   struct ObjRec	* ObjPtr;
+   char			* buf;
+{
+   int	ltx, lty, rbx, rby, real_ltx, real_lty, real_rbx, real_rby;
+
+   StretchedAbsXY (ObjPtr->obbox.ltx, ObjPtr->obbox.lty, &ltx, &lty);
+   StretchedAbsXY (ObjPtr->obbox.rbx, ObjPtr->obbox.rby, &rbx, &rby);
+   CalcBBox (ltx, lty, rbx, rby, &real_ltx, &real_lty, &real_rbx, &real_rby);
+   sprintf (buf, "%1dx%1d", abs(real_rbx-real_ltx), abs(real_rby-real_lty));
+}
+
 static
 void StretchBox (XGridOff, YGridOff, ObjPtr, Corner)
    int			XGridOff, YGridOff, Corner;
@@ -962,8 +1190,9 @@ void StretchBox (XGridOff, YGridOff, ObjPtr, Corner)
    int		stretched_sel_rby;
    int		ruler_ltx, ruler_lty, ruler_rbx, ruler_rby;
    int		sel_obj_ltx, sel_obj_lty, sel_obj_rbx, sel_obj_rby;
-   double	obj_w, obj_h;
    int		grid_x = XGridOff, grid_y = YGridOff;
+   char		buf[80];
+   double	obj_w, obj_h;
    XEvent	input, ev;
 
    if (numObjSelected == numObjLocked)
@@ -992,7 +1221,6 @@ void StretchBox (XGridOff, YGridOff, ObjPtr, Corner)
    ruler_lty = sel_obj_lty = OFFSET_Y(selObjLtY);
    ruler_rbx = sel_obj_rbx = OFFSET_X(selObjRbX);
    ruler_rby = sel_obj_rby = OFFSET_Y(selObjRbY);
-   BeginIntervalRulers (ruler_ltx, ruler_lty, ruler_rbx, ruler_rby);
 
    stretched_ltx = ltx = OFFSET_X(ObjPtr->obbox.ltx);
    stretched_lty = lty = OFFSET_Y(ObjPtr->obbox.lty);
@@ -1023,6 +1251,10 @@ void StretchBox (XGridOff, YGridOff, ObjPtr, Corner)
    obj_w = (double)(moveX - pivotX);
    obj_h = (double)(moveY - pivotY);
 
+   sprintf (buf, "%1dx%1d", ObjPtr->obbox.rbx-ObjPtr->obbox.ltx,
+         ObjPtr->obbox.rby-ObjPtr->obbox.lty);
+   StartShowMeasureCursor (grid_x, grid_y, buf, TRUE);
+   BeginIntervalRulers (ruler_ltx, ruler_lty, ruler_rbx, ruler_rby);
    XGrabPointer (mainDisplay, drawWindow, False,
          PointerMotionMask | ButtonReleaseMask,
          GrabModeAsync, GrabModeAsync, None, handCursor, CurrentTime);
@@ -1039,6 +1271,8 @@ void StretchBox (XGridOff, YGridOff, ObjPtr, Corner)
          stretching = FALSE;
 
          EndIntervalRulers (grid_x, grid_y);
+         GetMeasurement (ObjPtr, buf);
+         EndShowMeasureCursor (grid_x, grid_y, buf, TRUE);
          SelBox (drawWindow, revDefaultGC, stretched_ltx, stretched_lty,
                stretched_rbx, stretched_rby);
          SelBox (drawWindow, revDefaultGC, stretched_sel_ltx-2,
@@ -1057,6 +1291,9 @@ void StretchBox (XGridOff, YGridOff, ObjPtr, Corner)
       {
          int	proportional = input.xmotion.state & (ShiftMask|ControlMask);
 
+         GetMeasurement (ObjPtr, buf);
+         EndShowMeasureCursor (grid_x, grid_y, buf, TRUE);
+
          x = input.xmotion.x;
          y = input.xmotion.y;
          GridXY (x, y, &grid_x, &grid_y);
@@ -1125,6 +1362,8 @@ void StretchBox (XGridOff, YGridOff, ObjPtr, Corner)
                stretched_sel_lty-2, stretched_sel_rbx+2, stretched_sel_rby+2);
          SelBox (drawWindow, revDefaultGC, stretched_ltx, stretched_lty,
                stretched_rbx, stretched_rby);
+         GetMeasurement (ObjPtr, buf);
+         EndShowMeasureCursor (grid_x, grid_y, buf, TRUE);
          while (XCheckMaskEvent (mainDisplay, PointerMotionMask, &ev)) ;
       }
    }
@@ -1176,6 +1415,7 @@ void ScaleAnEPSObj (ObjPtr, ScalingFactor)
    StretchObj (ObjPtr, CORNER_RB, FALSE);
 }
 
+static
 char * FindColon(s)
    register char	* s;
 {
