@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,10 +25,11 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/text.c,v 2.116 1993/05/18 05:49:14 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/text.c,v 2.133 1994/03/21 04:57:13 william Exp $";
 #endif
 
 #include <stdio.h>
+#include <string.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
@@ -41,12 +43,16 @@ static char RCSid[] =
 #include "color.e"
 #include "copypaste.e"
 #include "cursor.e"
+#include "dialog.e"
 #include "drawing.e"
 #include "dup.e"
 #include "file.e"
 #include "font.e"
 #include "grid.e"
+#include "mainloop.e"
+#include "mark.e"
 #include "msg.e"
+#include "names.e"
 #include "obj.e"
 #include "pattern.e"
 #include "poly.e"
@@ -54,7 +60,12 @@ static char RCSid[] =
 #include "raster.e"
 #include "ruler.e"
 #include "scroll.e"
+#include "select.e"
 #include "setup.e"
+#include "stretch.e"
+#ifndef _NO_EXTERN
+#include "text.e"
+#endif
 #include "xpixmap.e"
 
 #define PAINT 1
@@ -105,9 +116,6 @@ static int	escPressed = FALSE;
 
 #define BLUR 32
 
-extern void	RedrawCurText ();
-extern void	DrawTextObj ();
-
 void BlurText (Win, gc, XOff, YOff, W, H)
    Window	Win;
    GC		gc;
@@ -181,6 +189,7 @@ void PaintText (Win,Str,Just,Rotate,XOff,YOff,xfs,ColorIndex,Pen)
 {
    int		orig_w, orig_h, w, len;
    XGCValues	values;
+   struct BBRec	text_bbox;
 
    if (Pen == NONEPAT) return;
 
@@ -215,6 +224,11 @@ void PaintText (Win,Str,Just,Rotate,XOff,YOff,xfs,ColorIndex,Pen)
          }
          break;
    }
+   text_bbox.ltx = ABS_X(XOff);
+   text_bbox.lty = ABS_Y(YOff);
+   text_bbox.rbx = ABS_X(XOff+w);
+   text_bbox.rby = ABS_Y(YOff+ZOOMED_SIZE(orig_h));
+   if (!BBoxIntersect (text_bbox, drawWinBBox)) return;
 
    values.foreground = colorPixels[ColorIndex];
    values.function = GXcopy;
@@ -238,6 +252,7 @@ void PaintCurText (Win, gc, Str, Just, XOff, YOff, xfs, ColorIndex, Pen,
 {
    int		w, h, left, right, len;
    XGCValues	values;
+   struct BBRec	text_bbox;
 
    len = strlen (Str);
 
@@ -250,6 +265,11 @@ void PaintCurText (Win, gc, Str, Just, XOff, YOff, xfs, ColorIndex, Pen,
       case JUST_C: XOff -= w/2; break;
       case JUST_R: XOff -= w; break;
    }
+   text_bbox.ltx = ABS_X(XOff);
+   text_bbox.lty = ABS_Y(YOff);
+   text_bbox.rbx = ABS_X(XOff+w);
+   text_bbox.rby = ABS_Y(YOff+h);
+   if (!BBoxIntersect (text_bbox, drawWinBBox)) return;
 
    if (Mode & PAINT)
    {
@@ -348,7 +368,6 @@ void NewCurText ()
 
    text_ptr = (struct TextRec *) calloc (1, sizeof(struct TextRec));
    text_ptr->font = curFont;
-   text_ptr->dpi = curFontDPI;
    text_ptr->style = curStyle;
    text_ptr->attr = NULL;
    text_ptr->size = curSize;
@@ -366,6 +385,10 @@ void NewCurText ()
    text_ptr->first = firstStr;
    text_ptr->last = lastStr;
 
+   text_ptr->read_only = FALSE;
+   text_ptr->orig_w = text_ptr->orig_h = 0;
+   text_ptr->font_name = NULL;
+
    curTextObj = (struct ObjRec *) calloc (1, sizeof(struct ObjRec));
    curTextObj->x = textAbsX;
    curTextObj->y = textAbsY;
@@ -404,6 +427,8 @@ void FreeTextObj (ObjPtr)
       if (ObjPtr->detail.t->cached_bitmap != None)
          XFreePixmap (mainDisplay, ObjPtr->detail.t->cached_bitmap);
 
+      if (ObjPtr->detail.t->font_name != NULL)
+         cfree (ObjPtr->detail.t->font_name);
       cfree (ObjPtr->detail.t);
    }
    cfree (ObjPtr);
@@ -637,6 +662,16 @@ int CreateTextObj ()
       UpdateAttr(curTextObj->detail.t, attr_ptr);
       textDrawn = FALSE;
       justDrawnTextObj = NULL;
+      if (AutoCenterAttr (attr_ptr->owner))
+      {
+         struct BBRec	bbox;
+
+         CenterObjInOBBox (attr_ptr->obj, attr_ptr->owner->obbox, &bbox);
+         if (bbox.ltx < ltx) ltx = bbox.ltx;
+         if (bbox.lty < lty) lty = bbox.lty;
+         if (bbox.rbx > rbx) rbx = bbox.rbx;
+         if (bbox.rby > rby) rby = bbox.rby;
+      }
       AdjObjBBox (attr_ptr->owner);
       if (outerSel != NULL) AdjAncestorsBBox ();
 
@@ -815,7 +850,7 @@ void XorText ()
 }
 
 static
-void PrepareEditExistingText (obj_ptr, abs_x, abs_y, x_off, y_off)
+int PrepareEditExistingText (obj_ptr, abs_x, abs_y, x_off, y_off)
    struct ObjRec	* obj_ptr;
    int			abs_x, abs_y, * x_off, * y_off;
 {
@@ -831,14 +866,27 @@ void PrepareEditExistingText (obj_ptr, abs_x, abs_y, x_off, y_off)
    editingText = TRUE;
 
    curFont = obj_ptr->detail.t->font;
-   curFontDPI = obj_ptr->detail.t->dpi;
    curStyle = obj_ptr->detail.t->style;
    curSize = obj_ptr->detail.t->size;
    textJust = obj_ptr->detail.t->just;
    textVSpace = obj_ptr->detail.t->v_space;
    curRotate = obj_ptr->detail.t->rotate;
    penPat = obj_ptr->detail.t->pen;
+   changingFontSizeFromRead = FALSE;
    SetCanvasFont ();
+   changingFontSizeFromRead = TRUE;
+   if (curSize != canvasFontSize)
+   {
+      char	msg[MAXSTRING+1];
+
+      sprintf (msg, "Text size=%1d not available.  Can not edit.", curSize);
+      Dialog (msg, NULL, NULL);
+      PopCurFont ();
+      SetCanvasFont ();
+      editingText = FALSE;
+      curTextModified = FALSE;
+      return (FALSE);
+   }
    ShowJust ();
    ShowPen ();
    colorIndex = obj_ptr->color;
@@ -939,8 +987,20 @@ void PrepareEditExistingText (obj_ptr, abs_x, abs_y, x_off, y_off)
       PrepareToReplaceAnObj (obj_ptr);
    else
       PrepareToReplaceAnObj (obj_ptr->detail.t->attr->owner);
+   return (TRUE);
 }
 
+static
+int PartOfAWord (ch)
+   char	ch;
+{
+   return (ch == '_' || (ch >= '0' && ch <= '9') ||
+         (ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z'));
+}
+
+static Time	lastClickTime;
+static int	textJustClicked=FALSE;
+
 static
 void HandleButton (Button_Ev)
    XButtonEvent	* Button_Ev;
@@ -989,27 +1049,46 @@ void HandleButton (Button_Ev)
       if (!pressed_in_same_text &&
             (obj_ptr = FindTextObj (x_off, y_off)) == NULL)
       {  /* cursor not within any existing text object */
-         GridXY (x_off, y_off, &grid_x, &grid_y);
+         if (curSize != canvasFontSize)
+         {
+            char	msg[MAXSTRING+1];
 
-         textOrigX = textCurX = grid_x;
-         textOrigY = textCurY = grid_y;
-         textAbsX = ABS_X(grid_x);
-         textAbsY = ABS_Y(grid_y);
-         tmpAdjX = tmpAdjY = 0;
-         NewCurText ();
-         RedrawCurText ();
-         CurFontMsg ();
-         PrepareToReplaceAnObj (curTextObj);
+            sprintf (msg, "Text size=%1d not available.  %s.", curSize,
+                  "Please try a different size");
+            Dialog (msg, NULL, NULL);
+         }
+         else
+         {
+            GridXY (x_off, y_off, &grid_x, &grid_y);
+
+            textOrigX = textCurX = grid_x;
+            textOrigY = textCurY = grid_y;
+            textAbsX = ABS_X(grid_x);
+            textAbsY = ABS_Y(grid_y);
+            tmpAdjX = tmpAdjY = 0;
+            NewCurText ();
+            RedrawCurText ();
+            CurFontMsg ();
+            PrepareToReplaceAnObj (curTextObj);
+         }
       }
       else
       {  /* cursor inside an existing text object */
+         Time	click_time=Button_Ev->time;
+         int	double_clicked=FALSE;
+         int	inherited_attr_left_index=(-1);
+
+         double_clicked = (pressed_in_same_text && textJustClicked &&
+               (click_time-lastClickTime) < doubleClickInterval);
          if (pressed_in_same_text)
          {
             obj_ptr = curTextObj;
             curStr = obj_ptr->detail.t->first;
+            if (textJustClicked && (click_time-lastClickTime) < doubleClickInterval)
+               double_clicked = TRUE;
          }
-         else
-            PrepareEditExistingText (obj_ptr, abs_x, abs_y, &x_off, &y_off);
+         else if (!PrepareEditExistingText (obj_ptr,abs_x,abs_y,&x_off,&y_off))
+            return;
 
          textCurY = textOrigY;
          if (pressed_in_same_text)
@@ -1056,10 +1135,24 @@ void HandleButton (Button_Ev)
                s[0] = *c_ptr;
                amount = XTextWidth (canvasFontPtr, s, 1);
 
-               if (x_off < textCurX+(amount>>1))
-                  break;
+               if (double_clicked)
+               {
+                  if (x_off < textCurX+amount)
+                     /* make sure it stops on the left side of the character */
+                     break;
+                  else
+                     textCurX += amount;
+               }
                else
-                  textCurX += amount;
+               {
+                  if (x_off < textCurX+(amount>>1))
+                     /* stops on the right side of the clicked */
+                     /*		character if the cursor is */
+                     /*		closer to the right side */
+                     break;
+                  else
+                     textCurX += amount;
+               }
                textCurIndex++;
             }
          }
@@ -1091,16 +1184,103 @@ void HandleButton (Button_Ev)
 
          attr_ptr = obj_ptr->detail.t->attr;
          if (attr_ptr != NULL && attr_ptr->inherited && textCurY == textOrigY &&
-               attr_ptr->shown && attr_ptr->nameshown &&
-               textCurIndex < strlen (attr_ptr->name))
-         {  /* clicked in the name of an inherited attribute */
-            textCurIndex = strlen (attr_ptr->name);
-            textCurX = left + XTextWidth (canvasFontPtr, attr_ptr->name,
-                  textCurIndex);
+               attr_ptr->shown && attr_ptr->nameshown)
+         {
+            inherited_attr_left_index = strlen (attr_ptr->name);
+            if (textCurIndex < inherited_attr_left_index)
+            {  /* clicked in the name of an inherited attribute */
+               textCurIndex = strlen (attr_ptr->name);
+               textCurX = left + XTextWidth (canvasFontPtr, attr_ptr->name,
+                     textCurIndex);
+            }
          }
          textCursorShown = TRUE;
          textHighlight = FALSE;
+
+         if (double_clicked)
+         {
+            char	cur_char=curStr->s[textCurIndex];
+            int		right_index=strlen(curStr->s);
+
+            highlightStartStr = highlightEndStr = endStr = curStr;
+            highlightStartY = textEndY = textCurY;
+            endStrW = curStrW;
+            textEndIndex = textCurIndex;
+
+            if (PartOfAWord (cur_char))
+            {
+               saved_end_index = textCurIndex;
+
+               if (inherited_attr_left_index != (-1))
+               {
+                  while (textCurIndex > inherited_attr_left_index)
+                     if (PartOfAWord (curStr->s[textCurIndex-1]))
+                        textCurIndex--;
+                     else
+                        break;
+               }
+               else
+               {
+                  while (textCurIndex > 0)
+                     if (PartOfAWord (curStr->s[textCurIndex-1]))
+                        textCurIndex--;
+                     else
+                        break;
+               }
+               SetTextCurX ();
+
+               highlightStartIndex = highlightEndIndex = textCurIndex;
+
+               textEndIndex = saved_end_index+1;
+               while (textEndIndex < right_index)
+                  if (PartOfAWord (curStr->s[textEndIndex]))
+                     textEndIndex++;
+                  else
+                     break;
+
+               highlightEndIndex = textEndIndex;
+            }
+            else if (cur_char != '\0')
+            {
+               saved_end_index = textCurIndex;
+
+               if (inherited_attr_left_index != (-1))
+               {
+                  while (textCurIndex > inherited_attr_left_index)
+                     if (curStr->s[textCurIndex-1] == cur_char)
+                        textCurIndex--;
+                     else
+                        break;
+               }
+               else
+               {
+                  while (textCurIndex > 0)
+                     if (curStr->s[textCurIndex-1] == cur_char)
+                        textCurIndex--;
+                     else
+                        break;
+               }
+               SetTextCurX ();
+
+               highlightStartIndex = highlightEndIndex = textCurIndex;
+
+               textEndIndex = saved_end_index+1;
+               while (textEndIndex < right_index)
+                  if (curStr->s[textEndIndex] == cur_char)
+                     textEndIndex++;
+                  else
+                     break;
+
+               highlightEndIndex = textEndIndex;
+            }
+            textHighlight = !(endStr==curStr && textEndIndex==textCurIndex);
+            RedrawCurText ();
+            textJustClicked = FALSE;
+            return;
+         }
          RedrawCurText ();
+         textJustClicked = TRUE;
+         lastClickTime = click_time;
 
          XGrabPointer (mainDisplay, drawWindow, FALSE,
                PointerMotionMask | ButtonReleaseMask,
@@ -1318,7 +1498,7 @@ int NumLines ()
 }
 
 static
-void DeleteHighlightedText ()
+void JustDeleteHighlightedText ()
 {
    struct StrRec	* s_ptr, * s_ptr1, * new_cur_str = NULL;
    int			highlighting = FALSE, new_cur_index = 0;
@@ -1410,7 +1590,7 @@ void HandleCRLF (ColorIndex)
    escPressed = FALSE;
 
    curTextModified = TRUE;
-   if (textHighlight) DeleteHighlightedText ();
+   if (textHighlight) JustDeleteHighlightedText ();
 
    y = textCurY;
    for (str_ptr = curStr; str_ptr != NULL; str_ptr = str_ptr->next)
@@ -1487,7 +1667,7 @@ void HandleBS (ColorIndex)
    if (textHighlight)
    {
       curTextModified = TRUE;
-      DeleteHighlightedText ();
+      JustDeleteHighlightedText ();
       RedrawCurText ();
 
       if (curTextObj->detail.t->cached_bitmap != None)
@@ -1630,10 +1810,10 @@ void HandleChar (Str, ColorIndex)
 
    if (((*Str)&0x80) && curFont != FONT_SYM && !ValidCharCode (Str)) return;
 
-   if (need_redraw = textHighlight)
+   if ((need_redraw = textHighlight))
    {
       curTextModified = TRUE;
-      DeleteHighlightedText ();
+      JustDeleteHighlightedText ();
    }
 
    if (textCurIndex+strlen (&(curStr->s[textCurIndex])) >= MAXSTRING)
@@ -1743,7 +1923,8 @@ void HandleTAB (key_ev, ColorIndex)
    abs_x = obj_ptr->x; x_off = OFFSET_X(abs_x);
    abs_y = obj_ptr->y; y_off = OFFSET_Y(abs_y);
 
-   PrepareEditExistingText (obj_ptr, abs_x, abs_y, &x_off, &y_off);
+   if (!PrepareEditExistingText (obj_ptr, abs_x, abs_y, &x_off, &y_off))
+      return;
 
    textCurIndex = 0;
    textCurY = textOrigY;
@@ -2010,8 +2191,50 @@ void DrawText (input)
          int	len;
 
          pasteInDrawTextMode = FALSE;
-         cut_buffer = (char *) XFetchBytes (mainDisplay, &len);
-         if (len == 0) { Msg ("Cut buffer is empty."); return; }
+         if (pasteFromFileInDrawTextMode)
+         {
+            FILE	* fp;
+            char	msg[MAXSTRING+1], inbuf[MAXSTRING+1];
+            int		size=0;
+
+            pasteFromFileInDrawTextMode = FALSE;
+            if ((fp = fopen (pasteFromFileName, "r")) == NULL)
+            {
+               sprintf (msg, "Can not open '%s' for read.", pasteFromFileName);
+               Dialog (msg, NULL, NULL);
+               return;
+            }
+            while (fgets (inbuf, MAXSTRING, fp) != NULL) size += strlen (inbuf);
+            fclose (fp);
+            if (size == 0)
+            {
+               sprintf (msg, "File '%s' is empty.", pasteFromFileName);
+               Dialog (msg, NULL, NULL);
+               return;
+            }
+            cut_buffer = (char *) calloc (size+2, sizeof(char));
+            if (cut_buffer == NULL)
+            {
+               sprintf (msg, "Can not malloc %1d bytes.", size+2);
+               Dialog (msg, NULL, NULL);
+               return;
+            }
+            if ((fp = fopen (pasteFromFileName, "r")) == NULL)
+            {
+               sprintf (msg, "Can not open '%s' for read.", pasteFromFileName);
+               Dialog (msg, NULL, NULL);
+               return;
+            }
+            len = 0;
+            while (fgets (&cut_buffer[len], MAXSTRING, fp) != NULL)
+               len += strlen (&cut_buffer[len]);
+            fclose (fp);
+         }
+         else
+         {
+            cut_buffer = (char *) XFetchBytes (mainDisplay, &len);
+            if (len == 0) { Msg ("Cut buffer is empty."); return; }
+         }
 
          if (escPressed)
          {
@@ -2197,6 +2420,23 @@ void DrawText (input)
             }
             break;
       }
+      if (textCursorShown)
+      {
+         PutTextCursor ();
+         MarkRulers (textCurX, textCurY);
+         SetFileModified (TRUE);
+      }
+   }
+}
+
+void DeleteHighLightedText ()
+{
+   if (!textCursorShown) return;
+
+   EraseTextCursor ();
+   HandleBS (colorIndex);
+   if (textCursorShown)
+   {
       PutTextCursor ();
       MarkRulers (textCurX, textCurY);
       SetFileModified (TRUE);
@@ -2253,7 +2493,6 @@ void DrawTextObj (Win, XOff, YOff, ObjPtr)
 
    SaveCurFont ();
    curFont = text_ptr->font;
-   curFontDPI = text_ptr->dpi;
    curStyle = text_ptr->style;
    curSize = text_ptr->size;
    textJust = text_ptr->just;
@@ -2263,7 +2502,7 @@ void DrawTextObj (Win, XOff, YOff, ObjPtr)
 
    pen = text_ptr->pen;
 
-   if ((zoomScale!=0 || curRotate!=ROTATE0) &&
+   if ((zoomScale!=0 || curRotate!=ROTATE0 || text_ptr->read_only) &&
          text_ptr->cached_bitmap==None)
    {
       int	watch_cursor = watchCursorOnMainWindow;
@@ -2314,7 +2553,7 @@ void DrawTextObj (Win, XOff, YOff, ObjPtr)
       XFillRectangle (mainDisplay, Win, drawGC, ltx, lty, rbx-ltx, rby-lty);
    }
 
-   if (zoomScale!=0 || curRotate!=ROTATE0)
+   if (zoomScale!=0 || curRotate!=ROTATE0 || text_ptr->read_only)
    {
       if (ObjPtr->obbox.ltx!=ObjPtr->obbox.rbx &&
             ObjPtr->obbox.lty!=ObjPtr->obbox.rby)
@@ -2359,12 +2598,26 @@ void DrawTextObj (Win, XOff, YOff, ObjPtr)
          y += yinc;
       }
    }
-
-   if (curFontDPI != text_ptr->dpi) text_ptr->dpi = curFontDPI;
    RestoreCurFont ();
 }
 
-#define GETVALUE(val,name) ScanValue("%d", (char *) &(val), name, "text")
+#define FONTS_PER_DPI (((MAXFONTS-1)*MAXFONTSTYLES+1)*MAXFONTSIZES)
+
+static
+int OldFontIndex (dpi_index, font_index, size_index, style_index)
+   register int dpi_index, font_index, size_index, style_index;
+   /* obsoleted procedure, kept to remain compatible with old versions */
+{
+   if (font_index == FONT_SYM)
+      return (size_index+MAXFONTSIZES*(MAXFONTSTYLES*font_index) +
+            dpi_index*FONTS_PER_DPI);
+   else
+      return (size_index+MAXFONTSIZES*(style_index+MAXFONTSTYLES*font_index) +
+            dpi_index*FONTS_PER_DPI);
+}
+
+#define GETVALUE(val,name) ScanValue("%d", &(val), name, "text")
+#define GETSTRNG(val,name) ScanValue("%s", (val), name, "text")
 
 void ReadTextObj (FP, Inbuf, ObjPtr)
    FILE			* FP;
@@ -2374,7 +2627,7 @@ void ReadTextObj (FP, Inbuf, ObjPtr)
    register int		i, max_len = 0, len;
    struct StrRec	* s_ptr;
    struct TextRec	* text_ptr;
-   char			color_str[80], * s, * c_ptr;
+   char			color_str[80], * s, * c_ptr, font_str[80];
    char			* tmp_str, inbuf[MAXSTRING+1], * line;
    int			num_lines, x, y, font, style, size, new_alloc, id=0;
    int			text_just, rotate, pen, cur_size, done, rotation;
@@ -2557,7 +2810,7 @@ void ReadTextObj (FP, Inbuf, ObjPtr)
       }
       if (id >= objId) objId = id+1;
    }
-   else
+   else if (fileVersion <= 29)
    {
       if (GETVALUE (x,         "x") == INVALID ||
           GETVALUE (y,         "y") == INVALID ||
@@ -2583,6 +2836,32 @@ void ReadTextObj (FP, Inbuf, ObjPtr)
       }
       if (id >= objId) objId = id+1;
    }
+   else
+   {
+      if (GETVALUE (x,         "x") == INVALID ||
+          GETVALUE (y,         "y") == INVALID ||
+          GETSTRNG (font_str,  "font_str") == INVALID ||
+          GETVALUE (style,     "style") == INVALID ||
+          GETVALUE (size,      "size") == INVALID ||
+          GETVALUE (num_lines, "num_lines") == INVALID ||
+          GETVALUE (text_just, "text_just") == INVALID ||
+          GETVALUE (rotate,    "rotate") == INVALID ||
+          GETVALUE (pen,       "pen") == INVALID ||
+          GETVALUE (bbox_w,    "bbox w") == INVALID ||
+          GETVALUE (bbox_h,    "bbox h") == INVALID ||
+          GETVALUE (id,        "id") == INVALID ||
+          GETVALUE (dpi,       "dpi") == INVALID ||
+          GETVALUE (asc,       "ascent") == INVALID ||
+          GETVALUE (des,       "descent") == INVALID ||
+          GETVALUE (fill,      "fill") == INVALID ||
+          GETVALUE (v_space,   "vertical spacing") == INVALID ||
+          GETVALUE (rotation,  "rotation") == INVALID ||
+          GETVALUE (locked,    "locked") == INVALID)
+      {
+         return;
+      }
+      if (id >= objId) objId = id+1;
+   }
    fill = UpgradePenFill (fill);
    pen = UpgradePenFill (pen);
 
@@ -2596,10 +2875,60 @@ void ReadTextObj (FP, Inbuf, ObjPtr)
 
    if (!PRTGIF) SaveCurFont ();
 
-   curFont = text_ptr->font = font;
-   curFontDPI = text_ptr->dpi = dpi;
+   if (PRTGIF)
+   {
+      if (fileVersion <= 29)
+      {
+         curFont = text_ptr->font = font;
+         text_ptr->font_name = NULL;
+      }
+      else
+      {
+         len = strlen (font_str);
+         text_ptr->font_name = (char *) calloc (len+1, sizeof(char));
+         if (*font_str == '\'' && font_str[len-1] == '\'')
+         {
+            font_str[len-1] = '\0';
+            strcpy (text_ptr->font_name, &font_str[1]);
+         }
+         else
+            strcpy (text_ptr->font_name, font_str);
+         curFont = text_ptr->font =
+               GetFontIndex (text_ptr->font_name, style, FALSE);
+      }
+   }
+   else
+   {
+      text_ptr->font_name = NULL;
+      if (fileVersion <= 29)
+         curFont = text_ptr->font = font;
+      else
+      {
+         char	*s;
+
+         len = strlen (font_str);
+         if (*font_str == '\'' && font_str[len-1] == '\'')
+         {
+            font_str[len-1] = '\0';
+            s = &font_str[1];
+         }
+         else
+            s = font_str;
+         curFont = text_ptr->font = GetFontIndex (s, style, TRUE);
+         if (curFont == INVALID)
+         {
+            char	msg[MAXSTRING];
+
+            sprintf (msg, "Can not find screen font for '%s'.", s);
+            TwoLineMsg (msg, "    Use Times instead.");
+            SetFileModified (TRUE);
+            curFont = text_ptr->font = FONT_TIM;
+         }
+      }
+   }
    curStyle = text_ptr->style = style;
-   curSize = text_ptr->size = size;
+   curSize = text_ptr->size =
+         (fileVersion<=29 ? GetCompatibleSize (dpi, size) : size);
    textJust = text_ptr->just = text_just;
    textVSpace = text_ptr->v_space = v_space;
    curRotate = text_ptr->rotate = rotate;
@@ -2611,9 +2940,9 @@ void ReadTextObj (FP, Inbuf, ObjPtr)
       if (fileVersion < 10)
       {
          canvasFontAsc =
-               pDrawFontAsc[FontIndex(curFontDPI,curFont,curSize,curStyle)];
+               pDrawFontAsc[OldFontIndex(dpi,curFont,size,curStyle)];
          canvasFontDes =
-               pDrawFontDes[FontIndex(curFontDPI,curFont,curSize,curStyle)];
+               pDrawFontDes[OldFontIndex(dpi,curFont,size,curStyle)];
       }
       else
       {
@@ -2621,13 +2950,37 @@ void ReadTextObj (FP, Inbuf, ObjPtr)
          canvasFontDes = des;
       }
       textCursorH = canvasFontAsc + canvasFontDes;
+      text_ptr->read_only = FALSE;
    }
    else
+   {
       SetCanvasFont ();
-
-   text_ptr->asc = canvasFontAsc;
-   text_ptr->des = canvasFontDes;
-
+      text_ptr->read_only = (curSize != canvasFontSize);
+   }
+   if (text_ptr->read_only)
+   {
+      text_ptr->asc = asc;
+      text_ptr->des = des;
+      switch (rotate)
+      {
+         case ROTATE0:
+         case ROTATE180:
+            text_ptr->orig_w = bbox_w;
+            text_ptr->orig_h = bbox_h;
+            break;
+         case ROTATE90:
+         case ROTATE270:
+            text_ptr->orig_w = bbox_h;
+            text_ptr->orig_h = bbox_w;
+            break;
+      }
+   }
+   else
+   {
+      text_ptr->asc = canvasFontAsc;
+      text_ptr->des = canvasFontDes;
+      text_ptr->orig_w = text_ptr->orig_h = 0;
+   }
    for (i = 0; i < num_lines; i++)
    {
       fgets (inbuf, MAXSTRING, FP);
@@ -2708,7 +3061,22 @@ void ReadTextObj (FP, Inbuf, ObjPtr)
    (*ObjPtr)->y = y;
 
    if (PRTGIF && fileVersion > 6)
-      SetTextBBox (*ObjPtr, text_just, bbox_w, bbox_h, rotate);
+   {
+      switch (rotate)
+      {
+         case ROTATE0:
+         case ROTATE180:
+            SetTextBBox (*ObjPtr, text_just, bbox_w, bbox_h, rotate);
+            break;
+         case ROTATE90:
+         case ROTATE270:
+            SetTextBBox (*ObjPtr, text_just, bbox_h, bbox_w, rotate);
+            break;
+      }
+   }
+   else if (!PRTGIF && text_ptr->read_only)
+      SetTextBBox (*ObjPtr, text_just, text_ptr->orig_w, text_ptr->orig_h,
+            rotate);
    else
       SetTextBBox (*ObjPtr, text_just, max_len,
             num_lines*textCursorH+(num_lines-1)*textVSpace, rotate);
@@ -2721,11 +3089,7 @@ void ReadTextObj (FP, Inbuf, ObjPtr)
    (*ObjPtr)->locked = locked;
    (*ObjPtr)->detail.t = text_ptr;
 
-   if (!PRTGIF)
-   {
-      if (curFontDPI != text_ptr->dpi) text_ptr->dpi = curFontDPI;
-      RestoreCurFont ();
-   }
+   if (!PRTGIF) RestoreCurFont ();
 }
 
 void RedrawCurText ()
