@@ -1,7 +1,7 @@
 %
-% Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+% Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
 %
-% Copyoright (C) 1992, William Cheng.
+% Copyright (C) 1992-1993, William Cheng.
 %
 % Permission limited to the use, copy, modify, and distribute this software
 % and its documentation for any purpose is hereby granted by the Author without
@@ -12,7 +12,8 @@
 % without specific, written prior permission.  The Author makes no
 % representations about the suitability of this software for any purpose.
 % It is provided "as is" without express or implied warranty.  All other
-% rights are reserved by the Author.
+% rights (including the right to sell "tgif" and the right to sell derivative
+% works of tgif) are reserved by the Author.
 %
 % THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 % INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -22,7 +23,7 @@
 % OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 % PERFORMANCE OF THIS SOFTWARE.
 %
-% @(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/tgif.pl,v 2.31 1993/04/22 04:25:02 william Exp $
+% @(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/tgif.pl,v 2.37 1994/05/02 06:08:08 william Exp $
 %
 
 % 1) Call tgif_clean/0 each time before step (2).
@@ -354,7 +355,7 @@ tgif_state(FileVersion,Obj,Parms) :-
 		_TextRotate,_RCBoxRadius,_UseGray,_PageLayoutMode,_PageArg1,
 		_PageArg2,_PageLineShownInTileMode),
 	tgif_chk_output(OutputObj,Obj),
-	FileVersion >= 29,
+	FileVersion =< 29,
 	(	(_PageLayoutMode == 0) -> % stack mode
 		_CurPageNum = _PageArg1, _LastPageNum = _PageArg2,
 		_PaperRow = 1, _PaperCol = 1
@@ -380,6 +381,89 @@ tgif_state(FileVersion,Obj,Parms) :-
 		cur_page_num=_CurPageNum,last_page_num=_LastPageNum],
 	!, abolish(tgif_file_version/1),
 	!, assert(tgif_file_version(FileVersion)).
+tgif_state(FileVersion,Obj,Parms) :-
+	% Mstsuda's Version
+	(	var(Obj) -> OutputObj = true,
+		current_predicate(state,
+			state(_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_))
+	;	OutputObj = false
+	),
+	Obj = state(_PageStyle,FileVersion,_PrintMag,_OrigX,_OrigY,_Zoom,
+		_EnglishGridSize,_Grid,_Color,_HoriAlign,_VertAlign,_LineWidth,
+		_Spline,_LineStyle,_ObjFill,_PenPat,_TextJust,_FontName,
+		_TextStyle,_PointSize,0,_Dash,_GridSystem,
+		_MetricGridSize,_TextVSpace,_ZoomIn,_GridShown,_MoveMode,
+		_TextRotate,_RCBoxRadius,_UseGray,_PageLayoutMode,_PageArg1,
+		_PageArg2,_PageLineShownInTileMode),
+	tgif_chk_output(OutputObj,Obj),
+	FileVersion =< 30,
+	(	(_PageLayoutMode == 0) -> % stack mode
+		_CurPageNum = _PageArg1, _LastPageNum = _PageArg2,
+		_PaperRow = 1, _PaperCol = 1
+	;	_PaperCol = _PageArg1, _PaperRow = _PageArg2,
+		_CurPageNum = 1, _LastPageNum = 1
+	),
+	Parms = [page_style=_PageStyle,file_version=FileVersion,
+		print_mag=_PrintMag,
+		orig_x=_OrigX,orig_y=_OrigY,zoom=_Zoom,
+		english_grid_size=_GridSize,snap_on=_Grid,color=_Color,
+		h_align=_HoriAlign,v_align=_VertAlign,line_width=_LineWidth,
+		line_type=_Spline,
+		line_style=_LineStyle,obj_fill=_ObjFill,pen_pat=_PenPat,
+		text_just=_TextJust,font_name=_FontName,text_style=_TextStyle,
+		point_size=_PointSize,line_dash=_Dash,
+		grid_system=_GridSystem,metric_grid=_MetricGridSize,
+		text_v_space=_TextVSpace,zoom_in=_ZoomIn,grid_shown=_GridShown,
+		move_mode=_ModeMode,text_rotate=_TextRotate,
+		rcb_radius=_RCBoxRadius,use_gray=_UseGray,
+		page_layout_mode=_PageLayoutMode,
+		page_line_shown_in_tile_mode=_PageLineShownInTileMode,
+		papge_col=_PaperCol,papge_row=_PaperRow,
+		cur_page_num=_CurPageNum,last_page_num=_LastPageNum],
+	!, abolish(tgif_file_version/1),
+	!, assert(tgif_file_version(FileVersion)).
+tgif_state(FileVersion,Obj,Parms) :-
+	% Mstsuda's Version
+	(	var(Obj) -> OutputObj = true,
+		current_predicate(state,
+			state(_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_))
+	;	OutputObj = false
+	),
+	Obj = state(_PageStyle,FileVersion,_PrintMag,_OrigX,_OrigY,_Zoom,
+		_EnglishGridSize,_Grid,_Color,_HoriAlign,_VertAlign,_LineWidth,
+		_Spline,_LineStyle,_ObjFill,_PenPat,_TextJust,_FontName,
+		_TextStyle,_PointSize,0,_Dash,_GridSystem,
+		_MetricGridSize,_TextVSpace,_ZoomIn,_GridShown,_MoveMode,
+		_TextRotate,_RCBoxRadius,_UseGray,_PageLayoutMode,_PageArg1,
+		_PageArg2,_PageLineShownInTileMode,_ColorDump),
+	tgif_chk_output(OutputObj,Obj),
+	FileVersion >= 31,
+	(	(_PageLayoutMode == 0) -> % stack mode
+		_CurPageNum = _PageArg1, _LastPageNum = _PageArg2,
+		_PaperRow = 1, _PaperCol = 1
+	;	_PaperCol = _PageArg1, _PaperRow = _PageArg2,
+		_CurPageNum = 1, _LastPageNum = 1
+	),
+	Parms = [page_style=_PageStyle,file_version=FileVersion,
+		print_mag=_PrintMag,
+		orig_x=_OrigX,orig_y=_OrigY,zoom=_Zoom,
+		english_grid_size=_GridSize,snap_on=_Grid,color=_Color,
+		h_align=_HoriAlign,v_align=_VertAlign,line_width=_LineWidth,
+		line_type=_Spline,
+		line_style=_LineStyle,obj_fill=_ObjFill,pen_pat=_PenPat,
+		text_just=_TextJust,font_name=_FontName,text_style=_TextStyle,
+		point_size=_PointSize,line_dash=_Dash,
+		grid_system=_GridSystem,metric_grid=_MetricGridSize,
+		text_v_space=_TextVSpace,zoom_in=_ZoomIn,grid_shown=_GridShown,
+		move_mode=_ModeMode,text_rotate=_TextRotate,
+		rcb_radius=_RCBoxRadius,use_gray=_UseGray,
+		page_layout_mode=_PageLayoutMode,
+		page_line_shown_in_tile_mode=_PageLineShownInTileMode,
+		papge_col=_PaperCol,papge_row=_PaperRow,
+		cur_page_num=_CurPageNum,last_page_num=_LastPageNum,
+		color_dump=_ColorDump],
+	!, abolish(tgif_file_version/1),
+	!, assert(tgif_file_version(FileVersion)).
 
 % --------------------------------------------------------------------- %
 
@@ -499,7 +583,7 @@ tgif_text(Obj,Parms) :-
 tgif_text(Obj,Parms) :-
 	current_predicate(tgif_file_version,tgif_file_version(_)),
 	tgif_file_version(FileVersion),
-	FileVersion >= 26, !,
+	FileVersion =< 29, !,
 	( var(Obj) -> OutputObj = true ; OutputObj = false ),
 	Obj = text(_Color,_X,_Y,_Font,_TextStyle,_TextSize,_NumLines,_TextJust,
 		_TextRotate,_PenPat,_BBoxW,_BBoxH,_Id,_TextDPI,_Asc,_Des,
@@ -512,6 +596,24 @@ tgif_text(Obj,Parms) :-
 		obj_fill=_ObjFill,v_space=_VSpace,rotation=_Rotation,
 		locked=_Locked,strs=StrList],
 	tgif_strs(StrList).
+tgif_text(Obj,Parms) :-
+	current_predicate(tgif_file_version,tgif_file_version(_)),
+	tgif_file_version(FileVersion),
+	FileVersion >= 30, !,
+	% From version 30 on, the PointSize is the actual size;
+	%	the TextSize in older versions is a size index.
+	( var(Obj) -> OutputObj = true ; OutputObj = false ),
+	Obj = text(_Color,_X,_Y,_FontName,_TextStyle,_PointSize,_NumLines,
+		_TextJust,_TextRotate,_PenPat,_BBoxW,_BBoxH,_Id,0,_Asc,_Des,
+		_ObjFill,_VSpace,_Rotation,_Locked,StrList),
+	tgif_chk_output(OutputObj,Obj),
+	Parms = [color=_Color,x=_X,y=_Y,font_name=_FontName,
+		text_style=_TextStyle,point_size=_PointSize,num_lines=_NumLines,
+		text_just=_TextJust,text_rotate=_TextRotate,pen_pat=_PenPat,
+		bbox_w=_BBoxW,bbox_h=_BBoxH,id=_Id,asc=_Asc,des=_Des,
+		obj_fill=_ObjFill,v_space=_VSpace,rotation=_Rotation,
+		locked=_Locked,strs=StrList],
+	tgif_strs(StrList).
 
 % --------------------------------------------------------------------- %
 
@@ -723,7 +825,7 @@ tgif_poly(Obj,Parms) :-
 tgif_poly(Obj,Parms) :-
 	current_predicate(tgif_file_version,tgif_file_version(_)),
 	tgif_file_version(FileVersion),
-	FileVersion >= 26, !,
+	FileVersion =< 30, !,
 	( var(Obj) -> OutputObj = true ; OutputObj = false ),
 	Obj = poly(_Color,_NumVs,_Vs,_LineStyle,_LineWidth,_PenPat,_Id,_Spline,
 		_ObjFill,_Dash,_Rotation,_ArrowHeadW,_ArrowHeadH,_Locked,
@@ -735,6 +837,21 @@ tgif_poly(Obj,Parms) :-
 		arrow_head_w=_ArrowHeadW,arrow_head_h=_ArrowHeadH,
 		locked=_Locked,attrs=AttrList],
 	tgif_attrs(AttrList).
+tgif_poly(Obj,Parms) :-
+	current_predicate(tgif_file_version,tgif_file_version(_)),
+	tgif_file_version(FileVersion),
+	FileVersion >= 31, !,
+	( var(Obj) -> OutputObj = true ; OutputObj = false ),
+	Obj = poly(_Color,_NumVs,_Vs,_LineStyle,_LineWidth,_PenPat,_Id,_Spline,
+		_ObjFill,_Dash,_Rotation,_ArrowHeadW,_ArrowHeadH,_Locked,
+		_SmoothSpecStr,AttrList),
+	tgif_chk_output(OutputObj,Obj),
+	Parms = [color=_Color,num_vs=_NumVs,vs=_Vs,line_style=_LineStyle,
+		line_width=_LineWidth,pen_pat=_PenPat,id=_Id,line_type=_Spilne,
+		obj_fill=_ObjFill,line_dash=_Dash,rotation=_Rotation,
+		arrow_head_w=_ArrowHeadW,arrow_head_h=_ArrowHeadH,
+		locked=_Locked,smooth_spec=_SmoothSpecStr,attrs=AttrList],
+	tgif_attrs(AttrList).
 
 % --------------------------------------------------------------------- %
 
@@ -797,7 +914,7 @@ tgif_polygon(Obj,Parms) :-
 tgif_polygon(Obj,Parms) :-
 	current_predicate(tgif_file_version,tgif_file_version(_)),
 	tgif_file_version(FileVersion),
-	FileVersion >= 26, !,
+	FileVersion =< 30, !,
 	( var(Obj) -> OutputObj = true ; OutputObj = false ),
 	Obj = polygon(_Color,_NumVs,_Vs,_ObjFill,_LineWidth,_PenPat,_Spline,
 		_Id,_Dash,_Rotation,_Locked,AttrList),
@@ -807,6 +924,19 @@ tgif_polygon(Obj,Parms) :-
 		id=_Id,line_dash=_Dash,rotation=_Rotation,locked=_Locked,
 		attrs=AttrList],
 	tgif_attrs(AttrList).
+tgif_polygon(Obj,Parms) :-
+	current_predicate(tgif_file_version,tgif_file_version(_)),
+	tgif_file_version(FileVersion),
+	FileVersion >= 31, !,
+	( var(Obj) -> OutputObj = true ; OutputObj = false ),
+	Obj = polygon(_Color,_NumVs,_Vs,_ObjFill,_LineWidth,_PenPat,_Spline,
+		_Id,_Dash,_Rotation,_Locked,_SmoothSpecStr,AttrList),
+	tgif_chk_output(OutputObj,Obj),
+	Parms = [color=_Color,num_vs=_NumVs,vs=_Vs,obj_fill=_ObjFill,
+		line_width=_LineWidth,pen_pat=_PenPat,line_type=_Spline,
+		id=_Id,line_dash=_Dash,rotation=_Rotation,locked=_Locked,
+		smooth_spec=_SmoothSpecStr,attrs=AttrList],
+	tgif_attrs(AttrList).
 
 % --------------------------------------------------------------------- %
 
