@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,7 +25,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/xbitmap.c,v 2.124 1993/06/03 07:23:29 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/xbitmap.c,v 2.145 1994/05/01 23:41:20 william Exp $";
 #endif
 
 #include <sys/types.h>
@@ -38,12 +39,14 @@ static char RCSid[] =
 #include "const.h"
 #include "types.h"
 
+#include "attr.e"
 #include "auxtext.e"
 #include "choice.e"
 #include "cmd.e"
 #include "color.e"
 #include "cursor.e"
 #include "dialog.e"
+#include "drawing.e"
 #include "dup.e"
 #include "eps.e"
 #include "file.e"
@@ -56,12 +59,40 @@ static char RCSid[] =
 #include "obj.e"
 #include "pattern.e"
 #include "raster.e"
+#include "rect.e"
 #include "select.e"
 #include "setup.e"
+#ifndef _NO_EXTERN
+#include "xbitmap.e"
+#endif
 #include "xpixmap.e"
 
-extern int	rand();
-extern double	atof();
+#ifndef _NO_EXTERN
+#ifdef _AIX
+extern void	srand ARGS_DECL((unsigned int));
+#else /* !_AIX */
+#ifdef __alpha
+extern void	srand ARGS_DECL((unsigned int));
+#else /* !__alpha */
+#ifdef sgi
+extern void	srand ARGS_DECL((unsigned int));
+#endif /* sgi */
+extern int	srand ARGS_DECL((int));
+#endif /* __alpha */
+#endif /* _AIX */
+#endif /* !_NO_EXTERN */
+
+extern int	rand ARGS_DECL((void));
+
+#ifdef __hpux
+extern double	atof ARGS_DECL((const char *));
+#else /* !__hpux */
+#ifndef _floatingpoint_h
+extern double	atof ARGS_DECL((char *));
+#endif /* !_floatingpoint_h */
+#endif /* __hpux */
+
+extern int	atoi ARGS_DECL((char *));
 
 GC	xbmGC = NULL;
 
@@ -1090,45 +1121,141 @@ void DumpXPmColors (FP)
       {
          for (i = 0; i < numColorsToDump-1; i++)
          {
-            fprintf (FP, "\"");
+            if (fprintf (FP, "\"") == EOF) writeFileFailed = TRUE;
             for (j = 0; j < charsPerPixel; j++)
-               fprintf (FP, "%c", colorChar[i*charsPerPixel+j]);
+               if (fprintf (FP, "%c", colorChar[i*charsPerPixel+j]) == EOF)
+                  writeFileFailed = TRUE;
             ColorStrToXPmStr (i, s);
-            fprintf (FP, "\", \"%s\"\n", s);
+            if (fprintf (FP, "\", \"%s\"\n", s) == EOF) writeFileFailed = TRUE;
          }
-         fprintf (FP, "\"");
+         if (fprintf (FP, "\"") == EOF) writeFileFailed = TRUE;
          for (j = 0; j < charsPerPixel; j++)
-            fprintf (FP, "%c", colorChar[i*charsPerPixel+j]);
+            if (fprintf (FP, "%c", colorChar[i*charsPerPixel+j]) == EOF)
+               writeFileFailed = TRUE;
          ColorStrToXPmStr (i, s);
-         fprintf (FP, "\", \"%s\"\n} ;\n", s);
+         if (fprintf (FP, "\", \"%s\"\n} ;\n", s) == EOF)
+            writeFileFailed = TRUE;
       }
       else
       {
          for (i = 0; i < numColorsToDump-1; i++)
          {
-            fprintf (FP, "   \"");
+            if (fprintf (FP, "   \"") == EOF) writeFileFailed = TRUE;
             for (j = 0; j < charsPerPixel; j++)
-               fprintf (FP, "%c", colorChar[i*charsPerPixel+j]);
-            fprintf (FP, "\", \"%s\",\n", colorStr[i]);
+               if (fprintf (FP, "%c", colorChar[i*charsPerPixel+j]) == EOF)
+                  writeFileFailed = TRUE;
+            if (fprintf (FP, "\", \"%s\",\n", colorStr[i]) == EOF)
+               writeFileFailed = TRUE;
          }
-         fprintf (FP, "   \"");
+         if (fprintf (FP, "   \"") == EOF) writeFileFailed = TRUE;
          for (j = 0; j < charsPerPixel; j++)
-            fprintf (FP, "%c", colorChar[i*charsPerPixel+j]);
-         fprintf (FP, "\", \"%s\"\n};\n", colorStr[i]);
+            if (fprintf (FP, "%c", colorChar[i*charsPerPixel+j]) == EOF)
+               writeFileFailed = TRUE;
+         if (fprintf (FP, "\", \"%s\"\n};\n", colorStr[i]) == EOF)
+            writeFileFailed = TRUE;
       }
    }
    else
    {  /* xpmOutputVersion is 3 */
       for (i = 0; i < numColorsToDump; i++)
       {
-         fprintf (FP, "\"");
+         if (fprintf (FP, "\"") == EOF) writeFileFailed = TRUE;
          for (j = 0; j < charsPerPixel; j++)
-            fprintf (FP, "%c", colorChar[i*charsPerPixel+j]);
-         fprintf (FP, " c %s\",\n", colorStr[i]);
+            if (fprintf (FP, "%c", colorChar[i*charsPerPixel+j]) == EOF)
+               writeFileFailed = TRUE;
+         if (fprintf (FP, " c %s\",\n", colorStr[i]) == EOF)
+            writeFileFailed = TRUE;
       }
    }
 }
 
+static
+void MapPixToEPSI(SrcX, SrcY, SrcW, SrcH, DestW, DestH, DestX, DestY)
+   int	SrcX, SrcY, SrcW, SrcH, DestW, DestH, * DestX, * DestY;
+{
+   *DestX = (int)round((((float)DestW)*((float)SrcX))/((float)SrcW));
+   *DestY = (int)round((((float)DestH)*((float)SrcY))/((float)SrcH));
+   if (*DestX >= DestW) *DestX = DestW-1; if (*DestX < 0) *DestX = 0;
+   if (*DestY >= DestH) *DestY = DestH-1; if (*DestY < 0) *DestY = 0;
+}
+
+void GenPreviewBitmap (FP, llxPage, llyPage, urxPage, uryPage)
+   FILE	* FP;
+   int	llxPage, llyPage, urxPage, uryPage;
+{
+   register int	col, row;
+   int		ltx, lty, w, h, * * data, num_image_bytes_per_row, x, y;
+   int		image_w=urxPage-llxPage, image_h=uryPage-llyPage, num_lines;
+   Pixmap	pixmap;
+   XImage	* image;
+
+   if (image_w == 0 || image_h == 0) return;
+   if ((pixmap = DrawAllOnPixmap (&ltx, &lty, &w, &h)) == None) return;
+
+   image = XGetImage (mainDisplay, pixmap, 0, 0, w, h, AllPlanes, ZPixmap);
+   if (image == NULL)
+   {
+      Dialog ("Can not generate image, print aborted!", NULL, NULL);
+      XFreePixmap (mainDisplay, pixmap);
+      return;
+   }
+   if (image_w < 0) image_w = (-image_w);
+   if (image_h < 0) image_h = (-image_h);
+   num_image_bytes_per_row = ((image_w & 0x7) ? (image_w>>3)+1 : (image_w>>3));
+   num_lines = ((image_w & 0x7) ? (((image_w>>3)+1)<<1) : ((image_w>>3)<<1));
+   num_lines = ((num_lines & 0x3f) ? (num_lines>>6)+1 : (num_lines>>6));
+
+   if ((data=(int**)calloc(image_h,sizeof(int*))) == NULL) return;
+   for (row=0; row < image_h; row++)
+      if ((data[row]=(int*)calloc(num_image_bytes_per_row,sizeof(int))) == NULL)
+         return;
+      else
+         for (col=0; col<num_image_bytes_per_row; col++)
+            data[row][col] = 0;
+
+   fprintf (FP, "%%%%BeginPreview: %1d %1d 1 %1d\n", image_w, image_h,
+         num_lines*image_h);
+   for (row=0; row < h; row++)
+   {
+      int	bit_count=0;
+
+      for (col=0; col < w; col++)
+      {
+         if (XGetPixel (image, col, row) != myBgPixel)
+         {
+            MapPixToEPSI(col, row, w, h, image_w, image_h, &x, &y);
+            data[y][x>>3] |= (1<<(7 - (x & 0x7)));
+         }
+         if (++bit_count == 8) bit_count = 0;
+      }
+   }
+   for (row=0; row < image_h; row++)
+   {
+      int	byte_count=0;
+
+      fprintf (FP, "%% ");
+      for (col=0; col < num_image_bytes_per_row; col++)
+      {
+         if (byte_count++ == 32)
+         {
+            byte_count = 1;
+            fprintf (FP, "\n%% ");
+         }
+         fprintf (FP, "%c", hexValue[(data[row][col]>>4) & 0xf]);
+         fprintf (FP, "%c", hexValue[data[row][col] & 0xf]);
+      }
+      fprintf (FP, "\n");
+   }
+   fprintf (FP, "%%%%EndImage\n");
+   fprintf (FP, "%%%%EndPreview\n");
+
+   for (row=0; row < image_h; row++)
+      cfree (data[row]);
+   cfree (data);
+   XDestroyImage (image);
+   XFreePixmap (mainDisplay, pixmap);
+}
+
 #define FS_SCALE 0x400
 #define HALF_FS_SCALE 0x200
 
@@ -1156,7 +1283,7 @@ void DumpXBitmapFile ()
       sprintf (&xbm_file_name[j], ".%s", XBM_FILE_EXT);
 
    ModifyOutputFileName (xbm_file_name);
-   if (short_name = IsPrefix (bootDir, xbm_file_name, &rest)) ++rest;
+   if ((short_name = IsPrefix (bootDir, xbm_file_name, &rest))) ++rest;
    if ((fp = fopen (xbm_file_name, "w")) == NULL)
    {
       if (short_name)
@@ -1176,6 +1303,7 @@ void DumpXBitmapFile ()
       sprintf (msg, "Printing into '%s'...", xbm_file_name);
    Msg (msg);
 
+   writeFileFailed = FALSE;
    if (colorDump)
    {
       BuildXPmColors ();
@@ -1228,7 +1356,7 @@ void DumpXBitmapFile ()
       BuildXPmBuckets (numColorsToDump, pixelValue, INVALID, NULL);
       for (i = 0; i < h; i++)
       {
-         fprintf (fp, "\"");
+         if (fprintf (fp, "\"") == EOF) writeFileFailed = TRUE;
          for (j = 0; j < w; j++)
          {
             data = XGetPixel (image,j,i);
@@ -1236,8 +1364,12 @@ void DumpXBitmapFile ()
             {
                switch (charsPerPixel)
                {
-                  case 1: fprintf (fp, "`"); break;
-                  case 2: fprintf (fp, "``"); break;
+                  case 1:
+                     if (fprintf (fp, "`") == EOF) writeFileFailed = TRUE;
+                     break;
+                  case 2:
+                     if (fprintf (fp, "``") == EOF) writeFileFailed = TRUE;
+                     break;
                }
             }
             else
@@ -1255,18 +1387,25 @@ void DumpXBitmapFile ()
                   return;
                }
                for (k = 0; k < charsPerPixel; k++)
-                  fprintf (fp, "%c", colorChar[index*charsPerPixel+k]);
+                  if (fprintf (fp,"%c",colorChar[index*charsPerPixel+k]) == EOF)
+                     writeFileFailed = TRUE;
             }
          }
          if (i == h-1)
          {
             if (xpmInXGrabSCFormat)
-               fprintf (fp, "\",\n} ;\n");
+            {
+               if (fprintf (fp, "\",\n} ;\n") == EOF)
+                  writeFileFailed = TRUE;
+            }
             else
-               fprintf (fp, "\"\n};\n");
+            {
+               if (fprintf (fp, "\"\n};\n") == EOF)
+                  writeFileFailed = TRUE;
+            }
          }
-         else
-            fprintf (fp, "\",\n");
+         else if (fprintf (fp, "\",\n") == EOF)
+            writeFileFailed = TRUE;
       }
    }
    else
@@ -1373,14 +1512,21 @@ void DumpXBitmapFile ()
                   if (byte_count++ == 12)
                   {
                      byte_count = 1;
-                     fprintf (fp, "\n   ");
+                     if (fprintf (fp, "\n   ") == EOF) writeFileFailed = TRUE;
                   }
-                  fprintf (fp, "0x%c", hexValue[(data>>4) & 0xf]);
+                  if (fprintf (fp, "0x%c", hexValue[(data>>4) & 0xf]) == EOF)
+                     writeFileFailed = TRUE;
 
                   if (i == h-1 && j == w-1)
-                     fprintf (fp, "%c};\n", hexValue[data & 0xf]);
+                  {
+                     if (fprintf (fp, "%c};\n", hexValue[data & 0xf]) == EOF)
+                        writeFileFailed = TRUE;
+                  }
                   else
-                     fprintf (fp, "%c, ", hexValue[data & 0xf]);
+                  {
+                     if (fprintf (fp, "%c, ", hexValue[data & 0xf]) == EOF)
+                        writeFileFailed = TRUE;
+                  }
 
                   bit_count = 0;
                   data = 0;
@@ -1391,15 +1537,22 @@ void DumpXBitmapFile ()
                if (byte_count++ == 12)
                {
                   byte_count = 1;
-                  fprintf (fp, "\n   ");
+                  if (fprintf (fp, "\n   ") == EOF) writeFileFailed = TRUE;
                }
                for (j = (w%8); j < 8; j++) data |= (1<<j);
 
-               fprintf (fp, "0x%c", hexValue[(data>>4) & 0xf]);
+               if (fprintf (fp, "0x%c", hexValue[(data>>4) & 0xf]) == EOF)
+                  writeFileFailed = TRUE;
                if (i == h-1)
-                  fprintf (fp, "%c};\n", hexValue[data & 0xf]);
+               {
+                  if (fprintf (fp, "%c};\n", hexValue[data & 0xf]) == EOF)
+                     writeFileFailed = TRUE;
+               }
                else
-                  fprintf (fp, "%c, ", hexValue[data & 0xf]);
+               {
+                  if (fprintf (fp, "%c, ", hexValue[data & 0xf]) == EOF)
+                     writeFileFailed = TRUE;
+               }
             }
          }
       }
@@ -1461,14 +1614,21 @@ void DumpXBitmapFile ()
                   if (byte_count++ == 12)
                   {
                      byte_count = 1;
-                     fprintf (fp, "\n   ");
+                     if (fprintf (fp, "\n   ") == EOF) writeFileFailed = TRUE;
                   }
-                  fprintf (fp, "0x%c", hexValue[(data>>4) & 0xf]);
+                  if (fprintf (fp, "0x%c", hexValue[(data>>4) & 0xf]) == EOF)
+                     writeFileFailed = TRUE;
 
                   if (i == h-1 && j == w-1)
-                     fprintf (fp, "%c};\n", hexValue[data & 0xf]);
+                  {
+                     if (fprintf (fp, "%c};\n", hexValue[data & 0xf]) == EOF)
+                        writeFileFailed = TRUE;
+                  }
                   else
-                     fprintf (fp, "%c, ", hexValue[data & 0xf]);
+                  {
+                     if (fprintf (fp, "%c, ", hexValue[data & 0xf]) == EOF)
+                        writeFileFailed = TRUE;
+                  }
 
                   bit_count = 0;
                   data = 0;
@@ -1479,15 +1639,22 @@ void DumpXBitmapFile ()
                if (byte_count++ == 12)
                {
                   byte_count = 1;
-                  fprintf (fp, "\n   ");
+                  if (fprintf (fp, "\n   ") == EOF) writeFileFailed = TRUE;
                }
                for (j = (w%8); j < 8; j++) data |= (1<<j);
 
-               fprintf (fp, "0x%c", hexValue[(data>>4) & 0xf]);
+               if (fprintf (fp, "0x%c", hexValue[(data>>4) & 0xf]) == EOF)
+                  writeFileFailed = TRUE;
                if (i == h-1)
-                  fprintf (fp, "%c};\n", hexValue[data & 0xf]);
+               {
+                  if (fprintf (fp, "%c};\n", hexValue[data & 0xf]) == EOF)
+                     writeFileFailed = TRUE;
+               }
                else
-                  fprintf (fp, "%c, ", hexValue[data & 0xf]);
+               {
+                  if (fprintf (fp, "%c, ", hexValue[data & 0xf]) == EOF)
+                     writeFileFailed = TRUE;
+               }
             }
          }
       }
@@ -1497,21 +1664,31 @@ void DumpXBitmapFile ()
    XDestroyImage (image);
    XFreePixmap (mainDisplay, pixmap);
 
-   if (short_name)
+   if (writeFileFailed)
    {
-      if (colorDump)
-         sprintf (msg, "XPM file printed into '%s'.", rest);
-      else
-         sprintf (msg, "XBM file printed into '%s'.", rest);
+      writeFileFailed = FALSE;
+      sprintf (msg, "Fail to write to '%s'.  File system may be full.",
+            xbm_file_name);
+      Dialog (msg, NULL, NULL);
    }
    else
    {
-      if (colorDump)
-         sprintf (msg, "XPM file printed into '%s'.", xbm_file_name);
+      if (short_name)
+      {
+         if (colorDump)
+            sprintf (msg, "XPM file printed into '%s'.", rest);
+         else
+            sprintf (msg, "XBM file printed into '%s'.", rest);
+      }
       else
-         sprintf (msg, "XBM file printed into '%s'.", xbm_file_name);
+      {
+         if (colorDump)
+            sprintf (msg, "XPM file printed into '%s'.", xbm_file_name);
+         else
+            sprintf (msg, "XBM file printed into '%s'.", xbm_file_name);
+      }
+      Msg (msg);
    }
-   Msg (msg);
 }
 
 void DumpXBmObj (FP, ObjPtr)
@@ -1643,7 +1820,7 @@ void DumpXBmObj (FP, ObjPtr)
          fprintf (FP, "%1d %1d lineto\n", ltx, rby);
          if (colorDump)
          {
-            fprintf (FP, "   closepath eoclip\n");
+            fprintf (FP, "   closepath eoclip newpath\n");
             DumpPatFill (FP, fill, 8, ObjPtr->bbox, "   ");
          }
          else
@@ -1978,9 +2155,9 @@ void DumpXBmData (FP, bitmap, image, W, H)
             if (nibble_count++ == 64)
             {
                nibble_count = 1;
-               fprintf (FP, "\n     ");
+               if (fprintf (FP, "\n     ") == EOF) writeFileFailed = TRUE;
             }
-            fprintf (FP, "%c", hexValue[data]);
+            if (fprintf (FP, "%c", hexValue[data]) == EOF) writeFileFailed = TRUE;
             bit_count = 0;
             data = 0;
          }
@@ -1991,9 +2168,9 @@ void DumpXBmData (FP, bitmap, image, W, H)
          if (nibble_count++ == 64)
          {
             nibble_count = 1;
-            fprintf (FP, "\n     ");
+            if (fprintf (FP, "\n     ") == EOF) writeFileFailed = TRUE;
          }
-         fprintf (FP, "%c", hexValue[data]);
+         if (fprintf (FP, "%c", hexValue[data]) == EOF) writeFileFailed = TRUE;
       }
    }
 }
@@ -2022,38 +2199,47 @@ void SaveXBmObj (FP, ObjPtr)
       image_w = xbm_ptr->image_w;
       image_h = xbm_ptr->image_h;
    }
-   fprintf (FP, "xbm('%s',", colorMenuItems[ObjPtr->color]);
-   fprintf (FP, "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,\n    ",
+   if (fprintf (FP, "xbm('%s',", colorMenuItems[ObjPtr->color]) == EOF)
+      writeFileFailed = TRUE;
+   if (fprintf (FP,
+         "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,\n    ",
          ltx, lty, rbx, rby, xbm_ptr->fill, ObjPtr->id, ObjPtr->rotation,
          image_w, image_h, xbm_ptr->rotate, xbm_ptr->flip, xbm_ptr->real_type,
          xbm_ptr->llx, xbm_ptr->lly, xbm_ptr->urx, xbm_ptr->ury,
-         no_bitmap, ObjPtr->locked, xbm_ptr->save_epsf);
+         no_bitmap, ObjPtr->locked, xbm_ptr->save_epsf) == EOF)
+      writeFileFailed = TRUE;
    switch (xbm_ptr->real_type)
    {
-      case XBM_XBM: fprintf (FP, "\"\",\"\","); break;
+      case XBM_XBM:
+         if (fprintf (FP, "\"\",\"\",") == EOF) writeFileFailed = TRUE;
+         break;
       case XBM_EPS:
-         fprintf (FP, "\"%s\",\"%s\",", xbm_ptr->write_date, xbm_ptr->filename);
+         if (fprintf (FP, "\"%s\",\"%s\",", xbm_ptr->write_date,
+               xbm_ptr->filename) == EOF)
+            writeFileFailed = TRUE;
          break;
    }
    if (xbm_ptr->save_epsf)
    {
-      fprintf (FP, "%1d,[", xbm_ptr->num_epsf_lines);
+      if (fprintf (FP, "%1d,[", xbm_ptr->num_epsf_lines) == EOF)
+         writeFileFailed = TRUE;
       for (i = 0; i < xbm_ptr->num_epsf_lines; i++)
-         fprintf (FP, "\n    \"%s\"%s,", xbm_ptr->epsflines[i],
-               (i==xbm_ptr->num_epsf_lines-1 ? "]" : ""));
+         if (fprintf (FP, "\n    \"%s\"%s,", xbm_ptr->epsflines[i],
+               (i==xbm_ptr->num_epsf_lines-1 ? "]" : "")) == EOF)
+            writeFileFailed = TRUE;
    }
    if (!no_bitmap)
    {
-      fprintf (FP, "\n    \"");
+      if (fprintf (FP, "\n    \"") == EOF) writeFileFailed = TRUE;
       DumpXBmData (FP, xbm_ptr->bitmap, &(xbm_ptr->image), image_w, image_h);
-      fprintf (FP, "\",");
+      if (fprintf (FP, "\",") == EOF) writeFileFailed = TRUE;
    }
 
    SaveAttrs (FP, ObjPtr->lattr);
-   fprintf (FP, ")");
+   if (fprintf (FP, ")") == EOF) writeFileFailed = TRUE;
 }
 
-#define GETVALUE(val,name) ScanValue("%d", (char *) &(val), name, "xbitmap")
+#define GETVALUE(val,name) ScanValue("%d", &(val), name, "xbitmap")
 
 void ReadXBmObj (FP, Inbuf, ObjPtr)
    FILE			* FP;
