@@ -1,8 +1,8 @@
 /*
- * Author:	William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ *
+ * Copyright (C) 1990-1994, William Cheng.
  *
- * Copyright (C) 1990, 1991, 1992, William Cheng.
- * 
  * Permission limited to the use, copy, modify, and distribute this software
  * and its documentation for any purpose is hereby granted by the Author without
  * fee, provided that the above copyright notice appear in all copies and
@@ -12,7 +12,8 @@
  * without specific, written prior permission.  The Author makes no
  * representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.  All other
- * rights are reserved by the Author.
+ * rights (including the right to sell "tgif" and the right to sell derivative
+ * works of tgif) are reserved by the Author.
  *
  * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
  * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
@@ -24,20 +25,23 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /amnt/kona/tangram/u/william/X11/TGIF2/RCS/xpixmap.c,v 2.126 1993/06/03 07:23:56 william Exp $";
+      "@(#)$Header: /amnt/maui/tangram/u/william/X11/TGIF2/RCS/xpixmap.c,v 2.140 1994/04/19 05:04:51 william Exp $";
 #endif
 
 #include <stdio.h>
+#include <string.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include "const.h"
 #include "types.h"
 
+#include "attr.e"
 #include "choice.e"
 #include "cmd.e"
 #include "color.e"
 #include "cursor.e"
 #include "dialog.e"
+#include "drawing.e"
 #include "dup.e"
 #include "file.e"
 #include "font.e"
@@ -48,9 +52,13 @@ static char RCSid[] =
 #include "obj.e"
 #include "pattern.e"
 #include "raster.e"
+#include "rect.e"
 #include "select.e"
 #include "setup.e"
 #include "xbitmap.e"
+#ifndef _NO_EXTERN
+#include "xpixmap.e"
+#endif
 
 struct BucketRec {
    int	pixel, index;
@@ -598,13 +606,15 @@ void SaveXPmColors (FP, ncolors, chars_per_pixel, color_char, color_str, pixels)
             break;
          }
 
-      fprintf (FP, "   \"");
+      if (fprintf (FP, "   \"") == EOF) writeFileFailed = TRUE;
       for (j = 0; j < chars_per_pixel; j++)
-         fprintf (FP, "%c", color_char[i*chars_per_pixel+j]);
-      fprintf (FP, "\", \"%s\", %1d, %1d, %1d,\n", color_str[i],
+         if (fprintf (FP, "%c", color_char[i*chars_per_pixel+j]) == EOF)
+            writeFileFailed = TRUE;
+      if (fprintf (FP, "\", \"%s\", %1d, %1d, %1d,\n", color_str[i],
             (int)(10000*tgifColors[found_index].red/maxRGB),
             (int)(10000*tgifColors[found_index].green/maxRGB),
-            (int)(10000*tgifColors[found_index].blue/maxRGB));
+            (int)(10000*tgifColors[found_index].blue/maxRGB)) == EOF)
+         writeFileFailed = TRUE;
    }
    found_index = 0;
    cur_pixel = pixels[i];
@@ -615,13 +625,15 @@ void SaveXPmColors (FP, ncolors, chars_per_pixel, color_char, color_str, pixels)
          break;
       }
 
-   fprintf (FP, "   \"");
+   if (fprintf (FP, "   \"") == EOF) writeFileFailed = TRUE;
    for (j = 0; j < chars_per_pixel; j++)
-      fprintf (FP, "%c", color_char[i*chars_per_pixel+j]);
-   fprintf (FP, "\", \"%s\", %1d, %1d, %1d],[\n", color_str[i],
+      if (fprintf (FP, "%c", color_char[i*chars_per_pixel+j]) == EOF)
+         writeFileFailed = TRUE;
+   if (fprintf (FP, "\", \"%s\", %1d, %1d, %1d],[\n", color_str[i],
          (int)(10000*tgifColors[found_index].red/maxRGB),
          (int)(10000*tgifColors[found_index].green/maxRGB),
-         (int)(10000*tgifColors[found_index].blue/maxRGB));
+         (int)(10000*tgifColors[found_index].blue/maxRGB)) == EOF)
+      writeFileFailed = TRUE;
 }
 
 void DumpXPmObj (FP, ObjPtr)
@@ -1111,7 +1123,7 @@ void SaveXPmPixels (FP,pixmap,image,w,h,ncolors,chars_per_pixel,color_char,
    }
    for (i = 0; i < h; i++)
    {
-      fprintf (FP, "   \"");
+      if (fprintf (FP, "   \"") == EOF) writeFileFailed = TRUE;
       for (j = 0; j < w; j++)
       {
          data = XGetPixel (*image,j,i);
@@ -1124,12 +1136,17 @@ void SaveXPmPixels (FP,pixmap,image,w,h,ncolors,chars_per_pixel,color_char,
             return;
          }
          for (k = 0; k < chars_per_pixel; k++)
-            fprintf (FP, "%c", color_char[index*chars_per_pixel+k]);
+            if (fprintf (FP, "%c", color_char[index*chars_per_pixel+k]) == EOF)
+               writeFileFailed = TRUE;
       }
       if (i == h-1)
-         fprintf (FP, "\"],");
+      {
+         if (fprintf (FP, "\"],") == EOF) writeFileFailed = TRUE;
+      }
       else
-         fprintf (FP, "\",\n");
+      {
+         if (fprintf (FP, "\",\n") == EOF) writeFileFailed = TRUE;
+      }
    }
 }
 
@@ -1154,12 +1171,14 @@ void SaveXPmObj (FP, ObjPtr)
    color_str = xpm_ptr->color_str;
    pixels = xpm_ptr->pixels;
 
-   fprintf (FP, "xpm('%s',", colorMenuItems[ObjPtr->color]);
-   fprintf (FP,
+   if (fprintf (FP, "xpm('%s',", colorMenuItems[ObjPtr->color]) == EOF)
+      writeFileFailed = TRUE;
+   if (fprintf (FP,
          "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,[\n",
          ltx, lty, rbx, rby, fill, ncolors, chars_per_pixel, first_pixel_is_bg,
          ObjPtr->id, ObjPtr->rotation, image_w, image_h, xpm_ptr->rotate,
-         xpm_ptr->flip, ObjPtr->locked);
+         xpm_ptr->flip, ObjPtr->locked) == EOF)
+      writeFileFailed = TRUE;
 
    BuildXPmBuckets (ncolors, pixels, INVALID, NULL);
    SaveXPmColors (FP, ncolors, chars_per_pixel, color_char, color_str, pixels);
@@ -1167,7 +1186,21 @@ void SaveXPmObj (FP, ObjPtr)
          ncolors, chars_per_pixel, color_char, pixels);
 
    SaveAttrs (FP, ObjPtr->lattr);
-   fprintf (FP, ")");
+   if (fprintf (FP, ")") == EOF) writeFileFailed = TRUE;
+}
+
+int MyStrICmp (Str1, Str2)
+   char	* Str1, * Str2;
+{
+   register char	c1, c2, * s1, * s2;
+
+   for (s1=Str1, s2=Str2; *s1 != '\0'; s1++, s2++)
+   {
+      c1 = (*s1 >= 'a' && *s1 <= 'z' ? *s1-'a'+'A' : *s1);
+      c2 = (*s2 >= 'a' && *s2 <= 'z' ? *s2-'a'+'A' : *s2);
+      if (c1 != c2) return (*s1 - *s2);
+   }
+   return (*s2 == '\0' ? 0 : (-(*s2)));
 }
 
 int QuickFindColorIndex (s, new_alloc)
@@ -1207,7 +1240,7 @@ int QuickFindColorIndex (s, new_alloc)
                valid = TRUE;
          }
 
-         if (valid && (strcmp(++s1,++s2) == 0))
+         if (valid && (MyStrICmp(++s1,++s2) == 0))
             return (i);
       }
 
@@ -1510,10 +1543,12 @@ int MyReadPixmapFile (file_name, image_w_return, image_h_return, w_return,
             while (*c_ptr != '\0')
             {
                while (*c_ptr == ' ' || *c_ptr == '\t') c_ptr++;
-               if (*c_ptr == 'c') break;
-               while (*c_ptr!=' ' && *c_ptr!='\0') c_ptr++;
-               while (*c_ptr == ' ') c_ptr++;
-               while (*c_ptr!=' ' && *c_ptr!='\0') c_ptr++;
+               if ((*c_ptr == 'c' || *c_ptr == 'm') &&
+                     (c_ptr[1]==' ' || c_ptr[1]=='\t'))
+                  break;
+               while (*c_ptr!=' ' && *c_ptr!='\t' && *c_ptr!='\0') c_ptr++;
+               while (*c_ptr == ' ' || *c_ptr == '\t') c_ptr++;
+               while (*c_ptr!=' ' && *c_ptr!='\t' && *c_ptr!='\0') c_ptr++;
             }
             if (*c_ptr++ == '\0')
             {
@@ -1521,15 +1556,34 @@ int MyReadPixmapFile (file_name, image_w_return, image_h_return, w_return,
                      *pixmap_return, *image_return);
                return (BitmapFileInvalid);
             }
-            while (*c_ptr == ' ') c_ptr++;
-            for (ptr = c_ptr; *ptr!=' ' && *ptr!= '"' && *ptr!='\0'; ptr++) ;
+            while (*c_ptr == ' ' || *c_ptr == '\t') c_ptr++;
+            for (ptr = c_ptr; *ptr !=  '"' && *ptr != '\0'; ptr++)
+               if ((*ptr == ' ' || *ptr == '\t') && (
+                     strncmp(&ptr[1], "m ", 2) == 0 ||
+                     strncmp(&ptr[1], "s ", 2) == 0 ||
+                     strncmp(&ptr[1], "g ", 2) == 0 ||
+                     strncmp(&ptr[1], "g4 ", 3) == 0))
+                  break;
             if (*ptr == '\0')
             {
                FreeAuxData (fp, i, *color_char, *color_str, *pixels,
                      *pixmap_return, *image_return);
                return (BitmapFileInvalid);
             }
-            *ptr = '\0';
+            while (ptr >= c_ptr)
+            {
+               if (*ptr == ' ' || *ptr == '\t' || *ptr == '"')
+                  ptr--;
+               else
+                  break;
+            }
+            if (ptr < c_ptr)
+            {
+               FreeAuxData (fp, i, *color_char, *color_str, *pixels,
+                     *pixmap_return, *image_return);
+               return (BitmapFileInvalid);
+            }
+            *(++ptr) = '\0';
             strcpy (s, c_ptr);
             if ((index = QuickFindColorIndex (s,&new_alloc)) == INVALID)
             {
@@ -1645,7 +1699,7 @@ int MyReadPixmapFile (file_name, image_w_return, image_h_return, w_return,
    return (BitmapSuccess);
 }
 
-#define GETVALUE(val,name) ScanValue("%d", (char *) &(val), name, "xpixmap")
+#define GETVALUE(val,name) ScanValue("%d", &(val), name, "xpixmap")
 
 void ReadXPmObj (FP, Inbuf, ObjPtr)
    FILE			* FP;
